#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: accCreateVCCTables.sql,v 1.45 2004/02/13 14:13:05 vltsccm Exp $"
#
# who       when      what
# --------  --------  ---------------------------------------------------
# clevin  12/04/96  created
# clevin  17/03/97  table "subnet" copied from accCreateACCTables.sql
# psivera 04/09/00  increased bsp lenght to 30
# psivera 28/11/01  changed 'email' field to 'backplane_name'
# mkiekebu 2003-12-17 replaced primary keys by indexes
#

#************************************************************************
#   NAME	   
#       accCreateVCCTables.sql
#
#   SYNOPSIS	   
#       msql -h $ACC_HOST acc < accCreateVCCTables.sql
# 
#   DESCRIPTION    
#       SQL script to create the VCC database tables. These tables
#       are currently used by the ACC and VCC modules.
#
#   ENVIRONMENT
#       ACC_HOST - ACC host name
#
#   CAUTIONS
#       - The mSQL server (msqld) should be running.
#       - The ACC database must be created previously 
#         with the msqladmin command.
#       - Only authorized users can run this command.
#
#   SEE ALSO
#       accServer(1)
#
#------------------------------------------------------------------------
#

####################   ENTITIES  #######################

# RTAP-CCSlite environments
#
# Attributes used only by VCC :
# attic    -> 1 (hidden) | 0 (normal)
# env_type -> "LCU" | "RTAP" | "QSEMU"
#
create table prog_environment (
        progenv_name  char(20),
        description   char(80),
        env_type      char(6),
        tcp_port      int,
        station_name  char(30),
        station_name0 char(10),
        station_name1 char(10),
        station_name2 char(10),
        attic         int
)
\g

create unique index idx on prog_environment (progenv_name)\g

# Stations
#
# active_status -> accACTIVE | accNOT_ACTIVE
# station_type  -> accUSER_STATION | accCONTROL_STATION
#
# Attributes used only by VCC :
# station_arch  -> subdirectory name under $VXROOT/GNU/
# station_cpu   -> subdirectory name under $VLTROOT/vw/bin/
# station_bsp   -> subdirectory name under $VXROOT/config/
# attic         -> 1 (hidden) | 0 (normal)
#
create table station (
        station_name    char(30),
        station_type    char(30),
        subnet_name     char(30),
        ip_address      char(20),
        active_status   char(15),
        location        char(80),
        responsible     char(80),
        telephone       char(80),
        backplane_name  char(40),
        station_arch    char(16),
        station_cpu     char(16),
        station_bsp     char(30),
        attic           int

)
\g

create unique index idx on station (station_name)\g

# Subnets
#
# enable_status  -> accENABLED | accDISABLED
# telescope_unit -> UT1 | UT2 | UT3 | UT4 | VLTI | (null)
#
create table subnet (
        subnet_name     char(30),
        description     char(80),
        ip_address      char(20),
        router_name     char(20),
        router_ip_addr  char(15),
        enable_status   char(15),
        instrument      char(30),
        telescope_unit  char(10)
)
\g

create unique index idx on subnet (subnet_name)\g

####################   RELATIONSHIPS  #######################

# LCU / RTAP-CCSlite environment (N <--> 1)
#
# -- Only used to configure RTAP-CCSlite environments --
#
create table lcu_progenv (
	progenv_name  char(30),
        bootenv_name  char(30)
)
\g

#
# ___oOo___
