#ifndef _iracq_h
#define _iracq_h

/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: iracq.h,v 1.113 2005/09/13 07:27:09 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jstegmei  20/05/99  created
* jstegmei  10/12/99  exposure status definitions changed according to
*                     SPR VLTSW990315
* jstegmei  29/06/04  increased iracqFITS_MAX_BLOCKS to 64
*/

/************************************************************************
 *
 *  iracq.h - iracq definitions
 *
 *----------------------------------------------------------------------
 */

#ifdef SUN_COMP
#include <sys/select.h>
#endif

#include "icmd.h"
#include "icmdSock.h"
#include "sdmaInt.h"
#include "tcomInt.h"

#include "iracqExt.h"

#ifndef NOCCS
#include "evhStates.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifndef USE
#define USE(x) x=x
#endif

/*
 *  dictionary revision
 */
#define iracqDICTIONARY "ESO-VLT-DIC.IRACE"

/*
 *  module id
 */
#define iracqMOD_ID     "iracq"
#define iracqDTT_NAME   "iracqDtt"

/*
 *  verbose
 */ 
extern int iracqXtermFlag;
extern int iracqVflag;
#define iracqVB(format, arg...) iracqVbHandler(format, ##arg)
/*#define iracqVB(format, arg...)*/

/*
 *  default system configuration file
 */
#define iracqDEFAULT_CONFIG     "sys.cfg"

/*
 *  global unknown tag
 */
#define iracqUNKNOWN    -1

/*
 *  link timeouts (in seconds)
 */
#define iracqTCOM_TIMEOUT          20
#define iracqTCOM_TIMEOUT_SHORT     2
#define iracqTCOM_TIMEOUT_STOP     60
#define iracqTCOM_TIMEOUT_STOP_F   30
#define iracqTCOM_TIMEOUT_CLKP    120
#define iracqSDMA_TIMEOUT          20

/*
 *  maximum number of acquisition devices
 */
#define iracqMAX_ACQ_DEV   8

/*
 *  maximum number of readout modes
 */
#define iracqMAX_RM       32

/*
 *  maximum number of setup modes
 */
#define iracqMAX_MODES    64

/*
 *  maximum number of detectors in mosaic
 */
#define iracqMAX_DET      64

/*
 *  maximum number of parameters
 */
#define iracqMAX_PARAM   100

/*
 *  destination id
 */
#define iracqPARAM_EXT    1   /* destination id for external adds   */
#define iracqPARAM_CTRL   2   /* destination id for control process */
#define iracqPARAM_IVLD   0   /* invalid                            */
#define iracqPARAM_NOSSD -1   /* no ssd parameter                   */

/*
 *  maximum number of SSD-files per sequence
 */
#define iracqMAX_SSD     tcomISEQ_MAX_CLKP
#define iracqSSD_N       (iracqMAX_SSD/32)

/*
 *  burst modes
 */
#define iracqNO_BURST              0
#define iracqBURST_PROC            1
#define iracqBURST_BUF             2

/*
 *  fits-header constants
 */
#define iracqFITS_LINE_LEN        80
#define iracqFITS_MAXKEY_SIZE     48
#define iracqFITS_MAXVAL_SIZE     (iracqFITS_LINE_LEN - iracqFITS_MAXKEY_SIZE)
#define iracqFITS_BLOCK_LINES     36
#define iracqFITS_MAX_BLOCKS      64
#define iracqFITS_BLOCK_LEN       (iracqFITS_LINE_LEN * iracqFITS_BLOCK_LINES)
#define iracqFITS_MAX_SIZE        (iracqFITS_BLOCK_LEN * iracqFITS_MAX_BLOCKS)
#define iracqFITS_MAX_ADD         64
#define iracqFITS_MAX_DYN         64


/*
 *  parameter types
 */
#define iracqTYPE_INT      1
#define iracqTYPE_FLOAT    2
#define iracqTYPE_DOUBLE   3
#define iracqTYPE_STRING   4
#define iracqTYPE_LOGICAL  5
#define iracqTYPE_CONST    6
#define iracqTYPE_COMMENT  7

/*
 *  operation modes
 */
#define iracqMODE_REAL 1
#define iracqMODE_SIM  2 

/*
 *  naming types
 */
#define iracqREQ_NAMING  1
#define iracqSEQ_NAMING  2
#define iracqAUTO_NAMING 3

/*
 *  windowing modes
 */
#define iracqWIN_FREE      0
#define iracqWIN_CENTER    1
#define iracqWIN_PARTITION 2

/*
 *  state mappings for DMA-acquisition
 */
#define iracqACQSTATE_IDLE     sdmaSTAT_IDLE
#define iracqACQSTATE_RUNNING  sdmaSTAT_WAITING
#define iracqACQSTATE_READY    sdmaSTAT_READY
#define iracqACQSTATE_ERROR    sdmaSTAT_FAILURE

/*
 *  server states
 */
#ifndef NOCCS
#define iracqSTATE_OFF         evhSTATE_OFF
#define iracqSTATE_LOADED      evhSTATE_LOADED
#define iracqSTATE_STANDBY     evhSTATE_STANDBY
#define iracqSTATE_ONLINE      evhSTATE_ONLINE
#else
#define iracqSTATE_OFF         1
#define iracqSTATE_LOADED      2
#define iracqSTATE_STANDBY     3
#define iracqSTATE_ONLINE      4
#endif

/*
 *  server sub-states
 */
#define iracqSUBSTATE_IDLE     1
#define iracqSUBSTATE_BUSY     2
#define iracqSUBSTATE_ACTIVE   6
#define iracqSUBSTATE_ERROR    8

/*
 *  exposure states
 */
#define iracqEXP_UNDEFINED           0
#define iracqEXP_INACTIVE            1
#define iracqEXP_PENDING             2
#define iracqEXP_INTEGRATING         4
#define iracqEXP_PAUSED              8
#define iracqEXP_READING_OUT        16
#define iracqEXP_PROCESSING         32
#define iracqEXP_TRANSFERRING       64
#define iracqEXP_COMPL_SUCCESS     128
#define iracqEXP_COMPL_FAILURE     256
#define iracqEXP_COMPL_ABORTED     512

/*
 *  data transfer states
 */
#define iracqDSTATE_IDLE      0
#define iracqDSTATE_RECV      1
#define iracqDSTATE_WR        2


#define iracqEXP_ISACTIVE(status) ((status == iracqEXP_PENDING) || \
                                   (status == iracqEXP_INTEGRATING) || \
                                   (status == iracqEXP_PAUSED) || \
                                   (status == iracqEXP_READING_OUT) || \
                                   (status == iracqEXP_PROCESSING) || \
                                   (status == iracqEXP_TRANSFERRING))

/*
 *  result codes
 */
#define iracqWARNING    -3
#define iracqERR_IO_ERR -2
#define iracqFAILURE    -1
#define iracqSUCCESS     0

/*
 *  CLDC boards definitions
 */
#define iracqMAX_CLDC                6

/*
 *  status bus definitions
 */
#define iracqMAX_ADC                16
#define iracqADC_MAX_DELAY          15
#define iracqADC_MAX_SAMP          255
#define iracqADC_MAX_HEADER         15
#define iracqADC_DELAY          0x000f   /* register 0 */
#define iracqADC_FILTER1_ON     0x0010   /* register 0 */
#define iracqADC_FILTER2_ON     0x0020   /* register 0 */
#define iracqADC_FIFO_FULL      0x0040   /* register 0 */
#define iracqADC_FIFO_EMPTY     0x0080   /* register 0 */
#define iracqADC_HEADER         0x000f   /* register 1 */
#define iracqADC_ENABLE         0x0001   /* register 2 */
#define iracqST_BACK_END             0
#define iracqST_FRONT_END            1
#define iracqADC_REVCODE        0x0007   /* register 3 */
#define iracqADC_REV_0               0
#define iracqADC_REV_B               2
#define iracqADC_REV_C               3
#define iracqADC_REV_D               4
#define iracqADC_DAY            0x00f8   /* register 3 */
#define iracqADC_MONTH          0x000f   /* register 4 */
#define iracqADC_YEAR           0x00f0   /* register 4 */

/*
 *  data transfer tags
 */
#define iracqDTAG_OPEN       1    /* open connection to data server     */
#define iracqDTAG_CLOSE      2    /* close connection to data server    */
#define iracqDTAG_TYPES      3    /* set requested frame types          */
#define iracqDTAG_WIN        4    /* set output window for a frame type */
#define iracqDTAG_SID        5    /* set setup-id for a frame type      */
#define iracqDTAG_NAMING     6    /* set naming scheme                  */
#define iracqDTAG_ST         7    /* info about stored frames           */
#define iracqDTAG_START      8    /* start transfer                     */
#define iracqDTAG_STOP       9    /* stop transfer                      */
#define iracqDTAG_ERR       10    /* error event                        */
#define iracqDTAG_FITS      11    /* FITS header                        */
#define iracqDTAG_STARTTIME 12    /* adjust start time                  */
#define iracqDTAG_SIZE      13    /* allocate new frame buffer size     */
#define iracqDTAG_CAM       14    /* RTD camera info                    */
#define iracqDTAG_DT        15    /* data transfer status               */
#define iracqDTAG_PERF      16    /* data transfer performance          */
#define iracqDTAG_IMG       17    /* image processing instruction       */
#define iracqDTAG_PROC_ENA  18    /* enable processing callback         */
#define iracqDTAG_OFFSET    19    /* offset event                       */
#define iracqDTAG_STAT_ENA  20    /* enable/disable statistics          */

/*
 *  data transfer task control word for endianess
 */
#define iracqDTT_ENDIAN 0x1A2B3C4D

/*
 *  image processing methods
 */
#define iracqIMG_NOP         0
#define iracqIMG_ENABLE      1
#define iracqIMG_DISABLE     2
#define iracqIMG_CLEAR       4
#define iracqIMG_ROT90      (1 << 16) 
#define iracqIMG_ROT180     (2 << 16)
#define iracqIMG_ROT270     (3 << 16)
#define iracqIMG_FLIPX      (4 << 16)
#define iracqIMG_FLIPY      (5 << 16)

/*
 *  transfer options
 */
#define iracqDOP_ANY_TRANSFER  -1
#define iracqDOP_NO_TRANSFER   -2

/*
 *  binary FITS table
 */
typedef struct
{
  unsigned int    r;
  char            T;
  unsigned int    Xn;
  char            ttype[iracqFITS_MAXKEY_SIZE];
  char            tunit[16];
  char            tdisp[16];
  int             tnull;
  double          tscal;
  double          tzero;
  char            tnullX;
  char            tscalX;
  char            tzeroX;
} iracqTFORM;

typedef struct
{
  unsigned int    naxis2;
  unsigned int    pcount;
  unsigned int    tfields;
  iracqTFORM     *tform;
  char            name[iracqFITS_MAXVAL_SIZE];
  int             ver;
  int             level;
  void          **dataTable;
  void           *dataAdd;
} iracqBFITS;


typedef struct
{
  char host[64];
  int  port;
} iracqDT_OPEN;

typedef struct
{
  int type;
  int start_x;
  int start_y;
  int nx;
  int ny;
} iracqDT_WIN;

typedef struct
{
  int type;
  int setupId;
} iracqDT_SID;

typedef struct
{
  int type;
  int num;
} iracqDT_CAM;

typedef struct
{
  int  status;
  char fileName[256];
} iracqDT_ST;

typedef struct
{
  char erms[256];
} iracqDT_ERR;

typedef struct
{
  char dataPath[256];
  char seqBaseName[256];
  char reqFileName[256];
  int  type;
  int  genDataCube;
  int  extension;
} iracqDT_NAMING;

typedef struct
{
  int  expNo;
  int  seqNamingIndex;
  int  cnt[32];
  int  cubeType[32];
  char cubeName[32][64];
} iracqDT_EXP;

typedef struct
{
  int nBlocksReq;
  int bufferSize;
} iracqDT_FITS;


typedef struct
{
  int sizeX;
  int sizeY;
  int usize;
  int numDet;
  int detNum;
} iracqDT_SIZE;

typedef struct
{
  double X;
  double Y;
} iracqDT_OFFSET;

typedef struct
{
  int   status;                 /* data transfer status                     */
  float postProcMax;            /* post-processing time maximum (ms)        */
  float postProcAvrg;           /* post-processing time average (ms)        */
  float callBackMax;            /* post-processing time maximum (ms)        */
  float callBackAvrg;           /* post-processing time average (ms)        */
  float openMax;                /* file open time maximum (ms)              */
  float openAvrg;               /* file open time average (ms)              */
  float closeMax;               /* file close time maximum (ms)             */
  float closeAvrg;              /* file close average (ms)                  */
  float readMin;                /* read from net minimum (MBytes/s)         */
  float readAvrg;               /* read from net average (Mbytes/s)         */
  float writeMin;               /* write to disk minimum (MByets/s)         */
  float writeAvrg;              /* write to disk average (MBytes/s)         */
} iracqDT_PERF;

typedef struct
{
  float        preProcLoad;     /* pre-processor CPU load (%)               */
  float        preProcTime;     /* pre-processing time (ms)                 */
  float        inputFrameRate;  /* input frame rate (Hz)                    */
  int          inputFrameSize;  /* input frame size (bytes)                 */
  iracqDT_PERF dt;              /* data transfer statistics                 */
} iracqPERF;

/*
 *  additional fits (via application extensions)
 */
typedef struct
{
  char  keyword[64]; 
  int   type;
  void *addr;
  char  constVal[iracqFITS_MAXVAL_SIZE];
  char  comment[iracqFITS_LINE_LEN+1];
} iracqFITS_ADD;

/*
 *  additional fits (via command)
 */
typedef struct
{
  char  keyword[64]; 
  int   type;
  char  constVal[iracqFITS_MAXVAL_SIZE];
  char  comment[iracqFITS_LINE_LEN+1];
} iracqFITS_DYN;

/*
 *  additional fits (dynamic parameters)
 */
typedef struct
{
  char  keyword[64];
  char  comment[iracqFITS_LINE_LEN+1];
} iracqFITS_PAR;

/*
 *  general window type
 */
typedef struct
{
  float startX;    /* has to be float (per definition) */
  float startY;    /* has to be float (per definition) */
  int   nx;
  int   ny;
  int   adjustMode;
  int   stepX;
  int   stepY;
  int   partNx;
  int   partNy;
} iracqWIN;

typedef struct
{
  char          fileName[256];               /* voltage file name        */
  char          buffer[32*1024];             /* voltage file buffer      */
  int           bufferLen;                   /* voltage file buffer size */
  tcomCLK_OBJ   clkv[tcomCLDC_MAX_CLK_CHAN]; /* clock voltages           */
  tcomDC_OBJ    dcv[tcomCLDC_MAX_DC_CHAN];   /* dc voltages              */
  int           state;                       /* state of board           */
  int           vset;                        /* voltage file loaded      */
} iracqCLDC;

typedef struct
{
  char name[64];
  int  addr;
  int  header;
  int  enable;
  int  filter1;
  int  filter2;
  int  delay;
  int  samp;
} iracqADC;

typedef struct
{
  char date[16];
  int  rev;
  int  revSet;
} iracqADC_REV;

/*
 *  configuration structure
 */
typedef struct
{
  char         sclHost[64];          /* name of the front-end LCU          */
  char         sclDev[256];          /* scl-module interface device name   */
  char         sclBootFile[256];     /* boot-file for the scl-module       */
  char         sclProtType[64];      /* protocol type (socket/sbt)         */
  int          sclSrvPort;           /* scl protocol converter server port */
  int          pid;                  /* process-id of server               */
  int          opMode;               /* operation mode                     */
  int          cmdLink;              /* command link                       */
  int          rspeedF;              /* global readspeed factor            */
  int          rspeedA;              /* global readspeed add               */
  int          seqCont;              /* continuous run of sequencer        */
  int          seqCalc;              /* calculation flag for sequence      */
  int          seqMode;              /* sequencer mode                     */
  int          seqSuspend;           /* flag to suspend sequencer start    */
  int          seqInt;               /* flag to interrupt sequence at stop */
  int          numAdc;               /* number of ADCs in system           */
  iracqADC     adc[iracqMAX_ADC];    /* array of ADC id's                  */
  iracqADC_REV adcRev[iracqMAX_ADC]; /* ADC revisions                      */
  int          numCldc;              /* number of cldc-boards in system    */
  iracqCLDC    cldc[iracqMAX_CLDC];  /* array of CLDC-boards               */
  float        maxVoltDiff;          /* maximum telemetry difference       */
} iracqCTRL;

/*
 *  state structure
 */
typedef struct
{
  int          iseq;              /* sequencer state                */
  int          cldcSel;           /* state of selected cldc-borad   */
  int          exp;               /* exposure status                */
  iracqDT_PERF dt;                /* data transfer                  */
  int          acq;               /* acq state                      */
  int          system;            /* system state                   */
  int          sub;               /* system substate                */
  int          subOrig;           /* system substate (saved value)  */
  char         errString[256];    /* error state string             */
  int          booted;            /* booted state                   */
  int          seqStarted;        /* sequencer started              */
  int          seqStopped;        /* sequencer stopped              */
  int          acqStarted;        /* acquisition started            */
  int          dmaErr1;           /* dma error toggle               */
  int          dmaErr2;           /* dma error toggle               */
  int          error;             /* general error flag             */
} iracqSTATE;

typedef struct
{
  int          acqState[iracqMAX_ACQ_DEV];
  int          seqState;
} iracqSTATE_STORE;

/*
 *  acquisition device structure
 */
typedef struct
{
  int               id;            /* own id                                */
  int               state;         /* status                                */
  char              sdmaHost[64];  /* name of the SDMA pre-processor host   */
  char              sdmaDev[256];  /* SDMA device name                      */
  int               sdmaCmdPort;   /* SDMA command server port              */
  int               sdmaDataPort;  /* SDMA data server port                 */
  int               sdmaDataPortB; /* SDMA data server port B               */
  int               sdmaErrPort;   /* SDMA error stack port                 */
  int               hdr;           /* DMA interface header                  */
  char              acqProc[256];  /* acquisition process                   */
  char              acqParam[128]; /* additional command line parameters    */
  char              acqName[64];   /* name of readout mode                  */
  int               opMode;        /* operation mode                        */
  int               pid;           /* process-id of server                  */
  int               cmdLink;       /* command link                          */
  int               dataLink;      /* data link                             */
  int               errLink;       /* error stack link                      */
  int               numParam;      /* number of parameters                  */
  sdmaPARAM_NAMES   paramId;       /* parameter names                       */
  sdmaPARAM_VALUES  paramBuf;      /* parameter values                      */
  int               paramType[sdmaMAX_PARAMSIZE]; /* parameter types        */
  int               dttConnect;    /* enable/disable DTT connection         */
} iracqDEV;

/*
 *  parameter type structure
 */
typedef struct
{
  int         value;              /* parameter value               */
  char        name[32];           /* parameter name                */
  int         type;               /* parameter type                */
  int         id;                 /* destination id                */
  int         ssdId[iracqSSD_N];  /* ssd-id                        */
  int         dsup;               /* from dsup file                */
} iracqPARAM;

/*
 *  parameter backup structure
 */
typedef struct
{
  char        name[32];           /* parameter name                */
  char        value[64];          /* parameter value               */
  int         dsup;
} iracqPARAM_S;

/*
 *  SSD type structure
 */
typedef struct
{
  char       name[256];
  int        id;
  int        loopCmd[tcomISEQ_MAX_LOOP_COM_LEN]; /* loop command      */
  int        loopCmdLen;                         /* loop length       */
  double     time;                               /* execution time    */
  int        doLoad;
} iracqSSD;

/*
 *  SEQ type structure
 */
typedef struct
{
  char       name[256];                          /* file name         */
  iracqSSD   frame[iracqMAX_SSD];                /* frames            */
  int        numFrames;                          /* number of frames  */
  char       buffer[1024*64];                    /* buffer for file   */
  int        bufferLen;                          /* length of buffer  */
  int        loopCmd[tcomISEQ_MAX_LOOP_COM_LEN]; /* loop command      */
  int        loopCmdLen;                         /* loop length       */
} iracqSEQ;

/*
 *  frame status structure
 */
typedef struct
{
  int         type;
  char        name[64];
  int         store;
  int         storeOld;
  int         gen;
  int         breakCnt;
  int         disp;
  int         valid;
  char        paramName[32];
} iracqFSTAT;

/*
 *  file cache structure
 */
#define iracqMAX_FILE_CACHE 16
typedef struct
{
  char        name[256];
  int         size;
  int         modTime;
  char       *buffer;
} iracqFILE_CACHE;

/*
 *  system confguration
 */
typedef struct
{
  iracqDEV     *dev;               /* acquisition devices                */
  char          cfgBase[256];      /* base path for configuration        */
  char          user[64];          /* current instrument user            */
  char          instance[64];      /* server instance                    */
  char          dicRev[32];        /* dictionary revision                */
  char          configFile[256];   /* system configuration file          */
  int           numDev;            /* number of acquisition devices      */
  int           opMode;            /* operation mode                     */
  char          voltageFile[256];  /* current voltage file               */
  int           cldc;              /* current cldc board number          */
  char          clkFile[256];      /* current clock pattern file         */
  char          pdef[256];         /* current parameter definition file  */
  char          acqName[64];       /* current readout mode name          */
  int           acqId;             /* current readout mode id            */
  int           burst;             /* burst mode                         */
  int           burstNum;          /* burst size                         */
  int           setupId;           /* setup id                           */
  iracqPARAM    param[iracqMAX_PARAM];       /* parameters               */
  int           numParam;                    /* number of parameters     */
  iracqPARAM_S  paramStore[iracqMAX_PARAM];  /* saved parameters         */
  int           numParamStore;               /* number of saved param    */
  iracqPARAM_S  paramDefault[iracqMAX_PARAM];/* default parameters       */
  int           numParamDefault;             /* number of default param  */
  tcomPAT_OBJ   subPat[tcomISEQ_MAXPAT];     /* subpatterns              */
  int           numPat;                      /* number of subpatterns    */
  int           patIndex[tcomISEQ_MAXPAT];   /* subpattern index         */
  int           seqLoaded;                   /* sequence loaded          */
  tcomFRAME_OBJ seqFrame[64];                /* sequencer frames         */
  int           numSeqFrame;                 /* number of seq. frames    */
  tcomCLKP_OBJ  clkp[tcomISEQ_MAX_CLKP];     /* clock patterns           */
  int           numClkp;                     /* number of clock patterns */
  icmdVAR       var[tcomMAX_VAR];            /* variables                */
  tcomEXPR      exprList[512];               /* expression list          */
  int           exprListLen;                 /* expression list length   */
  int           detConfLoaded;               /* detector config loaded   */
  int           detConfChanged;              /* detector config loaded   */
  char          dttName[256];        /* data transfer task name          */
  int           dttCmdLink;          /* data transfer task command link  */
  int           dttEvtLink;          /* data transfer task event link    */
  int           dttPid;              /* data transfer task process id    */
  char          dttHost[256];        /* data transfer task host          */
  int           dttProcEnabled;      /* data transfer proc. CB on/off    */
  int           dttStatEnabled;      /* data transfer statistics on/off  */
  iracqFSTAT    frame[32];           /* frame status                     */
  int           applyWin;            /* apply window to frame type       */
  char          dataPath[256];       /* path for data files              */
  char          seqBaseName[256];    /* sequence base name               */
  int           seqNamingIndex;      /* sequence naming index            */
  int           seqNamingIndexSet;   /* sequence naming index from setup */
  char          reqFileName[256];    /* requested file name              */
  char          lastFile[256];       /* name of last file stored         */
  int           genDataCube;         /* store data in data cube          */
  int           extension;           /* store data as image extension    */
  int           namingType;          /* naming type                      */
  int           setBufSize;          /* flag to update data buffer size  */
  int           expNo;               /* exposure number                  */
  int           expNoSet;            /* exposure number from setup       */
  float         expTime;             /* exposure time                    */
  int           nint;                /* number of INT per exposure       */
  float         minDit;              /* minimum DIT                      */
  float         dit;                 /* current DIT                      */
  int           ditSet;              /* new dit has been set             */
  float         intTime;             /* current DIT                      */
  char          fitsBuf[iracqFITS_MAX_SIZE+4]; /* global FITS-buffer     */
  int           fitsBufLen;          /* FITS-buffer size                 */
  int           numFitsBlocks;       /* number of FITS-blocks to create  */
  iracqFITS_ADD fitsAdd[iracqFITS_MAX_ADD]; /* additional FITS-keys      */
  int           fitsAddLen;          /* number of additional FITS-keys   */
  iracqFITS_DYN fitsDyn[iracqFITS_MAX_DYN]; /* dynamic FITS-keys         */
  int           fitsDynLen;          /* number of dynamic FITS-keys      */
  iracqFITS_PAR fitsPar[iracqMAX_PARAM]; /* dynamic param FITS-keys      */
  int           fitsParLen;          /* number of dynamic param FITS-keys*/
  int           extendedFits;        /* flag to use extended FITS-header */
  int           noNcReset;           /* no counter reset at exp.-start   */
  iracqSEQ      sequence;            /* readout sequence                 */
  int           ssdParse;            /* do parse the ssd files           */
  int           voltLoad;
  int           calcAllTimes;        /* do (re-)calculate readout times  */
  iracqFILE_CACHE fileCache[iracqMAX_FILE_CACHE];
  int           nextCache;
  int           cldcOn;              /* enable CLDC at online command    */
  int           startOn;             /* start at online command          */
  int           seqNoSuspend;        /* prevent sequencer suspension     */
  int           autoOnline;          /* go on-line at start-up           */
  int           testRun;             /* flag for test-run                */
  int           rtdInt;              /* flag for internal RTD-channel    */
  char          rtdDisp[64];         /* display for RTD                  */
  char          rtdHost[64];         /* host for RTD                     */
  char          rtdCmd[128];         /* host for RTD                     */
  int           numDataClient;       /* maximum number og data clients   */
  int           cubeType[32];        /* frame types in cube              */
  char          cubeName[32][64];    /* name of cube                     */
  int           ssdDit[iracqSSD_N];  /* ssd-id of DET.DIT                */
  int           ssdRsp[iracqSSD_N];  /* ssd-id of DET.RSPEED             */
} iracqSYS;

/*
 *  readout modes
 */
typedef struct
{
  int    id;                /* unique read-mode id                */
  char   name[64];          /* read-mode name                     */
  char   acqProc[iracqMAX_ACQ_DEV][256]; /* acquisition process   */
  char   seqFile[256];      /* readout sequence file              */
  char   pdef[256];         /* parameters definition file         */
  int    hwWin;
} iracqRM;

/*
 *  detector modes
 */
#define iracqMAX_DM       16    /* maximum number of modes          */
#define iracqMAX_DM_KEY   64    /* maximum number keywords per mode */
#define iracqMAX_DM_SIZE  ((iracqMAX_DM_KEY * 128) + iracqMAX_DM_KEY)

typedef struct
{
  char   name[64];                 /* mode name                */
  char   setup[iracqMAX_DM_SIZE];  /* setup buffer             */
  char  *key[iracqMAX_DM_KEY];     /* keyword list             */
  int    numKey;                   /* number of keywords       */
} iracqDM;

typedef struct
{
  int   x;
  int   y;
  int   no;
  float pixSpace;
  int   live;
  char  id[64];
} iracqCHIP;

/*
 *  detector configuration
 */
typedef struct
{
  iracqCHIP chip[iracqMAX_DET]; /* chip info for mosaics           */
  char      name[64];           /* chip name                       */
  char      type[64];           /* chip type                       */
  int       noChan;             /* number of data channels         */
  char      configFile[256];    /* detector configuration file     */
  int       numCldc;            /* number of CLDC-boards per DET.  */
  int       cldc[iracqMAX_CLDC];/* cldc-board number list          */
  char      cldcList[64];       /* cldc-board number list (string) */
  char      clkFile[256];       /* clock pattern file              */
  iracqRM  *rm;                 /* readout modes                   */
  int       numRm;              /* number of read modes            */
  int       rmDefault;          /* id of default read mode         */
  int       rmId;               /* read-mode process id            */
  int       sizeX;              /* chip size x-direction           */
  int       sizeY;              /* chip size y-direction           */
  int       usize;              /* additional space                */
  int       numDet;             /* number of detector instances    */
  int       dispNo;             /* selected instance for DTT-disp  */
  int       storeNo;            /* selected instance for DTT-store */
  iracqWIN  win;                /* current window                  */
  int       hwWindow;           /* flag for HW-window              */
  int       hwWindowAllowed;    /* flag for HW-window allowed      */
  iracqDM   dm[iracqMAX_DM];    /* detector modes                  */
  char      modeName[64];       /* detector mode name              */
  int       modeNum;            /* detector mode number            */
} iracqDET;

/*
 *  data event structure
 */
typedef struct
{
  char   fileName[256];
  char   erms[256];
  int    errorFlag;
  int    dtStatus;
  double offsetX;
  double offsetY;
  int    doOffset;
} iracqDATA_EVT;


/*
 *  prototypes
 */

/*
 *  file access
 */
const char *iracqGetFileExt(const char *);
const char *iracqProcName(const char *);
int    iracqReplaceEnv(char *, char *);
int    iracqLoadFile(const char *, char *, int *, char *);
int    iracqWriteFile(const char *, char *, int, char *);
int    iracqLoadCache(const char *, char **, int *, int, char *);
int    iracqDownloadData(int, int, const char *, char *);
int    iracqDownloadDataRaw(int, int, char *, int, char *);
int    iracqIsDir(const char *);
int    iracqWritePerm(const char *);
int    iracqReadPerm(const char *);
double iracqDiskSpace(char *);
int    iracqFindIndex(const char *, const char *, int, char *);

/*
 *  cldc
 */
int    iracqDac(char, char *);
int    iracqSetCldc(int, char *);
int    iracqSetVoltages(int, const char *, int, char *);
int    iracqSaveVoltages(const char *, char *);
int    iracqCldcReadVolt(int, float *, char *);
int    iracqCldcSetVolt(int, float, char *);
int    iracqCldcSetDcVolt(int, float, char *);
int    iracqCldcSetClkHiVolt(int, float, char *);
int    iracqCldcSetClkLoVolt(int, float, char *);
int    iracqCldcTel(int, float *, char *);
int    iracqCldcTelDc(int, float *, float *, char *);
int    iracqCldcTelClkLo(int, float *, char *);
int    iracqCldcTelClkHi(int, float *, char *);
int    iracqCldcCheck(char *);


/*
 *  window functions
 */
int    iracqAdjustSwWin(int *, int *, int *, int *);
int    iracqAdjustHwWin(int *, int *, int *, int *);
int    iracqChipOffset(int, int, char *);
int    iracqSetWin(int, int, int, int, int, char *);

/*
 *  setup functions
 */
void   iracqUpdateSimTime(void);			
int    iracqSetOpMode(int, char *);
int    iracqSetupId(int, char *);			
int    iracqSetDit(float, float *, char *);			
int    iracqSetRspeed(int, int, char *);			
int    iracqSetSeqMode(int, char *);
int    iracqSetDataPath(const char *, char *);			
int    iracqSetSeqBase(const char *, char *);			
int    iracqSetReqName(const char *, char *);			
int    iracqSetNamingType(int, char *);			
int    iracqSetDataCube(int, char *);			
int    iracqSetNamingScheme(char *);			
int    iracqSetBurst(int, char *);			
int    iracqSetReadoutMode(int, char *);			
int    iracqSetAcqProc(const char *, const char *, char *);
int    iracqSetClk(const char *, char *);
int    iracqLoadSeq(const char *, char *);
int    iracqLoadSeqExt(const char *, char *);
int    iracqLoadSeqProg(int *, int, char *);
int    iracqClearClkp(char *);
int    iracqLoadClkp(int, int *, int, char *);
int    iracqSetSeq(const char *, char *);
int    iracqLoadSSD(iracqSSD *, char *);
int    iracqGetSSD(const char *, char *);
int    iracqDetSetup(char *);
int    iracqUsrCmd(int, const char *, char *, char *);
int    iracqFrameTime(int, char *);
double iracqMinCv(int, char *);

/*
 *  parameter handling
 */
void   iracqClearParam(int);
int    iracqParamExists(const char *);
int    iracqAddParam(const char *, int, void *, int, int, char *);
int    iracqAddFreeParam(const char *, int, void *, char *);
int    iracqGetParamNum(const char *, int);
int    iracqGetParamSetup(char *);
int    iracqGetParam(const char *, char *, char *);
int    iracqParamType(const char *);
int    iracqSetParam(const char *, const char *, char *);
int    iracqSetParamList(char *, int, int, char *);
int    iracqApplyAcqParam(char *);
void   iracqParamStore(void);
void   iracqParamRestore(int);
void   iracqParamDefault(void);
int    iracqParamChanged(const char *, const char *);
int    iracqHandleSetupCmd(const char *, const char *, int  *, char *);
int    iracqHandleSetupList(char **, int, char *);
int    iracqHandleStatusCmd(const char *, char *, int *, char *);

/*
 *  ADC setup
 */
int    iracqAdcReset(char *);
int    iracqAdcInit(char *);
int    iracqAdcUpdate(char *);
int    iracqAdcGetRev(char *erms);
int    iracqAdcSetFilter1(int, int, char *);
int    iracqAdcSetFilter2(int, int, char *);
int    iracqAdcSetEnable(int, int, char *);
int    iracqAdcSetDelay(int, int, char *);
int    iracqAdcSetSamp(int, int, char *);
int    iracqAdcSetHeader(int, int, char *);
int    iracqAdcGetFilter1(int, int *, char *);
int    iracqAdcGetFilter2(int, int *, char *);
int    iracqAdcGetEnable(int, int *, char *);
int    iracqAdcGetDelay(int, int *, char *);
int    iracqAdcGetSamp(int, int *, char *);
int    iracqAdcGetHeader(int, int *, char *);
int    iracqAdcGetAddress(int, int *, char *);
int    iracqAdcSetGroupFilter1(char *, int, char *);
int    iracqAdcSetGroupFilter2(char *, int, char *);
int    iracqAdcSetGroupDelay(char *, int, char *);
int    iracqAdcSetGroupSamp(char *, int, char *);
int    iracqAdcSetGroupHeader(char *, int, char *);
int    iracqAdcSetGroupEnable(char *, int, char *);
int    iracqAdcGetGroupFilter1(char *, int *, char *);
int    iracqAdcGetGroupFilter2(char *, int *, char *);
int    iracqAdcGetGroupDelay(char *, int *, char *);
int    iracqAdcGetGroupSamp(char *, int *, char *);
int    iracqAdcGetGroupHeader(char *, int *, char *);
int    iracqAdcGetGroupEnable(char *, int *, char *);

/*
 *  frame setup
 */
int    iracqAddFrame(int, const char *, int, int, int, const char *, char *);
int    iracqInitFrame(char *);
int    iracqFramesAvailable(int *, char *);
int    iracqFrameAvailable(int);
int    iracqFrameGen(const char *, int, char *);
int    iracqFrameStore(const char *, int, char *);
int    iracqFrameWin(const char *, int, char *);
int    iracqFrameStored(const char *);
int    iracqFrameDisp(const char *, char *);
void   iracqFrameList(char *list);
void   iracqUpdateFrameInfo(void);
int    iracqFrameBreakCnt(const char *, int, char *);
int    iracqFrameDefault(const char *, int, int, int, char *);

/*
 *  run control
 */
int    iracqStart(char *);
int    iracqStop(char *);
int    iracqStartAcqProc(int, char *);
int    iracqStartAcq(char *);
int    iracqStopAcq(char *);
int    iracqResetCnt(char *);
int    iracqStartSeq(char *);
int    iracqStopSeq(char *);
int    iracqStartExp(char *);
int    iracqStopExp(char *);
int    iracqEndExp(char *);
int    iracqPause(int, char *);

/*
 *  fits header
 */
void   iracqFitsInit(char *, int *);
void   iracqFitsAppend(char *, int *, char *);
int    iracqFitsEnd(char *, int *, int, char *);
int    iracqFitsGen(char *, const char *, int, int, double, const char *, 
		    const char *, char *);
int    iracqFitsGetSize(char *);
int    iracqFitsGetKey(char *, char *, char *, char *);
int    iracqFitsCheckKey(char *, int, const char *, char *);
int    iracqFitsCombine(char *, int  *, char *, int, char *);
int    iracqFitsRead(char *, int, const char *, int *, char *, char *);
int    iracqFitsWrite(char *, int *, char *, char *);
char  *iracqFitsStrip(char *, int *, int *, char *);
int    iracqSetFits(char *);
int    iracqAddFits(const char *, int, void *, const char *, char *);
int    iracqAddFitsDynPar(const char *, const char *, char *);
int    iracqAddFitsDyn(const char *, int, const char *, const char *, char *);
void   iracqClearFitsDyn(void);
void   iracqExtendedFits(int);

/*
 *  acquisition process
 */
int    iracqInitAcq(int, int, char *);
int    iracqCallAcqProc(iracqDEV *, int, int, int, char *);
int    iracqAbortAcqProc(iracqDEV *, char *);
int    iracqDttClose(char *erms);
int    iracqDttOpen(char *erms);

/*
 *  data transfer interface
 */
int    iracqExecDtt(char *, char *, int *, int *, int *, char *);
int    iracqAbortDtt(char *, int, char *);
int    iracqDataOpen(int, int, char **, int *, char *);
int    iracqDataClose(int, char *);
int    iracqDataStart(int, char *);
int    iracqDataAdjustStartTime(int, char *);
int    iracqDataStop(int, char *);
int    iracqDataStatEnable(int, int, char *);
int    iracqDataGetEvt(int, iracqDATA_EVT *, char *);
int    iracqDataSetReq(int, int, char *);
int    iracqDataBufSize(int, int, int, int, int, int, char *);
int    iracqDataWin(int, int, int, int, int, int, char *);
int    iracqDataId(int, int, int, char *);
int    iracqDataNaming(int, const char *, const char *, const char *, 
		       int, int, int, char *);
int    iracqDataFits(int, char *, int, char *);
int    iracqDataPerf(int, iracqDT_PERF *, char *);
int    iracqDataCamType(int, int, int, char *);
int    iracqDataImgOp(int, int, char *);
int    iracqDataProcEnable(int, int, char *);

/*
 *  configuration
 */
int    iracqDefaultSysConf(char *);
int    iracqSysConf(const char *, int *, char *);
int    iracqDetConf(const char *, int *, char *);
int    iracqGetDsup(const char *, char *);
int    iracqGetMsup(const char *, char **, int *, char *);
int    iracqNumDataLink(void);

/*
 *  state handling
 */
int    iracqSeqStatus(void);
int    iracqAcqProcStatus(int);
int    iracqAcqStatus(void);
int    iracqGetStatus(char *);
int    iracqStoreState(iracqSTATE_STORE *, char *);
int    iracqRestoreState(iracqSTATE_STORE, char *);
void   iracqSetState(int);
void   iracqSetSubState(int);
void   iracqErrState(const char *);
void   iracqSetReadMask(fd_set *);
int    iracqHandleCtrlEvt(char *);
int    iracqHandleAcqCmdEvt(int, char *);
int    iracqHandleAcqErrEvt(int, char *);
int    iracqHandleDataEvt(char *);
int    iracqCheckTcomResult(int);
int    iracqCheckSdmaResult(int, int);
int    iracqCheckDataResult(int res);
void   iracqDmaRecover(void);

/*
 *  statistics
 */
int    iracqStatistics(iracqPERF *, char *);
float  iracqPerf(void);

/*
 *  status mapping functions
 */
void   iracqSeqStat2Str(char *, int);
void   iracqSeqStat2StrRaw(char *, int);
void   iracqCldcStat2Str(char *, int);
void   iracqAcqStat2Str(char *, int);
void   iracqSysStat2Str(char *, int);
void   iracqExpStat2Str(char *, int);
void   iracqSubStat2Str(char *, int);
void   iracqOpMode2Str(char *, int);
void   iracqSprintSysState(char *);
void   iracqSprintSubState(char *);
void   iracqSprintExpState(char *);
void   iracqSprintAcqState(char *);
void   iracqSprintCldcState(char *);
void   iracqSprintSeqState(char *);
void   iracqSprintSeqStateRaw(char *);
void   iracqSprintOpMode(char *);


/*
 *  verbose mode
 */
void   iracqDefaultVbHandler(const char *, ...);

/*
 *  online/initialization
 */
int    iracqInit(char *);
int    iracqFree(void);
int    iracqLoadDic(char *);
int    iracqOnline(int, char *);
int    iracqStandby(char *);
int    iracqAbort(char *);
int    iracqReset(char *);
void   iracqRtapEnv(int, char **, char *);

/*
 *  global variables
 */
extern iracqSYS      *iracqSys;
extern iracqCTRL     *iracqCtrl;
extern iracqSTATE    *iracqState;
extern iracqDET      *iracqDet;
extern int            iracqInitDone;
extern int            iracqUpdateAdc;
extern int            iracqUpdateVolt;
extern int            iracqUpdateRm;
extern int            iracqUpdateParam;

/*
 *  external state handler
 */
typedef void (*iracqSTATE_HANDLE)(int, int);
extern iracqSTATE_HANDLE iracqStateHandler;

/*
 *  external data event handler
 */
typedef void (*iracqDATA_HANDLE)(iracqDATA_EVT *);
extern iracqDATA_HANDLE iracqDataHandler;

/*
 *  external verbose handler
 */
typedef void (*iracqVB_HANDLE)(const char *, ...);
extern iracqVB_HANDLE iracqVbHandler;

/*
 *  external handler for sequencer program
 */
typedef int (*iracqSEQ_HANDLE)(const char *, int *, int *, char *);
extern iracqSEQ_HANDLE iracqSeqHandler;

/*
 *  external parameter setup handler
 */
typedef int (*iracqPAR_HANDLE)(const char *, char *, int *, char *);
extern iracqPAR_HANDLE iracqParamHandler;

/*
 *  external parameter status handler
 */
extern iracqPAR_HANDLE iracqParamStatHandler;


#ifdef __cplusplus
}
#endif

#endif /*!_iracq_h*/
