#ifndef _iracqDTT_h
#define _iracqDTT_h

/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: iracqDTT.h,v 1.113 2005/09/13 07:27:08 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jstegmei  27/07/99  created
*/

/************************************************************************
 *
 *  iracqDTT.h - data transfer task class definition (NOCCS)
 *
 *----------------------------------------------------------------------
 */


#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "rtdSem.h"
#include "rtdImageEvent.h"

#include "icmdSock.h"
#include "sdmaInt.h"
#include "iracq.h"
#include "iracqIMG.h"

#ifndef USE
#define USE(x) x=x
#endif

/*
 *  default data server
 */
#define iracqDTT_DEFAULT_DATA_SERVER "teb"

/*
 *  data transfer operation mode
 */
#define iracqDTT_MODE_STAND_ALONE  1  /* stand alone mode */
#define iracqDTT_MODE_CMD          2  /* command mode     */

/*
 *  maximum number of connections
 */
#define iracqDTT_MAX_CMD  4      /* maximum number of command connections */
#define iracqDTT_MAX_EVT  4      /* maximum number of event connections   */
#define iracqDTT_MAX_DATALINK 16 /* maximum number of data links          */

/*
 *  break condition
 */
#define iracqDTT_NO_BREAK  0x00000000

/*
 *  default data cube size
 */
#define iracqDTT_DEFAULT_CUBE_SIZE (1024 * 1024 * 512)

/*
 *  camera ringbuffer size
 */
#define iracqDTT_NUM_BUF 2

/*
 *  frame info structure
 */
typedef struct
{
  int         type;
  int         setupId;
  char        name[64];
  int         start_x;
  int         start_y;
  int         nx;
  int         ny;
  int         expCnt[iracqDTT_MAX_DATALINK];
  int         breakStat;
  int         breakCnt;
  FILE       *fdCube;
  int         maxCubeSize;
  int         numFrame;
  int         numCube;
  char        fileName[256];
  int         dataType;
  int         recNx;
  int         recNy;
  int         recStartX;
  int         recStartY;
} iracqDTT_FINFO;

class iracqDTT {
  public:

  iracqDTT();             // constructor  
  virtual ~iracqDTT();    // destructor 
  
  int verbose;
  int log;
  int active;

  char           *Version();
  int             ParseInputPars(int, char **);
  void            PrintUsage(const char *);
  const char     *ProcName(const char *);
  char           *Instance();

  // asynchronous handlers
  virtual void    Verbose(const char *, ...);
  virtual void    ExitHandler(int);
  void            VbHandlerGen(const char *, ...);
  void            ExitHandlerGen(int);

  // initialization routine
  int             Init(char*);
  void            FreeBuffers();

  // server main loop
  int             MainLoop();

  // socket server
  int             OpenServer(char *);
  int             CloseServer();
  int             CloseData();
  int             CloseClient(int);
  int             ConnectCmd(char *);
  int             ConnectEvt(char *);
  int             GetCmdSock();
  int             GetEvtSock();
  int             GetDataSock(int);
  void            HandleDataEvent(int);
  int             HandleCmd(int, char *);
  int             CheckStdIn();
  virtual void    AttachDataLink(int);
  virtual void    DetachDataLink(int);

  // data storage
  char           *DataPath();
  char           *FileName();
  int             ReplaceEnv(char *, char *);
  void            FrameHandled();
  int             Display(sdmaFRAME_T, char *);

  // detector information
  int             NumDet();

  // FITS header
  int             FitsAppend(const char *, int, const char *, char *);
  int             FitsAppend(const char *, float, const char *, char *);
  int             FitsAppend(const char *, double, const char *, char *);
  int             FitsAppend(const char *, const char *, const char *, char *);
  int             FitsAppend(const char *, const char *, char *);
  int             AddFits(const char *, int, void *, const char *, char *);
  int             AddFits(const char *, int *, const char *, char *);
  int             AddFits(const char *, float *, const char *, char *);
  int             AddFits(const char *, double *, const char *, char *);
  int             AddFits(const char *, char *, const char *, char *);
  int             AddFits(const char *, char *);
  int             BinFitsInit(iracqBFITS *bfits, int tfields);
  void            BinFitsClear(iracqBFITS *bfits);

  // application specific frame processing
  char           *GetFitsBuf();
  char           *ExpStartTime();
  double          ExpStartTimeMJD();
  char           *StartTransferTime();
  int             ExpStatus();
  int             ExpNo();
  int             NumFitsBlock();
  void            GetFrameName(int, char *);
  int             DetIdx();
  int             DetNum();
  int             SendFileEvent(const char *, char *);
  virtual int     ProcessFrame(sdmaFRAME_T *, char *);

  protected:


  private:
  // version string
  char            versionString_[256];

  // process name
  char            procName_[64];
  char            instance_[64];

  // socket handling
  char            host_[256];
  int             port_;
  char            srvName_[iracqDTT_MAX_DATALINK][256];     
  int             remoteFd_;
  int             srvCmdPort_;                
  int             srvCmdSock_;                
  int             cmdSock_[iracqDTT_MAX_CMD];
  int             srvEvtPort_;                
  int             srvEvtSock_;                
  int             evtSock_[iracqDTT_MAX_EVT]; 
  int             dataLink_[iracqDTT_MAX_DATALINK];
  int             numDataLink_;
  fd_set          readMask_;                  
  fd_set          readFds_;                   
  int             srvOpen_;
  void            AttachDataLink_(int);
  void            DetachDataLink_(int);

  // operational modes
  int             transferMode_;
  int             opMode_; 

  // exposure start time
  char            expStartTime_[64];;         
  double          expStartTimeMJD_;
  void            MJD2UTC_(double, char *);

  // start transfer time
  char            startTransferTime_[64];;         

  // file naming
  char            seqBaseName_[256]; 
  char            reqFileName_[256];
  char            propFileName_[256];
  int             namingType_;     
  int             seqNamingIndex_; 
  int             genDataCube_;    
  char            dataPath_[512];  
  char            user_[64];  

  // internal flags
  int             transfer_;  
  int             newSetup_;
  int             stopFlag_;           
  int             procEnabled_;

  // frame request
  int             sqPix_;
  char            reqPending_[iracqDTT_MAX_DATALINK];  
  sdmaREQ         request_;   
  iracqDTT_FINFO *dttFrame_;    
  char           *memPtr_;
  int             frameBufSize_;

  // exposure control
  unsigned int    breakCond_;   
  unsigned int    breakStat_;   
  int             expStatus_;   
  int             expNo_;

  // FITS header
  int             numFitsBlock_;  
  char           *fitsLoc_;      
  int             fitsLocLen_;   
  char           *fitsGlob_;     
  int             fitsGlobLen_;   
  iracqFITS_ADD   fitsAdd_[iracqFITS_MAX_ADD];
  int             fitsAddLen_;        
  char            padFitsBlock_[iracqFITS_BLOCK_LEN];
  iracqBFITS     *bfits_;
  int             numBfits_;

  // windowing
  int             sx_;
  int             sy_;
  int             nx_;
  int             ny_;
  int             sizeX_;
  int             sizeY_;
  int             numDet_;
  int             numDetOrig_;
  int             detNum_;
  int             detIdx_;

  // frame handling
  int             FrameIdxByType_(int);
  int             FrameIdxByName_(const char *);
  int             FrameSize_(sdmaFRAME_H);
  int             frameHandled_;
  int             HandleRecErr_();
  int             OpenData_(char *);
  int             Connect_(int, char *);
  int             Disconnect_(int, char *);
  int             DisconnectErr_(int, char *);
  int             Request_(int, char *);
  int             GetHeader_(int, sdmaFRAME_H *, char *);
  int             GetFrame_(int, sdmaFRAME_T *, int, char *);
  int             GetFrameBuf_(int, sdmaFRAME_T *, int, int, char *);
  int             AckFrame_(int, sdmaFRAME_T *, char *);
  int             PostOp_(sdmaFRAME_T *, int, char *);
  void            StoreFrame_(sdmaFRAME_T, int, int, int);
  int             ReceiveData_(int, char *);


  // command handler
  int             HandleTag_(int, int, char *);

  // events
  int             SendDataEvt_(int, const char *, char *);
  int             SendErrorEvt_(const char *);
  int             SendTransferEvt_(int, char *);
  int             SendOffsetEvt_(double, double, char *);

  // FITS header
  void            FitsInit_();
  int             FitsCreate_(sdmaFRAME_H, const char *, int, const char *, 
			      int, char *);    
  int             FitsCombine_(char *);
  int             FitsCubeCreate_(const char *, int, const char *, char *);    
  int             FitsCubeUpdate_(int, char *);
  int             FitsCubeClose_(int idx, char *erms);
  int             MergeFitsAdd_(char *);
  int             StoreBinFits_(int, iracqBFITS, char *);
  unsigned int    expFileCnt_;
  iracqCHIP       chip[iracqMAX_DET];

  // file storage
  int             StoreToFile_(sdmaFRAME_T, char *, int, int, char *);

  // data cube
  int             CubeIdxByType_(int);
  int             CreateDataCube_(int, char *);
  int             AddFrameToCube_(sdmaFRAME_T, int, char *);
  int             CloseDataCube_(int, char *);
  int             NextCube_(int, char *);
  void            CloseAllCubes_();

  // display
  int                doDisp_;
  char               camName_[256];
  char              *camShMemPtr_[iracqDTT_NUM_BUF];
  int                camShMemSize_;
  int                camShMemIdx_;
  rtdShm             camShMem_; 
  rtdIMAGE_EVT_HNDL  camEventHndl_;
  rtdIMAGE_INFO      camImageInfo_;
  int                camFrameType_;
  int                camDetNum_;
  int                camSizeX_;
  int                camSizeY_;
  int                camEventInit_;
  int                CamInit_(int, int, char *);
  int                CamClose_();
  int                CamLocked_();

  // statistics
  int                statEnabled_;           // disable statistics
  double             statPostProcMax_;       // seconds
  double             statPostProcAvrg_;      // seconds
  double             statPostProcCnt_;      
  double             statCallBackMax_;       // seconds
  double             statCallBackAvrg_;      // seconds
  double             statCallBackCnt_;      
  double             statOpenMax_;           // seconds
  double             statOpenAvrg_;          // seconds
  double             statOpenCnt_;      
  double             statCloseMax_;          // seconds
  double             statCloseAvrg_;         // seconds
  double             statCloseCnt_;      
  double             statReadMin_;           // MBytes/second
  double             statReadAvrg_;          // MBytes/second
  double             statReadCnt_;      
  double             statWriteMin_;          // MBytes/second
  double             statWriteAvrg_;         // MBytes/second
  double             statWriteCnt_;      

  // image processing
  iracqIMG           img_;
  int                extension_;
};

void iracqDttVbHandler(const char *, ...);
void iracqDttExitHandler(int);

#endif /*!_iracqDTT_h*/
