#ifndef oslxCONFIG_H
#define oslxCONFIG_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: oslxCONFIG.h,v 1.253 2005/08/17 13:09:21 vltsccm Exp $" 
*
* oslxCONFIG.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  30/01/96  created
* epozna    12/10/00  VLTSW20010283 - SaveConfigFile()  default value of arg 
*                      'useChangeRecord' is set to ccsFALSE (from NULL)
*/

/************************************************************************
*  Header file for the OSLX Configuration File Class
*----------------------------------------------------------------------
*/
#ifndef __cplusplus
#error This is a C++ include file which cannot be used from plain C
#endif

#include "oslxPRIVATE.h"
#include "oslxFITS_BASE.h"
#include "oslxDICTIONARY.h"
#include "oslxKEYW_FILTER.h"
#include "oslxSHORT_FITS.h"


// Contains methods and variables to handle a Short-FITS keyword buffer 
// in memory
class oslxCONFIG: public oslxSHORT_FITS
{
  public: 
    // Constructor method that associates a dictionary and
    // an alias table to the object.
    oslxCONFIG(const oslxDICTIONARY    *dictionary  = NULL, 
	       const oslxALIAS         *alias       = NULL);
    virtual ~oslxCONFIG();

    ccsCOMPL_STAT  SetConfigType(const char  *typeStr);
    ccsCOMPL_STAT  SetConfigType(const slxCONFIG_TYPE type);
    slxCONFIG_TYPE GetConfigType() const;

    ccsCOMPL_STAT LoadConfigFile(const slxFILENAME    fileName,
				 const slxCONFIG_TYPE type,
				 oslxKEYW_FILTER      *filter = NULL);
    ccsCOMPL_STAT SaveConfigFile(const slxCONFIG_TYPE type,
				 const slxFILENAME    fileName,
				 const vltLOGICAL     useChangeRecord = FALSE,
				 oslxKEYW_FILTER      *filter = NULL,
				 const vltLOGICAL     saveComments = FALSE,
				 const vltLOGICAL     saveIncludedKeys =FALSE);
    
    ccsCOMPL_STAT SetConfigFile(const slxFILENAME    fileName,
				const slxCONFIG_TYPE type=slxANY_CONFIG_FILE);

  protected:
    ccsCOMPL_STAT  ClearPri();

    // Local instance of OSLX Config File Header Class to hold header 
    // information locally
    ccsCOMPL_STAT   LoadConfigHeader(const slxFILENAME fileName);
    ccsCOMPL_STAT   CheckTypeAgainstFile(const slxCONFIG_TYPE type,
					 const slxFILENAME    fileName);

    slxCONFIG_TYPE      configType;

    oslxKEYW_FILTER     typeFilter;    

    // Make these methods unavailable for this class
    ccsCOMPL_STAT LoadShortFitsFile(const slxFILENAME   fileName,
				    const char          *envVar = NULL,
				    oslxKEYW_FILTER     *filter = NULL);
    ccsCOMPL_STAT SetShortFitsFile(const slxFILENAME fileName);
    ccsCOMPL_STAT Save(const vltLOGICAL       useChRecord =FALSE,
		       oslxKEYW_FILTER        *filter = NULL);
    ccsCOMPL_STAT SaveAs(const slxFILENAME    fileName,
			 const vltLOGICAL     useChRecord = FALSE,
			 oslxKEYW_FILTER      *filter = NULL);
    ////////////////////////////////////////////////////////////////////
};   // end-class oslxCONFIG


#endif /*!oslxCONFIG_H*/





