#ifndef oslxFITS_H
#define oslxFITS_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: oslxFITS.h,v 1.253 2005/08/17 13:09:21 vltsccm Exp $" 
*
* oslxFITS.h
*
* who       when       what
* --------  --------   ----------------------------------------------
* jknudstr  05/03/95   Created
* jknudstr  05/10/95   Lots of changes ...
* epozna    05/14/2000  CreateStoreFits functions to improve handling of pointers
* mnylund   14/06/04   Method GetHdrBlocks() added VLTSW20040124
* - - - - - - - - - - APR2005 - - - - - - - - - - - - - - - - - - - 
* mnylund   11/06/04  oslxFITS: Bug + ... VLTSW20040124
* mnylund   13/08/04  SPR VLTSW20040035 Cleanup of code
*                     UnloadDictionary and SetDictionary are removed use method 
*                     implemented in the parent class oslxFITS_BASE
* mnylund   11/11/04  VLTSW20040362 oslxFITS::LoadHeader seems to corrupt image .... 
*                     New functions are  added to make sure also empty lines are 
*                     handled when loading the header.
* mnylund   12/11/04  VLTSW20040319 Allow user to specify primary header size .... 
*                     A logging is implemented to warn in case an increased primary 
*                     header size is allocating a new 2880 block.
* - - - - - - - - - - JAN2006 - - - - - - - - - - - - - - - - - - - 
* mnylund   15/04/05  VLTSW20050133 seq_oslxCmd FITS duplicates FITS COMMENTs
*
*/

/************************************************************************
*  Header file for the OSLX FITS Class
*----------------------------------------------------------------------
*/

#include "oslxPRIVATE.h"
#include "oslxFITS_BASE.h"
#include "oslxDICTIONARY.h"
#include "oslxKEYWORD.h"
#include "oslxKEYW_FILTER.h"
#include "oslxDYN_BUF.h"

#ifndef __cplusplus
#error This is a C++ include file which cannot be used from plain C
#endif

#define oslxMAX_NO_OF_CATS  64

// Type used to distinguish between existing comments and logs and new ones
// entered when adding these in the SLX String List
typedef enum {
    oslxEXISTING_EL = 1,
    oslxNEW_EL
} oslxEL_ID;


// Class for handling the generation of FITS header information.
class oslxFITS: public oslxFITS_BASE
{
  public:
    // Constructor method that creates initializes an instance of the
    // class and makes an external reference to a Dictionary
    oslxFITS(const oslxDICTIONARY  *dictionary = NULL);
    ~oslxFITS();

    // Reset the object - i.e. set the Change Record to zero. The keywords
    // loaded into the object are not deleted
    ccsCOMPL_STAT Reset();

    // Clear the object and release all resources in use
    ccsCOMPL_STAT ClearInstance();

    // Method to load a Dictionary into the object and build up the
    // Hash Table



    ccsCOMPL_STAT LoadDictionary(const slxDICTIONARY dictionary,
				 const vltLOGICAL    merge = FALSE);
    //VLTSW20040035
    //ccsCOMPL_STAT UnloadDictionary();
    ccsCOMPL_STAT DicAutoLoading(const vltLOGICAL autoLoad);
    vltLOGICAL    DicAutoLoading();

    // Method to make the object reference to an external Dictionary
    //VLTSW20040035
    //ccsCOMPL_STAT SetDictionary(const oslxDICTIONARY *dictionary);

    ccsCOMPL_STAT DumpKeywords();

    // Method to copy keywords from one object to another
    ccsCOMPL_STAT Copy(oslxFITS  &srcObj);

    // Switch sorting of keywords on/off
    void          SortKeywords(const vltLOGICAL state);
    vltLOGICAL    SortKeywords();

    // Set the sequence of FITS Keywords in the header
    ccsCOMPL_STAT SetKeySequence(const char *keySequence);

    // Method to load the FITS header of a data file. The current filename is
    // set accordingly.
    ccsCOMPL_STAT LoadHeader(const slxFILENAME fileName,
			     const vltLOGICAL  merge = FALSE,
			     oslxKEYW_FILTER   *filter = NULL,
			     const vltLOGICAL  dontWriteChKeys = FALSE,
			     const vltLOGICAL  ignoreUnknownKeys = FALSE);

    // Method to load a FITS image file (header + data)
    ccsCOMPL_STAT LoadImageFile(const slxFILENAME fileName);

    // Saves the header and image buffer of the object into a file (new file)
    ccsCOMPL_STAT SaveImageFile(const slxFILENAME fileName = NULL);

    // Method to store a FITS image file into the image buffer of the
    // object
    ccsCOMPL_STAT StoreImage(const char      *image,
			     const vltINT32  size);

    // Method to return pointer to the image buffer
    char          *Image();

    // Method to return size of the image
    vltINT32      ImageSize();

    //VLTSW20040124
    // Method to return the number of 2880 blocks used by the header blocks
    vltINT32 NoOfHdrBlocks();

    //VLTSW20040124
    // Method to return the size of data blocks
    vltINT32 DataSize();

    // Methods to handle the Comment List
    void          ResetCommentList();
    ccsCOMPL_STAT ClearCommentList();
    ccsCOMPL_STAT AddComment(const slxKEYW_LINE comment);
    void          ResetCommentPtr();
    char          *GetNextComment();
    vltINT32      NumberOfComments();

    // Methods to handle the Log List
    void          ResetLogList();
    ccsCOMPL_STAT ClearLogList();
    ccsCOMPL_STAT AddLog(const slxKEYW_LINE log);
    void          ResetLogPtr();
    char          *GetNextLog();
    vltINT32      NumberOfLogs();

    // Methods to handle the History List
    void          ResetHistoryList();
    ccsCOMPL_STAT ClearHistoryList();
    ccsCOMPL_STAT AddHistory(const slxKEYW_LINE comment);
    void          ResetHistoryPtr();
    char          *GetNextHistory();
    vltINT32      NumberOfHistoryEntries();

    ccsCOMPL_STAT ClearAllAuxKeys();

    // Creates a new data file, with a number of FITS blocks, and write
    // the "END" keyword after the header.
    ccsCOMPL_STAT CreateFile(const slxFILENAME   fileName, 
			     vltUINT16           noOfBlocks);
       
    // Methods to create a FITS keyword and to write it in the local
    // buffer of the object. The Save() method must be called to store the
    // keywords in the FITS file header.
    // It is recommended first to build up the header in memory and then
    // set Save() the FITS lines in one go.
    ccsCOMPL_STAT CreateStoreFits(const slxKEYW_LINE    keyword,
				  const void            *value);
    ccsCOMPL_STAT CreateStoreFits(const slxKEYW_LINE    keyword,
	 		          const char            *value);
    ccsCOMPL_STAT CreateStoreFits(const slxKEYW_LINE    keyword,
				  const vltINT32        value);
    ccsCOMPL_STAT CreateStoreFits(const slxKEYW_LINE    keyword,
				  const vltLOGICAL      value);
    ccsCOMPL_STAT CreateStoreFits(const slxKEYW_LINE    keyword,
				  const vltDOUBLE       value);
    ccsCOMPL_STAT CreateStoreFits(const slxKEYW_LINE    keyword,
	 		          const vltDOUBLE       *value);
    ccsCOMPL_STAT CreateStoreFits(const slxKEYW_LINE    keyword,
	 		          const vltINT32        *value);
    ccsCOMPL_STAT CreateStoreFits(const slxKEYW_LINE    keyword,
	 		          const vltLOGICAL      *value); 

    ccsCOMPL_STAT StoreKeyword(oslxKEYWORD  &keyword);
    ccsCOMPL_STAT StoreKeyword(oslxKEYWORD  *keyword);
    ccsCOMPL_STAT StoreKeyword(const slxKEYW_INFO *keyword);
    ccsCOMPL_STAT StoreKeyword(const slxKEYW_LINE keywordLine);

    ccsCOMPL_STAT RemoveKeyword(const slxKEYW_LINE  keyword);

    // NOTE: Parameter "dontWriteChKeys" is obsolete
    ccsCOMPL_STAT Save(const slxFILENAME    fileName,
		       oslxKEYW_FILTER      *filter = NULL,
		       const vltLOGICAL     dontWriteChKeys = FALSE,
		       const vltLOGICAL     isExtension = FALSE);

    // Save the contents of the object in a read-able ASCII file.
    // The filename will not be the reference for the object.
    // No extension is appended to the filename.
    ccsCOMPL_STAT SaveAsciiFile(const slxFILENAME   fileName,
				const vltLOGICAL    useChangeRecord = FALSE,
				oslxKEYW_FILTER     *filter = NULL);

    // Return pointer to the buffer containing the FITS filename
    char          *FitsFileName() const;

    // Return the number of keywords stored in the object
    vltINT32      NoOfKeywords();

    // Methods to return FITS lines from a FITS file.
    ccsCOMPL_STAT ResetFitsHeaderPtr();
    ccsCOMPL_STAT GetNextFitsLine(slxKEYW_LINE    fitsLine);
    ccsCOMPL_STAT GetNextKeyword(const vltLOGICAL useChangeRecord = FALSE);

    ccsCOMPL_STAT ImageSize(vltINT32  *imageSize);

    vltINT32      NoOfHdrs();
    oslxFITS      *GetHdr(const vltINT32 hdrNo);

    vltINT32      ReqHdrBlocks();

    // Search for a keyword line and set the current keyword members
    // accordingly.
    ccsCOMPL_STAT GetKeyword(const slxKEYW_LINE keyword);

    // Check if a certain key is stored in the object
    vltLOGICAL    KeyInBuffer(const slxKEYW_LINE keyword);

    // Method to extract a number of keywords from a FITS header
    ccsCOMPL_STAT ExtractKeywords(const vltLOGICAL   useChangeRecord,
				  oslxKEYW_FILTER    *filter = NULL);
    char          *GetExtractBuffer();
    vltINT32      GetExtractBufferLen();
    vltINT32      NoOfExtractedKeywords();

    // Methods to handle the Change Record
    void          ResetChangeRecord();
    vltLOGICAL    CurrKeywordChanged();
    vltINT32      NoOfChangedKeywords() const;

    // Return reference to the Current Keyword which has been searched and
    // successfuly found
    oslxKEYWORD   &CurrKeyword();  
    oslxKEYWORD   &CK();  

    vltLOGICAL    FoundEndKey();

    ccsCOMPL_STAT AppendFile(const slxFILENAME  targetFile,
			     const slxFILENAME  sourceFile);

    ///////////////////////////////////////////////////////////////////
    // OBSOLETE METHODS - KEPT FOR BACKWARDS COMPATABILITY
    ///////////////////////////////////////////////////////////////////
    ccsCOMPL_STAT CreateStoreFits(const slxCATEGORY     category,
				  const slxSUBSYSTEM    subSystem1,
				  const slxSUBSYSTEM    subSystem2,
				  const slxPARAMETER    parameter,
				  const char            *value);
    ccsCOMPL_STAT CreateStoreFits(const slxCATEGORY     category,
				  const slxSUBSYSTEM    subSystem1,
				  const slxSUBSYSTEM    subSystem2,
				  const slxPARAMETER    parameter,
				  const vltLOGICAL      value);
    ccsCOMPL_STAT CreateStoreFits(const slxCATEGORY     category,
				  const slxSUBSYSTEM    subSystem1,
				  const slxSUBSYSTEM    subSystem2,
				  const slxPARAMETER    parameter,
				  const vltINT32        value);
    ccsCOMPL_STAT CreateStoreFits(const slxCATEGORY     category,
				  const slxSUBSYSTEM    subSystem1,
				  const slxSUBSYSTEM    subSystem2,
				  const slxPARAMETER    parameter,
				  const vltDOUBLE       value);

  protected:
    ccsCOMPL_STAT DelExtTable();
    ccsCOMPL_STAT DelSaveTable();

    ccsCOMPL_STAT ImageInfo(oslxFITS   &fits,
			    int        *imageSize,
			    int        *totalImageSize,
			    int        *noOfBlocks);

    ccsCOMPL_STAT GenFitsHdrBuf(const vltINT32  saveInstNo,
				oslxDYN_BUF     &hdrBuffer,
				const vltINT32  reqNoBlocks = 0);

    ccsCOMPL_STAT GetKeyword(const slxKEYW_LINE keyword,
			     ccsERROR           *intError);

    // Method used internally to create and write and FITS line in the object
    ccsCOMPL_STAT CreateStoreFitsInt(const slxCATEGORY     category,
				     const slxSUBSYSTEM    subSystem,
				     const slxSUBSYSTEM    subSystem2,
				     const slxPARAMETER    parameter,
				     const void            *value);

    ccsCOMPL_STAT LoadFitsHeaderList(slxFILENAME       fileName,
				     vltINT32          *headerBlocks,
				     slxKEYW_INFO      *keywordList,
				     miscHASH_TABLE    keywordHashTable,
				     slxCHANGE_RECORD  changeRecord,
				     const vltLOGICAL  dontWriteChKeys,
				     const vltLOGICAL  ignoreUnknownKeys,
				     oslxKEYW_FILTER   *filter = NULL);

    ccsCOMPL_STAT StoreFitsHeaderBufList(oslxDYN_BUF       &headerBuf,
					 vltINT32          *headerBlocks,
					 slxKEYW_INFO      *keywordList,
					 miscHASH_TABLE    keywordHashTable,
					 slxCHANGE_RECORD  changeRecord,
					 const vltLOGICAL  dontWriteChKeys,
					 const vltLOGICAL  ignoreUnknownKeys,
					 oslxKEYW_FILTER   *filter = NULL);

    ccsCOMPL_STAT StoreKeywordInList(const slxKEYW_LINE keywordLine,
				     slxKEYW_INFO       *keywordList,
				     miscHASH_TABLE     keywordHashTable,
				     slxCHANGE_RECORD   changeRecord,
				     slxDYN_BUF         *comBuf,
				     const vltLOGICAL   dontWriteChKeys,
				     const vltLOGICAL   ignoreUnknownKey,
				     slxDYN_BUF         *dataBuf = NULL);

    ccsCOMPL_STAT StoreKeywordInList(const slxKEYW_INFO *keyword,
				     slxKEYW_INFO       *keywordList,
				     miscHASH_TABLE     keywordHashTable,
				     slxCHANGE_RECORD   changeRecord,
				     slxDYN_BUF         *comBuf,
				     const vltLOGICAL   dontWriteChKeys,
				     const vltLOGICAL   ignoreUnknownKey,
				     slxDYN_BUF         *dataBuf = NULL);

    //VLTSW20040362 Methods to handle the Empty lines
    void          AddEmptyLine();
    void          ResetEmptyLine();
    void          RemoveEmptyLine();
    vltINT32      NoOfEmptyLines();
    vltINT32      LoadedHdrBlocks();

    //VLTSW20040319
    void          NewFileCreation();
    vltLOGICAL    NewFileCreated();
    void          AllocatedHeaderSize(vltUINT16 noOfBlocks);
    vltUINT16     AllocatedHeaderSize();
    void          HeaderIsLoaded();
    vltLOGICAL    LoadedHeader();

    // Contains the name of the FITS file currently related to the object
    slxFILENAME           fitsFile;         

    //VLTSW20050133 Save loaded fits file.
    slxFILENAME           loadedFitsFile;
    vltLOGICAL            updatedComments;
    vltLOGICAL            updatedLogs;
    vltLOGICAL            updatedHistory;

    // Defines the lay-out (order) of FITS Keywords in the header
    slxCATEGORY           headerLayout[oslxMAX_NO_OF_CATS];
    vltLOGICAL            sortKeywords;

    // Indicates the number of FITS blocks that constitute the header
    vltINT32              headerBlocks; 
    // Contains the information about the Current Keyword found
    oslxKEYWORD           currKeyword;     

    // Linked list of FITS keyword structures
    slxKEYW_INFO          header; 
    // Pointer to change record
    slxCHANGE_RECORD      changeRec;
    // Pointer to the current keyword        
    slxKEYW_INFO          *keywordPtr; 
    // Counts the number of keywords since last reset of keyword pointer    
    vltINT32              keywordsReturned; 
    // Number of keywords to return
    vltINT32              keywordsToReturn; 
    // Used to index the Change when returning keywords sequentially
    vltINT32              chRecIndex;      
    // Hash Table used for searching for FITS keywords
    miscHASH_TABLE        headerHashTable;  

    // Parameters for Dictionary Autoloading Feature
    miscHASH_TABLE        dicAutoLoadTable;
    vltLOGICAL            dicAutoLoading;

    // Hash Table containing pointers to the start of the various categories
    miscHASH_TABLE        catPtrTable;

    // Pointer to list of comments currently stored in the object
    slxSTRING_LIST        *commentList;
    // Number of comments stored in the object    
    vltINT32              noOfComments;
    // Pointer to list of comments    
    slxSTRING_LIST        *commentPtr;

    // Pointer to list of logs stored in the object         
    slxSTRING_LIST        *logList;
    // Number of logs stored in the object             
    vltINT32              noOfLogs;
    // Pointer to list of logs
    slxSTRING_LIST        *logPtr; 

    // Pointer to list of HISTORY keys currently stored in the object
    slxSTRING_LIST        *historyList;
    // Number of comments stored in the object    
    vltINT32              noOfHistoryEntries;
    // Pointer to list of comments    
    slxSTRING_LIST        *historyPtr;

    // Keywords extracted with a filter        
    slxDYN_BUF            extrKeywords; 
    vltINT32              noOfExtractedKeywords;

    // To handle when the user sets the order of the keywords
    oslxFITS              *saveOrderObj;

    miscHASH_TABLE        saveTable;
    vltINT32              saveTableCount;

    // Members to handle an image
    char                  *imageBuf;
    vltINT32              imageBufAlloc;
    vltINT32              imageSize;

    vltLOGICAL            foundEndKey;

    // To handle FITS File Extension
    vltINT32              noOfHdrs;
    miscHASH_TABLE        extTable;
    vltLOGICAL            appendingFitsFile;

    //VLTSW20040362 Number of empty lines stored in the object    
    vltINT32              noOfEmptyLines;

    //VLTSW20040319 Allocation of primary header
    vltLOGICAL            newFileCreated;
    vltUINT16             allocatedHeaderSize;
    vltLOGICAL            newHeaderLoaded;


};   // end-class oslxFITS


#define oslxBLANK_BUF_2880 \
"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                "

#endif /*!oslxFITS_H*/
