#ifndef oslxFITS_BASE_H
#define oslxFITS_BASE_H
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: oslxFITS_BASE.h,v 1.253 2005/08/17 13:09:22 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  11/06/97   created
* epozna    19/02/00   VLTSW20000031: new function - kwvalPrecision - 
*                        to declare the precision of a double number 
* epozna    20/06/00   VLTSW2000207 : function LoadAliasTable with
*                        (add argument vltLOGICAL merge)
* epozna    08/03/01  added sosKwPrefix, AddSOSPrefix(), RemoveSOSPrefix(), GetSOSPrefix()
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef __cplusplus
#error This is a C++ include file and cannot be used from plain C
#endif

#include "oslxBASE.h"
#include "oslxALIAS.h"
#include "oslxDICTIONARY.h"

class oslxFITS_BASE: public oslxBASE
{
  public: 
    oslxFITS_BASE();
    virtual ~oslxFITS_BASE();

    // Methods to handle Dictionaries
    virtual ccsCOMPL_STAT  LoadDictionary(const slxDICTIONARY  dicName,
					  const vltLOGICAL     merge = FALSE);
    ccsCOMPL_STAT  UnloadDictionary();
    oslxDICTIONARY *Dictionary();
    virtual ccsCOMPL_STAT  SetDictionary(const oslxDICTIONARY *dictionary);
    ccsCOMPL_STAT  AutoAddKeysInDic(const vltLOGICAL addKeys);
    vltLOGICAL     AutoAddKeysInDic();
    ccsCOMPL_STAT  AddKeyDic(const slxKEYW_LINE   keywordLine,
			     oslxDICTIONARY       *dictionary);

    // Load/unload an Alias Conversion Table
    ccsCOMPL_STAT LoadAliasTable(const slxALIAS_TABLE  aliasTablename,
				 const vltLOGICAL      merge = FALSE);

    ccsCOMPL_STAT UnloadAliasTable();
    ccsCOMPL_STAT SetAliasTable(const oslxALIAS *aliasTable);

    // Replaces the alias in a keyword line with the 
    ccsCOMPL_STAT ReplaceAlias(slxKEYW_LINE        keywordLine, 
			       const slxALIAS      alias,
			       ccsERROR            *intError) const;
    
    ccsCOMPL_STAT SetKeywordInfo(const slxKEYW_LINE   keywordLine,
				 slxKEYW_INFO         *keywordInfo,
				 oslxDICTIONARY       *dictionary,
				 slxDYN_BUF           *comBuf,
				 slxDYN_BUF           *dataBuf = NULL);
    ccsCOMPL_STAT _SetKeywordInfo(const slxKEYW_LINE   keywordLine,
				  slxKEYW_INFO         *keywordInfo,
				  oslxDICTIONARY       *dictionary,
				  slxDYN_BUF           *comBuf,
				  slxDYN_BUF           *dataBuf,
				  ccsERROR             *intError);
    
   vltINT32 KwValPrecision(slxVALUE svalue); 

  // functions operatin on sosKwPrefix, 
  // keywords with given prefix are not checked in the dictionary 
  ccsCOMPL_STAT  AddSOSPrefix(char * prefix); 
  ccsCOMPL_STAT  RemoveSOSPrefix(char *prefix); 
  ccsCOMPL_STAT  RemoveSOSPrefix(); 
  char *         GetSOSPrefix();
  vltLOGICAL     IsKeywordStartWithSOSPrefix(slxKEYW_LINE   keywordLine);
  
 protected: 
   
    virtual ccsCOMPL_STAT _DicCheck(const char *help);

    // Indicates if keywords not found in the Dictionary should be added 
    // automatically
    vltLOGICAL        autoAddKeysInDic;

    // Pointer to the dictionary
    oslxDICTIONARY    *dictionary;   
    // Flag to say if the dictionary is local or comes from outside
    vltLOGICAL        localDic; 
    // Indicates if Dictionary  loaded/available      
    vltLOGICAL        dicLoaded;     

    // Alias hash table
    oslxALIAS         *aliasTable; 
    // Local or external alias table   
    vltLOGICAL        localAliasTab; 
    // Indicates if Alias Conversion Table is loaded
    vltLOGICAL        aliasTabLoaded; 

private:
    // keywords with the declared prefix will not be checked in the dictionary 
    vltBYTES128       sosKwPrefix;

};

#endif /*!oslxFITS_BASE_H*/
