#ifndef oslxFITS_DICT_SUPPORT_H
#define oslxFITS_DICT_SUPPORT_H

/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: oslxFITS_DICT_SUPPORT.h,v 1.253 2005/08/17 13:09:22 vltsccm Exp $" 
*
* oslxFITS_DICT_SUPPORT.h
*
* who       when       what
* --------  --------   ----------------------------------------------
* epozna    01/10/00  Created after jknudstr oslxFITS
* - - - - - - - - - - APR2005 - - - - - - - - - - - - - - - - - - - 
* mnylund   13/08/04  SPR VLTSW20040035 Cleanup of code
*                     Parent class is changed to oslxFITS and the methods
*                     LoadDictionary, UnloadDictionary and SetDictionary are
*                     used from the parent class
* - - - - - - - - - - JAN2006 - - - - - - - - - - - - - - - - - - - 
* mnylund   07/06/05  SPR VLTSW20040035 Cleanup of code
*                     Parent class is changed again to oslxFITS_BASE and the 
*                     methods LoadDictionary, UnloadDictionary and SetDictionary 
*                     are again handle by the class itself
*/

/************************************************************************
*  Header file for the OSLX FITS Class
*----------------------------------------------------------------------
*/

#include "oslxPRIVATE.h"
#include "oslxFITS_BASE.h"
#include "oslxDICTIONARY.h"
#include "oslxKEYWORD.h"
#include "oslxKEYW_FILTER.h"
#include "oslxDYN_BUF.h"
#include "oslxFITS_BASE.h"
#include "fitsio.h"

#ifndef __cplusplus
#error This is a C++ include file which cannot be used from plain C
#endif

#define SORTINGLIST  "oslxSORT_REF_KEYW_LIST"   // name of the reflist filename
 
#define oslxSORTTYPE_NO_SORT       1
#define oslxSORTTYPE_REFLIST       2
#define oslxSORTTYPE_REFLIST_ABC   3

class oslxFITS_DICT_SUPPORT: public oslxFITS_BASE
{

 public: 
  
  // constructor/destructor
  oslxFITS_DICT_SUPPORT(const oslxDICTIONARY  *dictionary = NULL);
  ~oslxFITS_DICT_SUPPORT();

  // dictionary support functions
  ccsCOMPL_STAT LoadDictionary(const slxDICTIONARY dictionary,
  			       const vltLOGICAL    merge = FALSE);
  ccsCOMPL_STAT UnloadDictionary();
  
  ccsCOMPL_STAT DicAutoLoading(const vltLOGICAL autoLoad);
  vltLOGICAL    DicAutoLoading();
  
  // Method to make the object reference to an external Dictionary
  ccsCOMPL_STAT SetDictionary(const oslxDICTIONARY *dictionary);
  

  //ccsCOMPL_STAT DumpKeywords();
  
  // look up kw in dictionary, or add it to dictionary acc. to user decl.
  ccsCOMPL_STAT CheckKeywordInDictionary(const slxKEYW_LINE    keyword,
					 const void            *value,
					 int                   datatype );
    

  ccsCOMPL_STAT CheckKeywordInDictionary(const slxKEYW_LINE keyword, char* value );
 
  void          SetSortType(vltINT32 type=oslxSORTTYPE_REFLIST_ABC);
  vltINT32      GetSortType();

  protected:
  // sorting
  //static ccsCOMPL_STAT sortKeywordLines(char * list[slxMAX_FITS_LI_LEN],int nelem);
  static ccsCOMPL_STAT sortKeywordLines(char *list[slxMAX_FITS_LI_LEN],int nelem);
  static int compare_123_ABC(const void *kwl1, const void *kwl2);
  static int  comp3(const void *s1, const void *s2);
  static void printMatrix(const char * matrix[slxMAX_FITS_LI_LEN],int n);
  static int  numberOfLines(FILE *);
  static char** sortlist();
  static void sortlist_clean(char** matrix);

  ccsCOMPL_STAT CreateDotSeparatedKeyword( const slxKEYW_LINE  origKeyword,
		 			   slxKEYW_LINE        &newKeyword );


  char * Type2String(slxPAR_TYPE slxkwtype);
  private:
  // Parameters for Dictionary Autoloading Feature
  miscHASH_TABLE        dicAutoLoadTable;
  vltLOGICAL            dicAutoLoading;

  // following var are already in the  oslxFITS_BASE
  // vltLOGICAL dicLoaded; 
  // vltLOGICAL localDic;

  
  
};
 

#endif /*!oslxFITS_DICT_SUPPORT*/
