#ifndef oslxKEYW_FILTER_H
#define oslxKEYW_FILTER_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: oslxKEYW_FILTER.h,v 1.253 2005/08/17 13:09:23 vltsccm Exp $" 
*
* oslxKEYW_FILTER.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  22/09/95  Created
* jknudstr  05/10/95  Lots of changes ...
* marteau   11/06/99  Added Clone method and replaced GetFilter method
*                     to cope with new keyword naming scheme (up to
*                     5 subsystems)
*/

/************************************************************************
*  Header file for the OSLX class oslxKEYW_FILTER
*----------------------------------------------------------------------
*/
#ifndef __cplusplus
#error This is a C++ include file which cannot be used from plain C
#endif

#include <limits.h>
#if !defined(DBL_MAX)
    #define DBL_MAX 1.7976931348623157E+308	/* max decimal value of a */
						/* "double" */
#endif
#if !defined(DBL_MIN)
    #define DBL_MIN 2.2250738585072014E-308     /* min decimal value of a */
                                                /* "double" */
#endif

#define oslxDBL_MAX DBL_MAX
#define oslxDBL_MIN (-1 * (DBL_MAX * 0.999999999))
#define oslxINT_MAX INT_MAX
#define oslxINT_MIN INT_MIN

#include "oslxPRIVATE.h"
#include "oslxDICTIONARY.h"

// Number of filters in one instance of the class
#define oslxKEYW_FILTER_NO       25

// Actual number of filter available (filter #0 is not used)
#define oslxUSED_NO_KEYW_FILTER  (oslxKEYW_FILTER_NO - 1)

// Operators used when filtering
typedef enum {
  oslxOP_NOT_USED = 0,
  oslxOP_EQUAL,
  oslxOP_DIFFERENT,
  oslxOP_SMALLER,
  oslxOP_LARGER,
  oslxOP_SMALLER_EQUAL,
  oslxOPLARGER_EQUAL
} oslxOPERATOR;

// Structure to hold properties for one filter
typedef struct oslxK_FIL {
    slxCATEGORY           cat;
    slxSUBSYSTEM          subSyst[5];
    slxPARAMETER          par;

    // Used to keep track of if the Sub-Filter already matched 
    vltLOGICAL            matched;

    // Indicates if the value should be used when matching
    vltLOGICAL            matchValue;
    
    // Indicates if the range of the keyword should be used when matching
    // the value or if only one specific value should be used
    vltLOGICAL            matchRange;

    // Indicates the type of the keyword
    slxPAR_TYPE           type;

    // Strings
    char                  *lowLimStr;
    char                  *upLimStr; 

    // Logicals
    vltLOGICAL            valueLogical;

    // Integers
    vltINT32              lowLimInt;
    vltINT32              upLimInt;

    // Double
    vltDOUBLE             lowLimDbl;
    vltDOUBLE             upLimDbl;

    // Filter on class
    slxKEYWORD_CLASS      keywordClass;


    // This is the beginning of a re-implementation of the oslxKEYW_FILTER
    // whereby rather than using the lower/upper limits only we use also
    // a Low Operator and a High Operator
    oslxOPERATOR          lowOp;
    oslxOPERATOR          highOp;

    // Internal use
    oslxK_FIL             *next;
    oslxK_FIL             *prev;
} oslxFILTER_STRUCT;

// Class to handle aliasing of setup keywords.
class oslxKEYW_FILTER: public oslxBASE
{
  public:
    oslxKEYW_FILTER(const slxCATEGORY  category = NULL,
		    const slxSUBSYSTEM subSystem = NULL,
		    const slxPARAMETER parameter = NULL);
    ~oslxKEYW_FILTER();

    void Clear();

    oslxKEYW_FILTER *Clone() const ;

    void SetStrictFiltering(const vltLOGICAL onOff);
    vltLOGICAL StrictFiltering();

    ccsCOMPL_STAT AddFilter(const slxKEYW_LINE      keyword,
			    const slxKEYWORD_CLASS  keywClass = slxANY_CLASS);

    ccsCOMPL_STAT AddFilter(const slxKEYW_LINE      keyword,
			    const slxKEYWORD_CLASS  keywClass,
			    const char              *lowLim,
			    const char              *upLim = NULL,
			    const oslxOPERATOR      lowOp = oslxOP_NOT_USED,
			    const oslxOPERATOR      highOp = oslxOP_NOT_USED);

    ccsCOMPL_STAT AddFilter(const slxKEYW_LINE      keyword,
			    const slxKEYWORD_CLASS  keywClass,
			    const vltLOGICAL        value);

    ccsCOMPL_STAT AddFilter(const slxKEYW_LINE      keyword,
			    const slxKEYWORD_CLASS  keywClass,
			    const vltINT32          lowLim,
			    const vltINT32          upLim = oslxINT_MAX,
			    const oslxOPERATOR      lowOp = oslxOP_NOT_USED,
			    const oslxOPERATOR      highOp = oslxOP_NOT_USED);

    ccsCOMPL_STAT AddFilter(const slxKEYW_LINE      keyword,
			    const slxKEYWORD_CLASS  keywClass,
			    const vltDOUBLE         lowLim,
			    const vltDOUBLE         upLim = oslxDBL_MAX,
			    const oslxOPERATOR      lowOp = oslxOP_NOT_USED,
			    const oslxOPERATOR      highOp = oslxOP_NOT_USED);

    ccsCOMPL_STAT AddSymbolicFilter(const slxKEYW_LINE symbolicFilter);

    ccsCOMPL_STAT SetDictionary(const oslxDICTIONARY *dictionary);
    
    vltLOGICAL Match(const slxKEYW_INFO *keywordInfo);

    ccsCOMPL_STAT ApplyFilter(const slxKEYW_INFO      *keywordList,
			      slxCHANGE_RECORD        filterList,
			      const slxCHANGE_RECORD  changeRecord = NULL);

    ccsCOMPL_STAT ApplyFilter(const slxKEYW_INFO     *keywordList,
			      slxCHANGE_RECORD       filterList,
			      const slxCHANGE_RECORD changeRecord,
			      vltINT32               *noOfFilters,
			      vltINT32               *noOfMatches);

    ccsCOMPL_STAT GetFilter(const vltINT16 filterNo,
			    slxKEYW_LINE   keyword,
			    slxKEYW_LINE   valueMatch);

    ccsCOMPL_STAT AddKeywordClass(const slxKEYWORD_CLASS  keywordClass);
    ccsCOMPL_STAT DelKeywordClass(const slxKEYWORD_CLASS  keywordClass);

    void RejectIncludedKeys(const vltLOGICAL onOff);

    
    //////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////
    // OBSOLETE METHODS - KEPT FOR BACKWARDS COMPATABILITY REASONS
    //////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////

    ccsCOMPL_STAT GetFilter(const vltINT16 filterNo,
			    slxCATEGORY    category,
			    slxSUBSYSTEM   subSystem1,
			    slxSUBSYSTEM   subSystem2,
			    slxPARAMETER   parameter,
			    slxKEYW_LINE   valueMatch);

    ccsCOMPL_STAT AddFilter(const slxCATEGORY       category,
			    const slxSUBSYSTEM      subSystem,
			    const slxPARAMETER      parameter);

    vltLOGICAL Match(const slxCATEGORY   category,
		     const slxSUBSYSTEM  subSystem,
		     const slxPARAMETER  parameter);

    ccsCOMPL_STAT AddFilter(const slxCATEGORY       category,
			    const slxSUBSYSTEM      subSystem1,
			    const slxSUBSYSTEM      subSystem2, 
			    const slxPARAMETER      parameter,
			    const slxKEYWORD_CLASS  keywClass,
			    const char              *lowLim,
			    const char              *upLim = NULL);

    ccsCOMPL_STAT AddFilter(const slxCATEGORY       category,
			    const slxSUBSYSTEM      subSystem1,
			    const slxSUBSYSTEM      subSystem2, 
			    const slxPARAMETER      parameter,
			    const slxKEYWORD_CLASS  keywClass = slxANY_CLASS, 
			    const vltLOGICAL        matchValue = FALSE,
			    const vltLOGICAL        value = FALSE);

    ccsCOMPL_STAT AddFilter(const slxCATEGORY       category,
			    const slxSUBSYSTEM      subSystem1,
			    const slxSUBSYSTEM      subSystem2, 
			    const slxPARAMETER      parameter,
			    const slxKEYWORD_CLASS  keywClass,
			    const vltINT32          lowLim,
			    const vltINT32          upLim = oslxINT_MAX);

    ccsCOMPL_STAT AddFilter(const slxCATEGORY       category,
			    const slxSUBSYSTEM      subSystem1,
			    const slxSUBSYSTEM      subSystem2, 
			    const slxPARAMETER      parameter,
			    const slxKEYWORD_CLASS  keywClass,
			    const vltDOUBLE         lowLim,
			    const vltDOUBLE         upLim = oslxDBL_MAX);

    ccsCOMPL_STAT DelFilter(const slxCATEGORY  category,
			    const slxSUBSYSTEM subSystem1,
			    const slxSUBSYSTEM subSystem2,
			    const slxPARAMETER parameter);

  protected:
    ccsCOMPL_STAT PrepFilter(const slxKEYW_LINE  keyword,
			     oslxFILTER_STRUCT   **filter);

    void EnableMatchFlagCheck();
    void DisableMatchFlagCheck();

    // List containing the filters to use for the matching
    vltINT32              noOfFilters;
    oslxFILTER_STRUCT     *filterList;
    
    vltLOGICAL            useMatchFlag;

    vltLOGICAL            dictAvail;
    oslxDICTIONARY        *dictionary;

    vltLOGICAL            rejectIncludedKeys;

    vltLOGICAL            strictFiltering;

    //////////////////////////////////////////////////////////////////
    // OBSOLETE METHODS - KEPT FOR BACKWARDS COMPATABILITY
    //////////////////////////////////////////////////////////////////
    ccsCOMPL_STAT PrepFilter(const slxCATEGORY   category,
			     const slxSUBSYSTEM  subSystem1,
			     const slxSUBSYSTEM  subSystem2,
			     const slxPARAMETER  parameter,
			     oslxFILTER_STRUCT   **filter);
};   // end-class oslxKEYW_FILTER


#endif /*!oslxKEYW_FILTER_H*/





