#ifndef oslxSERVER_H
#define oslxSERVER_H
/******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: oslxSERVER.h,v 1.253 2005/08/17 13:09:24 vltsccm Exp $" 
*
* oslxSERVER.h
*
* who       when      what
* --------  --------  ----------------------------------------------
* jknudstr  20/11/96  Created
* - - - - - - - - - - -JAN 2006 - - - - - - - - - - - - - - - - - - - 
* mnylund   13/05/05  VLTSW20050136 Adding oslxFITS_EXT
*                     
*/

/************************************************************************
*  Header file for the OSLX Server
*----------------------------------------------------------------------
*/
#ifndef __cplusplus
#error This is a C++ include file which cannot be used from plain C
#endif

#include "oslx.h"

// Various macros
#define oslxMAX_NO_PARS 512

#define CMDINFO { \
    if (verbose) \
	{ \
	sprintf(cmdInfoBuf, "Handling command: %s", command); \
	INFO(cmdInfoBuf); \
	} \
}

#define oslxPACK_INFO_BUF { \
if (verbose) \
    { \
    sprintf(infoBuf, "Handling %s %s", command, par); \
    } \
}

#undef INFO
#define INFO(info) if (verbose) \
printf("\nOSLX Server> %s", info)
#undef INFO1
#define INFO1(info, opt1) if (verbose) \
    printf("\nOSLX Server> %s %s", info, opt1)
#undef INFO3
#define INFO3(info, opt1, opt2, opt3) if (verbose) \
    printf("\nOSLX Server> %s %s %s %s", info, opt1, opt2, opt3)
#undef INFO4
#define INFO4(info, opt1) if (verbose) \
    printf("\nOSLX Server> %s %d", info, opt1)

    // Set the error structure of the message, sends back an Error Reply, and
    // writes out a message that the command failed.
    // Used when a serious error occurred before forwarding any messages.
#define oslxSTD_ERROR_RETURN1 { \
    replySize = 0; \
    *replyBuffer = '\0'; \
    if (verbose) \
        { \
	sprintf(infoBuf, "Error handling command %s -%s", command, par); \
	INFO(infoBuf); \
	} \
    return FAILURE; \
}

// Makes a standard return after a successfull handling of a command, sends
// back a reply with the contents of oslxSERVER::replyBuffer 
// and prints out a message.
#define oslxSTD_CB_RETURN(cmd) { \
    if (verbose) \
        { \
	sprintf(infoBuf, "Successfully handled command %s -%s", \
		command, par); \
	INFO(infoBuf); \
	} \
    return SUCCESS; \
}

// Check if the instance number given is within the allowed range, and if 
// not send back an Error Reply to the source
#define oslxCHECK_INST_NO(no) { \
    if ((no < 1) || (no > oslxSERV_USED_INSTANCES)) \
        { \
	errAdd(&ErrStack(), oslxMODULE_ID, oslxERR_VALUE_OUT_OF_RANGE, \
	       __FILE__, NULL, "OSLX Instance Number", 1, \
	       oslxSERV_USED_INSTANCES); \
	oslxSTD_ERROR_RETURN1; \
        } \
}

#define oslxCHECK_PAR(parameter) \
    paramNb = 0; \
    if (strcmp(par, parameter) == 0) \
        paramNb = noOfPars; \
    if (CheckFitsExt(_cmdNo) == FAILURE) \
        oslxSTD_ERROR_RETURN1; \
    if (paramNb)
#define oslxCHECK_PAR_PAF oslxCHECK_PAR

typedef struct {
    int          fdIn;
    int          fdOut;
} oslxSERV_GLOB;

// Declare instance of the global (needed to be able to clean up in
// the Exit Handler
extern  oslxSERV_GLOB globalData;

// The base class for the OSS Server
class oslxSERVER: public eccsERROR_CLASS
{
  public:
    oslxSERVER();
    ~oslxSERVER();
  
    // Main Loop of the OSLX Server
    ccsCOMPL_STAT Server();
    ccsCOMPL_STAT CmdParser(const char  *command,
			    const char  *parameter,
			    const char  *instanceId,
			    const char  *parameters,
			    char        **replyBuffer,
			    vltINT32    *replyLen);
    ccsCOMPL_STAT GetInstance(const oslxCMD command,
			      const char    *instanceId);    
    ccsCOMPL_STAT _GetObjRef(miscHASH_TABLE       objTable, 
			     const char           *instanceId, 
			     const char           *objName,
			     const oslxCMD        instanceType,
			     oslxOBJ_TABLE_ENTRY  **objPtr,
			     vltLOGICAL           *created);

    ccsCOMPL_STAT CheckFitsExt(const oslxCMD command);

    // Reply methods
    ccsCOMPL_STAT GenReply(const char  *replyBuf);
    ccsCOMPL_STAT GenErrorReply();
    
    // Methods to handle the parsing of parameters
    ccsCOMPL_STAT Alias(char      *par,
			int       noOfPars, 
			char      *parValues[],
			char      *replyBuffer);
    ccsCOMPL_STAT Config(char      *par,
			 int       noOfPars, 
			 char      *parValues[],
			 char      *replyBuffer);
    ccsCOMPL_STAT Diction(char      *par,
			  int       noOfPars, 
			  char      *parValues[],
			  char      *replyBuffer);
    ccsCOMPL_STAT Filter(char      *par,
			 int       noOfPars, 
			 char      *parValues[],
			 char      *replyBuffer);
    ccsCOMPL_STAT Fits(char      *par,
		       int       noOfPars, 
		       char      *parValues[],
		       char      *replyBuffer);
    //VLTSW20050136
    ccsCOMPL_STAT FitsExt(char      *par,
		          int       noOfPars, 
		          char      *parValues[],
		          char      *replyBuffer);
    ccsCOMPL_STAT Gen(char      *par,
		      int       noOfPars, 
		      char      *parValues[],
		      char      *replyBuffer);
    ccsCOMPL_STAT InsDir(char      *par,
			 int       noOfPars, 
			 char      *parValues[],
			 char      *replyBuffer);
    ccsCOMPL_STAT Setup(char      *par,
			int       noOfPars, 
			char      *parValues[],
			char      *replyBuffer);
    ccsCOMPL_STAT ShortFits(char      *par,
			    int       noOfPars, 
			    char      *parValues[],
			    char      *replyBuffer);
    ccsCOMPL_STAT Tsf(char      *par,
		      int       noOfPars, 
		      char      *parValues[],
		      char      *replyBuffer);

    int PafParser(char            *cmd,
		  oslxCMD         cmdNo,
		  oslxSHORT_FITS  *instancePtr,
		  char            *par,
		  int             noOfPars, 
		  char            *paramValues[],
		  char            *replyBuffer);
	
    /////////////////////////////////////////////////////////////////////////
    // Various variables for handling Setup Files etc.
    /////////////////////////////////////////////////////////////////////////
    vltINT8           verbose; 
    oslxDICTIONARY    commonDictionary;
    oslxALIAS         commonAliasTable;
    oslxINS_DIR       insDir;

    // Flag to indicate if the new objects created automatically
    // should use the common dictionary
    vltLOGICAL        autoSetCommonDictionary;
    vltLOGICAL        autoSetCommonAliasTable;
    miscHASH_TABLE    aliasObjTable;
    miscHASH_TABLE    configObjTable;
    miscHASH_TABLE    dictionaryObjTable;
    miscHASH_TABLE    filterObjTable;
    miscHASH_TABLE    fitsObjTable;
    //VLTSW20050136
    miscHASH_TABLE    fitsExtObjTable;
    miscHASH_TABLE    setupObjTable;
    miscHASH_TABLE    shortFitsObjTable;
    miscHASH_TABLE    tsfObjTable;
    /////////////////////////////////////////////////////////////////////////

  protected:   
    ccsERROR              intCcsError;
    vltLOGICAL            errFound;
    ccsSTACK_ELEM         *errDescription;

    /////////////////////////////////////////////////////////////////////////
    // Various variables for handling commands in CB functions
    /////////////////////////////////////////////////////////////////////////
    char                  inputBuffer[msgBODYMAXLEN];
    char                  replyBuffer[50 * msgBODYMAXLEN];
    vltINT32              replySize;
    char                  tempBuf[msgBODYMAXLEN];
    char                  tempBuf2[50 * msgBODYMAXLEN];
    slxFILENAME           tempFileName;
 
    char                  *parValArray[oslxMAX_NO_PARS];
    vltINT32              paramNb;   
    vltLOGICAL            sendErrorReply; 
    oslxCMD               _cmdNo;
    slxKEYW_LINE          cmdInfoBuf;

    oslxSERV_GLOB         *gd;

    oslxALIAS             *aliasPtr;
    oslxCONFIG            *configPtr;
    oslxDICTIONARY        *dictionPtr;
    oslxKEYW_FILTER       *filterPtr;
    oslxFITS              *fitsPtr;
    oslxFITS              *fitsPtrOrg;
    //VLTSW20050136
    oslxFITS_EXT          *fitsExtPtr;
    oslxFITS_EXT          *fitsExtPtrOrg;
    oslxSETUP             *setupPtr;
    oslxSHORT_FITS        *shortFitsPtr;
    oslxTSF               *tsfPtr;
    /////////////////////////////////////////////////////////////////////////
    
    /////////////////////////////////////////////////////////////////////////
    // Help variables/pointers
    /////////////////////////////////////////////////////////////////////////
    char              *charP, **charPP;
    vltDOUBLE         dblVal, dblVal1, dblVal2;
    vltINT8           charVal;
    vltUINT8          *byteP;
    vltUINT8          byteVal;
    vltINT16          *int16P;
    vltINT16          int16Val;
    vltINT32          *intP;
    vltUINT32         *uintP;
    vltINT32          intVal, intVal1, intVal2, intVal3;
    vltFLOAT          *floatP;
    vltFLOAT          *floatP2;
    vltFLOAT          floatVal;
    vltLOGICAL        logVal, logVal1, logVal2;
    vltINT32          i, j, n, m;   
    vltINT32          no, no1, no2;
    slxFILENAME       tmpFileName1, tmpFileName2;
    slxKEYW_LINE      tmpBuf, tmpBuf1;
    slxKEYW_LINE      keywordLine;
    slxKEYW_LINE      infoBuf;
    slxKEYW_LINE      command;
    slxCATEGORY       category;
    slxSUBSYSTEM      subSystem;
    slxSUBSYSTEM      subSystem2;
    slxPARAMETER      parameter;
    slxKEYW_LINE      valueMatch;
    
    /////////////////////////////////////////////////////////////////////////
    // Private methods:
    /////////////////////////////////////////////////////////////////////////
    ccsCOMPL_STAT _DecodeFileSearch(char            **charPP,
				    vltINT32        paramNb,
				    slxFILENAME     baseName,
				    vltLOGICAL      *globalSearch,
				    vltLOGICAL      *stopFirstOccur,
				    vltLOGICAL      *skipInsUserRoot,
				    oslxKEYW_FILTER **filter,
				    vltLOGICAL      *ignoreUnknownKeys,
				    vltLOGICAL      *searchLocalFileList,
				    char            *errorMsg);
}; // end-class oslxSERVER

// OSLX Server Exit Handler
void oslxExitHandler(int sig);


#endif /*!oslxSERVER_H*/


//                              --- oOo ---






