/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: rmacPublic.h,v 1.26 2005/02/25 11:19:40 vltsccm Exp $"
*
* who       when        what
* --------  ----------  ----------------------------------------------
* pduhoux   2001-02-06  created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef RMAC_PUBLIC_H
#define RMAC_PUBLIC_H

/*
 ****************************************************************************
 * Header Files
 ****************************************************************************
 */
 
/* System Includes */

/* LCC Includes */

/* Local Includes */

#include "rmac.h"
#include "rmacDefines.h"

/* Function prototypes */
#ifdef MAKE_VXWORKS
ccsCOMPL_STAT   rmacOpen                  ( const char *rmnDev, ccsERROR *error);
ccsCOMPL_STAT   rmacClose                 ( void );
ccsCOMPL_STAT   rmacInit                  ( void );
vltINT32        rmacGetNodeId             ( void );
vltLOGICAL      rmacGetDataValid          ( int nodeId );
#endif

extern volatile rmacBUFFER *rmacDataBuffer;  /* Pointer to Reflective Memory data */
extern int      rmacBaseOpdIndex;            /* Index on OPD data                 */
extern int      rmacBaseDlIndex;             /* Index on DL data                  */

static __inline__
vltLOGICAL  rmacGetFringeDetectedFlag( int dlId )
{
    int idx = rmacOPD_MESSAGE + (rmacOffset * dlId);
    if (dlId == -1) idx = rmacBaseOpdIndex;
    return (vltLOGICAL)rmacDataBuffer[idx + rmacFRINGESDETECTED].ui;
}
static __inline__
void       rmacSetFringeDetectedFlag( int dlId, vltLOGICAL fdf )
{

    rmacDataBuffer[rmacOPD_MESSAGE + (rmacOffset * dlId) + rmacFRINGESDETECTED].ui = (vltUINT32)fdf;
}

static __inline__
vltLOGICAL  rmacGetOffsetValidFlag( int dlId )
{
    int idx = rmacOPD_MESSAGE + (rmacOffset * dlId);
    if (dlId == -1) idx = rmacBaseOpdIndex;
    return (vltLOGICAL)rmacDataBuffer[idx + rmacOFFSETVALID].ui;
}
static __inline__
void       rmacSetOffsetValidFlag( int dlId, vltLOGICAL ovf )
{

    rmacDataBuffer[rmacOPD_MESSAGE + (rmacOffset * dlId) + rmacOFFSETVALID].ui = (vltUINT32)ovf;
}

static __inline__
vltDOUBLE  rmacGetRealTimeOffset ( int dlId )
{
    int idx = rmacOPD_MESSAGE + (rmacOffset * dlId);
    if (dlId == -1) idx = rmacBaseOpdIndex;
    return rmacDataBuffer[idx + rmacRTOFFPOS].d;
}
static __inline__
void       rmacSetRealTimeOffset ( int id, vltDOUBLE rto )
{
    rmacDataBuffer[rmacOPD_MESSAGE + (rmacOffset * id) + rmacRTOFFPOS].d = rto;
}

static __inline__
vltDOUBLE  rmacGetOplMeasured ( int dlId )
{
    return rmacDataBuffer[rmacDL_MESSAGE + (rmacOffset * dlId) + rmacMEASPOS].d;
}
static __inline__
void       rmacSetOplMeasured ( vltDOUBLE opl )
{
    rmacDataBuffer[rmacBaseDlIndex + rmacMEASPOS].d = opl;
}

static __inline__
vltLOGICAL rmacGetTrackingOkFlag ( int dlId )
{
    return (vltLOGICAL)rmacDataBuffer[rmacDL_MESSAGE + (rmacOffset * dlId) + rmacTRKOKFLAG].ui;
}
static __inline__
void       rmacSetTrackingOkFlag ( vltLOGICAL tof )
{
    rmacDataBuffer[rmacBaseDlIndex + rmacTRKOKFLAG].ui = (vltUINT32)tof;
}

static __inline__
vltLOGICAL rmacGetCoherenceValidFlag ( void )
{
    return (vltLOGICAL)rmacDataBuffer[rmacVINCI_MESSAGE + rmacOPDCOHERVALID].ui;
}
static __inline__
void       rmacSetCoherenceValidFlag ( vltLOGICAL cvf )
{
    rmacDataBuffer[rmacVINCI_MESSAGE + rmacOPDCOHERVALID].ui = (vltUINT32)cvf;
}

static __inline__
vltLOGICAL rmacGetInsFringesDetectedFlag ( void )
{
    return (vltLOGICAL)rmacDataBuffer[rmacVINCI_MESSAGE + rmacINSFRINGESDETECTED].ui;
}
static __inline__
void       rmacSetInsFringesDetectedFlag ( vltLOGICAL ifdf )
{
    rmacDataBuffer[rmacVINCI_MESSAGE + rmacINSFRINGESDETECTED].ui = (vltUINT32)ifdf;
}

static __inline__
vltDOUBLE  rmacGetCoherenceOffset ( void )
{
    return rmacDataBuffer[rmacVINCI_MESSAGE + rmacOPDCOHERENCE].d;
}
static __inline__
void       rmacSetCoherenceOffset ( vltDOUBLE co )
{
    rmacDataBuffer[rmacVINCI_MESSAGE + rmacOPDCOHERENCE].d = co;
}

/*
 * PRIMA Metrology access functions
 */

static __inline__ void rmacSetPRIMETData( vltDOUBLE deltaL,
					  vltUINT32 deltaLValid )
{
    vltINT32 idx = rmacPRIMA_MESSAGE + (rmacOffset * rmacPRIMA_MET);
    rmacDataBuffer[idx + rmacPRIMET_DELTA].d = deltaL;
    rmacDataBuffer[idx + rmacPRIMET_DELTA_VALID].ui = deltaLValid;
}

static __inline__ void rmacGetPRIMETData( vltDOUBLE *deltaL,
					  vltUINT32 *deltaLValid )
{
    vltINT32 idx = rmacPRIMA_MESSAGE + (rmacOffset * rmacPRIMA_MET);
    *deltaL = rmacDataBuffer[idx + rmacPRIMET_DELTA].d;
    *deltaLValid = rmacDataBuffer[idx + rmacPRIMET_DELTA_VALID].ui;
}

/*
 * PRIMA FSU A access functions
 */

static __inline__ void rmacSetPRIFSUAData( vltDOUBLE opd,  
					   vltUINT32 opdValid,
					   vltDOUBLE gd,
					   vltUINT32 gdValid,
					   vltDOUBLE ampSquared,
					   vltUINT32 ampSquaredValid )
{
    vltINT32 idx = rmacPRIMA_MESSAGE + (rmacOffset * rmacPRIMA_FSU_A);
    rmacDataBuffer[idx + rmacPRIFSU_OPD].d = opd;
    rmacDataBuffer[idx + rmacPRIFSU_OPD_VALID].ui = opdValid;
    rmacDataBuffer[idx + rmacPRIFSU_GD].d = gd;
    rmacDataBuffer[idx + rmacPRIFSU_GD_VALID].ui = gdValid;
    rmacDataBuffer[idx + rmacPRIFSU_AMP].d = ampSquared;
    rmacDataBuffer[idx + rmacPRIFSU_AMP_VALID].ui = ampSquaredValid;
}

static __inline__ void rmacGetPRIFSUAData( vltDOUBLE *opd,  
					   vltUINT32 *opdValid,
					   vltDOUBLE *gd,
					   vltUINT32 *gdValid,
					   vltDOUBLE *ampSquared,
					   vltUINT32 *ampSquaredValid )
{
    vltINT32 idx = rmacPRIMA_MESSAGE + (rmacOffset * rmacPRIMA_FSU_A);
    *opd = rmacDataBuffer[idx + rmacPRIFSU_OPD].d;
    *opdValid = rmacDataBuffer[idx + rmacPRIFSU_OPD_VALID].ui;
    *gd = rmacDataBuffer[idx + rmacPRIFSU_GD].d;
    *gdValid = rmacDataBuffer[idx + rmacPRIFSU_GD_VALID].ui;
    *ampSquared = rmacDataBuffer[idx + rmacPRIFSU_AMP].d;
    *ampSquaredValid = rmacDataBuffer[idx + rmacPRIFSU_AMP_VALID].ui;
}

/*
 * PRIMA FSU B access functions
 */

static __inline__ void rmacSetPRIFSUBData( vltDOUBLE opd,  
					   vltUINT32 opdValid,
					   vltDOUBLE gd,
					   vltUINT32 gdValid,
					   vltDOUBLE ampSquared,
					   vltUINT32 ampSquaredValid )
{
    vltINT32 idx = rmacPRIMA_MESSAGE + (rmacOffset * rmacPRIMA_FSU_B);
    rmacDataBuffer[idx + rmacPRIFSU_OPD].d = opd;
    rmacDataBuffer[idx + rmacPRIFSU_OPD_VALID].ui = opdValid;
    rmacDataBuffer[idx + rmacPRIFSU_GD].d = gd;
    rmacDataBuffer[idx + rmacPRIFSU_GD_VALID].ui = gdValid;
    rmacDataBuffer[idx + rmacPRIFSU_AMP].d = ampSquared;
    rmacDataBuffer[idx + rmacPRIFSU_AMP_VALID].ui = ampSquaredValid;
}

static __inline__ void rmacGetPRIFSUBData( vltDOUBLE *opd,  
					   vltUINT32 *opdValid,
					   vltDOUBLE *gd,
					   vltUINT32 *gdValid,
					   vltDOUBLE *ampSquared,
					   vltUINT32 *ampSquaredValid )
{
    vltINT32 idx = rmacPRIMA_MESSAGE + (rmacOffset * rmacPRIMA_FSU_B);
    *opd = rmacDataBuffer[idx + rmacPRIFSU_OPD].d;
    *opdValid = rmacDataBuffer[idx + rmacPRIFSU_OPD_VALID].ui;
    *gd = rmacDataBuffer[idx + rmacPRIFSU_GD].d;
    *gdValid = rmacDataBuffer[idx + rmacPRIFSU_GD_VALID].ui;
    *ampSquared = rmacDataBuffer[idx + rmacPRIFSU_AMP].d;
    *ampSquaredValid = rmacDataBuffer[idx + rmacPRIFSU_AMP_VALID].ui;
}

/*
 * Monitor array access routines
 */

static __inline__
void       rmacSetMonitor ( vltUINT32 id, vltDOUBLE data )
{
    rmacDataBuffer[rmacMONITOR_MESSAGE + id].d = data;
}

static __inline__
vltDOUBLE  rmacGetMonitor ( vltUINT32 id )
{
    return rmacDataBuffer[rmacMONITOR_MESSAGE + id].d;
}

#endif /*!RMAC_PUBLIC_H*/
