#ifndef _sdmaDrv_h
#define _sdmaDrv_h

/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: sdmaDrv.h,v 1.104 2005/09/12 09:21:06 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jstegmei  10/12/98  created
*/

/************************************************************************
 *
 *  sdmaDrv.h  sdma driver library constants
 *
 *----------------------------------------------------------------------
 */


#ifdef SPARC_SBUS
#define SPARC_NOSIM
#define sdmaSCD
#endif /* SPARC_SBUS */


#ifdef SPARC_PCI

#define SPARC_NOSIM

#ifdef IRPCI
#include "irpci.h"
#define sdmaIPC
#define IRPCI_SPARC
#else
#define SYSV
#include "edtinc.h"
#define sdmaPCD
#endif /* IRPCI */

#endif /* SPARC_PCI */


#ifdef LINUX

#ifndef SPARC_SIM
#define SPARC_NOSIM
#undef __STRICT_ANSI__
#define _REENTRANT  
typedef unsigned char u_char;  
typedef unsigned int u_int;  
typedef unsigned short u_short;
#include "pthread.h"

#ifdef IRPCI
#include "irpci.h"
#define sdmaIPC
#define IRPCI_LINUX
#else
#include "edtinc.h"
#define sdmaPCD
#endif /* IRPCI */

#endif /* SPARC_SIM */

#endif /* LINUX */

#ifndef USE
#define USE(x) x=x
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifdef sdmaSCD
/*
 *  default input mode
 */
#define sdmaIM_DEFAULT "thread"

/*
 *  default device name
 */
#define sdmaDEV_DEFAULT "/dev/scd0"

/*
 *  maximum DMA-blocksize
 */
#define sdmaMAX_DMASIZE (1024*1024*2) 

#define sdmaHANDLE int

#define sdmaINVALID_LINK -1
#endif

#ifdef sdmaIPC
/*
 *  default input mode
 */
#define sdmaIM_DEFAULT "thread"

/*
 *  default device name
 */
#define sdmaDEV_DEFAULT "/dev/ipc0_dma"

/*
 *  maximum DMA-blocksize
 */
#define sdmaMAX_DMASIZE (1024*1024*2) 

#define sdmaHANDLE int

#define sdmaINVALID_LINK -1
#endif

#ifdef sdmaPCD
/*
 *  default input mode
 */
#define sdmaIM_DEFAULT "thread"

/*
 *  default device name
 */
#define sdmaDEV_DEFAULT "/dev/pcd0"

/*
 *  maximum DMA-blocksize
 */
#define sdmaMAX_DMASIZE (1024*32)

#define sdmaHANDLE EdtDev *

#define sdmaINVALID_LINK (EdtDev *)NULL
#endif

#ifndef sdmaHANDLE
#define sdmaHANDLE int
#endif

#ifndef sdmaINVALID_LINK
#define sdmaINVALID_LINK -1
#endif

/*
 *  prototypes
 */
#ifdef SPARC_NOSIM
int sdmaDrvAlloc (sdmaHANDLE, unsigned char **, int, unsigned long, char *);
void sdmaDrvFree (sdmaHANDLE, unsigned char **, int);
int  sdmaDrvOpen (char *, sdmaHANDLE *, char *);
int  sdmaDrvClose (sdmaHANDLE *, char *);
int  sdmaDrvWrite (sdmaHANDLE, void *, int, char *);
int  sdmaDrvInitRead (sdmaHANDLE, unsigned char **, int, unsigned long, 
		      unsigned int *, thread_t *, char *);
int  sdmaDrvStopDma (sdmaHANDLE, int, thread_t *, char *);
int  sdmaDrvSetHdr (sdmaHANDLE, short, char *);
int  sdmaDrvGetCnt (sdmaHANDLE, unsigned int *, char *);
int  sdmaDrvWaitCnt (sdmaHANDLE, unsigned int, char *);
int  sdmaDrvDmaStat (sdmaHANDLE, int *, char *);
int  sdmaDrvInputOverflow (sdmaHANDLE);
void sdmaDrvPrintRegs (sdmaHANDLE);
#else
/*
 *  dummy definitions
 */
#define sdmaDrvAlloc(fd, buf, n, size, err) 0
#define sdmaDrvFree(fd, buf, n)
#define sdmaDrvOpen(name, fd, err) \
    ((sprintf(err, "compiled for simulation only")<0) ? -1 : -1)
#define sdmaDrvClose(fd, err) 0
#define sdmaDrvWrite(fd, buf, size, err) 0
#define sdmaDrvInitRead(fd, buf, n, size, cnt, thid, err) 0
#define sdmaDrvStopDma(fd, num, thid, err) 0
#define sdmaDrvSetHdr(fd, val, err) 0
#define sdmaDrvGetCnt(fd, cnt, err) 0
#define sdmaDrvWaitCnt(fd, cnt, err) 0
#define sdmaDrvDmaStat(fd, cnt, err) 0
#define sdmaDrvInputOverflow(fd) 0
#define sdmaDrvPrintRegs(fd)
#endif

#ifdef __cplusplus
}
#endif

#endif /*!_sdmaDrv_h*/
