#ifndef _sdmaThr_h
#define _sdmaThr_h

/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: sdmaThr.h,v 1.104 2005/09/12 09:21:10 vltsccm Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jstegmei  31/08/98  created
*/

/************************************************************************
 *
 *  sdmaThr.h  sdma thread wrapper constants
 *
 *----------------------------------------------------------------------
 */

#ifndef THR_POSIX
#define THR_SOLARIS
#else

#ifdef HP
#define SEM_POSIX
#else
#define SEM_POSIX
#endif /* HP */

#endif /* THR_POSIX */

#ifdef THR_SOLARIS
#include <thread.h>
#include <sched.h>
#include <synch.h>
#include <sys/asynch.h>
#endif /* THR_SOLARIS */

#ifdef THR_POSIX
#include <sys/ipc.h>
#include <sys/sem.h>
#include <pthread.h>
#endif /* THR_POSIX */

#ifdef HP

#ifndef _SYS_PTHREAD_INCLUDED
#define HP_CMA
#endif

#endif /* HP */

#ifdef HP_CMA
#define sdmaSEL  int
#else
#define pthread_attr_default NULL
#define pthread_mutexattr_default NULL
#define pthread_condattr_default NULL
#define sdmaSEL  fd_set
#endif /* HP_CMA */

#ifdef __cplusplus
extern "C" {
#endif


/*
 *  thread attribute definitions
 */
#define sdmaTHR_BOUND     1
#define sdmaTHR_NEW_LWP   2
#define sdmaSEM_THREAD    1
#define sdmaSEM_PROCESS   2

#ifdef THR_SOLARIS
#define sdmaSemClean() 
#define sdmaSemTestClean() 
#endif

/*
 *  posix threads
 */
#ifdef THR_POSIX
#define sdmaTHID       pthread_t

int  sdmaThrPosixCreate(void *(*startFunc)(void *), void *, 
			long, sdmaTHID *);
void sdmaThrPosixExit(void *);
int  sdmaThrPosixJoin (sdmaTHID, sdmaTHID *, void **);
int  sdmaThrPosixKill (sdmaTHID, int);
#endif /* THR_POSIX */

/*
 *  ipc semaphores
 */
#ifdef SEM_IPC
#define sdmaCLEANUP_FILE "/tmp/sdmaSemId"
#define sdmaSEM int

int  sdmaSemIpcInit (sdmaSEM *, unsigned int, int, void *);
int  sdmaSemIpcWait (sdmaSEM *);
int  sdmaSemIpcPost (sdmaSEM *);
int  sdmaSemIpcTryWait (sdmaSEM *);
int  sdmaSemIpcDestroy (sdmaSEM *);
void sdmaSemIpcClean(void);
void sdmaSemIpcTestClean(void);
#endif /* SEM_IPC */

/*
 *  mutex semaphores
 */
#ifdef SEM_POSIX
typedef struct
{
  pthread_mutex_t mutex_v;
  pthread_cond_t  lck;
  unsigned int    val;
} sdmaSEM;


int  sdmaSemPosixInit (sdmaSEM *, unsigned int, int, void *);
int  sdmaSemPosixWait (sdmaSEM *);
int  sdmaSemPosixPost (sdmaSEM *);
int  sdmaSemPosixTryWait (sdmaSEM *);
int  sdmaSemPosixDestroy (sdmaSEM *);
void sdmaSemPosixClean(void);
void sdmaSemPosixTestClean(void);
#endif /* SEM_POSIX */

/*
 *  readers/writer locks using ipc semaphores
 */
#ifdef SEM_IPC
typedef struct
{
  sdmaTHID wrLock;
  int      id;
} sdmaLOCK;

int  sdmaRwIpcLockInit (sdmaLOCK *,int type, void *);
int  sdmaRwIpcLockDestroy (sdmaLOCK *);
int  sdmaRwIpcRdLock (sdmaLOCK *);
int  sdmaRwIpcTryRdLock (sdmaLOCK *);
int  sdmaRwIpcWrLock (sdmaLOCK *);
int  sdmaRwIpcTryWrLock (sdmaLOCK *);
int  sdmaRwIpcUnlock (sdmaLOCK *);
#endif /* SEM_IPC */

/*
 *  readers/writer locks using posix mutex
 */
#ifdef SEM_POSIX
typedef struct
{
  sdmaTHID        wrLock;
  pthread_mutex_t mutex_v;
  pthread_cond_t  lckr;
  pthread_cond_t  lckw;
  unsigned int    val;
  unsigned char   wr;
} sdmaLOCK;

int  sdmaRwPosixLockInit (sdmaLOCK *,int type, void *);
int  sdmaRwPosixLockDestroy (sdmaLOCK *);
int  sdmaRwPosixRdLock (sdmaLOCK *);
int  sdmaRwPosixTryRdLock (sdmaLOCK *);
int  sdmaRwPosixWrLock (sdmaLOCK *);
int  sdmaRwPosixTryWrLock (sdmaLOCK *);
int  sdmaRwPosixUnlock (sdmaLOCK *);
#endif /* SEM_POSIX */


/*
 *  wrappers for solaris thread implementation
 */
#ifdef THR_POSIX
#define THR_BOUND      sdmaTHR_BOUND
#define THR_NEW_LWP    sdmaTHR_NEW_LWP
#define USYNC_PROCESS  sdmaSEM_PROCESS
#define USYNC_THREAD   sdmaSEM_THREAD

#ifdef LINUX
  #ifdef SPARC_SIM
    #ifndef thread_t
      #define thread_t sdmaTHID
    #endif /* thread_t */
  #else
    #define thread_t sdmaTHID
  #endif /* SPARC_SIM */
#else
  #define thread_t sdmaTHID
#endif /* LINUX */


#define thr_create(stackBase,stackSize,func,arg,flags,thid) \
    sdmaThrPosixCreate(func,arg,flags,thid)

#define thr_join(thread,departed,status) \
    sdmaThrPosixJoin(thread,departed,status)

#define thr_exit(status) \
    sdmaThrPosixExit(status)

#define thr_kill(thread,sig) \
    sdmaThrPosixKill(thread,sig)
#endif /* THR_POSIX */

#ifdef SEM_IPC
#define sema_t   sdmaSEM
#define rwlock_t sdmaLOCK

#define sdmaSemClean() sdmaSemIpcClean()

#define sdmaSemTestClean() sdmaSemIpcTestClean()

#define sema_init(sp,count,type,arg) \
    sdmaSemIpcInit(sp,count,type,arg)

#define sema_wait(sp) \
    sdmaSemIpcWait(sp)

#define sema_post(sp) \
    sdmaSemIpcPost(sp)

#define sema_trywait(sp) \
    sdmaSemIpcTryWait(sp)

#define sema_destroy(sp) \
    sdmaSemIpcDestroy(sp)

#define rwlock_init(rwlp,type,arg) \
    sdmaRwIpcLockInit(rwlp,type,arg)

#define rwlock_destroy(rwlp) \
    sdmaRwIpcLockDestroy(rwlp)

#define rw_rdlock(rwlp) \
    sdmaRwIpcRdLock(rwlp)

#define rw_tryrdlock(rwlp) \
    sdmaRwIpcTryRdLock(rwlp)

#define rw_wrlock(rwlp) \
    sdmaRwIpcWrLock(rwlp)

#define rw_trywrlock(rwlp) \
    sdmaRwIpcTryWrLock(rwlp)

#define rw_unlock(rwlp) \
    sdmaRwIpcUnlock(rwlp)
#endif /* SEM_IPC */

#ifdef SEM_POSIX
#define sema_t   sdmaSEM
#define rwlock_t sdmaLOCK

#define sdmaSemClean() sdmaSemPosixClean()

#define sdmaSemTestClean() sdmaSemPosixTestClean()

#define sema_init(sp,count,type,arg) \
    sdmaSemPosixInit(sp,count,type,arg)

#define sema_wait(sp) \
    sdmaSemPosixWait(sp)

#define sema_post(sp) \
    sdmaSemPosixPost(sp)

#define sema_trywait(sp) \
    sdmaSemPosixTryWait(sp)

#define sema_destroy(sp) \
    sdmaSemPosixDestroy(sp)

#define rwlock_init(rwlp,type,arg) \
    sdmaRwPosixLockInit(rwlp,type,arg)

#define rwlock_destroy(rwlp) \
    sdmaRwPosixLockDestroy(rwlp)

#define rw_rdlock(rwlp) \
    sdmaRwPosixRdLock(rwlp)

#define rw_tryrdlock(rwlp) \
    sdmaRwPosixTryRdLock(rwlp)

#define rw_wrlock(rwlp) \
    sdmaRwPosixWrLock(rwlp)

#define rw_trywrlock(rwlp) \
    sdmaRwPosixTryWrLock(rwlp)

#define rw_unlock(rwlp) \
    sdmaRwPosixUnlock(rwlp)
#endif /* SEM_POSIX */

#ifdef __cplusplus
}
#endif

#endif /*!_sdmaThr_h*/
