# Restart with tcl: -*- mode: tcl; tab-width: 8; -*-
#************************************************************************
# E.S.O. - VLT project
# "@(#) $Id: audioLib.tcl.cpp,v 1.13 2005/10/28 08:40:23 vltsccm Exp $"
# Ensures the namespace ::audio exists
namespace eval ::audio {
    set cfg(port) 33962; # value picked up from a define
    set cfg(supportedTypes) "Snack NCD HP"; # value picked up from a define
    set cfg(ncdPath) /opt/tekxp/bin/hp700; # value picked up from a define
    set cfg(pollPeriod) 2000; # value picked up from a define
    if {![info exists refCnt]} {
        set refCnt 0; # counting number of active sounds
    }
    if {![info exists currentRate]} {
        set currentRate 0; # sampling rate of currently playing sound
    }
}
# dummy command to ensure that the above "namespace eval" cmd
# gets executed:
proc ::audio::_initNamespace {} {}
# Returns the hostname or IP-address of the server
proc ::audio::_serverHost {} {
    if {[info exists ::env(AUDIO)]} {
        return [lindex [split $::env(AUDIO) :] 0]
    } elseif {[info exists ::env(DISPLAY)]} {
        return [lindex [split $::env(DISPLAY) :] 0]
    } else {
        return localhost
    }
}
# This procedure will send a cmd without closing the socket (which
# is the return value)
proc ::audio::_sendCmd {cmd} {
    variable cfg
    variable refCnt
 # The sending of the previous audio-file must be finished
 # before we can send a next one.
    if {$refCnt > 0} {
        vwait audio::refCnt
    }
    set serverHost [_serverHost]
 # Check if there is an snack audioServer running on that host,
 # i.e. try to open binary socket connection to audioServer.
    for {set i 0} {$i < 25} {incr i} {
        if {![catch {socket $serverHost $cfg(port)} sock]} {
            fconfigure $sock -translation auto -blocking 1
            puts -nonewline $sock $cmd
            flush $sock
            return $sock
        } elseif {[string equal $sock "couldn't open socket: connection refused"]} {
 # This happens occasionally (at least on MS-Windows), as the number of
 # connections permitted per second is limited;
 # if it does, just forget about it
            after 100; # try again in 100 ms
        } else {
            error "no Snack audio server listening (anymore) at $serverHost/$cfg(port)"
        }
    }
 # if we get here, we have been trying for 2.5 s to connect, so give up
    error "Snack audio server on $serverHost not responding"
}

