#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bobDialog.tcl,v 1.145 2005/10/27 09:07:06 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# eallaert  1996-11-30  created
# mbec      1997-07-10  header update (cf. TemplateServerMode)
# mbec      1997-08-29  button relief fix
# eallaert  1997-10-21  removed TemplateServer stuff (put straight into tplsrv)
#

#************************************************************************
#   NAME
#   bobDialog - display a dialog box for BOB
# 
#   SYNOPSIS
#   bobDialog <message> <modality> \
#        ?<timeout> ?<type> ?<userLevel> ?<cancelButton>???? 
# 
#   DESCRIPTION
#   bobDialog will display a dialog box with:
#   <message> : text displayed in the dialog box
#   <modality>: determines the degree of blocking of the application.  Can be
#       "global" -> all applications are blocked while this dialog is active.
#       "application" -> only this application is blocked
#       "none" -> no application will be blocked
#   <timeout> : timeout value in ms after which the dialog will deactivate
#        automatically.  A 0 value indicates no timeout (default). Timing-out
#        has the same effect as pressing "Cancel" if this button is displayed
#        (see <cancelButton>); otherwise, it is same as pressing "OK".
#   <type> : type of dialog box; can be "error" (default), "info", "question",
#        "warning" or "powered" (the latter being reserved for the "About BOB"
#        dialog box).
#   <userLevel> : do this only if the configured user-level is <userLevel>
#        (0 for Beginner, 1 for Advanced, empty means all levels)
#   <cancelButton> : boolean, indicating whether "Cancel" button should be
#        shown (0 by default).
#
#   This procedure is implemented based on [incr Widgets]'s messagedialog.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#   Lines longer than 100 chars will be truncated.
#
#   EXAMPLES
#
#   SEE ALSO
#   [incr widgets] documentation (http://www.tcltk.com)
#
#   BUGS
#
#------------------------------------------------------------------------
#
# dialog box images
image create photo errorImg    -file [bobFindVlt bitmaps/bobStop.gif]
image create photo infoImg     -file [bobFindVlt bitmaps/bobInfo.gif]
image create photo questionImg -file [bobFindVlt bitmaps/bobQuestion.gif]
image create photo warningImg  -file [bobFindVlt bitmaps/bobWarning.gif]
image create photo poweredImg  -file [bobFindVlt bitmaps/bobPowered.gif]

proc bobDialog {text mod {timeout 0} {type error} {userLevel {}} {cancelButton 0 } } {
    #!!#
    ##puts "about to bring up a dialog box"

    if {[string length $userLevel] != 0 && $userLevel != $::bob::internal::config(bobUserLevel)} {
	return
    }

    # Check text length
    foreach line [split $text \n] {
	if {[string length $line] > 100} {
	    set line "[string range $line 0 96] ..."
	}
	append ltext "${line}\n"
    }

    if {[string equal $type powered]} {
	set title {About BOB}
    } else {
	set title "BOB $type"
    }


    # test here if there's already one instead of destroying it each time 
    if {[string length [info command .bobED]] == 0} {
	messagedialog .bobED -title $title -font $::bob::internal::config(bobDialogFont) \
		-image ${type}Img -text $ltext -modality $mod \
		-padx 10 -textpadx 10
	.bobED hide Apply
	.bobED hide Help
	.bobED buttonconfigure OK -padx 3 -pady 3 -borderwidth 2
	if { $cancelButton == 0 } {
	    .bobED hide Cancel
	} else {
	    .bobED show Cancel
	    .bobED buttonconfigure Cancel -padx 3 -pady 3 -borderwidth 2
	}

    } else {
	while {[winfo viewable .bobED]} {
	    # means the previous pass one has either an "after" command
	    # issued to deactivate after a timeout, or is waiting
	    # for acknowledge; in any case, let's wait till the window
	    # is gone
	    # This condition can only occur if an external process
	    # would send a command while the dialog box has a grab
	    # on the display, i.e. it cannot happen by pressing
	    # buttons.
	    after 50; update idletasks
	}
	.bobED configure -text $ltext -modality $mod -title $title \
		-font $::bob::internal::config(bobDialogFont) -image ${type}Img
	.bobED show OK
	if { $cancelButton == 0 } {
	    .bobED hide Cancel
	} else {
	    .bobED show Cancel
	}
    }
    # Prepare for automatic deactivation if so required.
    # Timing-out is same as pressing "Cancel" if this button is displayed;
    # otherwise, it is same as pressing "OK"
    if {$timeout > 0} {
	if {$cancelButton} {
	    set afterId [after $timeout {.bobED deactivate 0}]
	} else {
	    set afterId [after $timeout {.bobED deactivate 1}]
	}

	# Don't forget to cancel the after-events if a button gets pressed first,
	# and re-define the button's commands to their original settings
        set cmd [[[.bobED component bbox] component OK] cget -command]
	.bobED buttonconfigure OK -command \
	    "after cancel $afterId; $cmd; .bobED buttonconfigure OK -command {$cmd}"
        set cmd [[[.bobED component bbox] component Cancel] cget -command]
	.bobED buttonconfigure Cancel -command \
	    "after cancel $afterId; $cmd; .bobED buttonconfigure Cancel -command {$cmd}"
    }

    #!!#
    ##puts "about to activate .bobED"
    return [.bobED activate]
}
