#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bobTPL.tcl,v 1.145 2005/10/27 09:07:12 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# eallaert  14/11/1996  created
# eallaert  05/02/1997  fixed bug in switching textbox contents
# mbec      09/04/1997  modified
# mbec      16/04/1997  manpage & vltstandard - last touch
# eallaert  02/05/1997  abort scheme fixed, manpage update
# mbec      09/05/1997  added 'updateRawData' method - 
# mbec      13/05/1997  fixed editRawData method
#                       commented 'updateRawData' which is not used anyway
# mbec      14/05/1997  new check in checkTpl handles missing parameter in OBD
# mbec      17/06/1997  support the VALUE keyword (default without warning)
# eallaert  19/06/1997  sendCmd modified to allow parameters as separate args
# mbec      11/07/1997	header update
# eallaert  23/07/1997	better check of tsf correctness (missing keywords)
# eallaert  24/07/1997	added checkAbortFlag; set of commands not sent after
#                       the ABORT flag has been set is now configurable.
# eallaert  1997-09-25  TPL ID should have <ins>/<name>/<version> format
#                       Send TPL.<keywords> at START command to OS
#                       Use TPL PRESEQ to find sequencer script
# eallaert  1997-09-30  Check if all TPL keywords are also in TSF
# eallaert  1997-10-17  Use OBS(TPLNO) as ID with TemplateStatus
# eallaert  1998-04-07  mustRepeat added
# eallaert  1998-06-18  engineering mode (editing) improved
# eallaert  1999-05-06  complain about a fixed parameter being present in
#                       OBD only if its value is different from VALUE in TSF
# eallaert  1999-08-03  added sending of TPL.START to OS (SPR 990308)
# eallaert  1999-09-21  added sendObsKeys (SPR 9903055)
# eallaert  1999-09-28  made _tplMarkUse public
# eallaert  1999-11-05  do not use "status" as method argument (shadowing
#                       problem in itcl3.0 with respect to object vars)
# eallaert  2000-03-20  removed _tplMarkUse ("command" parameter for trace
#                       can consist of multiple words, i.e. "$this markUse")
# eallaert  2000-04-14 	consider parallel templates: add optional parameter to
#                       convenience procedures, get initial value of OBSinfo
#                       during construction                     
# eallaert  2000-05-02 	do not assign undefined keywords their DEFAULT value
#                       if the latter is set to "NODEFAULT". Complain about 
#                       keywords missing from the OBD if their default is set
#                       to "NODEFAULT". See SPR 20000145.
# eallaert  2000-06-05  default storage for paramfiles: $INS_ROOT/INS_USER/MISC;
#                       permit paramfile storage only under $INS_ROOT
#                       honour the TPL.FILE.KEEP flag for paramfiles
# eallaert  2000-06-25  added pafReady public proc
# eallaert  2000-09-12  markUse declared public
# eallaert  2000-09-19  empty paramfile is now legal (for VIMOS MM)
#                       deal with the optional .HIDE keyword from tsf
#                       implemented "sendObsKeys -array" for vimos
# eallaert  2001-04-19  added OCS and INS to categories which can have indices
#                       (SPRs 20010130 and 20010240)
#                       added COU and ISS categories (see SPR 20010183)
#                       added FITS ops-logs (SPR 20010098)
# eallaert  2001-08-14  added finishOB convenience procedure
# eallaert  2002-03-27  made markUse more robust against improper use of upvar
# eallaert  2002-04-16  added LGS to the list of legal categories
#                       more robust/complete trace call-back (SPR 20020236)
#                       removed "repopulate", added "re" as arg to "populate"
#                       no reset of VALUE keywords if repopulation (checkTpl)
#                       added seeBobVars (anticipating SPR 20020304)
# eallaert  2002-05-29  applied SPRs 20020304 & 20020361 to checkKeywordUse
# eallaert  2002-10-10  gvar(bobProcNameOS/OH) can contain list (SPR 20020527)
#                       added method getExpNr
# eallaert  2003-01-17  fixed seeBobVars for diffs between Tcl 8.3.3 and 8.3.5
# eallaert  2003-03-17  TPL.START not reset after editing a running TPL (SPR 20030090)
# eallaert  2003-04-07  fix RANGE check for filenames (SPR 20030099)
# eallaert  2003-04-28  don't 'storeParamFile' twice (SPR 20030144)
#
#

#************************************************************************
#   NAME
#   BobTPL - base class for template objects
# 
#   SYNOPSIS
#   BobTPL <obj_name>|#auto \
#          <rawData> <obdSource> <canvas> <scrolledText> \
#          <initialObsInfo> <tplNr> <args>
#
#   DESCRIPTION
#   defines template objects - inherits BobBaseObject
#   
#   METHODS
#   constructor: contruct the BobBaseObject, and do the store in array (always
#       done at creation time for templates). Do also a first check on
#       the parameters value;see [incr Tcl]
#   destructor: delete any saved paramfiles and proceed with base object
#       destruction;
#   configure: see [incr Tcl]
#
#   abort: request an abort of the template's execution. This will set a 
#       variable called "abort", which can be upvar-ed by the template.
#   ackabort: acknowledge the abort (sendStatus)
#   unmark: extend 'unmark' to also clear the log of this template
#   editRawData ?<cats>? ?<pars>?: pops up the TPL editor. The set of parameters
#       to be edited depends on the 2 arguments:
#       <cats> : name of a single keyword-category to be edited, or the word 
#                "all" to include all categories. Default: all
#       <pars> : name of specific keywords to edit (assuming <cats> is not set
#                to a "all"). Default: all
#   execute <OBSinfo> <seqNr> <statVarName>: execute the template, whereby
#       its sequential number in its parent OB is <seqNr>;
#       <OBSinfo> contains relevant info passed to the template for execution.
#       This method returns potentially messages and the exec-status in a 
#       variable whose name is given in <statVarName>. Possible values for 
#       exec-status:
#       -3 : MustRepeat
#       -2 : Paused
#       -1 : Aborted; returns also the string "ACK ABORT".
#        0 : OK - normal completion; return value will be empty
#       >0 : error - the return value of the method itself will contain an
#            error message
#             1: Could not find the template signature file
#             2: Could not find the sequencer script
#             3: TPL ID has illegal format
#            10: Error returned by the template
#            11: Template was aborted at OS-level
#       Template scripts should return the following way, with proper values:
#          return "<RA> <Dec>" : for successfully terminated acquisition 
#               templates, with <RA> and <Dec> the coordinates of the acquired
#               object (both in degrees, as a float number, identical to how
#               the primary FITS keywords RA and DEC are written)
#          return {} or return without value: if all went OK, for all but 
#               acquisition templates
#          error "message" : if there is some error within the template, or an 
#               error  returned from the OS-level (e.g. ABORT).  The setting
#               of the ABORT-flag within BOB causes the template to skip
#               SETUP and START commands.  When the template sees the abort-
#               flag (see above), it should return an "ACK ABORT".
#   getEditedData: returns the edited data in the form of "raw" template data
#   getExpNr: returns a list with the current exposure number and the number of
#             exposures (TPL(EXPNO) resp. TPL(NEXP)).
#   getTplId: returns TPL(ID); used to make nice tabs label wile editing an OB
#   getTplType: returns TPL(TYPE)
#   init_editRawData <pathName> ?<cats>? ?<pars>?: initializes the TPL-editor
#       widget <pathName> with the data as indicated by the two optional args:
#       <cats> : name of a single keyword-category to be edited, or the word 
#                "all" to include all categories. Default: all
#       <pars> : name of specific keywords to edit (assuming <cats> is not set
#                to a "all"). Default: all
#   mark: 'select' this Tpl and display its log in the corresponding text 
#       widget.
#   mustRepeat: request a MUSTREPEAT of the template's execution; this implies
#       an abort (see above). 
#   paste <object> <where> : paste <object> <where> relative to this 
#   pause: pause the template
#   populate: create all BobVisualTree child associated with this template
#   printLog: prints the log
#   showExpNr: shows current exposure number in canvas area display
#   updateRawData: builds a TPL raw-data string starting from the keyword
#       arrays.
#
#   CLASS PROCEDURES
#   checkAbortFlag: convenience procedure to check if bob's ABORT button was 
#       pressed.  It will automatically generate an "ACK ABORT" error message 
#       if the abort flag was set, otherwise it returns with an empty string
#
#   finishOB ?-status <status>?: tell the OB that when this template finishes,
#       it should skip the remaining templates and return the termination
#       status <status> (default: TERMINATED) to OH.
#
#   getResult <name> : retrieve the value for the previously stored <name>
#       variable. Remark that all variables are erased when the first TPL of
#       an OB starts it execution. In other words, result-passing from one
#       template to another is only possible within a single OB.
#       Bob itself stores the execution status (0 = OK, 1 = error) of the 
#       last template in the variable "tplExitStatus". This variable can
#       then be used by the call-back script to decide on proper actions.
#
#   pafReady <pathName> ?<keep>? : send an event to OH signalling that the
#       PAF-file <pathName> is ready for download. This file will be deleted by 
#       bob after OH fetched it, unless the boolean variable <keep> is set to
#       true (or 1). Unlike other events, this one will be sent to OH even if
#       the template executing this command is part of an OB which was not
#       downloaded from OH.
#       If <pathName> is a directory, there will be an event generated to OH
#       for every single file on that directory.
#
#   seeBobVars ?-localcopy <listOfCats>?: in the absence of the -localcopy
#       option, issues the proper set of "upvar" instructions, within the
#       context of the calling procedure, to make the various category-arrays
#       created/owned by BOB accessible as local arrays with the same name.
#       This procedure returns a list with as first element the list of 
#       category names which are then locally accessible.
#       If the "-localcopy" option is specified, <listOfCats> must be a list of
#       keyword category-arrays, for which there will be then a *copy* created
#       in the context of the calling procedure (i.e. these arrays are *not*
#       linked to the ones owned and accessed by BOB). The category OBS can
#       be included within <listOfCats>.
#
#   sendCmd <timeout> <cmd> ?<param1>? ?<param2>? ...: convenience procedure to
#       send commands to OS. This is the one and only way template-scripts are
#       allowed to use for communication with OS.
#       <timeout>: timeout value (in ms) for reply; 0 means return immediately
#                  with the commandId - do not wait for replies; this argument
#                  is not taken into account if the template-simulation mode 
#                  is on.
#       <cmd>    : command; this is either a commandname alone (in which case
#                  pararameters can be given in <paramX>) or it includes some 
#                  parameters (more parameters can be given as <paramX>).
#                  Typical examples of commandnames are SETUP and START, of 
#                  parameters are "-file <fileName>" and
#                  "-function <function1> <value1> [<function2> <value2> .....]"
#       <paramX> : parameter for <cmd>, if not already included as part of
#                  <cmd>.
#       This procedure returns:
#       - reply on command if waiting for last reply (<timeout> # 0).  Remark
#         that this reply is the new expoId value when the first SETUP command
#         for a new exposure is given (with an expoId-argument equal to 0).  
#         Only the last reply on the command is returned.
#       - cmdId if command was sent OK and <timeout> is set to 0; this can
#         then be used by the caller to e.g. attach an event (see seq_evtAttach)
#         to incoming replies on this command.
#       - error (with message) if any error occurs or if ABORT flag is 
#         acknowledged
#       There are a number of messages logged automatically logged by this 
#       procedure into BOB's template log window; the commands sent will be 
#       shown in green if operating in verbose mode (see configuration panel),
#       while in this case normal or error replies will be shown in green resp.
#       red. If <cmd> is WAIT, there will also be a message logged indicating
#       the termination of the exposure, when the final reply comes in.
#       Remark that sendCmd will not send any of a list of commands once the
#       abort flag has been set. This list can be defined/edited via the 
#       configuration panel. The skipping of such commands is indicated in BOB's
#       template log window.
#
#   sendObsKeys ?<timeout>? ?-array <listOfCats>?: convenience procedure to send
#       the OBS and DPR keywords, plus some TPL keywords, to OS. This is 
#       normally done automatically by bob when a "START" command is
#       given within the template; however, there are cases where data
#       can be produced without starting an exposure, i.e. without a START
#       command. The "sendObsKeys" procedure provides for these cases a
#       means to still have the proper OBS-, DPR- and TPL-keywords recorded
#       into the FITS header. The default timeout for OS to reply is 2000 ms.
#       Remark that a call to sendObsKeys will automatically increase the
#       exposure number (TPL EXPNO), so this proc should only be called once
#       per data file, and only if there is no START command involved.
#       The "-array <listOfCats>" optional pair is only legal in case OS is
#       not defined (i.e. BOB is used for particular engineering purposes).
#       <listOfCats> is now limited to OBS only. By using this optional 
#       argument, the array OBS will become available at the level that this
#       procedure is called. Remark that during normal operation (with OS)
#       there is no need to access the OBS keywords passed in the OB.
#       (This "-array <listOfCats>" option is actually deprecated: please use
#       "seeBobVars -localcopy <listOfCats>" instead).
#
#   setCallBack <script>: set the script to be evaluated upon termination of
#       this template to <script>, thereby overriding - for this template 
#       only - the setting in the configuration panel of the default template
#       call-back script.
#       <script> will be invoked whether or not the template returned an error.
#       It can find out the termination status of the template script via a
#       "getResult tplExitStatus" command (0 = OK, 1 = error). It
#       is up to the call-back to decide if and to what extent it should
#       continue if the template script returned with an error.
#       If the call-back script needs to get access to values the template
#       script has set, the "setResult" and "getResult" mechanism can be used.
#       Remark that these call-backs should not be seen as an extension of the
#       template's functionality (e.g. to prepare/start exposures). Instead,
#       their typical use is to log the template's termination or to put the
#       instrument in a safe state. For that reason. this call-back's return-
#       status does NOT influence the overall return-status of the template.
#
#   setResult <name> <value>: set the <value> for the variable <name>; this
#       value can then be retrieved by another template in the same OB -
#       see "getResult".
#
#   tplLog <text> ?<colour> ?<underline>??
#       append <text> to the current log-text, in colour <colour> (default is
#       standard default for text), and underlined if the boolean <underline>
#       is set. <text> is allowed to contain newline characters.
#       Remark that when "verbose communication" is on, the messages sent to
#       OS are logged in blue, and OK-replies are logged in SpringGreen4.
#       Errors come logged in red. The messages which are always added 
#       automatically by "sendCmd" (e.g. "starting exposure n of m...") are 
#       underlined. These colour-schemes of bob should be respected, in order
#       to avoid confusion.
#
#   PUBLIC VARIABLES
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#   ICD between the VLT Control SW and the Observation Handling Subsystem
#   (VLT-ICD-ESO-17240-19200)
#
#------------------------------------------------------------------------
#

#=============================================================================
#=============================================================================
itcl::class BobTPL {
    inherit BobBaseObject BobLogger

    constructor {rd obds cwin stwin args} {
	BobBaseObject::constructor TPL $rd $obds $cwin
	BobLogger::constructor $stwin
    } {}
    destructor {} 

    public method abort {args} {}
    public method ackabort {args} {}
    public method changeParamValue {cat par val} {}
    public method editRawData {args} {};
    public method execute {args} {}
    public method getEditedData {args} {}
    public method getExpNr {} {return [list $TPL(EXPNO) $TPL(NEXP)]}
    public method getTplId {} {return $tplId}
    public method getTplType {} {return $TPL(TYPE)}
    public method init_editRawData {mfw {cat all} {par all} } {};
    public method mark {args} {}
    public method markUse {args} {};	# mark keywords used
    public method mustRepeat {args} {}
    public method paste { obj where } {}  
    public method pause {} {};
    public method populate {args} {}
    public method printLog {} {}
    public method showExpNr {} {}
    public method unmark {args} {}
    public method updateLabel {args} {} ;
    public method updateRawData {} {} ;
    
    public proc checkAbortFlag {args} {};# called from the template execution.
    public proc finishOB {args} {};	# called from the template execution.
    public proc getResult {args} {};	# called from the template execution.
    public proc pafReady {args} {};	# called from the template execution.
    public proc seeBobVars {args} {};	# called from the template execution.
    public proc sendCmd {args} {};	# called from the template execution.
    public proc sendObsKeys {args} {};	# called from the template execution.
    public proc setCallBack {args} {};	# called from the template execution.
    public proc setResult {args} {};	# called from the template execution.
    public proc tplLog {args} {};	# called from the template execution.

    # List of legal values for TYPE keywords
    private common legalTypes {boolean coord file filename intlist intrect integer keywordlist keyword numlist number object paramfile pixel string target};
    # List of categories allowed in OBD and TSF (i.e. that bob can handle)
    # The keywords of these cats will be accessible in the template scripts
    # as elements of arrays (name of the array is the catecory name)
    private common legalCats {ADA AOS COU DEL DET DPR INS ISS LGS OCS SEQ TEL};
    private common maxCatIdx 9;		# max idx for cats with indices
    private common legalIndexedCats {DET INS OCS};	# cats which can have indices
    private common legalCatsWithIndices {};
    ::foreach cat $legalIndexedCats {
	::for {set idx 1} {$idx < $maxCatIdx} {incr idx} {
	    ::lappend legalCatsWithIndices ${cat}${idx}
	}
    }
    ::foreach cat "$legalCats $legalCatsWithIndices" {
	private variable $cat;		# array with <cat.> info from OBD
	private variable ${cat}tsf;	# array with <cat.> info from tsf
	private variable ${cat}usedKey;	# array with keys used by tpl script
	private variable ${cat}keyList {};# ordered list with keyword names
    }

    private variable tplCats {};	# list of cats passed to tpl script
    private variable executeLevel {};	# stack level of "execute" method
    private variable dfsKeys {};	# all keys from OBD and TSF
    private variable writeableKeys {TPL.EXPNO TPL.NEXP};
    private variable modifiedKeys {};	# list of keywords set by tpl script
    private variable namespaceKeys {};	# list of keywords upvar'd to other ns
    private variable unsetKeys {};	# list of keywords unset by tpl script
    private variable unknownKeys {};	# list of keywords from unknown cat <- upvar
    private variable callBack {};	# name of callback procedure
    private variable callBackPhase 0;	# flag dealing with call-back
    private variable filesToDelete {};	# list of paramfiles to delete
    private variable TPL;		# array with <TPL.> info from template
    private variable TPLusedKey;# array with <TPL.> keywords used by tpl script
    private variable OBSinfo;		# string with <OBS.> info from OB
    private variable tplNr;		# sequential number of template in OB
 
    private method checkKeywordUse {args} {}
    private method checkTpl {args} {};  # check the template against its tsf
					# file & do correction if needed
					# + tell the user (pop-up dialogs)
    private method checkValues {args} {};# check the param values against tsf
    private method resetStatus {} {};	# used to be public; no longer used
    private method storeParamFile {args} {};	# store paramfile value in file
    private method _seeBobVars {args} {}
    private method _sendCmd {args} {}
    private method _sendObsKeys {args} {}
    private method _sendStatus {status args} {}
    private method _setCallBack {args} {}
    private method _upvarObsKeys {args} {}
    
    # List of mandatory TPL keywords checked and sent (to OS) by BOB
    # THIS LIST SHOULD BE TAKEN FROM THE DICTIONARY !!!!!!!!!!!!!!!
    private common mandatoryKeys {DID ID NAME PRESEQ START VERSION}
    private common p2ppOnlyKeys "EXECTIME GUI INSTRUM OVERHEAD RESOURCES TYPE\
       VERSION";	# TPL keywords not used by template script, w/o MODE
    private common p2ppKeys "$p2ppOnlyKeys MODE";	# incl MODE
    private common tsfKeys "$p2ppOnlyKeys  DID PRESEQ REFSUP"; # keys taken from tsf
    private common tsfKeysTPL {};	# id, but with "TPL." prefix
    ::foreach key $tsfKeys {
	::lappend tsfKeysTPL TPL.$key
    }
    # List of "excused" keywords (for checking if touched by tpl script)
    private common exKeys "$p2ppKeys $mandatoryKeys EXPNO NEXP"
    private common simVar {};	# simulation variable
    private common THIStpl;	# necessary to get the proc working; tells which
				# tpl is running for calls issued from a proc
    private common nsc {not set};	# namespace of object; flag unitialized

    private variable insMode {};	# instrument mode for this template
    private variable labelBase \
	    {???? - TPL with problems - pls correct and reload OB};
	    # text next to pointer (w/o exp info)
    private variable parentWidget;	# widget where you're editing
    private variable tplId {};		# middle part of TPL ID
    private variable tpl {};		# name of template script
    private variable tplProcName {};	# proc name corresponding to script
    private variable tsf {};		# name of template signature file
    private variable typeCat {};	# matches types to children

    private proc _tplSim {args} {};	# handle sim delay for selected cmds
    private proc _tplSimDelay {args} {};# handle sim delay for other cmds
}

#=============================================================================

# ------------------------------------------------------------------
#                        CONSTRUCTOR
# ------------------------------------------------------------------
#
itcl::body BobTPL::constructor {rd obds cwin stwin initialObsInfo nr args} {
    set warned 0
    set TPL(ID) ????
    if {[catch {bobStoreInArray $rawData $type} msg]} {
	bobDialog "There is something wrong with this template's ($TPL(ID))\
		parameters:\n\
		--------------------------------------------------------\n\
		$msg\n\
		--------------------------------------------------------\n\
		I will proceed, but you may find this template crippled.\n\
		I suggest to check this, and eventually to reload this OB." \
		application
	set warned 1
    }

    if {[catch {eval configure $args} msg] && !$warned} {
	bobDialog "There is something wrong with the parameters of $TPL(ID):\n\
		--------------------------------------------------------\n\
		$msg\n\
		--------------------------------------------------------\n\
		I will proceed, but you may find this template crippled.\n\
		I suggest to check this, and eventually to reload this OB." \
		application
    }
    switch -- $nsc {not set} {
	set nsc [namespace current]
	switch -- $nsc :: {set nsc {}}
    }
    if {[info exists TPL(PARALLEL)]} { 
	# The tplNr could be wrong here (cf cut/paste), but is good enough
	# for parallel templates
	after idle "$this execute $initialOBSinfo $nr tttt"
    }
}

# ------------------------------------------------------------------
#                        DESTRUCTOR
# ------------------------------------------------------------------
#
itcl::body BobTPL::destructor {} {
    if {[llength $filesToDelete] > 0} {
	eval file delete -force -- $filesToDelete
    }
}
# ------------------------------------------------------------------
#                        PUBLIC METHOD abort
# ------------------------------------------------------------------
itcl::body BobTPL::abort { } {
    global gvar
    # !!!!!!!!!!!!!!!!! till we get the blessing
    set ::bob::internal::config(bobPropagateAbort) 0
    # !!!!!!!!!!!!!
    if {$::bob::internal::config(bobPropagateAbort)} {
	# This is sent without expoId !!!!! (timeout should prob. be config.)
	_sendCmd 20000 ABORT {}
    }
    # set abort flag last, as _sendCmd skips pending cmds if abort flag is set
    set abort 1
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD ackabort
# ------------------------------------------------------------------
# resetButtons will be done by OB
itcl::body BobTPL::ackabort { } {
    configure -status aborted
    if {!$mustRepeat} {
	_sendStatus ABORTED
	seq_logFitsAction TPL TPL EXEC ABORT
    }
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD changeParamValue
# ------------------------------------------------------------------
itcl::body BobTPL::changeParamValue {cat par val} {
    if {![string equal [set old [set ${cat}($par)]] $val]} {
	set ${cat}($par) $val
	seq_logFitsAction TPL TPL "" CHANGE
	seq_logFitsParRecord -string TPL TPL "" ID $TPL(ID)
	seq_logFitsComment "${cat}.$par \"$old\" -> \"$val\""
	updateRawData
    }
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD editRawData
# ------------------------------------------------------------------

itcl::body BobTPL::editRawData { {cat all} {par all} } {
    global ::gvar 
    
    panImportPanel bobEditor
    grab set .bobEditor

    # first (and only) page of tabnotebook is for this TPL
    set mfw [$gvar(bobEditorTnb) add -label $TPL(ID)]
    pack $mfw -fill both -expand yes -padx 1 -pady 1

    set uneditedRawData [init_editRawData $mfw $cat $par]
    $gvar(bobEditorTnb) select 0;	# select this page 
    vwait ::bob::editingDone
    set editingDone [set ::bob::editingDone]
    switch -- $editingDone 0 - 1 {
	set rawData {}
	if {[string equal $cat all] || [lsearch -exact $cat TPL] >= 0} {
	    # edited data have TPL info at front
	    append rawData [getEditedData] \n $uneditedRawData
	} else {
	    # unedited data have TPL info up front
	    append rawData $uneditedRawData \n [getEditedData]
	}	    
	[up] updateRawData;	# make sure the OB gets updated
    }
    panClosePanel no .bobEditor
}
# ------------------------------------------------------------------
#                        PUBLIC METHOD execute
# ------------------------------------------------------------------
# <obsInfo> contains OBS keyword info, to be passed on to OS.
# <statName> is the name of a variable that will be set to the execution status
# of the template when this method returns.  Possible values are:
#   -3 : MustRepeat
#   -2 : Paused
#   -1 : Aborted
#    0 : OK - normal completion
#   >0 : error - the return value of the method itself will contain an
#        error message
#        1: Could not find the template signature file
#        2: Could not find the sequencer script
#        3: Illegal format of TPL.ID
#       10: Error returned by the template
#       11: Template was aborted at OS-level
# Template scripts should return the following way, with proper values:
#   return {} : if all went OK
#   error "message" : if there is some error within the template, or an error
#               returned from the OS-level (e.g. ABORT).  The setting of the
#               ABORT-flag within BOB results automatically in an error
#               message with content "ACK ABORT" at the first opportunity.
#
itcl::body BobTPL::execute { obsInfo seqNr statName } {
    global gvar 
    upvar $statName stat

    # a skip completes as a normal completion.
    switch -- $flowCtrl SKIPPED {
	logLog "$tplId -- $TPL(NAME)" yellow 0 grey60
	logLog "\nSkipped at [clock format [clock seconds] \
		-format %Y-%m-%dT%H:%M:%S -gmt 1]\n" {} 1
	set stat 0
	disableFlowCtrlChange
	# Fold the OB back if so required
	if {$::bob::internal::config(bobAutoUnfold) && $fold_state=="unfolded"} {
	    foldAll folded
	}
	return {}
    }

    switch -- $status paused {
	# make sure not to delete any log-entries inserted by pause-action
    } default {
	# log-box should be OK, as we're starting with a "fresh" TPL
    }

    # Show the parameters of this template in its resp. listbox
    if {$::bob::internal::config(bobAutoUnfold) && [string equal $fold_state folded] && \
	    ![info exists TPL(PARALLEL)]} {
	foldAll unfolded
    }

    # only break if not yet paused on this item
    if {[string equal $flowCtrl BREAK] && [string equal $status idle]} {
	pause;	# takes care of log and sound-file
	set stat -2
	return {}
    }

    switch -- {} $tsf {
	set stat 1
	return "The template signature file for $tplId was not found\n\
		when this template was loaded.  Hence it cannot be executed,\n\
		and this is how far the OB to which it belongs gets."
    }

    switch -- {} $tpl {
	set stat 2
	return "The sequencer script for $tplId was not found\n\
		when this template was loaded.  Hence it cannot be executed,\n\
		and this is how far the OB to which it belongs will get."
    }

    source $tpl;         # get the proc definitions in

    # execute this template, but first check any pending abort
    if {$abort} {
	# this is a 'late' abort pending before the template is actually started
	set abort 0;	# template not started -> not aborted
	set stat -1;	# tell OB it got the abort
	return {};	# do not use 'ACK ABORT': it's detected before starting
    }
    configure -status running
    update;		# update the canvas (display 'running'-icon)

    set tplNr $seqNr;		# Template's sequential number in OB
    set OBSinfo $obsInfo;	# keep this locally, for _sendCmd/sendObsKeys
    # create a local array with the OBS keywords
    #!#while {![lempty $obsInfo]} {
    #!#	set key [lvarpop obsInfo]
    #!#	set value [lvarpop obsInfo]
    #!#	set index [string range $key 4 end]
    #!#	set OBS($index) $value
    #!#}

    set TPL(EXPNO) 0

    # make sure the DPR category is included (scripts could otherwise end up
    # creating a local DPR array, which is not seen by _sendCmd)
    if {![array exists DPR]} {
	set DPR(abc) abc
	array unset DPR abc
    }

    set tplCats [lrmdups "TPL $typeCat DPR"];	# make sure DPR is in there

    # set TPL.START before setting traces (=>one exception less)
    set time [clock format [set startTime [clock seconds]] \
	    -format %Y-%m-%dT%H:%M:%S -gmt 1]
    set TPL(START) $time
    if {$::bob::internal::info(interface)} {
	# Put traces on categories, to see if all keywords are used
	foreach cat $tplCats {
	    trace add variable $cat {read write unset} "$this markUse"
	    switch -- $cat TPL {
		foreach key [array names TPL] {lappend dfsKeys TPL.$key}
	    } default {
		foreach key [set ${cat}keyList] {lappend dfsKeys ${cat}.$key}
	    }
	}
    }

    # Reset the vars flagging the use of "gvar configuration vars" 
    array unset ::bob::internal::abusedGvars

    if {$seqNr != 0} {
	# means no parallel execution (independent from TPL(PARALLEL) setting)
	set THIStpl $this
    }


    logLog "$tplId -- $TPL(NAME)" yellow 0 grey60
    logLog "\nStarted at $time\n" {} 1
    _sendStatus STARTED
    seq_logFitsAction TPL TPL EXEC START
    seq_logFitsParRecord -string TPL TPL "" ID $TPL(ID)
    ##seq_logFitsParRecord -string TPL TPL "" NAME $TPL(NAME)
    
    set executeLevel [info level]

    # Fix up the body of the proc, to give automatic access to the <CAT> arrays
    set args [::info args $tplProcName]
    if {[llength $args] == 0} {
	set body [concat "foreach cat {$tplCats} {upvar \$cat \$cat};" \
		[::info body $tplProcName]]
	proc $tplProcName {} $body
	##puts "$tplProcName - body:\n$body"
	set execStat [catch {$tplProcName} reply]
    } else {
	set execStat [catch {$tplProcName $tplCats $this} reply]
    }

    # Do the call-back stuff; the return value of this call-back is
    # discarded, unless if we get an error.
    [up] setResult tplExitStatus $execStat
    set callBackPhase 1
    if {[clength $callBack] != 0} {
	set callBackScript $callBack
	logLog "\nEvaluating call-back script '$callBack'"
    } elseif {[string length [string trim $::bob::internal::config(tplCallBack)]] != 0} {
	set callBackScript $::bob::internal::config(tplCallBack)
	logLog "\nEvaluating default call-back script '$::bob::internal::config(tplCallBack)'"
    }
    if {[info exists callBackScript]} {
	set body [concat "foreach cat {$tplCats} {upvar \$cat \$cat};" \
		      $callBackScript]
	proc callBackProc {} $body
	if {[catch {callBackProc} msg]} {
	    logLog $msg red
	}
	logLog "Call-back script finished"
	rename callBackProc {};	# should no longer be defined
    }

    rename $tplProcName {};	# should no longer be defined

    if {$::bob::internal::info(interface)} {
	# Remove traces on categories
	foreach cat $tplCats {trace remove variable $cat {read write unset} "$this markUse"}
	checkKeywordUse $tplCats
    }

    logLog $reply red;	# display the error (if any) now

    set stopTime [clock seconds]

    logLog "\nFinished in [expr {$stopTime-$startTime}] seconds at\
	    [clock format $stopTime -format %Y-%m-%dT%H:%M:%S -gmt 1]\n" {} 1

    if {$::bob::internal::config(bobSoundOn) && 
	([llength $::bob::internal::config(bobTplSound)] > 0)} {
	::bob::bobPlaySound [bobFindSound $::bob::internal::config(bobTplSound)]
    }

    if {$::bob::internal::config(bobAutoUnfold) && [string equal $fold_state unfolded] && \
	    ![info exists TPL(PARALLEL)]} {
	foldAll folded
    }
    if {$execStat == 1} {
	switch -- $reply "ACK ABORT" {
	    # puts "got ACK ABORT reply in executing TPL"
	    if {$mustRepeat} {set stat -3} else {set stat -1}
	    ackabort
	} "OS ABORT/ERROR" {
	    # puts "got OS ABORT/ERROR reply in executing TPL"
	    set stat 11
	    configure -status fault
	    set reply "Exposure not terminated properly during execution\
		    of $tplId"
	    _sendStatus ABORTED {OS did not terminate exposure successfully}
	    seq_logFitsEvent -ufo {OS did not terminate exposure successfully}
	    seq_logFitsAction TPL TPL EXEC ABORT
	} default {
	    set stat 10
	    configure -status fault
	    if {$::bob::internal::config(bobUserLevel) > 0} {
	    	set reply "$reply\n\nError Stack :\n\n$::errorInfo"
	    }
	    # Save error stack for eventual "snapshot" later on
	    set ::bob::internal::tplErrorInfo $::errorInfo
	    set ::bob::internal::tplId $tplId
	    set ::bob::internal::tplExpNr [list $TPL(EXPNO) $TPL(NEXP)]
	    set reply "Problem during execution of $tplId:\n\n==> $reply <=="
	    _sendStatus ABORTED "template error: $reply"
	    seq_logFitsEvent -ufo {template error}
	    seq_logFitsAction TPL TPL EXEC ABORT
	}
    } else {
	# It could be that the abort-command is pending, and that the template
	# script did not look at the proper flags.  So let's correct that here.
	if {$abort} {
	    if {$mustRepeat} {set stat -3} else {set stat -1}
	    set reply "ACK ABORT"
	    ackabort
	} else {
	    set stat 0
	    configure -status done
	    _sendStatus TERMINATED $reply
	    seq_logFitsAction TPL TPL EXEC STOP
	}
    }
    return $reply
}
# ------------------------------------------------------------------
#                        PUBLIC METHOD getEditedData
# ------------------------------------------------------------------
itcl::body BobTPL::getEditedData { } {
    set childPrefix [namespace tail $this]

    # retrieve all the entries
    foreach {llab lent} [$parentWidget.sel getEntries] {}
    
    set maxindex [llength $llab]
    set maxindex2 [llength $lent]
    
    if {$maxindex != $maxindex2} {
	puts "BUG: label and entries have different size !\n\
		Please report to engineering team\n\
		I try to continue - please check the value later ..."
    }
    set rd "# TPL edited by BOB\n"
    for {set i 0} {$i < $maxindex} { incr i } {
	set key [lindex $llab $i]
	if {![lsearch -exact $tsfKeysTPL $key] >= 0} {
	    set val [lindex $lent $i]
	    set comp [split $key .]
	    set cat [lindex $comp 0]
	    set keyWoCat [join [lrange $comp 1 end] .]
	    set ${cat}($keyWoCat) $val
	    append rd [format $rdFormat ${key} ${val}]\n
	    switch -- $cat TPL {} default {
		${childPrefix}.$key configure -label "$keyWoCat = $val"
	    }
	}
    }
    updateLabel 0
    seq_logFitsAction TPL TPL "" CHANGE
    seq_logFitsParRecord -string TPL TPL "" ID $TPL(ID)
    return [append rd \n\n]
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD init_editRawData
# ------------------------------------------------------------------
itcl::body BobTPL::init_editRawData {mfw {cats all} {pars all}} {
    global ::gvar
    set parentWidget $mfw
    set uneditedRawData {}
    switch -- $cats all {
	set cats  "TPL $typeCat"
    } default {
	foreach line [split $rawData \n] {
	    if {[string equal $pars all]} {
		if {[string first ${cats}. $line] < 0} {
		    if {[string equal $cats TPL]} {
			# TPL-part must come up front
			set uneditedRawData "${line}\n$uneditedRawData"
		    } else {
			append uneditedRawData ${line}\n
		    }
		}
	    } elseif {[string first ${cats}.${pars} $line] < 0} {
		if {[string equal $cats TPL]} {
		    # TPL-part must come up front
		    set uneditedRawData "${line}\n$uneditedRawData"
		} else {
		    append uneditedRawData ${line}\n
		}
	    }
	}
	##puts "uneditedRawData = $uneditedRawData"
    }	
    set labels {}
    set values {}
    if {[llength $cats] > 1 || [string equal $pars all]} {
	foreach cat $cats {
	    switch -- $cat TPL {
		set keys [array names TPL]
	    } default {
		set keys [set ${cat}keyList]
	    }
	    foreach key $keys {
		lappend labels ${cat}.$key
		# use lappend, as we may have weird chars in these values
		lappend values [set ${cat}($key)]
	    }
	}
    } else {
	set labels ${cats}.$pars
	# make it a list, as we may have weird chars in this values
	set values [list [set ${cats}($pars)]]
    }

    # as we're not sure if this widget's parent is currently displayed,
    # we'd better set the -scrollbar option to "on" (vs default "auto")
    BobPanedComboBox $parentWidget.sel -relief flat \
	    -selectbackground Grey80 -selectforeground black \
	    -background Grey90 -entryselectbackground yellow \
	    -entrybackground Grey90 -font $::bob::internal::config(bobEditorFont) \
	    -scrollbar on \
	    -helpvariable $gvar(editorHelpVar) \
	    -leftlisthelp "Listbox with keywords of this template" \
	    -leftentryhelp "Keyword selected for editing" \
	    -rightlisthelp "Listbox with values for template keywords" \
	    -rightentryhelp "Selected keyword's value; press <Enter>\
	    to apply (<Escape> to restore, arrow keys to move up/down)"
    pack $parentWidget.sel -fill both -expand true -pady 0 -padx 0
    $parentWidget.sel insert [list $labels $values]

    return $uneditedRawData
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD mark
# ------------------------------------------------------------------
itcl::body BobTPL::mark {} {
    displayLog
    BobBaseObject::mark
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD markUse
# -------------------------------------------------------------------
# This method needs to be public for the cases where nested template
# procedures access <cat> elements (e.g. TPL(REFSUP))
# Remark that some users (e.g. NAOS/CONICA) do in their libs an "upvar"
# of the categories to a local variable (i.e. they prepend a namespace
# to the cat name). As a result, the trace will trigger with that local
# cat name, so ${cat}usedKey won't exist. Try to deal with that. In
# case these heuristics fail, make sure a log entry explains.
# Don't worry here about duplicate entries - we'll remove them later
# at reporting time (we want to be as fast as possible here).
itcl::body BobTPL::markUse {category el op} {
    if {[catch {$this info variable ${category}usedKey -name}]} {
	set cat [namespace tail $category]
	if {[catch {$this info variable ${cat}usedKey -name}]} {
	    lappend unknownKeys ${category}.$el
	} else {
	    set ${cat}usedKey($el) 1
	    lappend namespaceKeys ${category}.$el
	}
    } else {
	set ${category}usedKey($el) 1
    }

    switch -- $op write {
	lappend modifiedKeys ${category}.$el
    } unset {
	lappend unsetKeys ${category}.$el
    }

}

# ------------------------------------------------------------------
#                        PUBLIC METHOD mustRepeat
# ------------------------------------------------------------------
# Same as abort, except an additional flag to signal that no sendStatus
# is wanted
itcl::body BobTPL::mustRepeat { } {
    set mustRepeat 1
    abort
}

#---------------------------------------------------------------------------
#                        PUBLIC METHOD paste
#---------------------------------------------------------------------------
itcl::body BobTPL::paste { obj where } {
    global gvar
    BobBaseObject::paste $obj $where
    
    # make a copy of obj
    set copyobj [BobTPL #auto [$obj getRawData] [$obj getObdSource] $gvar(cvOB) $gvar(stTpl) "" 0]
    [up] add ${nsc}::$copyobj $where $this
    $copyobj populate
    [up] updateRawData
    return ${nsc}::$copyobj
}


# ------------------------------------------------------------------
#                        PUBLIC METHOD pause
# ------------------------------------------------------------------
itcl::body BobTPL::pause {} {
    global gvar
    configure -status paused
    ::bob::bobPlaySound [bobFindSound $::bob::internal::config(bobBreakpointSound)]
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD populate
# ------------------------------------------------------------------
# Important notice:
# Objects under TPL level are named following their tree position
# This will be used for the update of GUI after editing
# The optional arg indicates whether this is a re-population or not.
itcl::body BobTPL::populate {{re 0}} {
    set typeCat {}
    ##catch {unset names}
    foreach cat $legalCats {
	if {[lsearch -exact $legalIndexedCats $cat] >= 0} {
	    set maxIdx $maxCatIdx
	} else {
	    set maxIdx 0
	}
	if {[catch {bobStoreInArray $rawData $cat $maxIdx} msg]} {
	    bobDialog "There is a problem with some $cat parameters of\
		    this template ($TPL(ID)):\n\
		    --------------------------------------------------------\n\
		    $msg\n\
		    --------------------------------------------------------\n\
		    Please correct and reload this OB." \
		    application
	} else {
	    switch -- $msg {} {} default {
		append typeCat " $msg"
	    }
	}
    }
    set childPrefix [namespace tail $this]

    # create visual for CATegories
    foreach cat $typeCat {
	set ${cat}keyList $names($cat)
    }
    # next line updates the values, in particular for parmfile-types
    # this needs to be done here, before the sub-object labels are drawn
    updateLabel 1 $re;	# flag if this is a first-pass or not

    set canvas [$this canvas]
    foreach cat $typeCat {
	set obj [BobVisualTree $childPrefix.$cat $canvas -label $cat]
	
	foreach elm [set ${cat}keyList] {
	    if {![catch {set ${cat}tsf(${elm}.HIDE)} hide] || \
		    ![string equal $hide bob]} {
		set subobj [BobVisualTree $childPrefix.$cat.$elm $canvas \
			-label "$elm = [set ${cat}($elm)]" -fold_state nofold \
			-icon nofold]
		$obj add ${nsc}::$subobj
	    }
	}
	add ${nsc}::$obj
    }
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD printLog
# ------------------------------------------------------------------
# This will also print out the label of the OB; this should not be necessary
# in Bob's GUI, at least not if you assume the user has eyes in his/her skull.
itcl::body BobTPL::printLog {} {
    set header "TEMPLATE LOG: $labelBase"
    set len [string length $header]
    append header \n
    for {set i 0} {$i < $len} {incr i} {
	append header -
    }
    append header "\nParent OB: [[up] cget -label]\n\n"
    BobLogger::printLog $header
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD showExpNr
# ------------------------------------------------------------------
itcl::body BobTPL::showExpNr {} {
    logLog "starting exposure $TPL(EXPNO) of $TPL(NEXP) ([clock format\
	    [clock seconds] -format %Y-%m-%dT%H:%M:%S -gmt 1]) ..." {} 1
    configure -label "$labelBase (exp $TPL(EXPNO) of $TPL(NEXP))"
    update
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD unmark
# ------------------------------------------------------------------
itcl::body BobTPL::unmark {} {
    clearLog
    BobBaseObject::unmark
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD updateLabel
# ------------------------------------------------------------------
# This method exercises some checks and updates the label displayed.
itcl::body BobTPL::updateLabel {{firstPass 0} {repopulation 0}} {
    if {[catch {set TPL(MODE)} insMode]} {
	set insMode {};		# TPL.MODE not in OBD
	set modeSetInObd 0
    } else {
	unset TPL(MODE);	# take one from TSF further on
	set modeSetInObd 1
    }
    
    set tplId [split $TPL(ID) /]
    if {[llength $tplId] == 3} {
	set tplId [lindex $tplId 1]
    } elseif {[llength $tplId] != 1} {
	bobDialog "The TPL ID ($TPL(ID)) is not in the correct format;\n\
		Check the TPL dictionary $TPL(DID).\n\
		This template is illegal and cannot be coped with." \
		application
	return
    }

    set tsf [bobFindTsf $tplId $insMode]
    switch -- {} $tsf {
	bobDialog "Could not find the template signature file\n\
		for $TPL(ID).  This template cannot be executed, and\n\
		you will have to reload the OB to which it belongs,\n\
		after making the template signature file available." \
		application
	return
    }
    if {[catch {checkTpl $tsf $firstPass $repopulation} msg]} {
	bobDialog $msg application 15000 error
	return
    }

    if {[catch {bobFindSeq $TPL(PRESEQ) $insMode} tpl]} {
	bobDialog "The keyword TPL.PRESEQ is not set in the template signature\
		file.\n\
		I will look for the template script using naming rules." \
		application
	set tpl [bobFindSeq $tplId $insMode]
    }

    switch -- {} $tpl {
	bobDialog "Could not find the sequencer script for \n\
		for $TPL(ID).  This template cannot be executed, and\n\
		you will have to reload the OB to which it belongs,\n\
		after making the sequencer script available." application
    }

    switch -- [info exists TPL(MODE)] 0 {
	switch -- $modeSetInObd 0 {
	    bobDialog "The keyword TPL.MODE is not set in the template\
		    signature file,\nnor in the OB Description." application
	} 1 {
	    bobDialog "The keyword TPL.MODE is not set in the template\
		    signature file. Taking the value from the OBD." application
	    set TPL(MODE) $insMode
	}
    }

    # Decide on the name of the proc corresponding with this template
    set tplProcName [file rootname [file tail $tpl]]
    switch -- $tplProcName $tplId {
	set tplProcName $tplId
	catch {set labelBase "$tplId -- $TPL(NAME)"}
    } default {
	catch {set labelBase "$tplId ($tplProcName) -- $TPL(NAME)"}
    }
    configure -label $labelBase
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD updateRawData
# ------------------------------------------------------------------
# Method gets called only upon "cut" and "drag" operations, plus after
# editing keywords.
# Remark that ID has to be the first line !!!
# TPL-keywords that are supposed to be in the tsf should not be included
# in the raw data, nor is TPL.START
itcl::body BobTPL::updateRawData {} {
    set rawData "# TPL edited by BOB\n[format $rdFormat TPL.ID $TPL(ID)]\n"
    foreach key [array names TPL] {
	if {![lsearch -exact $tsfKeys $key] >= 0} {
	    switch -- $key ID {} START {} default {
		append rawData [format $rdFormat TPL.$key $TPL($key)]\n
	    }
	}
    }
    # now handle categories / parameters value
    foreach cat $typeCat {
	foreach key [lsort [set ${cat}keyList]] {
	    if {[info exists ${cat}(${key}.PARFILE)]} {
		# "old" value:
		###set value [set ${cat}($key.PARFILE)]
		# "new" value: read in the file
		set value [read_file [set ${cat}($key)]]
		regsub -all -- \n $value {\n} value
		regsub -all -- \" $value {\"} value
		append rawData [format $rdFormat $cat.$key $value]\n
	    } else {
		append rawData [format $rdFormat $cat.$key [set ${cat}($key)]]\n
	    }
	}
    }
    updateLabel 0
    [up] updateRawData
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# ------------------------------------------------------------------
#                        PRIVATE METHOD checkKeywordUse
#------------------------------------------------------------------------------
# checkKeywords will compare the list of keywords passed to the template
# script to the ones actually used, and log the list of unused ones.
# Remark that some TPL-keywords are not necessarily used by the template
# script - they are in the tsf for the needs of P2PP only (see p2ppOnlyKeys).
# The TPL-keywords in the mandatoryKeys list are always sent by BOB itself,
# unless there was no START command given (in which case there will be no
# FITS header). So no need to complain about these keywords either.
# "p2ppKeys" and "mandatoryKeys" are merged together in "exKeys",
# and EXPNO plus NEXP are also appended to this list.

# See also SPR 20020304 and 20020361 for more info on what can/should be
# listed in the TPL log.
### Remark: some of the "writeableKeys" should only be written into
### by BOB (i.e. TPL.EXPNO)!! Don't know how to deal with this ...
itcl::body BobTPL::checkKeywordUse {cats} {
    global gvar
    foreach cat $cats {
	foreach el [array names $cat] {
	    if {![info exists ${cat}usedKey($el)] && \
		    !([string equal $cat TPL] && [lsearch -exact $exKeys $el] >= 0)} {
		lappend unusedKeywords ${cat}.$el
	    }
	}
    }

    if {[info exists unusedKeywords]} {
	logLog "\nOBD and/or TSF keys not used by the template\
		script, nor sent by bob to OS:\n[lsort $unusedKeywords]" red
    }

    if {[llength $unknownKeys] > 0} {
	set unknownKeys [lrmdups $unknownKeys]
	logLog "\nThe template script uses keys of unknown category\
		(due to misuse of 'upvar'):\n$unknownKeys" red
    }

    if {$::bob::internal::config(tplInternals) && [llength $namespaceKeys] > 0} {
	set namespaceKeys [lrmdups $namespaceKeys]
	logLog "\nCAUTION: the template script links keys to other namespaces:\
		(via 'upvar'):\n$namespaceKeys" red
    }

    set lists [intersect3 [lrmdups $modifiedKeys] $dfsKeys]
    set newKeys [lindex $lists 0];	# in first list, but not in second
    set modKeys [lindex $lists 1];	# common to both lists
    set modKeys [lindex [intersect3 $modKeys $writeableKeys] 0]

    if {$::bob::internal::config(tplInternals) && [llength $newKeys] > 0} {
	logLog "\nINFO: the template script created the following\
		keywords:\n$newKeys" red
    }

    if {[llength $modKeys] > 0} {
	logLog "\nThe template script modified the following OBD/TSF\
		keywords:\n$modKeys" red
    }

    set unsetKeys [intersect [lrmdups $unsetKeys] $dfsKeys]
    if {[llength $unsetKeys] > 0} {
	logLog "\nThe template script unset the following OBD/TSF\
		keywords:\n$unsetKeys" red
    }

    if {[info exists ::bob::internal::abusedGvars(read)]} {
	logLog "\nThe template script read the following vars,\
		instead of using the public\
		API:\n$::bob::internal::abusedGvars(read)" red
    }

    if {[info exists ::bob::internal::abusedGvars(write)]} {
	logLog "\nThe template script wrote into the following vars,\
		instead of using the public\
		API:\n$::bob::internal::abusedGvars(write)" red
    }

    if {[info exists ::bob::internal::abusedGvars(unset)]} {
	logLog "\nThe template script attempted to unset the following gvar\
		elements:\n$::bob::internal::abusedGvars(unset)" red
    }
}
# ------------------------------------------------------------------
#                        PRIVATE METHOD checkTpl 
# ------------------------------------------------------------------
# Determine if there is a "TPL.PARAM" at the start of a line, i.e. possibly
# after a newline and only with whitespace in front, followed by whitespace.
# We'll check if there is in fact more than 1, by looking for multiple
# occurences.  As we store the resulting substring-indices, we can find
# out if there is more than 1 by looking at these index-pairs
# This piece of code should only be called once per template-object, unless
# editing is involved. In the latter case (firstPass = 0), some of the errors
# and warnings signalled in the first pass should not be signalled again.
itcl::body BobTPL::checkTpl {tsf {firstPass 0} {repopulation 0}} {
    set sig [read_file $tsf]
    set remain 1
    set idx2 0
    set error 0
    while {$remain} {
	set pattern "\[\t \]*TPL\\.PARAM\[\t \]+"
	#puts "pattern = $pattern"
	set pattern1 "^$pattern|\n$pattern"
	set pattern2 "\n$pattern"

	if {[regexp -indices -- ($pattern1) $sig l1]} {
	    set idx1 [lindex $l1 0]
	    if {$idx2==0} {
	    	# do a store in array from the values defined in your tsf
		if {[catch {bobStoreInArray [string range $sig 0 $idx1] TPL} msg]} {
		    bobDialog "There is a problem with some TPL parameters of\
			    $TPL(ID)'s signature file:\n\
			    -------------------------------------------------\n\
			    $msg\n\
			    -------------------------------------------------\n\
			    Please correct and reload this OB." \
			    application
		    return
		}
		switch -- $TPL(ID) GT {unset TPL(REFSUP)}
		set TPL(DID) $::bob::internal::bobTplDid
		if {![info exists TPL(START)]} {
		    set TPL(START) {};	# set here, to avoid complaints
		}
		# TPL(NEXP) can be modified by template script, set default here
		if {![info exists TPL(NEXP)]} {
		    if {[info exists SEQ(NEXPO)]} {
			set TPL(NEXP) $SEQ(NEXPO)
		    } else {
			set TPL(NEXP) 1
		    }
		}
		# THIS LIST SHOULD BE TAKEN FROM THE DICTIONARY !!!!!!!!!!!!!!!
		foreach el $mandatoryKeys {
		    # value must contain quotes !!!
		    if {![info exists TPL($el)]} {
			bobDialog "The keyword TPL $el is missing for the\
				template $TPL(ID).\n\
				It must either be in the OB description\n\
				or in the template signature file." application
		    }
		}
	    }
	    switch -- [string index $sig $idx1] \n {
		# the match starts with a newline - skip this newline
		incr idx1
	    }
	    # Check if there are more of these patterns following further
	    if {[regexp -indices -- ($pattern2) \
		    [string range $sig [expr {$idx1 +1}] end] l2]} {
		# yes, there are
		set idx2 [expr {[lindex $l2 0] + $idx1}]
	    } else {
		# set the substring range of the only remaining parameter
		set idx2 end
		set remain 0
	    }
	} else {
	    bobDialog "The template signature file $tsf has an\n\
		    illegal format: there is no TPL.PARAM" application
	    break;   # break out of the "while" loop
	}
	set thisPar [string range $sig $idx1 $idx2]
	if {$remain} {
	    set sig [string range $sig $idx2 end];	# ready for next loop
	}
	if {[catch {bobStoreInArray $thisPar TPL} msg]} {
	    bobDialog "There is a problem with some TPL parameters of\
		    $TPL(ID)'s signature file:\n\
		    ------------------------------------------------------\n\
		    $msg\n\
		    ------------------------------------------------------\n\
		    Please correct and reload this OB." \
		    application
	    return
	}
	set pos [string first "." $TPL(PARAM)]
	set cat [string range $TPL(PARAM) 0 [expr {$pos -1}]]
	set name [string trim [string range $TPL(PARAM) [expr {$pos +1}] end]]
	set child [lindex [contents] [lsearch -exact $typeCat $cat]]	
	#set value [$child getVal $name]
	# As we keep a local copy of the data, there's no need to interrogate
	# the child to obtain a value

        # Check if the category we found is a legal one
	if {[lsearch -exact "$legalCats $legalCatsWithIndices" $cat] < 0} {
	    bobDialog "$TPL(ID)'s signature file \n\
		    contains keywords of category of \"$cat\",\n\
		    which are illegal. Please correct and reload this OB." \
		    application
	    return
	}

	if {[lsearch -exact $legalIndexedCats $cat] >= 0} {
	    set maxIdx $maxCatIdx
	} else {
	    set maxIdx 0
	}
	if {[catch {bobStoreInArray $thisPar $cat $maxIdx tsf} msg]} {
	    bobDialog "There is a problem with some $cat parameters of\
		    $TPL(ID)'s signature file:\n\
		    ------------------------------------------------------\n\
		    $msg\n\
		    ------------------------------------------------------\n\
		    Please correct and reload this OB." \
		    application
	    return
	}

	# Should have 3 records: type, range, default value
	# if VALUE instead of DEFAULT, it has precedence ....
	# in which case I set it to this value without warning
	set fixedVal 0
	if {[catch {string trim [set ${cat}tsf(${name}.TYPE)]} ttype]} {
	    bobDialog "The template signature file\n$tsf\nassigns no value\n\
		    to the ${cat}.${name}.TYPE keyword" application
	    set error 1
	    continue
	} elseif {[lsearch -exact $legalTypes $ttype] == -1} {
	    bobDialog "The template signature file\n$tsf\nassigns the value\
		    \"$ttype\"\n\
		    to the ${cat}.${name}.TYPE keyword.\n\n\
		    This is not a legal type." application
	    set error 1
	    continue
	}	    
	
	if {[catch {set ${cat}tsf(${name}.VALUE)} default]} {
	    # The RANGE keyword is mandatory for all types, except for
	    # "boolean" and "file" (and for these 2 it should not be given)
	    if {![string equal $ttype boolean] && 
		![string equal $ttype file] &&
		[catch {set ${cat}tsf(${name}.RANGE)} range]} {
		bobDialog "The template signature file\n$tsf\nassigns no value\n\
			to the ${cat}.${name}.RANGE keyword" application
		set error 1
		continue
	    }
	    if {[catch {set ${cat}tsf(${name}.DEFAULT)} default]} {
		bobDialog "The template signature file\n$tsf\nassigns no value\
			\nto the ${cat}.${name}.VALUE nor\
			${cat}.${name}.DEFAULT keywords" application
		set error 1
		continue
	    }
	} else {
	    set fixedVal 1
	    # should in principle check here that DEFAULT is *not* set,
	    # but this may brake some of the sh** produced by oslxCompile
	}

	# missing keys handled here: name is in the signature but not
	# the OBD ... inform about this, update with default
	if {[catch {set ${cat}($name)} value]} {
	    # if the cat is missing, add it too; don't forget the ${cat}keyList
	    if {[lsearch $typeCat $cat] == -1} {
    		lappend typeCat $cat
	    }
	    lappend ${cat}keyList $name;	# see SPR 990040
	    
	    if {!$fixedVal && $firstPass} {
		# will not occur during repopulation, as the first population
		# must have added these keywords already to OB (rawdata).
		if {[string equal $default NODEFAULT]} {
		    bobDialog "The parameter ${cat}.$name for template $TPL(ID)\n\
			    is not defined in the OBD. There is no default value\n\
			    defined in the template signature file \n$tsf"\
			    application 15000 error
		    # Define the variable anyway, to avoid further troubles.
		    set ${cat}(${name}) ""
		} else {
		    bobDialog "The parameter ${cat}.$name for template $TPL(ID)\n\
			    is not defined in the OBD. It is set to the default\n\
			    \"$default\"\n\
			    as defined in the template signature file \n$tsf"\
			    application 15000 error 0
		    set ${cat}(${name}) $default
		}
	    } else {
		# For undeclared fixed-value keys, set them to VALUE
		set ${cat}(${name}) $default
	    }
	} elseif {$fixedVal && $firstPass && ![string equal $value $default]  && \
		!$repopulation} {
	    # a VALUE keyword that has been edited will now be in the OB
	    # (rawdata); do not complain about it if repopulating (after reset)
	    bobDialog "The parameter ${cat}.$name for template $TPL(ID)\n\
		    is defined in the OBD as \"$value\",\n\
		    but it is overridden to \"$default\".\n\
		    See ${cat}.${name}.VALUE in the template signature file\
		    \n$tsf"\
		    application 15000 error 0
	    set ${cat}(${name}) $default
	} elseif {$fixedVal && !$firstPass && ![string equal $value $default]} {
	    bobDialog "The parameter ${cat}.$name for template $TPL(ID)\n\
		    is set in the TSF to the fixed value \"$default\",\n\
		    but you have set it to \"$value\".\n\n\
		    The latter value will be used." application 15000 warning
	}
    }
    
    catch {unset TPL(PARAM)};	# was created for check purpose only

    # Now check (but not for Generic Templates)
    # For a generic TPL, the headers were read; the rest is irrelevant
    switch -- $TPL(ID) GT {
	checkValues SEQ;	# check parameters of cat SEQ only
    } default {
	foreach cat $typeCat {
	    foreach key [set ${cat}keyList] {
		if {![info exists ${cat}tsf(${key}.TYPE)]} {
		    bobDialog "The OBD-info for template $TPL(ID) contains\
			    the keyword\n\n${cat}.${key}\n\n\
			    which is not in the template signature file \n$tsf"\
			    application 0 error
		    set error 1
		}
	    }
	}
	if {!$error} {checkValues $typeCat}
    }
}   
# ------------------------------------------------------------------
#                        PRIVATE METHOD checkValues
# ------------------------------------------------------------------
# This method assumes that the TSF and template parameters in OBD are OK!
# Check keywords of any of the categories in <cats>
# If problems are found with the type of values, they will be flagged
# by errors-dialogs (without timeout), and there will be no attempt 
# to "correct" them automatically (like by truncating floats if the 
# type is supposed to be integer).
itcl::body BobTPL::checkValues {cats} {
    set idx 0
    foreach cat $cats {
	foreach name [array names $cat] {
	    catch {string trim [set ${cat}tsf(${name}.TYPE)]} ttype
	    catch {set ${cat}(${name})} value
	    if {[catch {set ${cat}tsf(${name}.VALUE)} default] } {
		catch {set ${cat}tsf(${name}.DEFAULT)} default
		catch {set ${cat}tsf(${name}.RANGE)} range
	    }
	    set child [lindex [contents] $idx]
	    switch -- $ttype {
		integer {
		    if {[string is integer -strict $value] != 1} {
			if {[string equal $default NODEFAULT]} {
			    bobDialog "The parameter ${cat}.${name} of the $tplId\n\
				    template has the value \"$value\",\n\
				    which is supposed to be an integer.\n\n\
				    There is also no default value in the signature file\n$tsf\n\
				    Please correct the OB and the signature file." \
				    application 0 error
			} elseif {[string first "ISF " [string trimleft $default]] == 0} {
			    # indirection: reference to ISF - skip
			#!#} elseif {[string is integer -strict $default] != 1} {
			} elseif {[catch {expr {int($default) - $default}} msg] || $msg != 0} {
			    bobDialog "The parameter ${cat}.${name} of the $tplId\n\
				    template has the value \"$value\",\n\
				    which is supposed to be an integer.\n\n\
				    Its value is either fixed (.VALUE entry)\n\
				    or the .DEFAULT value in the signature file\n$tsf\n\
				    is also not an integer.\n\n\
				    Please correct the OB and the signature file." \
				    application 0 error
			} else {
			    bobDialog "The parameter ${cat}.${name} of the $tplId\n\
				    template has the value \"$value\",\n\
				    which is supposed to be an integer.\n\n\
				    Please correct the OB." \
				    application 0 error
			    ##set ${cat}($name) $default
			    # Not sure here that arrays and children exist already!
			    ##if {![catch {lindex [$child contents] [lsearch -glob \
			    ##			   [$child contents] *$name ]} grandchild]} {
			    ##	if {![lempty $grandchild]} {
			    ##	    $grandchild configure -label "$name = $default"
			    ##	}
			    ##}
			}
		    }
		}
		number {
		    if {[string is double -strict $value] != 1} {
			if {[string equal $default NODEFAULT]} {
			    bobDialog "The parameter ${cat}.${name} of the $tplId\n\
				    template has the value \"$value\",\n\
				    which is supposed to be a float.\n\n\
				    There is no default value in the signature file\n$tsf\n\
				    Please correct the OB and the signature file." \
				    application 0 error
			} elseif {[string first "ISF " [string trimleft $default]] == 0} {
			    # indirection: reference to ISF - skip
			#!#} elseif {[string is double -strict $default] != 1} {
			} elseif {[catch {expr {$default}} msg]} {
			    bobDialog "The parameter ${cat}.${name} of the $tplId\n\
				    template has the value \"$value\",\n\
				    which is supposed to be a float.\n\n\
				    Its value is either fixed (.VALUE entry)\n\
				    or the .DEFAULT value in the signature file\n$tsf\n\
				    is also not a float.\n\n\
				    Please correct the OB and the signature file." \
				    application 0 error
			} else {
			    bobDialog "The parameter ${cat}.${name} of the $tplId\n\
				    template has the value \"$value\",\n\
				    which is supposed to be a float.\n\n\
				    Please correct the OB." \
				    application 0 error
			    ##set ${cat}($name) $default
			    # Not sure here that arrays and children exist already!
			    ##if {![catch {lindex [$child contents] [lsearch -glob \
			    ##		          [$child contents] *$name ]} grandchild]} {
			    ##	if {![lempty $grandchild]} {
			    ##	    $grandchild configure -label "$name = $default"
			    ##	}
			    ##}
			}
		    }
		}
		filename {
		    set match 0
		    foreach pattern $range {
			if {[string match $pattern $value]} {set match 1; break}
		    }
		    if {!$match} {
			bobDialog "The parameter ${cat}.${name} of the $tplId\n\
			    	template is set to \"$value\",\n\
			    	which should match (any of) the pattern(s)\
				\"$range\".\n\n\
				(Warning only - no corrective actions yet\
				in this version of bob)" \
				application 10000 warning
		    }
		}		    
		paramfile {
		    if {[info exists ${cat}(${name}.PARFILE)]} return
		    # an empty paramfile is legal (for the purposes of VIMOS MM)
		    switch -- [string trim $value] {} {
			set ${cat}(${name}.PARFILE) {}
			continue
		    }
		    # 'value' could already have been substituted before by a
		    # filename. In this case, don't attempt to store it again.
		    if {[file isfile $value]} {
			continue
		    }
		    if {[catch {storeParamFile $value keepVar} filename]} {
			bobDialog "Error:\n\n\
				$filename\n\n\
				while dealing with parameter ${cat}.${name}\n\
				of the $tplId template (type: paramfile).\n\n\
				Please correct and re-load this OB." \
				application 0 error
			continue
		    }
		    # edit the rawData: remove file-data by filename
		    set i1 [string first $value $rawData]
		    if {$i1 == -1} {
			regsub -all -- \n $value {\n} value
			regsub -all -- \" $value {\"} value
			set i1 [string first $value $rawData]
		    }
		    set i2 [expr {$i1 + [string length $value]}]
		    set rawData [string range $rawData 0 [incr i1 -1]]$filename[\
			    string range $rawData $i2 end]
		    set ${cat}(${name}) $filename
		    set ${cat}(${name}.PARFILE) $value
		    if {!$keepVar} {
			lappend filesToDelete $filename
		    }
		}
	    }
	}
	incr idx
    }
}   

# ------------------------------------------------------------------
#                        PRIVATE METHOD resetStatus
# ------------------------------------------------------------------
# This method used to public, and was called as part of the "Reset" action, for
# each template individually. However, as a consequence of SPR 20010380, the
# "Reset" action was changed, and now the OB will delete all its template
# object, and recreate new ones from scratch (via its "repopulate" method). 
# For the time being, we leave this method here, just in case.
itcl::body BobTPL::resetStatus {} {
    configure -status idle
    resetPC
    deleteLog

    # unset all keywords names of this template (the calling proc needs to
    # make sure that this gets done - see BobBaseObject::resetStatus)
    foreach cat "TPL $typeCat" {
	array unset $cat *
	array unset ${cat}usedKey *
    }
    set unknownKeys {};	# list of keywords of unknown cat (due to upvar misuse)
    set dfsKeys {};	# list of keywords in OBD and TSF
    set modifiedKeys {};# list of keywords modified by template script
    set namespaceKeys {};# list of keywords linked to other namespace
    set unsetKeys {};	# list of keywords unset by template script

    catch {configure -label $labelBase}
    set callBack {}
    set callBackPhase 0
}

# ------------------------------------------------------------------
#                        PRIVATE METHOD _seeBobVars
# ------------------------------------------------------------------
# We need to execute the "upvar" at the right stack level:
#   - uplevel 2, if template script called directly seeBobVars:
#        _seeBobVars
#           -> seeBobVars
#              -> calling template script
#                 -> BobTPL::execute
#   - uplevel 4, if called from another namespace, because then
#     "unknown/_tpl_unknown" enter into the play. This
#     can be detected by looking at [info level <uplevel>].
#        _seeBobVars
#           -> seeBobVars
#              -> namespace eval ::BobTPL seeBobVars
#                 (remark that Tcl 8.3.3 does not report such "namespace" commands
#                 properly when queried with an "info level" command)
#                 -> _tpl_unknown seeBobVars
#                    -> calling procedure/namespace
#   - upvar <stackLevelOfBobsExecuteMethod> <fullyQualifiedName> <name>
itcl::body BobTPL::_seeBobVars {} {
    set thisLevel [info level]
    set uplevel 2
    # Check if we got here via _tpl_unknown, which means from another namespace.
    # Can only be if "thisLevel" is sufficiently above "executeLevel".
    if {$thisLevel -3 > $executeLevel} {
	set callerLevel3 [lindex [info level [expr {[info level] - 3}]] 0]
	if {[string equal $callerLevel3 ::_tpl_unknown]} {
	    set uplevel 4
	}
    }
    foreach cat $tplCats {
	uplevel $uplevel "upvar #$executeLevel ${nsc}::$cat $cat"
    }
    return [list $tplCats $this]
}
# ------------------------------------------------------------------
#                        PRIVATE METHOD _sendCmd
# ------------------------------------------------------------------
# _sendCmd <timeout> <cmd> <params>: sends a command to the OS process as a 
# result of the Template script execution.
# This will at the same time check for any pending abort and exit with
# an aborted status if so.
# NB: the handling of the pause could have been implemented also here, but it
# would become a major problem if one wants to resume execution.
# Thus a template can only be paused 'before starting' at the moment.
# TBD in the future on how to improve things.
# Remark that the "ACK ABORT" must be done by the template script, not here!
# However, _sendCmd will not permit to send the following commands once the 
# abort flag has been set:
#   - all commands in the ::bob::internal::config(tplSkipList) list (see config). 
#     Remark that due to SPR20040269 this tplSkipList can also contain
#     exclusions, which need to be preceded by a dash.
#     
#
# For the seq_msgSendCommand, do not use "eval" involving $command: it causes
# braces and quotes to disappear from $command, at the receiving side, due to
# how seq_msgSendCommand works with multiple arguments; split off the 
# commandname from the rest instead
itcl::body BobTPL::_sendCmd {timeout cmd params} {

    set command $cmd
    switch -- {} $params {} default {append command { } $params}
    if {$abort && !$callBackPhase} {
	if {([lsearch -exact $::bob::internal::config(tplSkipList) *] >= 0 && \
		[lsearch -exact $::bob::internal::config(tplSkipList) -$cmd] < 0) || \
		[lsearch -exact $::bob::internal::config(tplSkipList) $cmd] >= 0} {
	    logLog "Abort pending - skipping $command"
	    return
	}
    }

    # show the exposure number for a START command, and tell also OS what it is
    switch -- $cmd START {
	set expoIdIdx [lsearch -exact [string toupper $params] -EXPOID]
	if {$expoIdIdx == -1} {
	    set expoId {}
	} else {
	    set expoId "-expoId [lindex $params [incr expoIdIdx]]"
	}
	_sendObsKeys $timeout $expoId
	showExpNr
    } START_NO_OS {
	incr TPL(EXPNO)
	showExpNr
	set cmd START;		# for rest identical to START command
    }

    if {$::bob::internal::config(tplVerbose)} {
	logLog "$command" blue
    }

    if {$::bob::internal::config(tplSim)} {
	if {[catch {_tplSim $timeout $cmd $params} reply]} {
	    if {!$::bob::internal::config(tplVerbose)} {logLog "$command" blue}
	    #logLog "$reply" red
	    error "$reply"
	}
	if {$::bob::internal::config(tplVerbose)} {logLog $reply SpringGreen4}
	switch -- $cmd WAIT {
	    # Map options passed to WAIT command into desired string
	    set optionList [lindex $::bob::internal::config(bobWaitOptions) 0]
	    for {set i 0} {$i < [llength $optionList]} {incr i} {
		if {[regexp -- [lindex $optionList $i] $params] > 0} {
		    break
		}
	    }
	    if {$i < [llength $optionList]} {
		set text [lindex [lindex $::bob::internal::config(bobWaitOptions) 1] $i]
		logLog "exposure $TPL(EXPNO) of $TPL(NEXP) ${text} ([clock format \
		    [clock seconds] -format %Y-%m-%dT%H:%M:%S -gmt 1])" {} 1
	    } else {
		logLog "ended exposure $TPL(EXPNO) of $TPL(NEXP) ([clock format \
		    [clock seconds] -format %Y-%m-%dT%H:%M:%S -gmt 1])" {} 1
	    }
	    configure -label "$labelBase ($TPL(EXPNO) of $TPL(NEXP) exp ready)"
	}
	return $reply
    }

    if {[string length $::bob::internal::config(bobEnvNameOS)] == 0 || \
	    [string length [lindex $::bob::internal::config(bobProcNameOS) 0]] == 0} {
	error "Cannot communicate with OS as long as\n\
		OS's environment name or process name are not set.\n\n\
		Check the Environment settings (under Configure menu).\n"
    }

    # needed in case some of the config vars refer to environment vars:
    global env 
    if {[catch {seq_msgSendCommand [subst $::bob::internal::config(bobEnvNameOS)] \
	    [subst [lindex $::bob::internal::config(bobProcNameOS) 0]] $cmd $params} cmdId]} {
	if {!$::bob::internal::config(tplVerbose)} {logLog $command blue}
	#logLog "seq_msgSendCommand failed: $cmdId" red
	error "seq_msgSendCommand failed: $cmdId"
    }
    if {$timeout == 0} {return $cmdId}

    # Display the replies here as they come in, i.e one by one
    set lR 0
    while {!$lR} {
	set error 0
	set clockStart [clock clicks -milliseconds]
	if {[catch {seq_msgRecvReply $timeout -ascii $cmdId error lR} \
		 reply ]} {
	    if {!$::bob::internal::config(tplVerbose)} {logLog $command blue}
	    #logLog "$reply" red
	    error "$reply"
	} 
	set elaps [expr {[clock clicks -milliseconds] - $clockStart}]

	if {$error != 0 } {  
	    error "received error reply: $reply"
	}
    
	set timeout [expr {$timeout - $elaps}]
	if {$timeout <= 0} { set timeout 1}
	#puts "timeout = $timeout; elapsed = $elaps milliseconds"
	if {$::bob::internal::config(tplVerbose)} {logLog "$reply" SpringGreen4}
    }

    switch -- $cmd WAIT {
	# reply must be as expStatus db-attribute; bit 7 = success
	# !!!!!!!!!!!!!!!!! Till we got the blessing .....
	set ::bob::internal::config(bobAbortOnExpFailure) 0
	# !!!!!!!!!!!!!!!!!
	if {$::bob::internal::config(bobAbortOnExpFailure) && ([expr {$reply & 128}] == 0)} {
	    error "OS ABORT/ERROR"
	} else {
	    # Map options passed to WAIT command into desired string
	    set logMsg "ended exposure"
	    set optionList [lindex $::bob::internal::config(bobWaitOptions) 0]
	    for {set i 0} {$i < [llength $optionList]} {incr i} {
		if {[regexp -- [lindex $optionList $i] $params] > 0} {
		    break
		}
	    }
	    if {$i < [llength $optionList]} {
		set text [lindex [lindex $::bob::internal::config(bobWaitOptions) 1] $i]
		logLog "exposure $TPL(EXPNO) of $TPL(NEXP) ${text} ([clock format \
		    [clock seconds] -format %Y-%m-%dT%H:%M:%S -gmt 1])" {} 1
	    } else {
		logLog "ended exposure $TPL(EXPNO) of $TPL(NEXP) ([clock format \
		    [clock seconds] -format %Y-%m-%dT%H:%M:%S -gmt 1])" {} 1
	    }
	    configure -label "$labelBase (exp $TPL(EXPNO) of $TPL(NEXP) ready)"
	}

    }
    return $reply
}
# ------------------------------------------------------------------
#                        PRIVATE METHOD _sendObsKeys
# ------------------------------------------------------------------
# The argument "expoId" here should be either an empty string, or a string
# of the form "-expoId <expoId>"
# A side-effect of calling this method is that TPL(EXPNO) is incremented!
itcl::body BobTPL::_sendObsKeys {timeout {expoId {}}} {
    foreach el $mandatoryKeys {
	append TPLinfo " TPL.${el} \"$TPL($el)\""
    }
    set DPRinfo {}
    foreach el [lsort [array names DPR]] {
	append DPRinfo " DPR.${el} \"$DPR($el)\""
    }
    _sendCmd $timeout SETUP "$expoId -function $OBSinfo $DPRinfo \
	    $TPLinfo TPL.NEXP $TPL(NEXP) TPL.EXPNO [incr TPL(EXPNO)]"
}
# ------------------------------------------------------------------
#                        PRIVATE METHOD _sendStatus
# ------------------------------------------------------------------
itcl::body BobTPL::_sendStatus {stat args} {
    switch -- [set obdS [getObdSource]] OH - templateClient {
    	set cmdStr "[[up] obsId ] $tplNr [clock format [clock seconds] \
	    -format %Y-%m-%dT%H:%M:%S -gmt 1] $stat $args"
    	BobBaseObject::sendStatus TemplateStatus $obdS $cmdStr
    }
}

# ------------------------------------------------------------------
#                        PRIVATE METHOD _setCallback
# ------------------------------------------------------------------
itcl::body BobTPL::_setCallBack {script} {
    set callBack $script
}
# ------------------------------------------------------------------
#                        PRIVATE METHOD storeParamFile
# -------------------------------------------------------------------
# storeParamFile <value> <keepVar>
#  <value> : string with the paramfile
#  <keepVar>: name of variable where the TPL.FILE.KEEP flag needs to be stored.
#
# Returns the filename where the data is supposed to be stored.
# Default is $INS_ROOT/$INS_USER/MISC/<PAR.NAME>. This default is only
# used if TPL.FIL.DIRNAME is empty (blank). 
# An error is returned when either of the following occurs:
#   - the TPL.FILE.DIRNAME keyword is not specified
#   - the TPL.FILE.DIRNAME keyword is specified, but it is not coming
#     on its "own" line (i.e. no newline before); this means we have
#     a "bad" obd
itcl::body BobTPL::storeParamFile {value keepVar} {
    global INS_ROOT INS_USER
    if {![regexp -indices -- "\n\[\t \]*PAF\.NAME\[\t \]+\[^\n\;\]+" $value l]} {
	error "The PAF.NAME keyword is not part of the PARFILE \
		string.\n\n\The format of this OBD is therefore not standard."
    }
    foreach {i1 i2} $l {}
    lassign [string range $value [incr i1] $i2] key filename
    set filename [string trim $filename]
    if {[regexp -indices -- "\n\[\t \]*TPL\.FILE\.DIRNAME\[\t \]+\[^\n\;\]+" $value l]} {
	foreach {i1 i2} $l {}
	lassign [string range $value [incr i1] $i2] key dirname
	set dirname [string trim $dirname]
	if {[string length $dirname] != 0} {
	    # some environment vars could be part of name - resolve them
	    set dirname [namespace eval :: "subst $dirname"]
	} else {
	    # default (if TPL.FILE.DIRNAME is empty)
	    set dirname $INS_ROOT/$INS_USER/MISC
	}
	set filename [file join $dirname $filename]
	set dirname [seq_relToAbsPath [file dirname $filename]]
	# Security check: make sure absolute path is under INS_ROOT
	if {[string equal $INS_ROOT $dirname] != 0} {
	    error "The path resulting from combining the TPL.FILE.DIRNAME\
		    and PAF.NAME keywords\n\
		    must point to a subdirectory of INS_ROOT\n($INS_ROOT)."
	}
	
	write_file $filename $value

	# Now look for TPL.FILE.KEEP
	upvar $keepVar keepFlag
	set keepFlag 0
	if {[regexp -indices -- "\n\[\t \]*TPL\.FILE\.KEEP\[\t \]+\[^\n\;\]+" $value l]} {
	    foreach {i1 i2} $l {}
	    lassign [string range $value [incr i1] $i2] key keepFlag
	    set keepFlag [expr {$keepFlag ? 1 : 0}]
	}
	    
    } elseif {[regexp -- "\[\t \]*TPL\.FILE\.DIRNAME\[\t \]+" $value]} {
	error "The TPL.FILE.DIRNAME keyword is not on its own line.\n\
		Keywords which are part of a PARFILE string have to be\n\
		separated by (escaped) newlines.\n\n\
		The format of this OBD is therefore not standard."
    } else {
	error "The TPL.FILE.DIRNAME keyword is not part of the PARFILE \
		string.\n\n\The format of this OBD is therefore not standard."
    }	
    return $filename
}

# ------------------------------------------------------------------
#                        PRIVATE METHOD _upvarObsKeys
# ------------------------------------------------------------------
# next block is to deal with making the OBS keywords available
# at the template level (only allowed if no OS is active).
# For more explanation about calculating the "uplevel", see
# the private method _seeBobVars
itcl::body BobTPL::_upvarObsKeys {cats} {
    set thisLevel [info level]
    set uplevel 2
    # Check if we got here via _tpl_unknown, which means from another namespace.
    # Can only be if "thisLevel" is sufficiently above "executeLevel".
    if {$thisLevel -3 > $executeLevel} {
	set callerLevel3 [lindex [info level [expr {[info level] - 3}]] 0]
	if {[string equal $callerLevel3 ::_tpl_unknown]} {
	    set uplevel 4
	}
    }
    foreach cat $cats {
	if {[lsearch -exact "OBS TPL $typeCat" $cat] <0} {
	    error "illegal keyword category: $cat"
	}
	upvar $uplevel $cat up$cat
	if {[array exists up$cat]} {
	    error "cannot create array $cat - this array exists already"
	} elseif {[info exists up$cat]} {
	    error "cannot create array $cat - variable $cat exists already"
	}
	switch -- $cat OBS {
	    set obsInfo $OBSinfo
	    while {![lempty $obsInfo]} {
		set key [lvarpop obsInfo]
		set value [lvarpop obsInfo]
		set index [string range $key 4 end]
		##if {[catch {set upOBS($index) $value}]} {
		##    error "can't set \"OBS($index)\": variable OBS isn't array"
		##}
		set upOBS($index) $value
	    }
	} default {
	    foreach par [array names $cat] {
		##if {[catch {set up${cat}($par) [set ${cat}($par)]}]} {
		##    error "can't set \"${cat}($index)\": variable $cat isn't array"
		##}
		set up${cat}($par) [set ${cat}($par)]
	    }
	}
    }
    return
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ------------------------------------------------------------------
#                        PUBLIC PROCEDURE checkAbortFlag
# ------------------------------------------------------------------
# Convenience procedure to check if bob's ABORT button was pressed.  To be
# called by any template script/procedure at any convenient time.  This
# proc will automatically generate an "ACK ABORT" error message if the abort
# flag was set, otherwise return with an empty string
itcl::body BobTPL::checkAbortFlag {args} {
    if {[llength $args] == 0} {
	set tpl $THIStpl
    } else {
	set arg [lvarpop args]
	switch -glob -- $arg {
	    -tpl {set tpl [lvarpop args]}
	    default {error "checkAbortFlag called with wrong arguments"}
	}
    }
    if {[$tpl cget -abort]} {error "ACK ABORT"}
}
# ------------------------------------------------------------------
#                        PUBLIC PROCEDURE getResult
# ------------------------------------------------------------------
# optional argument:   tpl-object
# mandatory argument:  name (array-index)
# 
itcl::body BobTPL::getResult {args} {
    set arg [lvarpop args]
    switch -glob -- $arg {
	"-tpl" {set tpl [lvarpop args]; set name [lvarpop args]}
	default {set tpl $THIStpl; set name $arg}
    }
    # There should be no arguments left
    if {[llength $args] != 0} {
	error {wrong # args: should be "getResult ?<option>? <name>"}
    }
    return [[$tpl up] getResult $name]
}

# ------------------------------------------------------------------
#                        PUBLIC PROCEDURE finishOB
# ------------------------------------------------------------------
# optional argument:   tpl-object, status
# mandatory argument:
# 
itcl::body BobTPL::finishOB {args} {
    set tpl $THIStpl
    set status TERMINATED
    while {![lempty $args]} {
	set arg [lvarpop args]
	switch -glob -- $arg {
	    "-tpl" {set tpl [lvarpop args]}
	    "-status" {set status [lvarpop args]}
	    default {set args [linsert $args 0 $arg]; break}
	}
    }
    # There must be no args left
    if {[llength $args] != 0} {
	error {wrong # args: should be "finishOB ?-status <status>?"}
    }

    # tell to skip remaining TPLs
    [$tpl up] forceFinish $status
}

# ------------------------------------------------------------------
#                        PUBLIC PROCEDURE pafReady
# ------------------------------------------------------------------
itcl::body BobTPL::pafReady {path {keep 0}} {
    global seq_ccsEnvName seq_ccsProcName
    
    set path [seq_relToAbsPath $path]
    if {[file isdirectory $path]} {
	set pathList [glob -nocomplain -- $path/*]
    } elseif {[file isfile $path]} {
	set pathList $path
    } else {
	error "$path is not a regular file"
    }

    if {[catch {expr {$keep}}]} {
	error "$keep is not a boolean flag"
    } elseif {$keep} {
	set transient false
    } else {
	set transient true
    }

    foreach path $pathList {
	BobBaseObject::sendStatus PAFReady OH \
		"$seq_ccsEnvName $seq_ccsProcName $path $transient" 1
    }
}

# ------------------------------------------------------------------
#                        PUBLIC PROCEDURE seeBobVars
# ------------------------------------------------------------------
# Returns the list of categories that the template script can use,
# after taking care of the "upvar" instructions in the caller's context.
itcl::body BobTPL::seeBobVars {args} {
    set tpl $THIStpl
    while {![lempty $args]} {
	set arg [lvarpop args]
	switch -glob -- $arg {
	    "-tpl" {set tpl [lvarpop args]}
	    "-localcopy" {set arrays [lvarpop args]}
	    default {error "wrong arg for seeBobVars: $arg"}
	}
    }
    if {[info exists arrays]} {
	return [$tpl _upvarObsKeys $arrays]
    } else {
	return [$tpl _seeBobVars]
    }
}
# ------------------------------------------------------------------
#                        PUBLIC PROCEDURE sendCmd
# ------------------------------------------------------------------
# The 2nd arguments (cmd) is either the command without params, or includes 
# some/all parameters.  In the latter case, the parameters
# need to be split off and merged with the 3rd argument before it can be
# passed on to _sendCmd.
# Remark that as "args" is treated as a list by Tcl, parameters surrounded by 
# quotes need special treatment - they would otherwise appear with braces on
# the receiving side.  For that purpose we use the "join" command, which 
# converts the list into a string.
# As for historic reasons <cmd> can also contain (part of) the arguments,
# we need to split them off. Do not use list operations for that (like
# "lrange $cmd 1 end") as quotes may get lost (subtle difference between 
# Tcl pre-8.0 and 8.0 list handling).
###itcl::body BobTPL::sendCmd {timeout cmd args}
itcl::body BobTPL::sendCmd {args} {
    set arg [lvarpop args]
    switch -glob -- $arg {
	"-tpl" {set tpl [lvarpop args]; set timeout [lvarpop args]}
	default {set tpl $THIStpl; set timeout $arg}
    }
    # There must be at least one arg left
    if {[llength $args] < 1} {
	error {wrong # args: should be "sendCmd ?option? timeout cmd ?args?"}
    }
    set cmd [lvarpop args]
    set command [lindex $cmd 0]
    set idx [expr {[string first $command $cmd]+[string length $command] + 1}]
    set a [string trim [string range $cmd $idx end]]
    switch -- {} $args {} default {append a { } [join $args { }]}
    $tpl _sendCmd $timeout $command $a
}
# ------------------------------------------------------------------
#                        PUBLIC PROCEDURE sendObsKeys
# ------------------------------------------------------------------
# A side-effect of calling this proc is that TPL(EXPNO) is incremented!
# (unless we deal with the -array option)
# Arguments: ?-tpl <objectId>? ?-array <catList> ?timeout?
itcl::body BobTPL::sendObsKeys {args} {
    set tpl $THIStpl
    while {![lempty $args]} {
	set arg [lvarpop args]
	switch -glob -- $arg {
	    "-tpl" {set tpl [lvarpop args]}
	    "-array" {
		global gvar
		if {[clength $::bob::internal::config(bobEnvNameOS)] == 0 && \
			[string length [lindex $::bob::internal::config(bobProcNameOS) 0]] == 0} {
		    set arrays [lvarpop args]
		} else {
		    error "use of -array is not allowed with OS defined"
		}
	    }
	    default {set timeout $arg}
	}
    }
    # next block is to deal with making the OBS keywords available
    # at the template level (only allowed if no OS is active).
    if {[info exists arrays]} {
	$tpl _upvarObsKeys $arrays
    } else {
	if {[clength $timeout] == 0} {set timeout 2000}
	$tpl _sendObsKeys $timeout
    }
}
# ------------------------------------------------------------------
#                        PUBLIC PROCEDURE setCallBack
# ------------------------------------------------------------------
itcl::body BobTPL::setCallBack {args} {
    set arg [lvarpop args]
    switch -glob -- $arg {
	"-tpl" {set tpl [lvarpop args]; set script [lvarpop args]}
	default {set tpl $THIStpl; set script $arg}
    }
    $tpl _setCallBack $script
}

# ------------------------------------------------------------------
#                        PUBLIC PROCEDURE setResult
# ------------------------------------------------------------------
# optional argument:   tpl-object
# mandatory arguments: name (array-index) and value of result
# 
itcl::body BobTPL::setResult {args} {
    set arg [lvarpop args]
    switch -glob -- $arg {
	"-tpl" {set tpl [lvarpop args]; set name [lvarpop args]}
	default {set tpl $THIStpl; set name $arg}
    }
    # There should be 1 argument left
    if {[llength $args] != 1} {
	error {wrong # args: should be "setResult ?<option>? <name> <result>"}
    }
    [$tpl up] setResult $name [lvarpop args]
}

# ------------------------------------------------------------------
#                        PUBLIC PROCEDURE tplLog
# ------------------------------------------------------------------
itcl::body BobTPL::tplLog {args} {
    set arg [lvarpop args]
    switch -glob -- $arg {
	"-tpl" {set tpl [lvarpop args]}
	default {set tpl $THIStpl; set args [linsert $args 0 $arg]}
    }
    eval $tpl logLog $args
    update
}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ------------------------------------------------------------------
#                        PRIVATE PROCEDURES 
# ------------------------------------------------------------------
# ------------------------------------------------------------------
#                        PRIVATE proc _tplSim
# ------------------------------------------------------------------

itcl::body BobTPL::_tplSim {timeout cmd params} {
    global ::gvar

    foreach command $::bob::internal::config(tplSimList) {
	switch -- $cmd $command {
	    #puts "found special command: $cmd"
	    if {[catch {_tplSimDelay $::bob::internal::config(tplSpeDelay) $timeout}]} {
		error  "simulated reply timeout"
	    } else {
		return "OK_SIM"
	    }
	}
    }
    # If we get here, this command was not in the list of "special" commands
    if {[catch {_tplSimDelay $::bob::internal::config(tplDefDelay) $timeout}]} {
	# puts "error return: $timeout"
	error "simulated reply timeout"
    } else {
	return "OK_SIM"
    }
}

# ------------------------------------------------------------------
#                        PRIVATE PROCEDURE _tplSimDelay
# ------------------------------------------------------------------
itcl::body BobTPL::_tplSimDelay {ms timeout} {
    set ms [expr {int($ms)}];	# temporary - slider returns floats?!
    if {$ms == 0} return
    set simVar 0
    ##puts "delaying for $ms ms, timeout = $timeout ..."
    if {$timeout <= $ms && $timeout != 0} {
	after $timeout [code {set simVar 1}]
	set clockStart [clock clicks -milliseconds]
	tkwait variable [scope simVar]
	set elaps [expr {[clock clicks -milliseconds] - $clockStart}]
	##puts "delay = $ms ms; timeout = $timeout; elapsed = $elaps millisec"
	error  "timeout"
    } else {
	after $ms [code {set simVar 1}]
	set clockStart [clock clicks -milliseconds]
	tkwait variable [scope simVar]
	set elaps [expr {[clock clicks -milliseconds] - $clockStart}]
	##puts "delay = $ms ms; timeout = $timeout; elapsed = $elaps millisec"
	return {}
    }
}


# ___oOo___





