#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bobVisualTree.tcl,v 1.145 2005/10/27 09:07:09 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# mbec      09/04/97    created
# mbec      16/04/97    manpage & vltstandard - last touch
# mbec      07/05/97    pixmap are private common defined this class
#                       changed tags for lines between objects
# mbec      12/05/97    moved all the 'engineering' (copy paste drag + edit)
#                       stuff here
# mbec      06/06/97    cleaned up few stuff (edit - paste)
#                       try to remove the 'isa' call as much as possible
# mbec      13/06/97    many mods - check the drag generating Enter events !
# eallaert  2001-04-24  several modifications / log editing of param
# eallaert  2002-10-16  editParams: parent widget correction 
# eallaert  2002-12-18  <canvas> is sinc 1.114 pure canvas (instead of scrolledcv),
#                       so entryfield should not look for canvas components.
# eallaert  2005-01-10  check ::bob::internal::info(interpreter) for folding
#

#************************************************************************
#   NAME
#   BobVisualTree - define visual tree nodes concept
#
#   SYNOPSIS
#   BobVisualTree <obj_name>|#auto <canvas> <args>
#
# 
#   DESCRIPTION
#   this class inherits from Tree and BobVisual and deals with all the aspects
#   of representing tree nodes on a canvas.
#   already it implements the standard functions allowing to draw the tree
#   with a folding mechanism (with incremental display and actual
#   representation of the tree by lines joining the parent/child pair)
#
#   METHODS
#   constructor: construct the Tree and BobVisual object and bind the Visual to
#         mouse event if so required for folding operations
#   destructor: see [incr Tcl]
#   configure: see [incr Tcl]
#
#   busyCursor: display a busy cursor on the canvas
#   draw <ulx> <uly>: same as BobVisual and additionaly draws a line 
#         between parent and child
#   foldAll <mode> <<pleaseRefresh>>: mode is one of folded / unfolded
#         this will unfold recursively the object and all his children
#         second argument is used to know when to actually refresh the
#         display 
#   normalCursor: display a normal cursor on the canvas
#   refresh: refresh the whole canvas. For reason SPECIFIC to this
#         application, the drawing at the root point doesn't include the
#         vertical lines linking to the child.
#         (basically: the root point is just an abstraction and we don't
#         want for cosmetic reasons the OBs (=1st child level) to have
#         these lines showing)
#   toggle_fold: toggles the fold state
#   toggle_foldAll: toggles the fold state recursively
#
#   CLASS PROCEDURES
#
#   VARIABLES
#   fold_state: one of  folded / unfolded / nofold - this value also
#         determine if the object is an end point in the tree (nofold)
#         or an intermediate node (folded/unfolded) in which case the
#         it's binded to mouse event.
#         
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#=============================================================================
#=============================================================================
itcl::class BobVisualTree {
    inherit BobTree BobVisual

    constructor {cwin args} {
	BobTree::constructor
	BobVisual::constructor $cwin
    } {}
 
    public method bottomY {} {}
    public method busyCursor {} {}
    public proc dirtyTrickdrop_me {who xx yy args} {
	[set $who] drop_me $xx $yy
    }
    public method displayVerticalLines {} {}
    public method drag_me {x y} {}
    public method draw {ulx uly} {}
    public method drop_me {x y} {}
    public method editParamValue {} {}
    public method enter_me {xx yy mm args} {}
    public method familytag {} {}
    public method foldAll { mode {pleaseRefresh 1} } {}
    public method hifuzzULy {} {
    	return [expr {[ULy] + $fuzzyHeight}]
    }
    public method hifuzzLRy {} {
    	return [expr {[LRy] + $fuzzyHeight}]
    }
    public method lowfuzzULy {} {
    	return [expr {[ULy] - $fuzzyHeight}]
    }
    public method lowfuzzLRy {} {
    	return [expr {[LRy] - $fuzzyHeight}]
    }
    public method normalCursor {} {}
    public method pick_me { x y } {}
    public method refresh {} {}
    public method restoreValue {args} {}
    public method toggle_fold {} {}
    public method toggle_foldAll {} {}

    public method validate {args} {}
    public method LRy {} {
    	return [lindex [$canvas bbox childtag-$this] 3]
    }

    public method Px {} {
   	return [lindex [$canvas bbox $this-icon] 0]
    }
    public method ULy {} {
    	return [lindex [$canvas bbox $this] 1]
    }

    public proc root { } {}
    public proc setRoot { obj } {}
    public proc cleanupCanvas {} {}

    public variable fold_state "folded"
   
    private method paste { obj where } {}
    private method editRawData {args} {}	;
    private method hilite {{tagname motion} {raise 1} {color Grey70} } {}
    private method lowlite {{tagname motion}} {}	

    private variable familytag ""
    private variable doit_action_flag 1;# to see if action cut/copy was canceled
    private common img_folded [image create pixmap img_folded -file [bobFindVlt bitmaps/bobFolded.xpm]]
    private common img_unfolded [image create pixmap img_unfolded -file [bobFindVlt bitmaps/bobUnfolded.xpm]]
    private common img_nofold [image create pixmap img_nofold -file [bobFindVlt bitmaps/bobNoFold.xpm]]
    private common img_root [image create pixmap img_root -file [bobFindVlt bitmaps/bobRoot.xpm]]

    # check for common size
    foreach img {img_folded img_unfolded img_nofold img_root} {
	set x [image height $img]
	if {$x > $icon_h} {set icon_h $x}
	set x [image width $img]
	if {$x > $icon_w} {set icon_w $x}
    }
    

    private common pasteBufferObject ""
    private common cutBufferObject "";	# after 2 successive cut: delete the object
    private common root "";	# root point of the tree
    private common InsertionObject ""
    private common InsertionDirection ""
    # trick to paste before at 1st time
    private common lastEnterY 1000000
    private common dragHeight
    
    private common firstInsertion 1
    private common fuzzyHeight [expr {$separator_h/2}]

    private common dragSelection ""
    # dummy
    global gvar
    private common nsc
    set nsc [namespace current]
    switch -- $nsc :: {set nsc {}}
    private common dummyPasteObject "${nsc}::[BobVisualTree #auto $gvar(cvOB)]"
}

#=============================================================================
#=============================================================================

# ------------------------------------------------------------------
#                        CONSTRUCTOR
# ------------------------------------------------------------------
itcl::body BobVisualTree::constructor {cwin args}  {
    eval configure $args
    switch -- $fold_state nofold {} default {
	$canvas bind $this-icon <ButtonPress-1> "$this toggle_fold"
	$canvas bind $this-icon <Control-ButtonPress-1> "$this toggle_foldAll"
	if { [$this isa BobBaseObject] } {
	    $canvas bind $this <Enter> "$this enter_me %x %y %m %#"
	}
    }
    # engineering bindings ...
    $canvas bind $this <ButtonPress-2> "$this pick_me %x %y"
}


# ------------------------------------------------------------------
#                        PUBLIC METHOD foldAll
# ------------------------------------------------------------------
itcl::body BobVisualTree::foldAll { mode {pleaseRefresh 1} } {
    if {[string equal $::bob::internal::info(interpreter) bobWish]} {
	return
    }

    # fold/unfold all the children recursively
    switch -- $fold_state nofold {} default {
	switch -- $mode {
	    folded {
		set fold_state folded
		set icon "folded"
		foreach child [contents] {
		    $child foldAll $mode 0
		}
		if {$pleaseRefresh} refresh
	    }
	    unfolded {
		set fold_state "unfolded"
		set icon "unfolded"
		foreach child [contents] {
		    $child foldAll $mode 0
		}
		if {$pleaseRefresh} refresh
	    }
	    default {
		##puts "invalid fold_state value : $this $fold_state"
	    }
	}
    }
}


# ------------------------------------------------------------------
#                        PUBLIC PROCEDURES
# ------------------------------------------------------------------
itcl::body BobVisualTree::bottomY {} {
    return [lindex [$canvas bbox all] 3]
}
# ------------------------------------------------------------------
itcl::body BobVisualTree::root { } {
    return $root
}

itcl::body BobVisualTree::setRoot {obj} {
    set root $obj
    set cvs [$root canvas]
    $cvs delete all
    $cvs bind $root-icon <ButtonPress-1> ""
    $cvs bind $root-icon <Control-ButtonPress-1> ""
    $cvs bind $root <Motion> ""
    $cvs bind $root <Enter> ""
    $cvs bind $root <ButtonPress-2> ""
    $cvs configure -closeenough 0
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD toggle_foldAll
# ------------------------------------------------------------------
itcl::body BobVisualTree::toggle_foldAll {} {
    if {[string equal $::bob::internal::info(interpreter) bobWish]} {
	return
    }

    switch -- $fold_state {
	unfolded {
	    foldAll folded
	}
	folded {
	    foldAll unfolded
	}
	nofold {
	    #won't do anything
	}
	default {
	    ##puts "invalid fold_state value : $fold_state"
	}
    }
}


# ------------------------------------------------------------------
#                        PUBLIC METHOD toggle_fold
# ------------------------------------------------------------------
itcl::body BobVisualTree::toggle_fold {} {
    if {[string equal $::bob::internal::info(interpreter) bobWish]} {
	return
    }

    switch -- $fold_state {
	unfolded {
	    set fold_state "folded"
	    set icon "folded"
	    refresh
	}
	folded {
	    set fold_state "unfolded"
	    set icon "unfolded"
	    refresh
	}
	nofold {
	    #won't do anything
	}
	default {
	    ##puts "invalid fold_state value : $fold_state"
	}
    }
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD familytag
# ------------------------------------------------------------------
itcl::body BobVisualTree::familytag {} {
    set ftag ""
    set me $this
    while {![string equal [set p [$me up]] ""]} {
	set me $p
	append ftag "childtag-$p "
    }
    set familytag $ftag
    return $ftag
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD draw
# ------------------------------------------------------------------
itcl::body BobVisualTree::draw {ulx uly} {
    
    set moretags [$this familytag]
    append moretags " childtag-$this"

    set uly [BobVisual::draw $ulx $uly $moretags]
    set ulx [expr {$ulx + $icon_w}]

    switch -- $fold_state folded {
	foreach obj [contents] {
	    $canvas delete $obj
	}
    } default {
	foreach obj [contents] {
	    set uly [$obj draw $ulx $uly]
	}
    }
    return $uly
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD busyCursor
# ------------------------------------------------------------------
itcl::body BobVisualTree::busyCursor {} {
    $canvas configure -cursor watch
    # put a grab on the menu bar ..
    # grab set .bob.topMenu
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD normalCursor
# ------------------------------------------------------------------
itcl::body BobVisualTree::normalCursor {} {
    $canvas configure -cursor {}
    # release the grab
    # grab release .bob.topMenu
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD refresh
# ------------------------------------------------------------------
itcl::body BobVisualTree::refresh {} {
    
    $canvas configure -cursor watch
    switch -- $this $root {
	# first refresh done from root
    	$canvas delete all
	
	set ulx 10
    	set uly 10
	set uly [draw $ulx $uly]
    } default {
    	set lryBefore [LRy]
    	$canvas addtag move_below overlapping 0 [hifuzzLRy] 1500 [bottomY]
	 	
    	set  ulx [Px]
    	set  uly [ULy]
    	
    	$canvas delete $this childtag-$this
    	
    	set uly [draw $ulx $uly]
    	
    	set lryNow  [LRy]
    	$canvas move move_below 0 [expr {$lryNow - $lryBefore}]
    	$canvas dtag move_below
    }

    $canvas delete vertical-line $root-line
    $root displayVerticalLines
    $canvas lower vertical-line

    set bbox [$canvas bbox all]
    #set bbox [$canvas bbox text]
    $canvas configure -cursor {} -scrollregion $bbox
}
# ------------------------------------------------------------------
#                        PUBLIC METHOD displayVerticalLines
# ------------------------------------------------------------------
itcl::body BobVisualTree::displayVerticalLines {} {
    
    switch -- $fold_state folded {} default {
        foreach obj [contents] {
	    $obj displayVerticalLines
	}
        if {[info exists obj] && ![string equal $this $root]} {	
	    # draw line for the last
	    set x [expr {[Px] + $icon_w/2}]
	    set moretags [$this familytag]
	    append moretags " childtag-$this"
	    $canvas create line $x [expr {[ULy]+ $icon_h/2}] $x [expr {[$obj ULy]+ $icon_h/2}] -tags [eval list vertical-line $this-vertical-line $moretags]
	}
    }
}


# ------------------------------------------------------------------
#                        PUBLIC METHOD 
# ------------------------------------------------------------------
itcl::body BobVisualTree::enter_me {xx yy mm args} {
    global gvar
    if {$::bob::internal::info(interface)} {
	#puts "$this --- enter me"
	switch -- $gvar(engineer_mode) paste {
	    # first check if there's something to paste
	    set objects [itcl::find objects $pasteBufferObject]
	    switch -- $objects {} {
		bobDialog "NO OBJECT ready for pasting : use Cut or Copy first !" application
		set gvar(engineer_mode) "copy"
	    } default {
		# add a dummy object to know where you insert
		# call back automatically detect if it's a TPL or OB
		if {[string equal [$this info class] [$pasteBufferObject info class]]} {
		    if {[expr {$yy - $lastEnterY}] <= 0} {
			set InsertionDirection "before"
			##puts "$args paste before $this"
			set InsertionObject $this
			
			set zzzx [Px]
			set zzzy [ULy]
			$canvas move pasteMoved 0 [expr {-$icon_h -$separator_h}]
			$canvas dtag pasteMoved
			
			$canvas addtag pasteMoved overlapping 0 [lowfuzzULy] 1500 [bottomY]
			
			# $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
			#$canvas delete $dummyPasteObject
			#$dummyPasteObject lowlite pastetag
			
			$canvas move pasteMoved 0 [expr {$icon_h +$separator_h}]
			
			if {$firstInsertion} {
			    set zzzy [$dummyPasteObject draw $zzzx $zzzy]
			    $dummyPasteObject hilite pastetag
			    set firstInsertion 0
			} else {
			    $canvas move $dummyPasteObject 0 [expr {$zzzy - [$dummyPasteObject ULy]}]
			}
			
			
		    } else {
			set InsertionDirection "after"
			set InsertionObject $this
			
			$canvas move pasteMoved  0  [expr {-$icon_h -$separator_h}]
			$canvas dtag pasteMoved
			
			# $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
			#$canvas delete $dummyPasteObject childtag-$dummyPasteObject 
			#$dummyPasteObject lowlite pastetag
			
			$canvas addtag pasteMoved overlapping 0 [hifuzzLRy] 1500 [bottomY]
			$canvas move pasteMoved 0 [expr {$icon_h +$separator_h}]
			
			set zzzx [Px]
			set zzzy [expr {[LRy] +$separator_h}]
			
			if {$firstInsertion} {
			    set zzzy [$dummyPasteObject draw $zzzx $zzzy]
			    $dummyPasteObject hilite pastetag
			    set firstInsertion 0
			} else {
			    $canvas move $dummyPasteObject 0 [expr {$zzzy - [$dummyPasteObject ULy]}]
			}
		    }
		    
		    set lastEnterY $yy
		}
	    }
	} drag {
	    switch -- $mm NotifyWhileGrabbed {
		##puts "$this enter for a drag"
		switch -- [$this info class] [$dragSelection info class] {
		    ##puts "enter $this"
		    
		    set InsertionObject $this
		    
		    if {[expr {$yy - $lastEnterY}] <= 0} {
			set InsertionDirection "before"
			
			# keep this one now as it will be shifted & back
			set uly [ULy]
			
			# create tag of what will be shifted back
			$canvas addtag dragMovedBack overlapping 0 [$dragSelection hifuzzLRy] 1500 [bottomY]
			
			# create free space for insertion
			$canvas addtag dragMovedInsert overlapping 0 [lowfuzzULy] 1500 [bottomY]
			$canvas move dragMovedInsert 0 $dragHeight
			$canvas dtag dragMovedInsert
			
			# insert the drag selection in it
			set deltaMove [expr {$uly - [$dragSelection ULy]}]
			$canvas move childtag-$dragSelection 0 $deltaMove
			#$canvas move $dragSelection-vertical-line 0 $deltaMove
			
			
			# shift back to close the gap from previous position of drag selection
			$canvas move dragMovedBack 0 [expr {-$dragHeight}]
			$canvas dtag dragMovedBack
		    } else {
			set InsertionDirection "after"
			
			# keep this one now as it will be shifted & back
			set uly [ULy]
			set lryds [$dragSelection hifuzzLRy]
			
			# create free space for insertion
			$canvas addtag dragMovedInsert overlapping 0 [hifuzzLRy] 1500 [bottomY]
			$canvas move dragMovedInsert 0 $dragHeight
			$canvas dtag dragMovedInsert
			
			# insert the drag selection in it
			set zzzy [expr {[LRy] + $separator_h}]
			set deltaMove [expr {$zzzy - [$dragSelection ULy]}]
			$canvas move childtag-$dragSelection 0 $deltaMove
			#$canvas move $dragSelection-vertical-line 0 $deltaMove
			
			
			# shift back to close the gap from previous position of drag selection
			$canvas addtag dragMovedBack overlapping 0 $lryds 1500 [bottomY]
			$canvas move dragMovedBack 0 [expr {-$dragHeight}]
			$canvas dtag dragMovedBack
		    }
		    set lastEnterY $yy
		    update
		}
	    }
	} default {
	    ##puts "$this x [Px] uly [ULy]lry [LRy] --------- x $xx y $yy"
	}
    }
}


# ------------------------------------------------------------------
#                        PUBLIC METHOD 
# ------------------------------------------------------------------
itcl::body BobVisualTree::drag_me {x y} {
    global gvar
    # active only for engineering interface ....
    if {$::bob::internal::info(interface)} {
	#puts "$this --- drag_me"
	switch -- $gvar(engineer_mode) drag {
	    # just generate a Enter event - the update is done by the receiver !
	    event generate $canvas <Enter> -x $x -y $y -detail NotifyInferior -mode NotifyWhileGrabbed
	    update
		
	} cut - copy - paste {
	    # check if mouse outside hilite: cancel action if so ...
	    if {$x < [expr {[Px] - $icon_w/2}] || $y < [ULy] || $y > [LRy]} {
		set doit_action_flag 0
		# this event to cancel the action prorerly (caught by drop_me)
		event generate $canvas <ButtonRelease-2> -when head
	    }
	}
    }
}



# ------------------------------------------------------------------
#                        PUBLIC METHOD 
# ------------------------------------------------------------------
# active only for engineering interface ....
itcl::body BobVisualTree::drop_me {x y} {
    global gvar
    
    if {$::bob::internal::info(interface)} {
	
	switch $gvar(engineer_mode) {
	    paste {
		if {$doit_action_flag} {
		    busyCursor
		    set newobj [$InsertionObject paste $pasteBufferObject $InsertionDirection]
		    set zzzx  [Px]
		    set zzzy  [ULy]
		    set zzzy [$newobj draw $zzzx $zzzy]
		    # refresh vertical lines
		    $canvas delete vertical-line
		    $root displayVerticalLines
		    $canvas lower vertical-line
		} else {
		    $canvas move pasteMoved  0  [expr {- $icon_h - $separator_h}]
		    $canvas dtag pasteMoved
		}
		set InsertionObject ""
		#  need to put back this binding which was removed originally
		$canvas delete $dummyPasteObject  childtag-$dummyPasteObject
		$dummyPasteObject lowlite pastetag
		$canvas dtag pasteMoved
		set gvar(engineer_mode) "drag"
		set firstInsertion 1
	    }

	    copy {
		if {$doit_action_flag} {
		    busyCursor
		    set pasteBufferObject $this
		    # incremental buffer ...
		    set objects [itcl::find objects $cutBufferObject] 
		    switch -- $objects {} {} default {
			delete object $cutBufferObject
		    }
		    $dummyPasteObject configure -label $label
		    $canvas bind $dummyPasteObject <ButtonPress-2> "$dummyPasteObject pick_me  %x %y"
		    set lastEnterY [bottomY]
		}
	    }

	    cut {
		if {$doit_action_flag} {
		    busyCursor
		    # cut this from it's original tree node
		    cut
		    #update the canvas
		    set ULY [ULy]
		    set LRY [LRy]
		    $canvas addtag shift_cut overlapping 0 [hifuzzLRy]  1500 [bottomY]
		    $canvas delete $this childtag-$this
		    $canvas move shift_cut 0 [expr {$ULY-$LRY - $separator_h}]
		    $canvas dtag shift_cut
		    
		    # update raw data of parent - N.A. for OB themselves though!
		    if {![$this isa BobOB]} {
			[up] updateRawData
		    }
		    # incremental buffer ...
		    set objects [itcl::find objects $cutBufferObject] 
		    switch -- $objects {} {} default {
			delete object $cutBufferObject
		    }
		    # set this one as the one to paste
		    set pasteBufferObject $this
		    set cutBufferObject $this
		    $dummyPasteObject configure -label $label
		    $canvas bind $dummyPasteObject <ButtonPress-2> "$dummyPasteObject pick_me  %x %y"
		    set lastEnterY [bottomY]
		    # refresh vertical lines
		    $canvas delete vertical-line
		    $root displayVerticalLines
		    $canvas lower vertical-line
		}
	    }
	    
	    drag {
		switch -- $InsertionObject {} {} default {
		    busyCursor
		    $dragSelection cut
		    set papa [$InsertionObject up]
		    $papa add $dragSelection $InsertionDirection $InsertionObject
		    set InsertionObject ""
		    # update raw data of parent - N.A. for OB themselves though!
		    if {[$this isa BobTPL]} {
			[up] updateRawData
		    }
		}
		# put back this binding which has been removed originally
		$canvas bind $this <Enter> "$this enter_me %x %y %m %#"
		$this lowlite dragtag
		# I need to update the tags of the dragged selection
		$dragSelection refresh
		set dragSelection ""
		# refresh vertical lines
		$canvas delete vertical-line
		$root displayVerticalLines
		$canvas lower vertical-line
	    }
	}
	# end of switch
	

	# clean up all the bindings
	bind $canvas <ButtonRelease-2> ""
	$canvas bind $this <ButtonRelease-2> ""
	lowlite stdtag
	# go back to original <motion> binding (if needed)
	$canvas bind $this <Motion> ""
	bind $canvas <Motion> ""
	normalCursor
    }
}  



# ------------------------------------------------------------------
#                        PUBLIC METHOD 
# ------------------------------------------------------------------
# active only for engineering interface ....
# drag/copy/cut paste are valid for TPL and OB ONLY !!!!
# only edit is supported for PAR and CAT
itcl::body BobVisualTree::pick_me { x y } {
    global gvar

    if {$::bob::internal::info(interface)} {
	if { (([$this isa BobOB] || [$this isa BobTPL])  && ! [string equal $gvar(engineer_mode) edit]) || [string equal $gvar(engineer_mode) paste]} {
	    switch -- $gvar(engineer_mode) paste {
		switch -- $this $dummyPasteObject {
		    #hilite pastetag
		    set doit_action_flag 1
		    $canvas bind $this <Motion> "$this drag_me %x %y"
		    # add drag/drop binding
		    $canvas bind $this <ButtonRelease-2> "$this drop_me %x %y"
		}
	    } drag {
		set lastEnterY $y
		hilite dragtag
		
		$canvas delete [up]-vertical-line
		
		set doit_action_flag 1
		set dragSelection $this
		$canvas bind $this <Enter> ""
		bind $canvas <Motion> "$this drag_me %x %y"
		bind $canvas <ButtonRelease-2> "BobVisualTree::dirtyTrickdrop_me dragSelection %x %y"
		bind $canvas <Leave> {BobVisualTree::cleanupCanvas}
		set dragHeight [expr {[LRy] -[ULy] + $separator_h}]
	    } default {
		hilite stdtag
		set doit_action_flag 1
		$canvas bind $this <Motion> "$this drag_me %x %y"
		# add drag/drop binding
		$canvas bind $this <ButtonRelease-2> "$this drop_me %x %y"
	    }
	} else {
	    hilite stdtag
	    busyCursor
	    switch -- $gvar(engineer_mode) edit {
		editRawData
	    } default {
		bobDialog "Selected object supports only Edit mode ..." \
			application 
	    }
	    normalCursor
	    lowlite stdtag
	}
    }
}


# ------------------------------------------------------------------
#                        PRIVATE METHODS 
# ------------------------------------------------------------------


itcl::body BobVisualTree::hilite {{tagname motion} {raise 1} {color Grey70} } {
    # add drag tag to everything enclosed in the selected
    $canvas addtag $tagname enclosed [Px] [ULy] 1500 [LRy]
    # show what's dragged with hilite rectangle
    $canvas create rectangle [Px] [expr {[ULy] + 1}] 1500 [expr {[LRy] -1}] \
	    -outline black -fill $color \
	    -tags [list $this $tagname rec-$tagname childtag-$this]
    $canvas lower rec-$tagname
    #$canvas create line 0 [expr {[ULy] + $icon_h/2}] 1500 [expr {[ULy] + $icon_h/2}] -fill green -tags [list $this $tagname rec-$tagname childtag-$this]
} 

itcl::body BobVisualTree::lowlite {{tagname motion}} {
    $canvas delete rec-$tagname
    $canvas dtag all $tagname
} 


# it's either a CAT or a PAR we're dealing with at this stage
# I climb up to TPL ...
itcl::body BobVisualTree::editRawData {} {
    if {[[up] isa BobTPL]} {
	# edit a CAT; the CAT comes from $this ...
	panImportPanel bobEditor
	set cat [lindex [split $this .] end]
	[up] editRawData $cat
    } else {
	# edit a PAR; the PAR key comes from $this ...
	##panImportPanel bobEditor
	##set cat [lindex [split [up] .] end]
	##set tempo [split $this .]
	##set index [lsearch $tempo $cat]
	##set par [join [lrange $tempo [incr index] end] .]
	##[[up] up] editRawData $cat $par
	editParamValue
    }
}

# returns a list with 2 elements: the original and the new value.
itcl::body BobVisualTree::editParamValue {} {
    global gvar

    if {$labelTruncated} {
	bobDialog "This keyword's value is too long to edit directly on the canvas.\n\n\
                   If you're sure you want to modify this value,\n\
                   edit the entire category, template or OB instead." \
	    application
	return
    }
    set labelTags [$canvas gettags $this-text]
    set labelCoords [$canvas coords $this-text]
    set labelCoordX [lindex $labelCoords 0]
    set labelCoordY [lindex $labelCoords 1]

    set line [split $label " "]
    set param [lindex $line 0] 
    set origValue [join [lrange $line 2 end] " "]

    # write the parameter again, just to know the pixel size of this
    set paramId [$canvas create text $labelCoordX $labelCoordY \
	    -text "$param =" -anchor w -font $::bob::internal::config(bobCanvasFont)]
    foreach {xl yl xr yr} [$canvas bbox $paramId] {}; # upper left, lower right
    $canvas delete $paramId
    set boxId [$canvas create rectangle \
	    [incr xl -20] [incr yl -3] $xr [incr yr 2]\
	    -outline black -fill Grey70 \
	    -tags [concat $this-rec $labelTags]]
    $canvas lower $this-rec

    set entryField ${canvas}.entryField
    if {![winfo exists $entryField]} {
	entry $entryField -exportselection no -background Grey90 \
		-selectbackground yellow -selectforeground black \
		-takefocus 0 -foreground black -relief sunken \
		-highlightthickness 1 -font $::bob::internal::config(bobEditorFont)
	bind $entryField <Enter> "set $gvar(mainHelpVar) \
		{Enter new value here; press <Enter> to apply, <Escape> to cancel}"
	bind  $entryField <Leave> "set $gvar(mainHelpVar) {}"
    }
    $entryField delete 0 end
    $entryField insert 0 $origValue

    set valueId [$canvas create window $xr $labelCoordY \
	    -window $entryField -anchor w -width 1300 -tags $labelTags]

    bind $entryField <KeyPress-Return> \
	    [code $this validate %W $param $boxId $valueId]
    bind $entryField <Key-Escape> \
	    [code $this restoreValue %W $param $boxId $valueId]
    bind $entryField <Map> [code focus $entryField]
    if {![winfo viewable $entryField]} {tkwait visibility $entryField}
    selection clear
    grab set $entryField
    $entryField selection range 0 end
}

itcl::body BobVisualTree::validate {entryField par boxId valueId} {
    set value [$entryField get]
    grab release $entryField
    $canvas delete $boxId
    $canvas delete $valueId
    configure -label "$par = $value"
    ##destroy $entryField
    set cat [[up] cget -label]
    [[up] up] changeParamValue $cat $par $value
}

itcl::body BobVisualTree::restoreValue {entryField par boxId valueId} {
    grab release $entryField
    $canvas delete $boxId
    $canvas delete $valueId
    ##destroy $entryField
}

itcl::body BobVisualTree::paste { obj where} {
    bobDialog "this is very unfortunate:\nthe call is virtual in this context" application 
}

itcl::body BobVisualTree::cleanupCanvas {} {
    
    global ::gvar
    if {$::bob::internal::info(interface)} {
	##puts "cleanup"
	# maybe there's no root yet
	if { ![catch {set cns [$root canvas]}]} {
	    bind $cns <Leave> ""
	    set please_refresh 0
	    switch -- $dragSelection {} {} default {
		$cns bind $dragSelection <Enter> "$dragSelection enter_me %x %y %m %#"
		$cns bind $dragSelection <ButtonRelease-2> ""
		$cns bind $dragSelection <Motion> ""
		$dragSelection lowlite dragtag
		set dragSelection ""
	    }
	    bind $cns <ButtonRelease-2> ""
	    $root lowlite stdtag
	    switch -- $InsertionObject {} {} default {
		set please_refresh 1
		set InsertionObject ""
	    }
	    $dummyPasteObject lowlite pastetag
	    $cns delete $dummyPasteObject childtag-$dummyPasteObject
	    bind $cns <Motion> ""
	    if {$please_refresh} {
		$root refresh
	    }
	    set firstInsertion 1
	}
    }
}










