#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: feditConfigOperations.tcl,v 2.11 2003/11/12 09:10:33 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin   13/11/95   created
# mcomin   04/12/95   Fixed bug : syntax error"
# mcomin   21/12/95   Fixed bug in feditErrorCheck and feditRecAdd
# ------------------
# mcomin   14/03/96   Ported to TK4.0 and itcl2.0
#          12/06/96   Removed SetShortHelp
#          25/06/96   Add check on "%s" format statement.
#          25/02/97   Replace setCursor with setSelection
#                     Fixed bug when selecting an element using 'arrows'
#          31/03/97   Fixed backwards compatibility in setSelection method
#
#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#------------------------------------------------------------------------
# PROCEDURE : feditGetConfig - The procedure displays the configuration of
#                              the a record.
#------------------------------------------------------------------------

proc feditGetConfig { mode idx } {
   global debug par

   if {[set size [$par(wdgErrList) getSize]] == 0 } { return $idx }

   set maxIndex [expr $size -1 ]

   if {[cequal $mode PREV]} {           # Display previous error   

      if {$idx == 0} {
         set newIndex $maxIndex                      
      } else {
         set newIndex [expr $idx -1]      
      }

   } elseif {[cequal $mode NEXT]} {     # Display next error
   
      if {$idx >= $maxIndex} {
         set newIndex 0                      
      } else {
         set newIndex [expr $idx + 1]      
      }                                          
   }

   $par(wdgErrList) clearSelection 
   $par(wdgErrList) setSelection $newIndex   
   displayError $newIndex 
   return $newIndex
}

#------------------------------------------------------------------------
# PROCEDURE : feditErrorCheck - 
#------------------------------------------------------------------------

proc feditErrorCheck { myErr } {
   global par recList myNumber

   if { $myErr == 0 } {

      $par(SH_AREA) displayText $par(ERR_MSG2) 
      set tmp "Error Number CANNOT be 0 " 
      tk_dialog .dialog "Dialog Box" $tmp info 0 OK
      return 0

   } elseif { $myErr < 0 } { 

      incr par(LAST_ID)
      set  myNumber $par(LAST_ID)
      return $myNumber
   }

   # Error number already exist
   if { [info exists recList($myErr)] } { return -1 }

   # New Error number : set it to the next available value   
   if {$myNumber > $par(LAST_ID)} { set par(LAST_ID) $myNumber }

   return $myNumber
}



#------------------------------------------------------------------------
# PROCEDURE : feditRecModify - The procedure modifies a format definition
#------------------------------------------------------------------------

proc feditRecModify { } {
   global par recList errMemo rec_memo myIndex 
   global myNumber  myMnemonic  myFormat   mySeverity  myHelpFile myOffset   

   if { [$par(wdgErrList) getSize] == 0 } { return }

   set usrMsg  "REPLACE  Done"
   set myId    $par(CURRENT_ID)
   set myIndex [feditGetInsertIdx $myId]

   if {[info exists errMemo($myMnemonic)] } {

      # Check if there if the same error mnemonic used in another definition.
      # If the current insertion index and the index of the error definition
      # using the mnemonic are not the same ... the replace is aborted.

      set oldId $errMemo($myMnemonic)

      # Check if we modify the same error definition
      if {[feditGetInsertIdx $oldId] != $myIndex} {
         $par(SH_AREA) displayText $par(ERR_MSG3)
         return 
      }
   }
   
   # Check if the error number has a valid value
   if { [set ret [feditErrorCheck $myNumber ]] == 0 } { return }

   # Check if the error number already exits in another definition

   if {$ret < 0 && $par(OLD_MEMO) != [feditGetRec $myNumber $rec_memo]} {      
      $par(SH_AREA) displayText $par(ERR_MSG1)
      return
   }

   if {$par(WHAT) == "ERRORS"} {

      feditStoreHelpFile $myHelpFile

      if {[set tmp [feditCheckLog $myFormat String]] != "" } {
         set myFormat $tmp
         append usrMsg "   -   Format modified the following characters are not allowed  %s "
      }

      set myList [list {} {} $myNumber $mySeverity $myMnemonic \
                        $myFormat $myOffset $myHelpFile ]

      unset errMemo($par(OLD_MEMO))
      set errMemo($myMnemonic)  $myNumber
      set par(CURRENT_ID)       $myNumber
      set par(OLD_MEMO)         $myMnemonic

   } elseif {$par(WHAT) == "LOGS"} {
      set do nothing
      return
   }

   # Delete previous definition

   unset recList($myId)
   $par(wdgErrList) deleteItem $myIndex

   # Add new definition

   set recList($myNumber)   $myList   
   set myIndex              [feditGetInsertIdx $myNumber]   

   $par(wdgErrList)  addItem      $myIndex [feditErrorFormat $myList]
   $par(wdgErrList)  setSelection $myIndex
    
   incr par(SAVE_FLAG);

   $par(SH_AREA) displayText $usrMsg
}      



#------------------------------------------------------------------------
# PROCEDURE : feditRecDelete - The procedure deletes a format definition
#------------------------------------------------------------------------

proc feditRecDelete { } {
   global par myIndex recList  errMemo  myNumber  myMnemonic myHelpFile

   if { [$par(wdgErrList) getSize] == 0 } { 
      feditClearConfig
      set tmp "No Elements to DELETE" 
      tk_dialog .dialog "Dialog Box" $tmp info 0 OK
      return
   } 

   if {[set myIndex [$par(wdgErrList) getIndex]] == ""} {
      return ;# No elements selected
   }

   feditRemoveHelp $myHelpFile 1 ; # Force removal of help file, if file name
                                   # is derived from the menmonic

   unset recList($myNumber) errMemo($myMnemonic)

   if {[$par(wdgErrList) getSize] && $myIndex != "" } {
      $par(wdgErrList) deleteItem   $myIndex
      $par(wdgErrList) setSelection $myIndex
      displayError           $myIndex
   }

   incr par(SAVE_FLAG)
     
   $par(SH_AREA) displayText "DELETE Done"         
}      


#------------------------------------------------------------------------
# PROCEDURE : feditRecAdd - The procedure adds a new definition
#------------------------------------------------------------------------

proc feditRecAdd {} {
   global par recList  errMemo  myIndex
   global myNumber   myMnemonic  myFormat  mySeverity  myHelpFile myOffset
 
   if { [info exists errMemo($myMnemonic) ] } {
      $par(SH_AREA) displayText "ADD Fail : Mnemonic already exist "
      return
   }

   set usrMsg "ADD Done"

   if {[feditErrorCheck $myNumber] <= 0 } { 
      incr   par(LAST_ID)
      set    myNumber $par(LAST_ID)
      append usrMsg " :   GENERATED New Error Number ($myNumber)"
   }

   if {$par(WHAT) == "ERRORS" } {

      if {[set tmp [feditCheckLog $myFormat String]] != "" } {
         set myFormat $tmp
         append usrMsg "   -   Format modified the following characters are not allowed  \"  \[  \]  ;  $  and   %s "
      }

      set myList [list {} {} $myNumber $mySeverity $myMnemonic \
                        $myFormat $myOffset $myHelpFile ]
      set errMemo($myMnemonic)  $myNumber

      feditStoreHelpFile  $myHelpFile

   } elseif {$par(WHAT) == "LOGS" } {
      set myList "NOT IMPLEMENTED"
      return
   }

   set recList($myNumber)   $myList
   set myIndex              [feditGetInsertIdx $myNumber]
   set par(OLD_MEMO)        $myMnemonic
   set par(CURRENT_ID)      $myNumber

   $par(wdgErrList)  addItem      $myIndex [feditErrorFormat $myList]
   $par(wdgErrList)  setSelection $myIndex
   
   if {[info exists recList(0)]} { unset recList(0) }

   $par(SH_AREA) displayText $usrMsg
   incr par(SAVE_FLAG)     
}

#------------------------------------------------------------------------
# PROCEDURE : feditRecSearch - The procedure looks for a format definition
#------------------------------------------------------------------------

proc feditRecSearch {} {
   global par errMemo myNumber myMnemonic  myIndex gvar

   $par(SH_AREA) displayText ""
   set myId $myNumber

   if { $gvar(searchMode) == $par(BY_MEMO)} {
      if { [info exists errMemo($myMnemonic)] == 0 } {
         $par(SH_AREA) displayText "Error Definition for $myMnemonic Not Found"
         feditClearConfig
         return
      }
      set myId $errMemo($myMnemonic)
   }


   if {[set idx  [feditGetInsertIdx $myId]] < 0 } {
      $par(SH_AREA) displayText "Error Definition for error number $myNumber Not Found " 
      feditClearConfig
      return
   }

   set myIndex $idx
   $par(wdgErrList)  clearSelection 
   $par(wdgErrList)  setSelection   $myIndex
   displayError $myIndex
}


#------------------------------------------------------------------------
# PROCEDURE : feditGetInsertIdx - Return the insertion index for the 
#                                 current new definition
#------------------------------------------------------------------------

proc feditGetInsertIdx { newId } {
   global recList

   if {[info exists recList($newId)] == 0 } { return -1 }
   set list [lsort -integer [array names recList]]
   set idx  [lsearch $list $newId]

   return $idx
}

#
# ___oOo___
