#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: feditErrRoutines.tcl,v 2.11 2003/11/12 09:10:34 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    13/11/95  created
# mcomin    04/12/95  Add check if offsetRec is defined
#           21/09/96  Replace feditReadHelpFile with errReadHelpFile
#                     (The procedure is now included in the seq tcl library)
#
#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#------------------------------------------------------------------------
# PROCEDURE : feditErrSetMnemonic - Generates the first part of the 
#                                   error mnemonic
#------------------------------------------------------------------------

proc feditErrSetMnemonic { module } {  

   switch $module {   
      
      "ccs"   { return "ccsERR_" }       
      "db"    { return "ccsERR_" }            
      "msg"   { return "ccsERR_" }
      "tims"  { return "ccsERR_" }       
      "scan"  { return "ccsERR_" }            
      "tims"  { return "ccsERR_" }
      "evt"   { return "ccsERR_" }       
      "cmd"   { return "ccsERR_" }            
      "tims"  { return "ccsERR_" }
    
      default {  return "${module}ERR_" }
   }
}
 

#-----------------------------------------------------------------------------
# PROCEDURE : feditErrorFormat -
#-----------------------------------------------------------------------------

proc feditErrorFormat { text } {

    global  rec_id rec_type rec_memo rec_format
    global  err_help err_offset
             
    set errNumb   [lindex $text $rec_id]
    
    switch [lindex $text $rec_type] {     
      "W"      { set errSev "Warning" }
      "F"      { set errSev "Fatal  " }
      "S"      { set errSev "Serious" }
      default  { set errSev "Undefined" }
    }    
    
    set errOff    [lindex $text $err_offset]
    set errMemo   [lindex $text $rec_memo]
    set errFormat [lindex $text $rec_format]
    set errHelp   [lindex $text $err_help]    

    set newText   [ format "%-25s  %-15s %6d  %-10s   %s" \
                           $errMemo $errOff $errNumb  $errSev $errFormat ]
    return $newText
}            

#-----------------------------------------------------------------------------
# PROCEDURE : feditGenErrDefFile - Generates the Error Definition File
#-----------------------------------------------------------------------------


proc feditGenErrDefFile { module } {
   global recList  errMemo  offsetRec totalElem deleteHelpList myHelpFile
   global rec_offset rec_id  rec_type rec_memo rec_format   
   global err_help err_offset
   
   #  Initialization

   set myFile   "${module}_ERRORS";   # File name

   if { [ set filePath [ findModuleFile "ERRORS/$myFile" ]] == "" } {
       set filePath  [getRelPath $module "ERRORS" ]
       append filePath "/$myFile"       
   }
   
   ###########################################
   #      Generate  Error Definition File    #
   ###########################################


   set fh [open $filePath w+ ] ;      # Open and create a new file 

   feditWriteHeader "format" $fh "Error Definition File"
   
   #
   # Save offset definitions
   #
 
   puts $fh " "

   if {[info exists offsetRec]} { 
      foreach name [array names offsetRec] { 
         if {$name != "none" } { puts $fh "define $name $offsetRec($name)" } 
      }
      puts $fh " " 
   }
      
   foreach errNum [lsort -integer [array names recList]] {
      
      set     line1   [feditGetRec $errNum $rec_id]
      lappend line1   [feditGetRec $errNum $rec_type]
      lappend line1   [feditGetRec $errNum $err_offset]

      set line2       [feditGetRec $errNum $rec_memo]
      append line2    " : [feditGetRec $errNum $rec_format]"
      
      set line3       [feditGetRec $errNum $err_help]        
      
      if { [string trim $line3 " " ] == ""} { set line3 "noHelp" }

      # Store first line of the error definition and get 
      # the byte offset counter for direct access of the error format

      puts $fh $line1;  
      feditSaveRec $errNum $rec_offset [tell $fh]           
       
      puts $fh $line2;
      puts $fh $line3;

#      puts "line1 = $line1"   
#      puts "line2 = $line2" 
#      puts "line3 = $line3"       
   } 
   
   puts  $fh "#\n# \"@(#) \$Id\$\" \n";  # Write cmm identifier and the EOF 
   close $fh;                            # Close the file

   # Delete unused help files

   if {[info exists deleteHelpList] == 0 } { return }

   set msg "DELETE not used Help Files"
   if {[tk_dialog .dialog "Dialog Box" $msg question 1 Yes No]} { return }


   set currHelpFiles {}
   set tmpList {}

   puts "\nDelete unused help files :"

   # Build list of currently used files

   foreach name [array names recList] {
      lappend currHelpFiles [lindex $recList($name) 7]
   }

   # Build list of files that have to be removed

   foreach item $deleteHelpList {

      # Check if this help file is used in another definition
      if {[lsearch $currHelpFiles $item] == -1 } {
         lappend tmpList $item
      }
   }

   foreach item [getFileList ../ERRORS/HELP/${module}ERR*.hlp] {

      # Extract the files in HELP directory which are not used
      set tmpFile [file tail $item]
      if {[lsearch $currHelpFiles $tmpFile] == -1 } {
         lappend tmpList $tmpFile
      }
   }

   foreach item $tmpList {
      set filePath [findModuleFile ERRORS/HELP/$item]
      if { $filePath != "" && [file exists $filePath] } {
         puts "FILE > $filePath  REMOVED"
         exec mv $filePath $filePath~ ; # Remove old help file
      }
   }

   foreach item $deleteHelpList {

      # Check if this help file is used in another definition
     
      if {[lsearch $currHelpFiles $item] != -1 } { continue }

      set filePath [findModuleFile ERRORS/HELP/$item]
      if { $filePath != "" && [file exists $filePath] } {
         puts "FILE > $filePath  REMOVED"
         exec mv $filePath $filePath~ ; # Remove old help file
      }
   }

   set deleteHelpList {} ; # Clear the list
}


#-----------------------------------------------------------------------------
# PROCEDURE : displayError - Displays the definition of the current error.
#-----------------------------------------------------------------------------

proc displayError { index } {
   global par rec_offset rec_id  rec_type rec_memo rec_format   
   global err_help err_offset recList
   global myNumber  myMnemonic  myFormat  mySeverity  myHelpFile myOffset
   
#   puts "DBG> displayError index = $index"

   set myErr [feditWhichRec $index]

   if {[info exists recList($myErr)] == 0} { 
      feditClearConfig
      set myMnemonic ""
      return 
   }

   if {$index < 0 } { set index 0 }

   set par(CURRENT_ID) $myErr ;        # Store current record identifier
                                       # Used by feditRecModify
         
   set myNumber   [feditGetRec $myErr $rec_id]
   set myMnemonic [feditGetRec $myErr $rec_memo]  
   set mySeverity [feditGetRec $myErr $rec_type]   
   set myFormat   [feditGetRec $myErr $rec_format]  
   set myHelpFile [feditGetRec $myErr $err_help]
   set myOffset   [feditGetRec $myErr $err_offset]

   set par(OLD_MEMO) $myMnemonic ; # Save current mnemonic to later update
                                   # Help file

   # Display help file

   if {$myHelpFile != ""} {
      $par(wdgErrHelp) deleteText 0.0 end ;     # Clear Help File
      set tmp [internal::errReadHelpFile $myHelpFile $par(EDIT_MODE)]
      $par(wdgErrHelp) insertText 0.0 $tmp   
   }

   set par(STORE_HELP) 0
}

#-----------------------------------------------------------------------------
# PROCEDURE : feditCleanHelp - Clean the ERRORS directory from unused
#                              help files.
#-----------------------------------------------------------------------------

proc feditCleanHelp { mod helpDir } {
   global  recList err_help

   #
   # set list of currently used help files
   #

   set usedHelpFiles {}
   foreach item [array names recList] {
      if {[set tmp [lindex $recList($item) $err_help]] != ""} {
         lappend usedHelpFiles $tmp
      }
   }

   set listOfFiles [getFileList ${helpDir}/${mod}*.hlp]

#   puts "DBG> used = $usedHelpFiles \n"
#   puts "DBG> dir = $listOfFiles \n"

   foreach item $listOfFiles {

      set item [file tail $item]
 
      if {[lsearch $usedHelpFiles $item] == "-1"} {
         puts "DBG > Remove file $item"
        exec rm ${helpDir}/${item}
      }
   }
}

#
# ___oOo___
