#
# E.S.O. - VLT project - 1995
#
# "@(#) $Id: feditErrorConfig.tcl,v 2.11 2003/11/12 09:10:34 vltsccm Exp $" 
#
# who        when      what
# ---------  --------  ----------------------------------------------
# M.COMIN    ??/??/??  First release
# M.COMIN    08/06/95  Fixed bug when defining default offset
# mcomin     13/11/95  Moved from err module to fedit module
# mcomin     04/12/95  Add bind on RETURN for mnemonics
# ------------------
# mcomin    14/03/96  Ported to TK4.0 and itcl2.0
#

#------------------------------------------------------------------------------
# PROCEDURE : errConfig -  
#------------------------------------------------------------------------------

proc feditErrorConfig { top } {
   global par myNumber myFormat  mySeverity  myIndex
   global myHelpFile myMnemonic  myOffset wdgErrHelp  

   ##################################################
   #             Creation of main frames
   ##################################################

   set titleFrame [frame $top.configTitle ]
   pack $titleFrame -anchor nw -fill x

   set form    [frame $top.errConfig -relief raised -bd 2]
   set actBut1 [frame $top.actFrame ]

   set row1 [frame $form.row1] ; set row3 [frame $form.row3]
   set row5 [frame $form.row5] ; set row7 [frame $form.row7]
   set row9 [frame $form.row9]

   pack $form -fill x -pady 5
   pack $row1  $row3  $row5 -anchor nw -fill x
   pack $row7 -anchor nw -fill x -pady 5
   pack $row9 -anchor nw -fill x

   ##################################################
   #       Creation of Format Definition GUI
   ##################################################

   #
   # Create title
   #

   label $titleFrame.lmodule -text "Module Name : "
   label $titleFrame.module  -textvariable  par(MODULE) -width 8 -fg Blue \
         -font -Adobe-courier-bold-r-normal--14* -anchor w

   label $titleFrame.lblFile -text "File Path "

   label $titleFrame.filePath  -textvariable par(FILE) -fg Blue \
         -font -Adobe-courier-bold-r-normal--12* -anchor w 

   pack $titleFrame.lmodule $titleFrame.module -side left -padx 3
   pack $titleFrame.lblFile $titleFrame.filePath -side left -padx 3

   #
   # Create Error Configuration Entry Form
   #

   UifEntry $row1.errNumb -label "Error Number" -width 4\
            -labelFont $par(FONT12) -variable myNumber

   UifEntry $row1.errMemo -label "Error Mnemonic" -width 30\
            -labelFont $par(FONT12) -labelWidth 16 -variable myMnemonic

   label $row1.lbl1 -text "Error Severity" -font $par(FONT12)

   UifOptionButton $row1.errSeverity -default W -length 7 \
                   -itemList {Warning Serious Fatal} -selectMode value \
	           -itemValue { W S F } -variable mySeverity \
                   -foreground blue

   set mySeverity "W"

   pack $row1.errMemo -side left -pady 3 -padx 2
   pack $row1.errNumb -side left -pady 3 -padx 5
   pack $row1.lbl1 -side left  -padx 10
   pack $row1.errSeverity -side left

   UifEntry $row3.errOffset -label "Error Offset" -width 30\
            -labelFont $par(FONT12) -labelWidth 16 -variable myOffset

   button $row3.defOff -width 12 -text "Default Offset" -command setDefaultOffset
             
   button $row3.setOff -width 12 -text "Set Offset" -command errDefineOffset

   pack $row3.errOffset $row3.defOff $row3.setOff -side left -pady 3 -padx 2

   UifEntry $row5.errFormat -label "Error Format" -width 85 \
            -labelFont $par(FONT12) -labelWidth 16 -variable myFormat

   pack $row5.errFormat -side left -pady 3 -padx 2

   #
   # Create Error Help reference File
   #
 
   $row7 configure -relief groove -bd 2

   set h1 [frame $row7.h1]
   set h2 [frame $row7.h2]
   set par(wdgErrHelp) $h2.helpText

   pack $h1 $h2 -anchor w

   UifEntry $h1.errHelp -label "Help File Name" -width 30\
            -labelFont $par(FONT12) -labelWidth 16 -variable myHelpFile

   button  $h1.setTemplate -text "Help Template" -width 12 \
	                   -command {feditHelpTemplate}

   button  $h1.clearTmp -text "Clear Help" -width 12 \
                        -command "$par(wdgErrHelp) deleteText 0.0 end"

   button  $h1.restoreTmp -text "Restore Help File" -width 20 \
                        -command feditRestoreHelpFile

   pack $h1.errHelp $h1.setTemplate $h1.clearTmp $h1.restoreTmp \
        -side left -padx 3 -pady 3

   UifText $par(wdgErrHelp) -height 8 -font "-Adobe-courier-medium-r-normal--14*"

   pack  $par(wdgErrHelp) -padx 5 -pady 5 

   #
   # Set specific bindings
   #

   bind [$row1.errMemo component uifEntry] <Return> { feditSetError }

   bind  [$par(wdgErrHelp) component uifText] <Any-Key> "+feditSetHelpFlag %A"
   bind  [$h1.errHelp  component uifEntry]    <Any-Key> "+feditSetHelpFlag %A"

   # Set short helps

   UifShortHelp::add $par(SH_AREA) $row1.errMemo \
        "Press RETURN to update the Help File Name"

   UifShortHelp::add $par(SH_AREA) $row5.errFormat \
        "The format follows the printf specifications : with strings specify the maximum number of chars., for example %.80s "


   UifShortHelp::add $par(SH_AREA) $h1.restoreTmp \
     "Replace the current help text with the contents of <helpFileName>.hlp~, if any"

}

#-----------------------------------------------------------------------------
# PROCEDURE : setDefaultOffset -  
#-----------------------------------------------------------------------------

proc setDefaultOffset {} {
   global offsetRec myOffset par

   if {[info exists offsetRec($par(MODULE)ErrOffset)] == 0 } {
      set offsetRec($par(MODULE)ErrOffset) 0
   }
   set myOffset  "$par(MODULE)ErrOffset"
}

#-----------------------------------------------------------------------------
# PROCEDURE : setError -  
#-----------------------------------------------------------------------------

proc feditSetError {} {
   global par recList errMemo myMnemonic myHelpFile myNumber

   $par(SH_AREA) displayText ""
   
   if { [info exists errMemo($myMnemonic)] } {

      feditRecSearch
    
   } else {

     set oldMemo     $myMnemonic
     feditClearConfig
     set myMnemonic  $oldMemo
#     set myHelpFile  "${myMnemonic}.hlp"
     incr par(LAST_ID)
     set myNumber    $par(LAST_ID)
     
     $par(SH_AREA) displayText "ERROR DOES NOT EXIST : Complete definition and ADD"  
   }

   set myHelpFile  "${myMnemonic}.hlp"
}


