#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: iracqUifCmd.tcl,v 1.113 2005/09/13 07:27:24 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# jstegmei  26/05/99  created
# jstegmei  27/10/99  added RTD functions
# jstegmei  12/01/00  dynamic add/remove of available frame types
# jstegmei  21/01/00  introduced LOADED state, OFF command now goes
#                     to LOADED state instead of exiting
# jstegmei  21/01/00  issue warning at RESET command
# jstegmei  05/03/00  fixed tcl-error in parameter-apply function
# jstegmei  30/06/00  check change of system/detector configuration
#                     files on disk before going standby/online
# jstegmei  28/12/00  added handling of multiple acquisition processes
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc iracqCmd1 {} {
    global gvar
    iracqFunction "cmd1" 1 tmpStr
}

proc iracqCmd2 {} {
    global gvar

    iracqFunction "cmd1" 0 tmpStr
}

# general command execution
proc iracqCmdGen {} {
    global gvar

    if {$gvar(iracqGenFunction) == "wait"} {
	iracqTkDialog1 "Error" \
		"This function cannot be called from here!"
	return 0
    }

    if {![iracqFunction $gvar(iracqGenFunction) 1 tmpStr 1]} {
	return 0
    }

    return 1
}

# off
proc iracqCmdOff {} {
    global gvar

    if {$gvar(iracqSrvState) == "OFF"} {
	if {![iracqRestartServer]} {
	    iracqTkDialog1 "Error" "Unable to start server"
	    return 0
	}
    } else {
	if {$gvar(iracqSrvState) == "LOADED" } {
	    if {[iracqFileChanged sysCfg \
		    "$gvar(iracqSysConfigFile)" 1]} {
		if {[iracqTkDialog2 \
			"Attention" \
			"The system configuration file has changed on disk!\
			\nReload?"]} {
		    iracqCmdSysConf $gvar(iracqSysConfigFile)
		    iracqFileChanged detCfg \
			    "$gvar(iracqDetConfigFile)" 1
		    
		    if {![iracqGetConfig]} {
			return 0
		    }
		    return 1
		}
	    }
	}
    }

    if {![iracqFunction "off" 1 tmpStr]} {
	return 0
    } 

    if {![iracqGetConfig]} {
	return 0
    } 

    return 1
}

# exit
proc iracqCmdExit {} {
    global gvar

    if {![iracqFunction "exit" 1 tmpStr]} {
	return 0
    }

    iracqFileChanged sysCfg \
	    "$gvar(iracqSysConfigFile)" 1
    iracqFileChanged detCfg \
	    "$gvar(iracqDetConfigFile)" 1
    
    if {!$gvar(iracqCCS)} {
	# close socket connections
	iracqCloseEventNoCcs
	iracqCloseServerNoCcs
	set gvar(iracqSrvState) "OFF"
	set gvar(iracqSrvSubState) "IDLE"
    }

    return 1
}

# load system configuration
proc iracqCmdSysConf {fileName} {
    global gvar

    if {![iracqFunction "sysconf $fileName" 1 tmpStr]} {
	return 0
    }

    return 1
}

# load detector configuration
proc iracqCmdDetConf {fileName} {
    global gvar

    if {![iracqFunction "detconf $fileName" 1 tmpStr]} {
	return 0
    }
    
    return 1
}

# online
proc iracqCmdOnline {} {
    global gvar

    if {$gvar(iracqSrvState) == "OFF"} {
	if {![iracqRestartServer]} {
	    iracqTkDialog1 "Error" "Unable to start server"
	    return 0
	}
    } else {
	if {[iracqFileChanged sysCfg \
		"$gvar(iracqSysConfigFile)" 1]} {
	    if {[iracqTkDialog2 \
		    "Attention" \
		    "The system configuration file has changed on disk!\
		    \nReload?"]} {
		iracqCmdSysConf $gvar(iracqSysConfigFile)
		iracqFileChanged detCfg \
		    "$gvar(iracqDetConfigFile)" 1
	    }
	} else {
	    if {[iracqFileChanged detCfg \
		    "$gvar(iracqDetConfigFile)" 1]} {
		if {[iracqTkDialog2 \
			"Attention" \
			"The detector configuration file has changed on disk!\
			\nReload?"]} {
		    iracqCmdDetConf $gvar(iracqDetConfigFile)
		}
	    }
	}
    }

    if {![iracqFunction "online" 1 tmpStr]} {
	return 0
    } else {
	if {![iracqGetConfig]} {
	    return 0
	} 
    }
    
    return 1
}

# standby
proc iracqCmdStandby {} {
    global gvar

    if {$gvar(iracqSrvState) == "OFF"} {
	if {![iracqRestartServer]} {
	    iracqTkDialog1 "Error" "Unable to start server"
	    return 0
	}
    } else {
	if {$gvar(iracqSrvState) == "LOADED" || \
	    $gvar(iracqSrvState) == "STANDBY"} {
	    if {[iracqFileChanged sysCfg \
		    "$gvar(iracqSysConfigFile)" 1]} {
		if {[iracqTkDialog2 \
			"Attention" \
			"The system configuration file has changed on disk!\
			\nReload?"]} {
		    iracqCmdSysConf $gvar(iracqSysConfigFile)
		    iracqFileChanged detCfg \
			    "$gvar(iracqDetConfigFile)" 1
		}
	    }
	}
    }


    if {![iracqFunction "standby" 1 tmpStr]} {
	return 0
    } else {
	if {![iracqGetConfig]} {
	    return 0
	} 
    }

    return 1
}

# reset
proc iracqCmdReset {} {
    global gvar

    if {[tk_dialog .dialog "Warning" \
	    "This will reset the whole IRACE system!\nContinue?" \
	    warning 0 "OK" "Dismiss"]} {
	return 1
    }


    if {![iracqFunction "reset" 1 tmpStr]} {
	return 0
    } else {
	if {![iracqGetConfig]} {
	    return 0
	} 
    }

    return 1
}

# set operation mode
proc iracqCmdSetOpMode {mode} {
    global gvar

    if {![iracqFunction "setopmode $mode" 1 tmpStr]} {
	return 0
    } else {
	if {![iracqGetConfig]} {
	    return 0
	}
    }

    return 1
}

# set readout mode
proc iracqCmdSetReadoutMode {mode} {
    global gvar

    if {!$gvar(iracqCCS)} {
	if {[iracqFunction "setrmode $mode" 1 tmpStr]} {
	    set gvar(iracqRm) $mode
	    if {![iracqGetConfig]} {
		return 0
	    }
	} else {
	    return 0
	}
    } else {
	iracqBusy 1 1
	if {[iracqSendCmd "SETUP" "-function DET.NCORRS.NAME $mode"]} {
	    set gvar(iracqRm) $mode
	    if {![iracqGetConfig]} {
		iracqBusy 0 1
		return 0
	    }
	} else {
	    iracqBusy 0 1
	    return 0
	}
	iracqBusy 0 1
    }

    return 1
}

proc iracqCmdGetParam {doRes} {
    global iracqParam iracqParamSet gvar

    if {![iracqFunction "getparam" $doRes tmpStr]} {
	return 0
    }

    if {[info exists iracqParam]} {
	set oldList [lsort [array names iracqParam]]
	foreach ele $oldList {
	    set oldParam($ele) $iracqParam($ele)
	    set iracqParam($ele) "undefined"
	    set iracqParamSet($ele) ""
	}
    }

    while {[set paramString [iracqSgets tmpStr]] != -1} {
        if {[string length $paramString] != 0} {
	    scan $paramString "%s %s" p1 p2
	    set iracqParam($p1) $p2
	    set iracqParamSet($p1) ""
	}
    }

    if {[info exists oldList]} {
	set gvar(iracqNewParam) 0
	set newList [lsort [array names iracqParam]]
	foreach ele $newList {
	    if {[lsearch -exact $oldList $ele] == -1} {
		# new element added
		set gvar(iracqNewParam) 1
		break
	    }
	    if {$iracqParam($ele) == "undefined" && \
		    $oldParam($ele) != "undefined"} {
		# element has been marked as invalid
		set gvar(iracqNewParam) 1
		break
	    }
	    if {$iracqParam($ele) != "undefined" && \
		    $oldParam($ele) == "undefined"} {
		# element has been marked as valid again
		set gvar(iracqNewParam) 1
		break
	    }
	}
    } else {
	set gvar(iracqNewParam) 1
    }

    return 1    
}

# load voltage file
proc iracqCmdLoadVoltage {file} {
    global gvar

    if {$file == ""} {
	return 1
    }

#if {($file != $gvar(iracqDetVoltage)) && \
#	    ($file != $gvar(iracqSysVoltageT))} {
#	if {![iracqTkDialog2 "Warning" \
#		"The voltage file does not match detector configuration!\
#		\nSetup?"]} {
#	    set gvar(iracqSysVoltage) $gvar(iracqSysVoltageT)
#	    return 0
#	}	
#    }

    if {![iracqFunction "ldv $file" 1 tmpStr]} {
	set gvar(iracqSysVoltage) $gvar(iracqSysVoltageT)
	return 0
    }
    set gvar(iracqSysVoltageT) $file

    if {[iracqFunction "vsetup" 0 tmpStr]} {
	iracqScanVsetup $tmpStr
    }

    return 1
}    

# save sequence file
proc iracqCmdSaveVoltage {file} {
    global gvar

    if {$file == ""} {
	return 1
    }

    if {![iracqFunction "savevolt $file" 1 tmpStr]} {
	return 0
    }

    return 1
}    

# load clock file
proc iracqCmdLoadClk {file} {
    global gvar

    if {$file == ""} {
	return 1
    }

    if {![iracqFunction "ldclk $file" 1 tmpStr]} {
	set gvar(iracqSysClk) $gvar(iracqSysClkT)
	return 0
    } else {
	if {![iracqGetConfig]} {
	    return 0
	} 
    }
    set gvar(iracqSysClkT) $file

    return 1
}    

# load sequence file
proc iracqCmdLoadSeq {file} {
    global gvar

    if {$file == ""} {
	return 1
    }

    if {![iracqFunction "ldseq $file" 1 tmpStr]} {
	set gvar(iracqSysSeq) $gvar(iracqSysSeqT)
	return 0
    } else {
	if {![iracqGetConfig]} {
	    return 0
	} 
    }
    set gvar(iracqSysSeqT) $file

    return 1
}    

# load acquisition process
proc iracqCmdLoadAcqProc {file} {
    global gvar

    if {$file == ""} {
	set file "none"
    }

    if {![iracqFunction "ldacq $file" 1 tmpStr]} {
	set gvar(iracqSysAcqProc) $gvar(iracqSysAcqProcT)
	return 0
    } else {
	if {![iracqGetConfig]} {
	    set gvar(iracqSysAcqProc) $gvar(iracqSysAcqProcT)
	    return 0
	} 
    }

    if {$file == "none"} {
	set gvar(iracqSysAcqProcT) ""
    } else {
	set gvar(iracqSysAcqProcT) $file
    }
    return 1
}    

# set cldc number
proc iracqCmdSetCldc {num} {
    global gvar 

    if {$num == ""} {
	return 1
    }

    if {$num == $gvar(iracqSysCldc)} {
	# nothing has changed
	return 1
    }

#    if {$num != $gvar(iracqDetCldc)} {
#	if {![iracqTkDialog2 "Warning" \
#		"The CLDC board number does not match detector configuration!\
#		\nSetup?"]} {
#	    return 0
#	}
#    }

    if {![iracqFunction "setcldc $num" 1 tmpStr]} {
	return 0
    }

    iracqGetConfig

    return 1
}    

# get voltage setup
proc iracqCmdVsetup {} {

    if {![iracqFunction "vsetup" 1 tmpStr 1]} {
	return 0
    }

    iracqScanVsetup $tmpStr
    
    return 1
}

# get telemetry
proc iracqCmdTel {} {

    if {![iracqFunction "tel" 1 tmpStr 1]} {
	return 0
    }
    
    return 1
}

# get single telemtry value
proc iracqCmdTelChan {channel} {

    if {![iracqFunction "telchan $channel" 1 tmpStr]} {
	return -1
    }
    set telVal [iracqScanString [iracqSgets tmpStr]]
    
    return $telVal
}

# set voltage on a single channnel
proc iracqCmdVoltChan {channel value doRes} {

    if {$channel == ""} {
	return 1
    }
    if {$value == ""} {
	return 1
    }

    if {![iracqFunction "voltchan $channel $value" $doRes tmpStr]} {
	if {[iracqFunction "vsetup" 0 tmpStr]} {
	    iracqScanVsetup $tmpStr
	}
	return 0
    }
    
    return 1
}

# enable dac
proc iracqCmdDacEnable {} {
    global gvar
    global cv

    $cv.iracqDacEnable disable

    if {![iracqFunction "dac enable" 1 tmpStr]} {
	iracqGetState
	return 0
    }

    iracqGetState

    return 1
}    

# disable dac
proc iracqCmdDacDisable {} {
    global gvar
    global cv

    $cv.iracqDacEnable disable

    if {![iracqFunction "dac disable" 1 tmpStr]} {
	iracqGetState
	return 0
    }

    iracqGetState

    return 1
}    

# start sequencer
proc iracqCmdStartSeq {} {
    global gvar

    if {![iracqFunction "startseq" 1 tmpStr]} {
	return 0
    }

    if {![iracqGetState]} {
	return 0
    }

    return 1
}    

# stop sequencer
proc iracqCmdStopSeq {} {
    global gvar

    if {![iracqFunction "stopseq" 1 tmpStr]} {
	return 0
    }

    if {![iracqGetState]} {
	return 0
    }

    return 1
}    

# start acquisition
proc iracqCmdStartAcq {} {
    global gvar

    if {![iracqFunction "startacq" 1 tmpStr]} {
	return 0
    }

    if {![iracqGetState]} {
	return 0
    }

    return 1
}    

# stop acquisition
proc iracqCmdStopAcq {} {
    global gvar

    if {![iracqFunction "stopacq" 1 tmpStr]} {
	return 0
    }

    if {![iracqGetState]} {
	return 0
    }

    return 1
}    

# start all
proc iracqCmdStart {} {
    global gvar

    if {![iracqFunction "start" 1 tmpStr]} {
	return 0
    }

    if {![iracqGetState]} {
	return 0
    }

    return 1
}    

# stop sequencer
proc iracqCmdStop {} {
    global gvar

    if {![iracqFunction "stop" 1 tmpStr]} {
	return 0
    }

    if {![iracqGetState]} {
	return 0
    }

    return 1
}    

# set default parameters
proc iracqCmdParamDefault {} {
    global gvar
    global iracqParam

    if {![iracqFunction "pdef" 1 tmpStr]} {
	return 0
    } else {
	if {![iracqGetConfig]} {
	    return 0
	}
    }

    iracqCmdFrameList
    return 1
}    

# apply all parameters
proc iracqCmdParamApply {} {
    global gvar
    global iracqParam
    global iracqParamSet

    set paramString ""
    foreach i [lsort [array names iracqParamSet]] {
	if {$iracqParamSet($i) != "" && $iracqParam($i) != "undefined"} {
	    set paramString "$paramString $i $iracqParamSet($i)"
	}  
    }

    if {![iracqFunction "papply $paramString" 1 tmpStr]} {
	return 0
    } else {
	if {![iracqGetConfig]} {
	    return 0
	}
    }

    iracqCmdFrameList

    foreach i [lsort [array names iracqParamSet]] {
	set iracqParamSet($i) ""
    }

    return 1
}    

# set parameter
proc iracqCmdParamSet {paramName} {
    global gvar
    global iracqParam
    global iracqParamSet

    if {$iracqParamSet($paramName) == ""} {
	return 1
    }

    if {![iracqFunction "pset $paramName $iracqParamSet($paramName)"\
	    1 tmpStr]} {
	set iracqParamSet($paramName) ""
	return 0
    } else {
	if {![iracqGetConfig]} {
	    set iracqParamSet($paramName) ""
	    return 0
	}
    }

    iracqCmdFrameList

    set iracqParamSet($paramName) ""
    return 1
}    

# set readspeed
proc iracqCmdRspeed {} {
    global gvar

    if {$gvar(iracqRspeedFSetup) == ""} {
	set factor $gvar(iracqRspeedF)
    } else {
	set factor $gvar(iracqRspeedFSetup)
    }

    if {$gvar(iracqRspeedASetup) == ""} {
	set add $gvar(iracqRspeedA)
    } else {
	set add $gvar(iracqRspeedASetup)
    }

    if {![iracqFunction "rspeed $factor $add" \
	    1 tmpStr]} {
	iracqGetConfig
	return 0
    }

    iracqGetConfig

    set gvar(iracqRspeedFSetup) ""
    set gvar(iracqRspeedASetup) ""

    return 1
}

# start exposure
proc iracqCmdStartExp {} {
    global gvar

    if {![iracqFunction "startexp" \
	    1 tmpStr]} {
	return 0
    }

    if {![iracqGetState]} {
	return 0
    }

    return 1
}

# stop exposure
proc iracqCmdStopExp {} {
    global gvar

    if {![iracqFunction "stopexp" \
	    1 tmpStr]} {
	return 0
    }

    if {![iracqGetState]} {
	return 0
    }

    return 1
}

# end exposure
proc iracqCmdEndExp {} {
    global gvar

    if {![iracqFunction "endexp" \
	    1 tmpStr]} {
	return 0
    }

    if {![iracqGetState]} {
	return 0
    }

    return 1
}

# reset acquisition process loop counters
proc iracqCmdResetCnt {} {
    global gvar

    if {![iracqFunction "rstcnt" \
	    1 tmpStr]} {
	return 0
    }

    return 1
}

# set sequence base name
proc iracqCmdSeqBaseName {name} {
    global gvar

    if {$name == ""} {
	return 1
    }

    if {![iracqFunction "seqbase $name" \
	    1 tmpStr]} {
	return 0
    }

    set gvar(iracqSeqBaseName) $name

    return 1
}

# set request name
proc iracqCmdReqFileName {name} {
    global gvar

    if {$name == ""} {
	return 1
    }

    if {![iracqFunction "reqname $name" \
	    1 tmpStr]} {
	return 0
    }

    set gvar(iracqReqFileName) $name

    return 1
}

# store in data cube
proc iracqCmdDataCube {} {
    global gvar 

    if {![iracqFunction "dcube $gvar(iracqDataCube)" \
	    1 tmpStr]} {
	set gvar(iracqDataCube) [expr {1 - $gvar(iracqDataCube)}]
    }

    return 1
}

# sequence naming
proc iracqCmdNamingType {value} {
    global gvar 

    if {![iracqFunction "ntype $gvar(iracqNamingType)" \
	    1 tmpStr]} {
    }

    return 1
}

# set DIT to minimum DIT
proc iracqCmdSetMinDit {} {
    global gvar

    if {![iracqFunction "setdit $gvar(iracqMinDit)" \
	    1 tmpStr]} {
	iracqGetConfig
	return 0
    }

    iracqGetConfig

    return 1
}

# set DIT
proc iracqCmdSetDit {dit} {
    global gvar

    if {$dit == ""} {
	return 1
    }

    if {![iracqFunction "setdit $dit" \
	    1 tmpStr]} {
	iracqGetConfig
	return 0
    }

    iracqGetConfig

    return 1
}

# adc setup
proc iracqCmdAdcSet {tag num value} {
    global gvar

    if {$num < 0} {
	return 0
    }

    if {![iracqFunction "adcgset $tag $num $value" \
	    1 tmpStr]} {
	return 0
    }

    return 1
}

# adc status
proc iracqCmdAdcGet {tag num doRes} {
    global gvar

    if {![iracqFunction "adcget $tag $num" \
	    $doRes tmpStr]} {
	return -1
    }
    
    set value [iracqScanString [iracqSgets tmpStr]]
    
    return $value
}

# use hardware window
proc iracqCmdHwWin {} {
    global gvar

    # set window as hardware window
    if {![iracqFunction "setwin $gvar(iracqHwWin) $gvar(iracqWinStartX) \
	    $gvar(iracqWinStartY) $gvar(iracqWinNx) $gvar(iracqWinNy)" \
	    1 tmpStr]} {
	iracqGetConfig
	return 0
    }

    iracqGetConfig
    return 1
}

# set window
proc iracqCmdSetWin {} {
    global gvar

    if {$gvar(iracqWinStartXSetup) == ""} {
	set sx $gvar(iracqWinStartX)
    } else {
	set sx $gvar(iracqWinStartXSetup)
    }

    if {$gvar(iracqWinStartYSetup) == ""} {
	set sy $gvar(iracqWinStartY)
    } else {
	set sy $gvar(iracqWinStartYSetup)
    }

    if {$gvar(iracqWinNxSetup) == ""} {
	set nx $gvar(iracqWinNx)
    } else {
	set nx $gvar(iracqWinNxSetup)
    }

    if {$gvar(iracqWinNySetup) == ""} {
	set ny $gvar(iracqWinNy)
    } else {
	set ny $gvar(iracqWinNySetup)
    }

    if {![iracqFunction "setwin $gvar(iracqHwWin) \
	    $sx $sy $nx $ny" \
	    1 tmpStr]} {
	iracqGetConfig
	return 0
    }
    
    iracqGetConfig

    set gvar(iracqWinStartXSetup) ""
    set gvar(iracqWinStartYSetup) ""
    set gvar(iracqWinNxSetup) ""
    set gvar(iracqWinNySetup) ""

    return 1
}

# set window to full frame
proc iracqCmdFullFrame {} {
    global gvar

    set gvar(iracqWinStartXSetup) 1
    set gvar(iracqWinStartYSetup) 1
    set gvar(iracqWinNxSetup) 0
    set gvar(iracqWinNySetup) 0

    iracqCmdSetWin

}

# define window in RTD
proc iracqCmdRtdWin {} {
    global gvar env seq_ccsEnvName

    if {!$gvar(iracqCCS)} {
	return 0
    }

    if {$gvar(iracqRtdCmd) == "rtdir"} {
	set target "rtdir"
	set cmd "docmd DEFWIN"
    } else {
	set target "rtdb"
	set cmd "defwin"
    }
    set reply [panSendCommand $seq_ccsEnvName $gvar(iracqRtdCmd) \
	    "SCRIPT" \
	    NO_CHECK 200000 \
	    $cmd]

    if {$reply == "FAILURE"} {
	return 0
    }

    if {[string first "-1" $reply] != -1} {
	iracqTkDialog1 "RTD-Error" \
		"Problem defining window parameters\n\
		in the RTD or operation was\n\
		cancelled.\n\n\
		Window parameters not changed"
	return 0
    }

    set STARTX [lindex $reply 0]
    set STARTY [lindex $reply 1]
    set NX     [expr {int([lindex $reply 2])}]
    set NY     [expr {int([lindex $reply 3])}]

    set reply [panSendCommand $seq_ccsEnvName $gvar(iracqSrvName) \
	    "SETUP" \
	    CHECK_ALL 200000 \
	    "-function DET$gvar(iracqDetIdx)\.WIN.STARTX $STARTX \
	    DET$gvar(iracqDetIdx)\.WIN.STARTY $STARTY \
	    DET$gvar(iracqDetIdx)\.WIN.NX $NX \
	    DET$gvar(iracqDetIdx)\.WIN.NY $NY"]

    if {$reply == "FAILURE"} {
	return 0
    }

    return 1
}

# get frame list
proc iracqCmdFrameList {} {
    global gvar iracqFrame iracqFrameGen iracqFrameStore iracqFrameDisp
   
    if {$gvar(iracqCCS)} {
	# this is not needed here
	return 1
    }

    if {![iracqFunction "getframes" \
	    0 tmpStr]} {
	return 0
    }

    while {[set frameString [iracqSgets tmpStr]] != -1} {
        if {[string length $frameString] != 0} {
	    scan $frameString "%s %s %s %s" p1 p2 p3 p4
	    set iracqFrame($p1) $p1
	    set iracqFrameGen($p1) $p2
	    set iracqFrameStore($p1) $p3
	    set iracqFrameDisp($p1) $p4
	    set valid($p1) 1
	    iracqAddFrame $gvar(iracqFrameWidget) $p1
	}
    }
    
    if {[info exists iracqFrame]} {
	foreach ele [array names iracqFrame] {
	    if {$ele != ""} {
		if {![info exists valid($ele)]} {
		    iracqRemoveFrame $gvar(iracqFrameWidget) $ele
		}
	    }
	}
    }
    

    return 1
}

# display frame on/off
proc iracqCmdFrameDisp {name} {
    global gvar iracqFrameDisp

    set tmp [expr {1 - $iracqFrameDisp($name)}]
    if {!$iracqFrameDisp($name)} {
	set fname "NO_FRAME"
    } else {
	set fname $name
    }
    if {!$gvar(iracqCCS)} {
	if {![iracqFunction "camtype $fname" \
		1 tmpStr]} {
	    set iracqFrameDisp($name) $tmp
	    return 0
	}
    
	# try to get parameter setup
	if {[iracqCmdGetParam 0]} {
	    if {$gvar(iracqNewParam)} {
		iracqParamUpdate
	    }
	}
    } else {
	if {![iracqSendCmd "FRAME" "-name $fname -display"]} {
	    set iracqFrameDisp($name) $tmp
	}
    }

    iracqGetConfig
    iracqCmdFrameList

    return 1
}

# generate frame on/off
proc iracqCmdFrameGen {name} {
    global gvar iracqFrameGen

    set tmp [expr {1 - $iracqFrameGen($name)}]

    if {!$gvar(iracqCCS)} {
	if {![iracqFunction "framegen $iracqFrameGen($name) $name" \
		1 tmpStr]} {
	    set iracqFrameGen($name) $tmp
	    return 0
	}
	
	# try to get parameter setup
	if {[iracqCmdGetParam 0]} {
	    if {$gvar(iracqNewParam)} {
		iracqParamUpdate
	    }
	}
    } else {
	if {$iracqFrameGen($name)} {
	    set val "T"
	} else {
	    set val "F"
	}
	if {![iracqSendCmd "FRAME" "-name $name -gen $val"]} {
	    set iracqFrameGen($name) $tmp
	}
    }

    iracqCmdFrameList

    return 1
}

# store frame on/off
proc iracqCmdFrameStore {name} {
    global gvar iracqFrameStore

    set tmp [expr {1 - $iracqFrameStore($name)}]

    if {!$gvar(iracqCCS)} {
	if {![iracqFunction "framestore $iracqFrameStore($name) $name" \
		1 tmpStr]} {
	    set iracqFrameStore($name) $tmp
	    return 0
	}
	
	# try to get parameter setup
	if {[iracqCmdGetParam 0]} {
	    if {$gvar(iracqNewParam)} {
		iracqParamUpdate
	    }
	}
    } else {
	if {$iracqFrameStore($name)} {
	    set val "T"
	} else {
	    set val "F"
	}
	if {![iracqSendCmd "FRAME" "-name $name -store $val"]} {
	    set iracqFrameStore($name) $tmp
	}
    }

    iracqCmdFrameList

    return 1
}

# reset sequence naming index
proc iracqCmdResetSeqIdx {} {
    global gvar 

    if {[tk_dialog .confirm "Confirm"\
	    "This will set the sequence naming index to 1!"\
	    warning 0 "Continue" "Dismiss"]} {
	return 1
    }
    
    if {![iracqFunction "pset DET.EXP.SEQ.NO 1" \
	    1 tmpStr]} {
	return 0
    }

    return 1
}

# reset exposure number
proc iracqCmdResetExpNo {} {
    global gvar 

    if {![iracqFunction "pset DET.EXP.NO 1" \
	    1 tmpStr]} {
	return 0
    }

    return 1
}

# enable/disable extended FITS-header
proc iracqCmdExtFits {} {
    global gvar 

    set tmp [expr {1 - $gvar(iracqExtFits)}]

    if {![iracqFunction "extfits $gvar(iracqExtFits)" \
	    1 tmpStr]} {
	set gvar(iracqExtFits) $tmp
	return 0
    }

    return 1
}

# enable/disable sequencer continuos mode
proc iracqCmdSeqCont {} {
    global gvar 

    set tmp [expr {1 - $gvar(iracqSeqCont)}]

    if {![iracqFunction "seqcont $gvar(iracqSeqCont)" \
	    1 tmpStr]} {
	set gvar(iracqSeqCont) $tmp
	return 0
    }

    return 1
}

# enable/disable number cruncher reset at start command
proc iracqCmdNoNcReset {} {
    global gvar 

    set tmp [expr {1 - $gvar(iracqNoNcReset)}]

    if {![iracqFunction "noncrst $gvar(iracqNoNcReset)" \
	    1 tmpStr]} {
	set gvar(iracqNoNcReset) $tmp
	return 0
    }

    return 1
}

# download flatfield
proc iracqCmdDownloadFlatfield {} {
    global gvar 

    if {$gvar(iracq_flatfield) == ""} {
	return 1
    }

    if {![iracqFunction "lddata flatfield $gvar(iracq_flatfield) $gvar(iracq_flatfieldDev)" \
	    1 tmpStr]} {
	return 0
    }

    return 1
}

# download bad-pixel-mask
proc iracqCmdDownloadBadpix {} {
    global gvar 

    if {$gvar(iracq_badpix) == ""} {
	return 1
    }
 
    if {![iracqFunction "lddata badpix $gvar(iracq_badpix) $gvar(iracq_badpixDev)" \
	    1 tmpStr]} {
	return 0
    }

    return 1
}

# set sequencer mode
proc iracqCmdSeqMode {value} {
    global gvar 

    if {![iracqFunction "seqmode $gvar(iracqSeqMode)" \
	    1 tmpStr]} {
	iracqGetConfig
	return 0
    }

    return 1
}

# get current performance
proc iracqCmdPerf {doRes} {
    global gvar 

    if {![iracqFunction "perf" $doRes tmpStr $doRes]} {
	return 0.0
    }

    set str [iracqScanString [iracqSgets tmpStr]]
    set res [scan $str "%f" p]
    if {$res < 1} {
	return 0.0
    }

    set gvar(iracqPerf) $p

    return $p
}


#
# ___oOo___
