#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: iracqUifEvt.tcl,v 1.113 2005/09/13 07:27:24 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# jstegmei  18/06/99  created
# jstegmei  27/10/99  added RTD functions
# jstegmei  13/12/99  support multiple server instances
# jstegmei  12/01/00  dynamic add/remove of available frame types
# jstegmei  24/07/01  added general event disable feature
# jstegmei  15/07/04  enable action-history by default
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

# read attribute
proc iracqReadDb {attr value} {
    global gvar
    upvar $value dbVal

    set newVal [seq_dbReadSymbolic $attr]
    
    set dbVal $newVal

    return 1	
}

# read attribute
proc iracqCfgDb {} {
    global gvar

    if {!$gvar(iracqSrvOpen)} {
	return 1
    }

    seq_dbMultiRead $gvar(iracqCfgListId)
    
    set res [seq_dbListExtract $gvar(iracqCfgListId) \
	    $gvar(iracqDbAttrAcqProcName)]
    set gvar(iracqSysAcqProc) [lindex $res 4]
    set gvar(iracqSysAcqProcT) $gvar(iracqSysAcqProc)

    set res [seq_dbListExtract $gvar(iracqCfgListId) \
	    $gvar(iracqDbAttrHwWindow)]
    set gvar(iracqHwWin) [lindex $res 4]

    set res [seq_dbListExtract $gvar(iracqCfgListId) \
	    $gvar(iracqDbAttrSeqMode)]
    set gvar(iracqSeqMode) [lindex $res 4]

    return 1	
}

# initialize database variables
proc iracqDbInitVars {} {
    global gvar env seq_ccsEnvName

    set gvar(iracqActionEvt) ""
    set gvar(iracqFileEvt) ""

    # database points
    set gvar(iracqDbPointIrace) "$gvar(iracqDbPoint):irace"
    set gvar(iracqDbPointDet)   "$gvar(iracqDbPoint):detector"
    set gvar(iracqDbPointRtd)   "$gvar(iracqDbPoint):rtd"
    set gvar(iracqDbPointExp)   "$gvar(iracqDbPoint):exposure"

    # irace class database attributes
    set gvar(iracqDbAttrUpdate)    "$gvar(iracqDbPointIrace).update"
    set gvar(iracqDbAttrAlarm)    "$gvar(iracqDbPointIrace).alarm"
    set gvar(iracqDbAttrActionLog) "$gvar(iracqDbPointIrace).currentAction"
    set gvar(iracqDbAttrSubState) "$gvar(iracqDbPointIrace).subStateN"
    set gvar(iracqDbAttrRmDef) "$gvar(iracqDbPointIrace).rmDef"
    set gvar(iracqDbAttrFrames) "$gvar(iracqDbPointIrace).frames"
    set gvar(iracqDbAttrDynPars) "$gvar(iracqDbPointIrace).dynPars"
    set gvar(iracqDbAttrAdcStatus) "$gvar(iracqDbPointIrace).adcStatus"
    set gvar(iracqDbAttrClkVolt) "$gvar(iracqDbPointIrace).clkVolt"
    set gvar(iracqDbAttrDcVolt) "$gvar(iracqDbPointIrace).dcVolt"

    set gvar(iracqDbAttrVersion) "$gvar(iracqDbPointIrace).version"
    set gvar(iracqDbAttrState) "$gvar(iracqDbPointIrace).stateN"
    set gvar(iracqDbAttrSeqStatus) "$gvar(iracqDbPointIrace).seqStatusN"
    set gvar(iracqDbAttrSeqMode) "$gvar(iracqDbPointIrace).seqMode"
    set gvar(iracqDbAttrRspeedFactor) "$gvar(iracqDbPointIrace).rspeedFactor"
    set gvar(iracqDbAttrRspeedAdd) "$gvar(iracqDbPointIrace).rspeedAdd"
    set gvar(iracqDbAttrCldcStatus) "$gvar(iracqDbPointIrace).cldcStatusN"
    set gvar(iracqDbAttrAcqStatus) "$gvar(iracqDbPointIrace).acqStatusN"
    set gvar(iracqDbAttrPerf) "$gvar(iracqDbPointIrace).perf"
    set gvar(iracqDbAttrOpMode) "$gvar(iracqDbPointIrace).opModeN"
    set gvar(iracqDbAttrReadoutMode) "$gvar(iracqDbPointIrace).readoutMode"
    set gvar(iracqDbAttrDetCldcBoard) "$gvar(iracqDbPointIrace).detCldcBoard"
    set gvar(iracqDbAttrDetClkFile) "$gvar(iracqDbPointIrace).detClkFile"
    set gvar(iracqDbAttrSysVoltFile) "$gvar(iracqDbPointIrace).sysVoltFile"
    set gvar(iracqDbAttrSysClkFile) "$gvar(iracqDbPointIrace).sysClkFile"
    set gvar(iracqDbAttrSysSeqFile) "$gvar(iracqDbPointIrace).sysSeqFile"
    set gvar(iracqDbAttrCldcBoard) "$gvar(iracqDbPointIrace).cldcBoard"
    set gvar(iracqDbAttrNumAdc) "$gvar(iracqDbPointIrace).numAdc"
    set gvar(iracqDbAttrNumRm) "$gvar(iracqDbPointIrace).numRm"
    set gvar(iracqDbAttrConfigPath) "$gvar(iracqDbPointIrace).configPath"
    set gvar(iracqDbAttrDetConfigFile) "$gvar(iracqDbPointIrace).detConfigFile"
    set gvar(iracqDbAttrAcqProcName) "$gvar(iracqDbPointIrace).acqProcName"

    # RTD class database attributes
    set gvar(iracqDbAttrRtdInt) "$gvar(iracqDbPointRtd).rtdInt"
    set gvar(iracqDbAttrRtdDisp) "$gvar(iracqDbPointRtd).rtdDisp"
    set gvar(iracqDbAttrRtdHost) "$gvar(iracqDbPointRtd).rtdHost"
    set gvar(iracqDbAttrRtdCmd) "$gvar(iracqDbPointRtd).rtdCmd"

    # exposure class database attributes
    set gvar(iracqDbAttrMINDIT) "$gvar(iracqDbPointExp).MINDIT"
    set gvar(iracqDbAttrDIT) "$gvar(iracqDbPointExp).DIT"
    set gvar(iracqDbAttrSTARTX) "$gvar(iracqDbPointExp).STARTX"
    set gvar(iracqDbAttrSTARTY) "$gvar(iracqDbPointExp).STARTY"
    set gvar(iracqDbAttrNX) "$gvar(iracqDbPointExp).NX"
    set gvar(iracqDbAttrNY) "$gvar(iracqDbPointExp).NY"
    set gvar(iracqDbAttrHwWindow) "$gvar(iracqDbPointExp).hwWindow"
    set gvar(iracqDbAttrExpStatus) "$gvar(iracqDbPointExp).expStatusN"
    set gvar(iracqDbAttrDtStatus) "$gvar(iracqDbPointExp).dtStatus"
    set gvar(iracqDbAttrExpTime) "$gvar(iracqDbPointExp).expTime"
    set gvar(iracqDbAttrExpCountDown) "$gvar(iracqDbPointExp).expCountDown"
    set gvar(iracqDbAttrSeqCont) "$gvar(iracqDbPointExp).seqCont"
    set gvar(iracqDbAttrNoNcReset) "$gvar(iracqDbPointExp).noNcReset"
    set gvar(iracqDbAttrExtendedFits) "$gvar(iracqDbPointExp).extendedFitsHeader"
    set gvar(iracqDbAttrSeqBaseName) "$gvar(iracqDbPointExp).seqBaseName"
    set gvar(iracqDbAttrReqFileName) "$gvar(iracqDbPointExp).reqFileName"
    set gvar(iracqDbAttrNamingType) "$gvar(iracqDbPointExp).dataFileNamingType"
    set gvar(iracqDbAttrDataCube) "$gvar(iracqDbPointExp).genDataCube"
    set gvar(iracqDbAttrFileHistory) "$gvar(iracqDbPointExp).newDataFileName"

    # detector class database attributes
    set gvar(iracqDbAttrCldcList) "$gvar(iracqDbPointDet).cldcList"
    set gvar(iracqDbAttrChipName) "$gvar(iracqDbPointDet).NAME"
    set gvar(iracqDbAttrChipSizeX) "$gvar(iracqDbPointDet).NX"
    set gvar(iracqDbAttrChipSizeY) "$gvar(iracqDbPointDet).NY"
}

# initialize database events
proc iracqEvtInit {} {
    global gvar env seq_ccsEnvName

    # attach database events
    if {[catch {set gvar(iracqEvtIdUpdate) [seq_evtAttach \
	    $gvar(iracqDbAttrUpdate) "w" {iracqEvtUpdate %D}]}]} {
	iracqHandleSrvErr "cannot attach database event to \
		$gvar(iracqDbAttrUpdate)"
	return 0
    }
    #iracqAddEvent $gvar(iracqEvtIdUpdate)

    if {[catch {set value \
	    [seq_dbReadSymbolic $gvar(iracqDbAttrSubState)]}]} {
	iracqHandleSrvErr "cannot attach database event to \
		$gvar(iracqDbAttrSubState)"
	return 0
    }
    iracqEvtSetSubState [concat 0 $value $value]
    if {[catch {set gvar(iracqEvtIdSubState) [seq_evtAttach \
	    $gvar(iracqDbAttrSubState) "!=" {iracqEvtSetSubState %D}]}]} {
	iracqHandleSrvErr "cannot attach database event to \
		$gvar(iracqDbAttrSubState)"
	return 0
    }
    iracqAddEvent $gvar(iracqEvtIdSubState)

    if {[catch {iracqAddEvent [seq_evtAttach \
	    $gvar(iracqDbAttrAlarm) "w" {iracqEvtAlarm %D}]}]} {
	iracqHandleSrvErr "cannot attach database event to \
		$gvar(iracqDbAttrAlarm)"
	return 0
    }

    iracqAttachActionHistoryEvent

    iracqAttachFileHistoryEvent

    set attrList {}
    
    # irace
    lappend attrList Version
    lappend attrList State
    lappend attrList SeqStatus
    lappend attrList SeqMode
    lappend attrList RspeedFactor
    lappend attrList RspeedAdd
    lappend attrList CldcStatus
    lappend attrList AcqStatus
    lappend attrList OpMode
    lappend attrList ReadoutMode
    lappend attrList DetCldcBoard
    lappend attrList DetClkFile
    lappend attrList SysVoltFile
    lappend attrList SysClkFile
    lappend attrList SysSeqFile
    lappend attrList CldcBoard
    lappend attrList NumAdc
    lappend attrList NumRm
    lappend attrList ConfigPath
    lappend attrList DetConfigFile
    lappend attrList AcqProcName
    lappend attrList RtdInt
    lappend attrList RtdHost
    lappend attrList RtdDisp
    lappend attrList RtdCmd

    # exposure
    lappend attrList ExpStatus
    lappend attrList DtStatus
    lappend attrList ExpTime
    lappend attrList ExpCountDown
    lappend attrList STARTX
    lappend attrList STARTY
    lappend attrList NX
    lappend attrList NY
    lappend attrList HwWindow
    lappend attrList SeqCont
    lappend attrList NoNcReset
    lappend attrList SeqBaseName
    lappend attrList ReqFileName
    lappend attrList NamingType
    lappend attrList DataCube
    lappend attrList ExtendedFits
    lappend attrList MINDIT
    lappend attrList DIT

    # detector
    lappend attrList CldcList
    lappend attrList ChipName
    lappend attrList ChipSizeX
    lappend attrList ChipSizeY
    
    foreach attr $attrList {
	if {[catch {set value \
		[seq_dbReadSymbolic $gvar(iracqDbAttr$attr)]}]} {
	    iracqHandleSrvErr "cannot attach database event to \
		    $gvar(iracqDbAttr$attr)"
	    return 0
	}
	iracqEvt$attr [list 0 $value $value]
	if {[catch {iracqAddEvent [seq_evtAttach \
		$gvar(iracqDbAttr$attr) "!=" "iracqEvt$attr %D"] $attr}]} {
	    iracqHandleSrvErr "cannot attach database event to \
		    $gvar(iracqDbAttr$attr)"
	    return 0
	}
    }

    # readout mode list
    if {[catch {iracqEvtRmDef}]} {
	    iracqHandleSrvErr "cannot attach database event to \
		    $gvar(iracqDbAttrRmDef)"
	    return 0
    }
    if {[catch {iracqAddEvent [seq_evtAttach \
	    $gvar(iracqDbAttrRmDef) "w" {iracqEvtRmDef}]}]} {
	iracqHandleSrvErr "cannot attach database event to \
		$gvar(iracqDbAttrRmDef)"
	return 0
    }

    # frame list
    if {![iracqAttachFrameEvent]} {
	return 0
    }

    # dynamic parameters
    if {![iracqAttachParamEvent]} {
	return 0
    }

    # adc-status
    if {[catch {set gvar(iracqSysNumAdc) \
	    [seq_dbReadSymbolic $gvar(iracqDbAttrNumAdc)]}]} {
	    iracqHandleSrvErr "cannot read database attribute \
		    $gvar(iracqDbAttrNumAdc)"
	    return 0
    }
    if {[catch {iracqEvtAdcStatus}]} {
	    iracqHandleSrvErr "cannot attach database event to \
		    $gvar(iracqDbAttrAdcStatus)"
	    return 0
    }
    if {[catch {iracqAddEvent [seq_evtAttach \
	    $gvar(iracqDbAttrAdcStatus) "w" {iracqEvtAdcStatus}]}]} {
	iracqHandleSrvErr "cannot attach database event to \
		$gvar(iracqDbAttrAdcStatus)"
	return 0
    }

    # clock voltages
    if {[catch {iracqEvtClkVolt}]} {
	    iracqHandleSrvErr "cannot attach database event to \
		    $gvar(iracqDbAttrClkVolt)"
	    return 0
    }
    if {[catch {set gvar(iracqClkVoltEvtId) [seq_evtAttach \
	    $gvar(iracqDbAttrClkVolt) "w" {iracqEvtClkVolt}]}]} {
	iracqHandleSrvErr "cannot attach database event to \
		$gvar(iracqDbAttrClkVolt)"
	return 0
    }
    iracqAddEvent $gvar(iracqClkVoltEvtId)

    # DC voltages
    if {[catch {iracqEvtDcVolt}]} {
	    iracqHandleSrvErr "cannot attach database event to \
		    $gvar(iracqDbAttrDcVolt)"
	    return 0
    }
    if {[catch {set gvar(iracqDcVoltEvtId) [seq_evtAttach \
	    $gvar(iracqDbAttrDcVolt) "w" {iracqEvtDcVolt}]}]} {
	iracqHandleSrvErr "cannot attach database event to \
		$gvar(iracqDbAttrDcVolt)"
	return 0
    }
    iracqAddEvent $gvar(iracqDcVoltEvtId)

    # create list for configuration update
    if {[catch {set gvar(iracqCfgListId) \
	    [seq_dbListCreate $gvar(iracqEnv) r]}]} {
	iracqHandleSrvErr "cannot create database list for \
		configuration update"
	return 0
    }

    seq_dbListAdd $gvar(iracqCfgListId) \
	    $gvar(iracqDbAttrAcqProcName) \
	    $gvar(iracqDbAttrHwWindow) \
	    $gvar(iracqDbAttrSeqMode)
    
    # align with default values
    iracqFileHistoryOnOff
    iracqActionHistoryOnOff

    set gvar(iracqSrvOpen) 1
    return 1
}

# attach action history
proc iracqAttachActionHistoryEvent {} {
    global gvar

    if {$gvar(iracqActionEvt) != ""} {
	return 1
    }

    if {[catch {set gvar(iracqActionEvt) [seq_evtAttach \
	    $gvar(iracqDbAttrActionLog) "w" {iracqEvtActionLog %D}]}]} {
	iracqHandleSrvErr "cannot attach database event to \
		$gvar(iracqDbAttrActionLog)"
	set gvar(iracqActionEvt) ""
	set gvar(iracqActionEvtEnabled) 0
	return 0
    }

    set gvar(iracqActionEvtEnabled) 1
    return 1
}

# detach action history
proc iracqDetachActionHistoryEvent {} {
    global gvar

    if {$gvar(iracqActionEvt) != ""} {
	seq_evtDetach $gvar(iracqActionEvt)
	set gvar(iracqActionEvt) ""
    }

    set gvar(iracqActionEvtEnabled) 0
    return 1
}

# attach file history
proc iracqAttachFileHistoryEvent {} {
    global gvar

    if {$gvar(iracqFileEvt) != ""} {
	return 1
    }

    if {[catch {set gvar(iracqFileEvt) [seq_evtAttach \
	    $gvar(iracqDbAttrFileHistory) "w" {iracqEvtFileHistory %D}]}]} {
	iracqHandleSrvErr "cannot attach database event to \
		$gvar(iracqDbAttrFileHistory)"
	set gvar(iracqFileEvt) ""
	set gvar(iracqFileEvtEnabled) 0
	return 0
    }

    set gvar(iracqFileEvtEnabled) 1
    return 1
}

# detach file history
proc iracqDetachFileHistoryEvent {} {
    global gvar

    if {$gvar(iracqFileEvt) != ""} {
	seq_evtDetach $gvar(iracqFileEvt)
	set gvar(iracqFileEvt) ""
	set gvar(iracqFileEvtEnabled) 0
    }

    set gvar(iracqFileEvtEnabled) 0
    return 1
}


# attach window event
proc iracqAttachWindowEvent {} {
    global gvar

    if {!$gvar(iracqCCS)} {
	return 1
    }
    
    # hw-window
    if {[catch {set value \
	    [seq_dbReadSymbolic $gvar(iracqDbAttrHwWindow)]}]} {
	iracqHandleSrvErr "cannot attach database event to \
		$gvar(iracqDbAttrHwWindow)"
	return 0
    }
    iracqEvtHwWindow [concat 0 $value $value]
    if {[catch {iracqAddEvent [seq_evtAttach \
	    $gvar(iracqDbAttrHwWindow) "!=" "iracqEvtHwWindow %D"]}]} {
	iracqHandleSrvErr "cannot attach database event to \
		$gvar(iracqDbAttrHwWindow)"
	return 0
    }
    
    return 1
}

# attach read-mode event
proc iracqAttachReadModeEvent {} {
    global gvar

    if {!$gvar(iracqCCS)} {
	return 1
    }

    if {[catch {iracqEvtRmDef}]} {
	    iracqHandleSrvErr "cannot attach database event to \
		    $gvar(iracqDbAttrRmDef)"
	    return 0
    }
    if {[catch {iracqAddEvent [seq_evtAttach \
	    $gvar(iracqDbAttrRmDef) "w" {iracqEvtRmDef}]}]} {
	iracqHandleSrvErr "cannot attach database event to \
		$gvar(iracqDbAttrRmDef)"
	return 0
    }
    
    if {[catch {set value \
	    [seq_dbReadSymbolic $gvar(iracqDbAttrReadoutMode)]}]} {
	iracqHandleSrvErr "cannot attach database event to \
		$gvar(iracqDbAttrReadoutMode)"
	return 0
    }
    iracqEvtReadoutMode [concat 0 $value $value]
    if {[catch {iracqAddEvent [seq_evtAttach \
	    $gvar(iracqDbAttrReadoutMode) "!=" "iracqEvtReadoutMode %D"]}]} {
	iracqHandleSrvErr "cannot attach database event to \
		$gvar(iracqDbAttrReadoutMode)"
	return 0
    }
    
    return 1
}

# attach parameter event
proc iracqAttachParamEvent {} {
    global gvar

    if {!$gvar(iracqCCS)} {
	return 1
    }

    if {$gvar(iracqParamAttached)} {
	return 1
    }

    if {[catch {set attrInfo [seq_dbGetAttrInfo \
	    $gvar(iracqDbAttrDynPars)]}]} {
	    iracqHandleSrvErr "cannot attach database event to \
		    $gvar(iracqDbAttrDynPars)"
	    return 0
    }
    set gvar(iracqDynParTableSize) [lindex $attrInfo 2]
    if {[catch {iracqEvtDynPars $gvar(iracqDbAttrDynPars)}]} {
	    iracqHandleSrvErr "cannot attach database event to \
		    $gvar(iracqDbAttrDynPars)"
	    return 0
    }
    if {[catch {iracqAddEvent [seq_evtAttach \
	    $gvar(iracqDbAttrDynPars) "w" {iracqEvtDynPars %A}]}]} {
	iracqHandleSrvErr "cannot attach database event to \
		$gvar(iracqDbAttrDynPars)"
	return 0
    }
    
    set gvar(iracqParamAttached) 1

    return 1
}

# attach frame event
proc iracqAttachFrameEvent {} {
    global gvar

    if {!$gvar(iracqCCS)} {
	return 1
    }

    if {[catch {iracqEvtFrames}]} {
	    iracqHandleSrvErr "cannot attach database event to \
		    $gvar(iracqDbAttrFrames)"
	    return 0
    }
    if {[catch {iracqAddEvent [seq_evtAttach \
	    $gvar(iracqDbAttrFrames) "w" {iracqEvtFrames}]}]} {
	iracqHandleSrvErr "cannot attach database event to \
		$gvar(iracqDbAttrFrames)"
	return 0
    }
    
    return 1
}

proc iracqEvtRtdInt {data} {
    global gvar

    set gvar(iracqRtdInt) [lindex $data 2]
}

proc iracqEvtRtdDisp {data} {
    global gvar

    set gvar(iracqRtdDisp) [lindex $data 2]
}

proc iracqEvtRtdCmd {data} {
    global gvar

    set gvar(iracqRtdCmd) [lindex $data 2]
    if {$gvar(iracqRtdCmd) == ""} {
	$gvar(iracqBcv).iracqStartRtd disable
    } else {
	$gvar(iracqBcv).iracqStartRtd enable
    }
}

proc iracqEvtRtdHost {data} {
    global gvar

    set gvar(iracqRtdHost) [lindex $data 2]
}

# handle update event
proc iracqEvtUpdate {data} {
    global gvar seq_ccsProcName
 
    set orgName [lindex $data 2]
    
    if {$orgName != "@$gvar(iracqEnv):$seq_ccsProcName"} {
	iracqGetConfig
    }
}

# handle alarm event
proc iracqEvtAlarm {data} {
    global gvar

    set newAlarm "[join [lindex $data 2]]"
    if {$newAlarm != ""} {
	iracqDialog1 "Alarm" "$newAlarm"
    }
}

# handle action log event
proc iracqEvtActionLog {data} {
    global gvar

    set newAction "[join [lindex $data 2]]"
    if {$newAction != ""} {
	iracqActionLog "$newAction\n"
    }
}

# handle file history event
proc iracqEvtFileHistory {data} {
    global gvar

    set newFile [lindex $data 2]
    
    if {$newFile != ""} {
	iracqFileHistory "$newFile"
    }
}

# handle sub-state event
proc iracqEvtSetSubState {data} {
    global gvar

    set gvar(iracqSrvSubState) [lindex $data 2]
}

# handle readout mode config event
proc iracqEvtRmDef {} {
    global gvar

    set numRm [seq_dbReadSymbolic $gvar(iracqDbAttrNumRm)]
    set rmTable [seq_dbReadSymbolic $gvar(iracqDbAttrRmDef)]

    for {set i 0} {$i<$numRm} {incr i 1} {
	set record [lindex $rmTable $i]
	set ele [lindex $record 0]
	set gvar(iracqDetRm$i) $ele
    }

    set gvar(iracqDetNumRm) $numRm

    # update readout mode options
    iracqRmUpdate
    
}

# handle frame configuration event
proc iracqEvtFrames {} {
    global gvar
    global iracqFrame iracqFrameGen iracqFrameStore iracqFrameDisp

    set frameTable [seq_dbReadSymbolic $gvar(iracqDbAttrFrames)]

    for {set i 0} {$i<32} {incr i 1} {
	set record [lindex $frameTable $i]
	set name [lindex $record 0]
	if {$name != ""} {
	    set generate [lindex $record 1]
	    set store [lindex $record 2]
	    set disp [lindex $record 3]
	    set iracqFrame($name) $name
	    set iracqFrameGen($name) $generate
	    set iracqFrameStore($name) $store
	    set iracqFrameDisp($name) $disp
	    set valid($name) 1
	    iracqAddFrame $gvar(iracqFrameWidget) $name
	}
    }

    if {[info exists iracqFrame]} {
	foreach ele [array names iracqFrame] {
	    if {$ele != ""} {
		if {![info exists valid($ele)]} {
		    iracqRemoveFrame $gvar(iracqFrameWidget) $ele
		}
	    }
	}
    }
}

# handle dynamic parameter event
proc iracqEvtDynPars {attr} {
    global gvar iracqParam 

    if {[info exists iracqParam]} {
	set oldList [lsort [array names iracqParam]]
	foreach ele $oldList {
	    set oldParam($ele) $iracqParam($ele)
	    set iracqParam($ele) "undefined"
	}
    }

    set paramTable [seq_dbReadSymbolic $attr]

    for {set i 0} {$i<$gvar(iracqDynParTableSize)} {incr i 1} {
	set record [lindex $paramTable $i]
	set name [lindex $record 0]
	if {$name != ""} {
	    set value [lindex $record 1]
	    set iracqParam($name) $value
	} else {
	    break
	}
    }

    if {[info exists oldList]} {
	set gvar(iracqNewParam) 0
	set newList [lsort [array names iracqParam]]
	foreach ele $newList {
	    if {[lsearch -exact $oldList $ele] == -1} {
		# new element added
		set gvar(iracqNewParam) 1
		break
	    }
	    if {$iracqParam($ele) == "undefined" && \
		    $oldParam($ele) != "undefined"} {
		# element has been marked as invalid
		set gvar(iracqNewParam) 1
		break
	    }
	    if {$iracqParam($ele) != "undefined" && \
		    $oldParam($ele) == "undefined"} {
		# element has been marked as valid again
		set gvar(iracqNewParam) 1
		break
	    }
	}
    } else {
	set gvar(iracqNewParam) 1
    }

    if {$gvar(iracqNewParam)} {
	iracqParamUpdate
    }

    if {[info exists iracqParam(DET.CHOP.ST)]} {
	set gvar(iracq_chopState) $iracqParam(DET.CHOP.ST)
    }
}

# handle adc-status event
proc iracqEvtAdcStatus {} {
    global gvar

    
    set adcStatus [seq_dbReadSymbolic $gvar(iracqDbAttrAdcStatus)]

    if {$gvar(iracqAdcSelNum) < 0} {
	if {$gvar(iracqSysNumAdc) > 0} {
	    iracqAdcSelect 0
	}
    }

    set record [lindex $adcStatus $gvar(iracqAdcSelNum)]
    set name [lindex $record 0]
    set addr [lindex $record 1]
    set header [lindex $record 2]
    set enable [lindex $record 3]
    set filter1 [lindex $record 4]
    set filter2 [lindex $record 5]
    set delay [lindex $record 6]

    if {$gvar(iracqAdcSelNum) < $gvar(iracqSysNumAdc) &&\
	    $gvar(iracqAdcSelNum) >= 0} {
	set gvar(iracqAdcFilter1Sel) $filter1
	set gvar(iracqAdcFilter2Sel) $filter2
	set gvar(iracqAdcEnableSel) $enable
	set gvar(iracqAdcDelaySel) $delay
	set gvar(iracqAdcHeaderSel) $header
    }
}

# handle clock voltages event
proc iracqEvtClkVolt {} {
    global gvar
    global iracqVset
    global cv
    global i

    if {$gvar(iracqSelClkV) == "CLK"} {
	$gvar(iracqVoltSelOption) deleteEntry 0 last  
    }

    set clkTable [seq_dbReadSymbolic $gvar(iracqDbAttrClkVolt)]
	
    for {set i 0} {$i<16} {incr i 1} {
	set record [lindex $clkTable $i]
	set iracqVset(ckLoName_$i) [lindex $record 0]
	set iracqVset(ckHiName_$i) [lindex $record 1]
	set iracqVset(ckLoVal_$i) [lindex $record 2]
	set iracqVset(ckHiVal_$i) [lindex $record 3]
	set iracqVset(ckLoDac_$i) [lindex $record 4] 
	set iracqVset(ckHiDac_$i) [lindex $record 5] 
	set iracqVset(ckLoTel_$i) [lindex $record 6] 
	set iracqVset(ckHiTel_$i) [lindex $record 7] 

	set value [lindex $record 8]
	iracqRound value 3
	set iracqVset(ckLoR1_$i) $value

	set value [lindex $record 9]
	iracqRound value 3
	set iracqVset(ckHiR1_$i) $value

	set value [lindex $record 10]
	iracqRound value 3
	set iracqVset(ckLoR2_$i) $value

	set value [lindex $record 11]
	iracqRound value 3
	set iracqVset(ckHiR2_$i) $value

	if {$gvar(iracqSelClkV) == "CLK"} {
	    if {$iracqVset(ckLoDac_$i) != -1} {
		$gvar(iracqVoltSelOption) addItem $iracqVset(ckLoName_$i)\
			"iracqSelClkLo $i"
	    }
	    if {$iracqVset(ckHiDac_$i) != -1} {
		$gvar(iracqVoltSelOption) addItem $iracqVset(ckHiName_$i)\
			"iracqSelClkHi $i"
	    }
	}
    }

    if {$gvar(iracqSelClkV) == "CLK"} {
	if {$gvar(iracqSetVoltId) != ""} {
	    iracqUpdateVoltId $gvar(iracqSetVoltId)
	    set value $iracqVset($gvar(iracqSetVoltId))
	    iracqRound value 3
	    set gvar(iracqSetVoltValue) $value
	    set gvar(iracqVoltT) [iracqVolt2Slide $gvar(iracqSetVoltValue)]
	    $cv.iracqVoltTuner setValue $gvar(iracqVoltT)
	    iracqUpdateTelValue
	}
	
	set gvar(iracqVsetupDone) 1
    }
}

# handle DC voltages event
proc iracqEvtDcVolt {} {
    global gvar
    global iracqVset
    global cv
    global i

    if {$gvar(iracqSelClkV) == "DC"} {
	$gvar(iracqVoltSelOption) deleteEntry 0 last  
    }

    set dcTable [seq_dbReadSymbolic $gvar(iracqDbAttrDcVolt)]
    
    for {set i 0} {$i<16} {incr i 1} {
	set record [lindex $dcTable $i]
	set iracqVset(dcName_$i) [lindex $record 0]
	set iracqVset(dcVal_$i) [lindex $record 1]
	set iracqVset(dcDac_$i) [lindex $record 2] 
	set iracqVset(dcTel_$i) [lindex $record 3] 

	set value [lindex $record 4]
	iracqRound value 3
	set iracqVset(dcR1_$i) $value

	set value [lindex $record 5]
	iracqRound value 3
	set iracqVset(dcR2_$i) $value

	if {$gvar(iracqSelClkV) == "DC"} {
	    if {$iracqVset(dcDac_$i) != -1} {
		$gvar(iracqVoltSelOption) addItem $iracqVset(dcName_$i)\
			"iracqSelDc $i"
	    }
	}
    }

    if {$gvar(iracqSelClkV) == "DC"} {
	if {$gvar(iracqSetVoltId) != ""} {
	    iracqUpdateVoltId $gvar(iracqSetVoltId)
	    set value $iracqVset($gvar(iracqSetVoltId))
	    iracqRound value 3
	    set gvar(iracqSetVoltValue) $value
	    set gvar(iracqVoltT) [iracqVolt2Slide $gvar(iracqSetVoltValue)]
	    $cv.iracqVoltTuner setValue $gvar(iracqVoltT)
	    iracqUpdateTelValue
	}

	set gvar(iracqVsetupDone) 1
    }
}

proc iracqEvtVersion {data} {
    global gvar

    set gvar(iracqVersion) [lindex $data 2]
}

proc iracqEvtState {data} {
    global gvar

    set gvar(iracqSrvState) [lindex $data 2]

    if {$gvar(iracqSrvState) == "OFF"} {
	iracqPlotStop $gvar(iracqPerfMon)
	set gvar(iracqPerfEnabled) 0
	set gvar(iracqSrvStateRst) 1
    } else {
	if {$gvar(iracqSrvStateRst)} {
	    set gvar(iracqSrvStateRst) 0
	    iracqFileChanged sysCfg \
		    "$gvar(iracqSysConfigFile)" 1
	    iracqFileChanged detCfg \
		    "$gvar(iracqDetConfigFile)" 1
	}
    }
}

proc iracqEvtSeqStatus {data} {
    global gvar

    set gvar(iracqSeqState) [lindex $data 2]
}

proc iracqEvtSeqMode {data} {
    global gvar

    set gvar(iracqSeqMode) [lindex $data 2]
}

proc iracqEvtCldcStatus {data} {
    global gvar

    set gvar(iracqCldcState) [lindex $data 2]
}

proc iracqEvtAcqStatus {data} {
    global gvar

    set gvar(iracqAcqState) [lindex $data 2]
}

proc iracqEvtExpStatus {data} {
    global gvar

    set gvar(iracqExpStatus) [lindex $data 2]
}

proc iracqEvtDtStatus {data} {
    global gvar

    iracqHandleDtStatus [lindex $data 2]
}

proc iracqEvtExpCountDown {data} {
    global gvar

    set value [lindex $data 2]

    set gvar(iracqExpCountdown) [iracqSec2Str $value]
}

proc iracqEvtExpTime {data} {
    global gvar

    set value [lindex $data 2]
    iracqRound value 3 "e"

    set gvar(iracqExpTime) [iracqSec2Str $value]
}

proc iracqEvtOpMode {data} {
    global gvar

    set gvar(iracqOpMode) [lindex $data 2]
}

proc iracqEvtReadoutMode {data} {
    global gvar

    set gvar(iracqRm) [lindex $data 2]
}

proc iracqEvtDetCldcBoard {data} {
    global gvar

    set gvar(iracqDetCldc) [lindex $data 2]
}

proc iracqEvtDetClkFile {data} {
    global gvar

    set gvar(iracqDetClk) [lindex $data 2]
}

proc iracqEvtSysVoltFile {data} {
    global gvar cv

    set gvar(iracqSysVoltage) [lindex $data 2]
    [$cv.iracqSysVoltage component uifEntry] xview end
    set gvar(iracqSysVoltageT) $gvar(iracqSysVoltage)
}

proc iracqEvtSysClkFile {data} {
    global gvar cv

    set gvar(iracqSysClk) [lindex $data 2]
    [$cv.iracqSysClk component uifEntry] xview end
    set gvar(iracqSysClkT) $gvar(iracqSysClk)
}

proc iracqEvtSysSeqFile {data} {
    global gvar cv

    set gvar(iracqSysSeq) [lindex $data 2]
    [$cv.iracqSysSeq component uifEntry] xview end
    set gvar(iracqSysSeqT) $gvar(iracqSysSeq)
}

proc iracqEvtCldcBoard {data} {
    global gvar

    set gvar(iracqSysCldc) [lindex $data 2]
}

proc iracqEvtNumAdc {data} {
    global gvar

    set gvar(iracqSysNumAdc) [lindex $data 2]

    iracqAdcCfg
}

proc iracqEvtNumRm {data} {
    global gvar

    set gvar(iracqDetNumRm) [lindex $data 2]
}

proc iracqEvtConfigPath {data} {
    global gvar

    set gvar(iracqCfgBase) [lindex $data 2]
    set gvar(iracqSysCfgBase) "$gvar(iracqCfgBase)/SYS";
    set gvar(iracqDetCfgBase) "$gvar(iracqCfgBase)/DET";
    set gvar(iracqSysVoltBase) "$gvar(iracqCfgBase)/VOLTAGE";
    set gvar(iracqSysClkBase) "$gvar(iracqCfgBase)/CLK";
    set gvar(iracqSysSeqBase) "$gvar(iracqCfgBase)/SEQ";
}

proc iracqEvtDetConfigFile {data} {
    global gvar

    set gvar(iracqDetConfigFile) [lindex $data 2]
    set gvar(iracqDetCfgBase) [file dirname $gvar(iracqDetConfigFile)]
    set tmp [file tail $gvar(iracqDetConfigFile)]
    set gvar(iracqDetConfigName) [file rootname $tmp]
}

proc iracqEvtSysConfigFile {data} {
    global gvar

    set gvar(iracqSysConfigFile) [lindex $data 2]
    set gvar(iracqSysCfgBase) [file dirname $gvar(iracqSysConfigFile)]
}

proc iracqEvtChipName {data} {
    global gvar

    set gvar(iracqDetName) [lindex $data 2]
}

proc iracqEvtHwWindow {data} {
    global gvar

    set gvar(iracqHwWin) [lindex $data 2]
}

proc iracqEvtSTARTX {data} {
    global gvar

    set gvar(iracqWinStartX) [lindex $data 2]
}

proc iracqEvtSTARTY {data} {
    global gvar

    set gvar(iracqWinStartY) [lindex $data 2]
}

proc iracqEvtNX {data} {
    global gvar

    set gvar(iracqWinNx) [lindex $data 2]
}

proc iracqEvtNY {data} {
    global gvar

    set gvar(iracqWinNy) [lindex $data 2]
}

proc iracqEvtChipSizeX {data} {
    global gvar

    set gvar(iracqWinNxMax) [lindex $data 2]
}

proc iracqEvtChipSizeY {data} {
    global gvar

    set gvar(iracqWinNyMax) [lindex $data 2]
}

proc iracqEvtAcqProcName {data} {
    global gvar

    set gvar(iracqSysAcqProc) [lindex $data 2]
    set gvar(iracqSysAcqProcT) $gvar(iracqSysAcqProc)
}

proc iracqEvtSeqCont {data} {
    global gvar

    set gvar(iracqSeqCont) [lindex $data 2]
}

proc iracqEvtNoNcReset {data} {
    global gvar

    set gvar(iracqNoNcReset) [lindex $data 2]
}

proc iracqEvtSeqBaseName {data} {
    global gvar

    set gvar(iracqSeqBaseName) [lindex $data 2]
}

proc iracqEvtReqFileName {data} {
    global gvar

    set gvar(iracqReqFileName) [lindex $data 2]
}

proc iracqEvtNamingType {data} {
    global gvar

    set gvar(iracqNamingType) [lindex $data 2]
}

proc iracqEvtDataCube {data} {
    global gvar

    set gvar(iracqDataCube) [lindex $data 2]
}

proc iracqEvtExtendedFits {data} {
    global gvar

    set gvar(iracqExtFits) [lindex $data 2]
}

proc iracqEvtRspeedFactor {data} {
    global gvar

    set gvar(iracqRspeedF) [lindex $data 2]
}

proc iracqEvtRspeedAdd {data} {
    global gvar

    set gvar(iracqRspeedA) [lindex $data 2]
}

proc iracqEvtMINDIT {data} {
    global gvar

    set value [lindex $data 2]

    iracqRound value 4 "e"

    set gvar(iracqMinDit) $value
}

proc iracqEvtDIT {data} {
    global gvar

    set value [lindex $data 2]

    iracqRound value 4 "e"

    set gvar(iracqDit) $value 
}

proc iracqEvtCldcList {data} {
    global gvar

    set gvar(iracqCldcList) [lindex $data 2]
    iracqUpdateCldcList 
}

#
# ___oOo___
