#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: iracqUifInit.tcl,v 1.113 2005/09/13 07:27:25 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# jstegmei  26/05/99  created
# jstegmei  14/10/99  added init routines for _uifClasses
# jstegmei  09/12/99  added iracqDisableWindowSetting function
# jstegmei  13/12/99  support multiple server instances
# jstegmei  11/07/00  added stack for entry widgets
# jstegmei  06/09/01  added select-background to result widget
# jstegmei  11/11/03  updated check for running CCS environment
# jstegmei  23/04/04  fixed bug when aborting file-manager
# jstegmei  15/07/04  enable action-history by default
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

# quit session
proc iracqQuitSession {} {
    global gvar

    bind . <Destroy> {}

    # close file manager
    if {[info exists gvar(iracqFileMgr)]} {
	if {$gvar(iracqFileMgr) != 0} {
	    set pid ""
	    catch {set pid [eval exec ps -p $gvar(iracqFileMgr)]}
	    if {$pid != ""} {
		catch [eval exec kill $gvar(iracqFileMgr)]
	    }
	    set gvar(iracqFileMgr) 0
	}
    }

    # stop monitoring
    iracqPerfStop

    # close server
    iracqCloseServer

    iracqVB "exiting..."
    if {$gvar(iracqCCS)} {
	seq_ccsExit
    }
    
    exit

    return 1
}

proc iracqPrintUsage {} {
    global gvar

    puts "\nusage: $gvar(iracqProcName) \[options\]\n"
    puts "options:"
    puts "   -host <host name>   - host name of the server"
    puts "                         (default: $gvar(iracqHost))"
    puts "   -load <module>      - load plug-in's in module"
    puts "                         (searches lib<module>*.tcl)"
    puts "   -noccs              - do not use CCS"
    puts "                         (default: use CCS)"
    puts "   -port <port number> - server port number (NOCCS only)"
    puts "                         (default: $gvar(iracqPort))"
    puts "   -server <name>      - server name"
    puts "                         (default: $gvar(iracqSrvName))"
    puts "   -dtt <instance>     - process name of data transfer task"
    puts "                         (default: $gvar(iracqDttName))"
    puts "   -verbose <level>    - set verbose level"
    puts "                         (default: $gvar(iracqVerbose))"
    puts "   -log <level>        - set log level"
    puts "                         (default: $gvar(iracqLogLevel))"
    puts "   -inst <instance>    - server instance"
    puts "                         (default: <$gvar(iracqInstance)>)"
    puts ""
    exit 1
}

proc iracqMessageBox {} {
    set w .iracqMessage

    if {[iracqPopUp $w]} {
	return
    }

    toplevel $w
    wm withdraw $w
    wm title $w "INFO..."
    label $w.msg -width 30 -relief ridge 
    pack $w.msg -padx 20 -pady 20
    update idletasks
    set x [expr {[winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
		     - [winfo vrootx [winfo parent $w]]}]
    set y [expr {[winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
		     - [winfo vrooty [winfo parent $w]]}]
    wm geom $w +$x+$y
    wm resizable $w 0 0
    wm deiconify $w

    set speed 100
    iracqRunInfo $w.msg "W E L C O M E  T O  I R A C E!" $speed
    iracqRunInfo $w.msg "Latest news: " $speed
    iracqRunInfo $w.msg \
	    "Double-Click on a filename in the file history\
	    will show the FITS-header in the result display...\
	    \nPress right mouse-button to go directly into the parameter\
	    setup panel...\nPress middle mouse-button on 'BROWSE' widgets\
	    to edit configuration files...\n" $speed
    

}

proc iracqCreateInit {} {
    global gvar

    wm withdraw .
    set w .iracqCreate
    
    toplevel $w
    wm withdraw $w
    wm title $w "Creating Panel..."
    $w configure -cursor watch
    label $w.msg -width 40 \
	    -text "Creating Infrared Acquisition Control-Panel..."
    pack $w.msg -padx 20 -pady 20

    # upper left corner
    update idletasks
    wm geom $w +0+0
    wm deiconify $w
}

proc iracqCreateExit {} {
    catch {destroy .iracqCreate}
    wm deiconify .
}

proc iracqCreateInfo {text} {
    if {[winfo exists .iracqCreate.msg]} {
	.iracqCreate.msg configure -text $text
	update idletasks
    }
}

# initialization procedure
proc iracqInitX {} {
    global gvar cv bcv argc argv argv0
    global env ckCCS
    global iracqFctList
    global iracqVset

    iracqCreateInfo "Initializing..."

    # lock panel
    iracqLockPanel

    # initialize global variables
    iracqInitVars

    # scan command line arguments
    set instance ""
    for {set i 0} {$i < $argc} {incr i 1} {
	if {[switch -glob -- [lindex $argv $i] {
	    "iracqCtrl.pan"  {
		set gvar(iracqEDITING) 1
	    }
	    "-host"  {
		catch {set gvar(iracqHost) [lindex $argv [expr {$i + 1}]]; \
			incr i 1}
	    }
	    "-load"  {
		catch {lappend gvar(iracqLoadList) \
			   [lindex $argv [expr {$i + 1}]]; \
			   incr i 1}
	    }
	    "-noccs"  {
		set gvar(iracqCCS) 0
	    }
	    "-xterm"  {
		set gvar(iracqInXterm) 1
	    }
	    "-inst"  {
		catch {set gvar(iracqInstance) [lindex $argv [expr {$i + 1}]]; \
			incr i 1}
	    }
	    "-port"  {
		catch {set gvar(iracqPort) [lindex $argv [expr {$i + 1}]]; \
			incr i 1}
	    }
	    "-server"  {
		catch {set gvar(iracqSrvName) [lindex $argv [expr {$i + 1}]]; \
			incr i 1}
	    }
	    "-dtt"  {
		catch {set gvar(iracqDttName) [lindex $argv [expr {$i + 1}]]; \
			incr i 1}
	    }
	    "-log"  {
		catch {set gvar(iracqLogLevel) [lindex $argv [expr {$i + 1}]]; \
			incr i 1}
	    }
	    "-cfg"  {
		catch {set value [lindex $argv [expr {$i + 1}]]; \
			incr i 1}
		if {[file extension $value] != ".cfg"} {
		    set value "$value\.cfg"
		}
		if {[string index $value 0] != "/"} {
		    set gvar(iracqSysConfigFile) \
			    "$gvar(iracqSysCfgBase)/$value"
		} else {
		    set gvar(iracqSysConfigFile) $value
		    set gvar(iracqSysCfgBase) \
			    [file dirname $gvar(iracqSysConfigFile)]
		}
	    }
	    "-verbose"  {
		catch {set gvar(iracqVerbose) [lindex $argv [expr {$i + 1}]]; \
			incr i 1}
	    }
	    "-usage"  {
		iracqPrintUsage
		exit 1
	    }
	}] == ""} {
	}
    }

    if {!$gvar(iracqCCS)} {
	if {$gvar(iracqSrvName) == "iracqServer"} {
	    # use NOCCS version of server
	    set gvar(iracqSrvName) "iracqServerNoCcs"
	}
	if {$gvar(iracqPort) == 0} {
	    # set default port
	    if {[catch {set gvar(iracqPort) $env(IRACQ_PORT)}]} {
		set gvar(iracqPort) 0
	    }
	}
	# change root window title
	set oldTitle [wm title .]
	wm title . "$oldTitle - $gvar(iracqHost)"
    }

    # save original server name (needed for later restart)
    set gvar(iracqSrvNameOrig) $gvar(iracqSrvName)

    if {$gvar(iracqCCS) && $gvar(iracqInstance) != ""} {

	# set database point to selected instance
	set gvar(iracqDbPoint) "<alias>iracq_$gvar(iracqInstance)"

	# re-intialize database variables
	iracqDbInitVars

	# set new server name to the instance registered to CCS
	set gvar(iracqSrvName) "$gvar(iracqSrvName)_$gvar(iracqInstance)"
	
	# change root window title
	set oldTitle [wm title .]
	wm title . "$oldTitle - Instance_$gvar(iracqInstance)"
    }

    # action history
    if {$gvar(iracqActionLogOn)} {
	$cv.iracqActionHistoryBut enable   
	$cv.iracqActionHistoryBut configure -cursor left_ptr  
    } else {
	$cv.iracqActionHistoryBut disable
    }

    bind [$cv.iracqGenFunction component uifEntry] <Return> {
	iracqCmdGen
	iracqGetConfig
    }

    bind [$cv.iracqAdcDelay component uifEntry] <Return> \
	    {iracqAdcSetDelay}

    bind [$cv.iracqAdcHeader component uifEntry] <Return> \
	    {iracqAdcSetHeader}

    bind [$cv.iracqSysCldc component uifEntry] <Return> {
	iracqCmdSetCldc $gvar(iracqSysCldcSetup)
	set gvar(iracqSysCldcSetup) ""
    }
    
    bind [$cv.iracqSysAcqProc component uifEntry] <Return> \
	    {iracqCmdLoadAcqProc $gvar(iracqSysAcqProc)}
    
    bind [$cv.iracqSysVoltage component uifEntry] <Return> \
	    {iracqCmdLoadVoltage $gvar(iracqSysVoltage)}
    
    bind [$cv.iracqSysClk component uifEntry] <Return> \
	    {iracqCmdLoadClk $gvar(iracqSysClk)}

    bind [$cv.iracqSysSeq component uifEntry] <Return> \
	    {iracqCmdLoadSeq $gvar(iracqSysSeq)}

    bind [$cv.iracqSeqBaseName component uifEntry] <Return> {
	iracqCmdSeqBaseName $gvar(iracqSeqBaseNameSetup)
	iracqStackPush iracqSeqBaseName $gvar(iracqSeqBaseNameSetup)
	set gvar(iracqSeqBaseNameSetup) ""
    }
    bind [$cv.iracqSeqBaseName component uifEntry] <Up> {
	set gvar(iracqSeqBaseNameSetup) [iracqStackDown iracqSeqBaseName]
    }
    bind [$cv.iracqSeqBaseName component uifEntry] <Down> {
	set gvar(iracqSeqBaseNameSetup) [iracqStackUp iracqSeqBaseName]
    }

    bind [$cv.iracqReqFileName component uifEntry] <Return> {
	iracqCmdReqFileName $gvar(iracqReqFileNameSetup)
	iracqStackPush iracqReqFileName $gvar(iracqReqFileNameSetup)
	set gvar(iracqReqFileNameSetup) ""
    }
    bind [$cv.iracqReqFileName component uifEntry] <Up> {
	set gvar(iracqReqFileNameSetup) [iracqStackDown iracqReqFileName]
    }
    bind [$cv.iracqReqFileName component uifEntry] <Down> {
	set gvar(iracqReqFileNameSetup) [iracqStackUp iracqReqFileName]
    }

    bind [$cv.iracqRspeedF component uifEntry] <Return> {
	iracqCmdRspeed 
    }

    bind [$cv.iracqRspeedA component uifEntry] <Return> {
	iracqCmdRspeed
    }

    bind [$cv.iracqDit component uifEntry] <Return> {
	iracqCmdSetDit $gvar(iracqDitSetup)
	set gvar(iracqDitSetup) ""
    }

    bind [$cv.iracqSetVoltValue component uifEntry] <Return> {
	iracqCmdVoltChan $gvar(iracqSetVoltChan) $gvar(iracqSetVoltValue) 1
	if {$gvar(iracqSetVoltId) != ""} {
	    set iracqVset($gvar(iracqSetVoltId)) $gvar(iracqSetVoltValue)
	}
	set gvar(iracqVoltT) [iracqVolt2Slide $gvar(iracqSetVoltValue)]
	$cv.iracqVoltTuner setValue $gvar(iracqVoltT)
	after 100 iracqUpdateTelValue
    }

    bind [$cv.iracqSetVoltChan component uifEntry] <Return> {
	iracqUpdateVoltChan $gvar(iracqSetVoltChan)
	set gvar(iracqVoltT) [iracqVolt2Slide $gvar(iracqSetVoltValue)]
	$cv.iracqVoltTuner setValue $gvar(iracqVoltT)
	iracqUpdateTelValue
    }

    bind $cv.iracqVoltTuner.uifScale  <Enter> {
	if {$gvar(iracqVsetEnabled)} {
	    iracqDetachUpdate
	    iracqVoltTunerEnable
	}
    }
    
    bind $cv.iracqVoltTuner.uifScale  <Leave> {
	if {$gvar(iracqVsetEnabled)} {
	    iracqVoltTunerDisable
	    iracqAttachUpdate
	}
    }

    # disable CLDC functions per default
    $cv.iracqDacEnable disable
    iracqDisableVoltageSetting
    iracqDisableCfg
    
    bind [$cv.iracqWinStartX component uifEntry] <Return> \
	    {iracqCmdSetWin}

    bind [$cv.iracqWinStartY component uifEntry] <Return> \
	    {iracqCmdSetWin}

    bind [$cv.iracqWinNx component uifEntry] <Return> \
	    {iracqCmdSetWin}

    bind [$cv.iracqWinNy component uifEntry] <Return> \
	    {iracqCmdSetWin}

    $cv.iracqSeqModeWdg  disableEntry 2

    # setup user area
    $cv.iracqUserArea.uifFrame configure -relief sunken -borderwidth 1

    if {$gvar(iracqEDITING)} {
	# no notebook due to bug in panel-editor
	set nb ""
    } else {
	# create notenbook for plug-ins
	set nb [iracqNoteBookCreate $cv.iracqUserArea.uifFrame]
    }

    foreach ele $gvar(iracqLoadList) {
	if {[iracqIsStdExt $ele]} {
	    lappend gvar(iracqClassList) $ele
	} else {
	    lappend gvar(iracqModList) $ele
	}
	    
    }

    # irace standard components
    iracqLoadIraceModule $nb

    # iracq extension classes
    iracqLoadStdExt $nb $gvar(iracqClassList)

    # application modules
    iracqLoadModList $nb $gvar(iracqModList)

    # initialize notebook
    iracqNoteBookInit $nb

    # set frames-widget
    set wnb [iracqNoteBookPath $nb Frames]
    if {$wnb != ""} {
	set gvar(iracqFrameWidget) \
		[$wnb.iracq.cv.iracqFrames-main component uifFrame]
    } else {
	set gvar(iracqFrameWidget) [frame .iracqFrames-main]
    }

    # set parameter-widget
    set wnb [iracqNoteBookPath $nb Param]
    if {$wnb != ""} {
	iracqInitParam iracq $wnb.iracq.cv
    } 

    # setup performance monitor
    if {[winfo exists $cv.iracqPerf]} {
	set gvar(iracqPerfMon) [iracqPlotWidget \
		[$cv.iracqPerf component uifFrame]]
    }

    # pack result box
    iracqTextBox [$cv.iracqDiagBox component uifFrame] \
	    80 15 iracqText
    $gvar(iracqText) configure -selectforeground white \
	    -selectbackground grey


    # pack action history box
    iracqTextBox [$cv.iracqActionHistory component uifFrame] \
	    80 5 iracqActionHistoryText

    # pack file history box
    iracqTextBox [$cv.iracqFileHistory component uifFrame] \
	    42 5 iracqFileHistoryText
    $gvar(iracqFileHistoryText) tag bind act <Double-Button-1> {
	set curX %x
        set curY %y
	set li [$gvar(iracqFileHistoryText) index "@$curX,$curY linestart"]
	set selFile [$gvar(iracqFileHistoryText) get $li "$li lineend"]
	if {$selFile != ""} {
	    if {[string index $selFile 0] == "/"} {
		iracqShowFits $selFile
	    } else {
		# get INS_ROOT environment
		if {[catch {set path $env(INS_ROOT)}]} {
		    iracqTkDialog1 "Error" \
			    "Environment variable \"INS_ROOT\" is not defined!"
		} else {
		    set fileName "$path/$gvar(iracqINS_USER)/DETDATA/$selFile"
		    iracqShowFits $fileName
		}
	    }
	}
    }
    #$gvar(iracqFileHistoryText) configure -cursor hand2


    # pack readout mode option button
    iracqOptionBox [$cv.iracqRmFrame component uifFrame] \
	    0 iracqRmOption ""
    set gvar(iracqRm) "undefined"

    # pack voltage option button
    iracqOptionBox [$cv.iracqVoltSelFrame component uifFrame] \
	    0 iracqVoltSelOption ""

    # bind mouse button 3 to parameter setup panel
    bind all <ButtonPress-3> {
	if {$gvar(iracqSrvState) != "OFF"} {
	    iracqParamSetup
	}
    }

    bind [$cv.iracqBrowseVolt component uifButton] <ButtonPress-2> {
	if {$gvar(iracqCfgEnabled) && $gvar(iracqVsetEnabled)} {
	    $cv.iracqBrowseVolt configure -label "Edit"
	}
    }
    bind [$cv.iracqBrowseVolt component uifButton] <ButtonRelease-2> {
	if {[lindex [$cv.iracqBrowseVolt configure -label] 4] == "Edit"} {
	    iracqEditFile $gvar(iracqSysVoltage)
	}
	$cv.iracqBrowseVolt configure -label "Browse"
    }
    bind [$cv.iracqBrowseVolt component uifButton] <Leave> {
	$cv.iracqBrowseVolt configure -label "Browse"
    }

    bind [$cv.iracqBrowseClk component uifButton] <ButtonPress-2> {
	if {$gvar(iracqCfgEnabled)} {
	    $cv.iracqBrowseClk configure -label "Edit"
	}
    }
    bind [$cv.iracqBrowseClk component uifButton] <ButtonRelease-2> {
	if {[lindex [$cv.iracqBrowseClk configure -label] 4] == "Edit"} {
	    iracqEditFile $gvar(iracqSysClk)
	}
	$cv.iracqBrowseClk configure -label "Browse"
    }
    bind [$cv.iracqBrowseClk component uifButton] <Leave> {
	$cv.iracqBrowseClk configure -label "Browse"
    }

    bind [$cv.iracqBrowseSeq component uifButton] <ButtonPress-2> {
	if {$gvar(iracqCfgEnabled)} {
	    $cv.iracqBrowseSeq configure -label "Edit"
	}
    }
    bind [$cv.iracqBrowseSeq component uifButton] <ButtonRelease-2> {
	if {[lindex [$cv.iracqBrowseSeq configure -label] 4] == "Edit"} {
	    iracqEditFile $gvar(iracqSysSeq)
	}
	$cv.iracqBrowseSeq configure -label "Browse"
    }
    bind [$cv.iracqBrowseSeq component uifButton] <Leave> {
	$cv.iracqBrowseSeq configure -label "Browse"
    }


    iracqCreateExit

    # open server connection
    if {![iracqOpenServer]} {
	iracqBusy 0 0
	iracqUnlockPanel
	return 0
    } else {
	# get configuration
	if {![iracqGetConfig]} {
	    iracqBusy 0 0
	    iracqUnlockPanel
	    return 0
	}

	set gvar(iracqSysCfgBase) "$gvar(iracqCfgBase)/SYS";
	set gvar(iracqDetCfgBase) "$gvar(iracqCfgBase)/DET";
        set gvar(iracqSysVoltBase) "$gvar(iracqCfgBase)/VOLTAGE";
        set gvar(iracqSysClkBase) "$gvar(iracqCfgBase)/CLK";
	set gvar(iracqSysSeqBase) "$gvar(iracqCfgBase)/SEQ";
	[$cv.iracqSysVoltage component uifEntry] xview end
	[$cv.iracqSysClk component uifEntry] xview end
	[$cv.iracqSysSeq component uifEntry] xview end
    }
 
    # reset busy state
    iracqBusy 0 0

    # release panel lock
    iracqUnlockPanel

    return 1
}

# exit routine called by panClosePanel
proc iracqExitX {} {
    global gvar cv env
    
    iracqQuitSession
}

proc iracqInitVars {} {
    global gvar cv bcv argc argv argv0
    global env ckCCS
    global seq_ccsEnvName EDITING

    set gvar(iracqCCS) $ckCCS
    set gvar(iracqVerbose) 0
    set gvar(iracqInXterm) 0
    set gvar(iracqActionLogOn) 1
    set gvar(iracqActionEvtEnabled) 0
    set gvar(iracqFileEvtEnabled) 0
    set gvar(iracqInstance) ""

    if {[info exists EDITING]} {
	set gvar(iracqEDITING) $EDITING
    } else {
	set gvar(iracqEDITING) 0
    }

    if {![info exists gvar(iracqDbPoint)]} {
	set gvar(iracqDbPoint) "<alias>iracq"
    }

    set gvar(iracqProcName) [file tail $argv0]

    # check, whether RTAP is running
    if {$gvar(iracqCCS)} {
	if {![catch {set RTAPENV $env(RTAPENV)}]} {
	    if {![catch \
		    "exec ps -ef | grep Scheduler | grep $env(USER) | grep -v grep" \
		      msg]} {
		if {$msg != ""} {
		} else {
		    set gvar(iracqCCS) 0
		}
	    } else {
		set gvar(iracqCCS) 0
	    }
	} else {
	    set gvar(iracqCCS) 0
	}
    }

    set gvar(iracqDttName) iracqDtt
    set gvar(iracqLogLevel) 0

    if {$gvar(iracqCCS)} {
	# set default server name
	if {![info exists gvar(iracqSrvName)]} {
	    set gvar(iracqSrvName) iracqServer
	}
	set gvar(iracqSrvNameOrig) $gvar(iracqSrvName)

	# set default port (no socket connection)
	set gvar(iracqPort) 0

	# set environment
	if {[catch {set gvar(iracqEnv) $env(RTAPENV)}]} {
	    set gvar(iracqEnv) $seq_ccsEnvName
	}

	# intialize database variables
	iracqDbInitVars

    } else {
	# set default server name
	set gvar(iracqSrvName) iracqServerNoCcs

	# set default port
	if {[catch {set gvar(iracqPort) $env(IRACQ_PORT)}]} {
	    set gvar(iracqPort) 0
	}
    }

    # initialize loadable module list
    set gvar(iracqModList) {}
    set gvar(iracqClassList) {}
    set gvar(iracqLoadList) {}

    # set default host
    if {[catch {set gvar(iracqHost) $env(HOST)}]} {
	set gvar(iracqHost) ""
    }
    

    # uif-classes parameter
    set gvar(iracqParamInstance) ""
    set gvar(iracqParamWidget) ""
    set gvar(iracqFramesInstance) ""
    set gvar(iracqFramesWidget) ""
    set gvar(iracqReadModeInstance) ""
    set gvar(iracqReadModeWidget) ""
    set gvar(iracqWindowInstance) ""
    set gvar(iracqWindowWidget) ""
    set gvar(iracqDitInstance) ""
    set gvar(iracqDitWidget) ""

    # general
    if {[catch {set gvar(iracqINS_USER) $env(INS_USER)}]} {
	set gvar(iracqINS_USER) "SYSTEM"
    }
    if {[catch {set gvar(iracqINS_ROOT) $env(INS_ROOT)}]} {
	set gvar(iracqINS_ROOT) ""
    }

    if {![info exists gvar(iracqDetIdx)]} {
	set gvar(iracqDetIdx) ""
    }
    set gvar(iracqText) ""
    set gvar(iracqString) ""
    set gvar(iracqEvtString) ""
    set gvar(iracqTimeout) 0
    set gvar(iracqTimer) 0
    set gvar(iracqTimerMax) 0
    set gvar(iracqAborted) 0
    set gvar(iracqBusyState) 0 
    set gvar(iracqActionHistory) 1
    set gvar(iracqFileHistory) 1
    set gvar(iracqCv) $cv
    set gvar(iracqBcv) $bcv
    set gvar(iracqDefaultTimeout) 10
    set gvar(iracqSrvOpen) 0
    set gvar(iracqEvtOpen) 0
    set gvar(iracqCmdFd) 0
    set gvar(iracqVersion) ""
    set gvar(iracqGenFunction) ""
    set gvar(iracqEvtPort) 0
    set gvar(iracqIoStream) -1
    set gvar(iracqEvtStream) -1
    set gvar(iracqCmdFd) -1
    set gvar(iracqUpdateEvt) 1
    set gvar(iracqParamAttached) 0
    set gvar(iracqMaxParam) 64

    # statistics
    set gvar(iracqDtRecv) 0
    set gvar(iracqDtStore) 0
    set gvar(iracqPerf) 0.0
    set gvar(iracqPerfEnabled) 0

    # sequencer
    set gvar(iracqSeqState) ""
    set gvar(iracqSeqMode) 0
    set gvar(iracqRspeedF) 1
    set gvar(iracqRspeedFSetup) ""
    set gvar(iracqRspeedA) 0 
    set gvar(iracqRspeedASetup) "" 
    set gvar(iracqSysClk) ""
    set gvar(iracqSysClkT) ""
    set gvar(iracqSysSeq) ""
    set gvar(iracqSysSeqT) ""
    set gvar(iracqSeqCont) 0
    set gvar(iracqNoNcReset) 0

    # cldc
    set gvar(iracqCldcList) ""
    set gvar(iracqCldcState) ""
    set gvar(iracqSetVoltChan) ""
    set gvar(iracqSetVoltValue) ""
    set gvar(iracqSetVoltValueStore) ""
    set gvar(iracqSetTelChan1) ""
    set gvar(iracqSetTelChan2) ""
    set gvar(iracqSetTelVal1) ""
    set gvar(iracqSetTelVal2) ""
    set gvar(iracqSetVoltR1) -10.0
    set gvar(iracqSetVoltR2) 10.0
    set gvar(iracqSetVoltId) ""
    set gvar(iracqVsetupDone) 0
    set gvar(iracqVoltT) 2048
    set gvar(iracqSysCldc) 0
    set gvar(iracqSysCldcSetup) ""
    set gvar(iracqSysVoltage) ""
    set gvar(iracqSysVoltageT) ""
    set gvar(iracqNewParam) 0

    # readout mode
    set gvar(iracqAcqState) ""
    set gvar(iracqSysAcqProc) ""
    set gvar(iracqSysAcqProcT) ""
    set gvar(iracqRmOption) ""

    # adc 
    set gvar(iracqSysNumAdc) 0
    set gvar(iracqSysNumAdcT) 0
    set gvar(iracqAdcFilter1Sel) 0
    set gvar(iracqAdcFilter2Sel) 0
    set gvar(iracqAdcEnableSel) 0
    set gvar(iracqAdcDelaySel) 0
    set gvar(iracqAdcDelaySelSetup) ""
    set gvar(iracqAdcHeaderSel) 0
    set gvar(iracqAdcHeaderSelSetup) ""
    set gvar(iracqAdcSel) None
    set gvar(iracqAdcSelNum) -1

    # common parameters
    set gvar(iracqMinDit) 0.0
    set gvar(iracqDit) 0.0

    # window parameters
    set gvar(iracqWinStartX) 1
    set gvar(iracqWinStartXSetup) ""
    set gvar(iracqWinStartY) 1
    set gvar(iracqWinStartYSetup) ""
    set gvar(iracqWinNxMax) 1024
    set gvar(iracqWinNyMax) 1024
    set gvar(iracqWinNx) $gvar(iracqWinNxMax)
    set gvar(iracqWinNxSetup) ""
    set gvar(iracqWinNy) $gvar(iracqWinNyMax)
    set gvar(iracqWinNySetup) ""
    set gvar(iracqHwWin) 0

    # system
    set gvar(iracqOpMode) ""
    set gvar(iracqSrvState) ""
    set gvar(iracqSrvStateRst) 1
    set gvar(iracqSrvSubState) ""
    set gvar(iracqExtFits) 0


    # RTD parameters
    set gvar(iracqRtdInt) 0
    if {[catch {set gvar(iracqRtdHost) $env(HOST)}]} {
	set gvar(iracqRtdHost) ""
    }
    if {[catch {set gvar(iracqRtdDisp) $env(DISPLAY)}]} {
	set gvar(iracqRtdDisp) ""
    }
    set gvar(iracqRtdCmd) "rtd"

    # exposure parameters
    set gvar(iracqSeqBaseName) ""
    set gvar(iracqSeqBaseNameSetup) ""
    set gvar(iracqReqFileName) ""
    set gvar(iracqReqFileNameSetup) ""
    set gvar(iracqExpTime) [iracqSec2Str 0.0]
    set gvar(iracqExpCountdown) [iracqSec2Str 0.0]
    set gvar(iracqExpStatus) "INACTIVE"
    set gvar(iracqDtStatus) 0
    set gvar(iracqDataCube) 0
    set gvar(iracqNamingType) 1

    # configuration files
    if {[catch {set gvar(iracqCfgBase) \
	    "$env(IRACE_ROOT)"}]} {
	if {$gvar(iracqINS_ROOT) == ""} {
	    set gvar(iracqCfgBase) "[pwd]"
	    set gvar(iracqSysCfgBase) $gvar(iracqCfgBase);
	    set gvar(iracqDetCfgBase) $gvar(iracqCfgBase);
	    set gvar(iracqSysVoltBase) $gvar(iracqCfgBase);
	    set gvar(iracqSysClkBase) $gvar(iracqCfgBase);
	    set gvar(iracqSysSeqBase) $gvar(iracqCfgBase);
	} else {
	    set gvar(iracqCfgBase) "$gvar(iracqINS_ROOT)/$gvar(iracqINS_USER)/MISC/IRACE"
	    set gvar(iracqSysCfgBase) "$gvar(iracqCfgBase)/SYS";
	    set gvar(iracqDetCfgBase) "$gvar(iracqCfgBase)/DET";
	    set gvar(iracqSysVoltBase) "$gvar(iracqCfgBase)/VOLTAGE";
	    set gvar(iracqSysClkBase) "$gvar(iracqCfgBase)/CLK";
	    set gvar(iracqSysSeqBase) "$gvar(iracqCfgBase)/SEQ";
	}
    } else {
	set gvar(iracqSysCfgBase) "$gvar(iracqCfgBase)/SYS";
	set gvar(iracqDetCfgBase) "$gvar(iracqCfgBase)/DET";
	set gvar(iracqSysVoltBase) "$gvar(iracqCfgBase)/VOLTAGE";
	set gvar(iracqSysClkBase) "$gvar(iracqCfgBase)/CLK";
	set gvar(iracqSysSeqBase) "$gvar(iracqCfgBase)/SEQ";
    }
    set gvar(iracqSysConfigFile) "$gvar(iracqSysCfgBase)/sys.cfg"
    set gvar(iracqDetConfigFile) ""
    set gvar(iracqDetConfigName) ""
    set gvar(iracqDetName) ""

    # data-plugin
    set gvar(iracq_flatfield) ""
    set gvar(iracq_flatfieldDev) 0
    set gvar(iracq_badpix) ""
    set gvar(iracq_badpixDev) 0
    if {[catch {set gvar(iracq_flatfieldBase) \
	    "$env(INS_ROOT)/$gvar(iracqINS_USER)/DETDATA"}]} {
	set gvar(iracq_flatfieldBase) "[pwd]"
    }
    if {[catch {set gvar(iracq_badpixBase) \
	    "$env(INS_ROOT)/$gvar(iracqINS_USER)/DETDATA"}]} {
	set gvar(iracq_badpixBase) "[pwd]"
    }
    if {[catch {set gvar(iracqFitsBase) \
	    "$env(INS_ROOT)/$gvar(iracqINS_USER)/DETDATA"}]} {
	set gvar(iracqFitsBase) "[pwd]"
    }

}

proc iracqDisableWindowSetting {} {
    global gvar cv

    $cv.iracqWinStartX disable
    $cv.iracqWinStartY disable
    $cv.iracqWinNx disable
    $cv.iracqWinNy disable
    destroy $cv.iracqWinStartX \
	    $cv.iracqWinStartY \
	    $cv.iracqWinNx \
	    $cv.iracqWinNy \
	    $cv.iracqWinStartXSet \
	    $cv.iracqWinStartYSet \
	    $cv.iracqWinNxSet \
	    $cv.iracqWinNySet \
	    $cv.iracqWinRtd \
	    $cv.iracqFullFrame \
	    $cv.iracqHwWin
}

proc iracqEnableCfg {} {
    global gvar cv

    set gvar(iracqCfgEnabled) 1
    $cv.iracqSysAcqProc enable
    $cv.iracqSysClk enable
    $cv.iracqSysSeq enable
    $cv.iracqBrowseClk enable
    $cv.iracqBrowseSeq enable

    if {$gvar(iracqVsetEnabled)} {
	$cv.iracqSysCldc enable
	$cv.iracqSysVoltage enable
	$cv.iracqBrowseVolt enable
    }
}

proc iracqDisableCfg {} {
    global gvar cv

    set gvar(iracqCfgEnabled) 0
    $cv.iracqSysAcqProc disable
    $cv.iracqSysClk disable
    $cv.iracqSysSeq disable
    $cv.iracqBrowseClk disable
    $cv.iracqBrowseSeq disable

    $cv.iracqSysCldc disable
    $cv.iracqSysVoltage disable
    $cv.iracqBrowseVolt disable
}

proc iracqCfgSetting {} {
    global gvar cv

    if {$gvar(iracqCfgEnabled)} {
	iracqEnableCfg
    } else {
	iracqDisableCfg
    }
}


#
# ___oOo___
