#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: iracqUifIoCcs.tcl,v 1.113 2005/09/13 07:27:25 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# jstegmei  18/06/99  created
# jstgemei  15/12/99  added iracqWaitExit function
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#


# handle server error
proc iracqHandleSrvErrCcs {errString} {
    global gvar

    seq_errAdd $errString "iracqUif"
    seq_errDisplay

    return 0
}

# open server
proc iracqOpenServerCcs {} {
    global gvar env seq_ccsEnvName

    if {$gvar(iracqSrvOpen)} {
	return 1
    }

    # initialize database events
    iracqVB "attaching to database..."
    if {![iracqEvtInit]} {
	return 0
    }

    # check server
    iracqVB "checking server..."
    set reply [panSendCommand $gvar(iracqEnv) $gvar(iracqSrvName) \
	    "VERSION" "CHECK_ALL -noErrDisplay" 10000 ""]
    
    if {$reply == "FAILURE"} {
	iracqVB "server not alive..."
	return 0
    }

    iracqVB "server is: $gvar(iracqSrvName)/$gvar(iracqEnv)"

    seq_ccsAsyncInput open

    return 1
}

# close server
proc iracqCloseServerCcs {} {
    global gvar

    if {$gvar(iracqSrvOpen)} {
	seq_ccsAsyncInput close

	# detach events
	iracqVB "detaching from database events..."
	iracqDetachEvents
	
	set gvar(iracqSrvOpen) 0
    }

    return 1
}

# abort function
proc iracqAbortCcs {} {
    global gvar

    set gvar(iracqAborted) 1

    # abort function
    set reply [panSendCommand $gvar(iracqEnv) $gvar(iracqSrvName) \
	    "BREAK" CHECK_ALL 5000 ""]

}

# busy function
proc iracqBusyCcs {busy doRes} {
    global gvar

    if {$busy} {
	#set gvar(iracqBusyState) 1
	if {$doRes == 1} {
	    $gvar(iracqCv) config -cursor watch
	    if {[winfo exists $gvar(iracqBcv).iracqAbortButton]} {
		$gvar(iracqBcv).iracqAbortButton enable
	    }

	    # disable function buttons
	}
    } else {
	$gvar(iracqCv) config -cursor left_ptr
	if {[winfo exists $gvar(iracqBcv).iracqAbortButton]} {
	    $gvar(iracqBcv).iracqAbortButton disable
	}

	# enable function buttons

	#set gvar(iracqBusyState) 0
    }
}

# call irace function via i/o-stream
proc iracqFunctionCcs {func doRes resStr {show 0}} {

    upvar $resStr diagString
    global gvar 

    if {$gvar(iracqSrvState) == "OFF"} {
	if {$doRes == 1} {
	    iracqTkDialog1 "INFO" "The server is not running!"
	    return 0
	} else {
	    return 0
	}
    }

    if {$gvar(iracqText) == ""} {
	set doRes 0
    }

    if {$gvar(iracqBusyState)} {
	if {$doRes == 1} {
	    iracqTkDialog1 "Busy" "The server is still busy!"
	    return 0
	} else {
	    return 0
	}
    }

    set gvar(iracqAborted) 0
    iracqBusy 1 $doRes

    # send command
    if {[catch \
	    {seq_msgSendCommand $gvar(iracqEnv) $gvar(iracqSrvName) \
	    "IRACE" $func} \
	    cmdHandle]} {
	seq_errDisplay
	iracqBusy 0 $doRes
	return 0
    }

    # wait for result
    set lastReply 0
    while {$lastReply == 0} {
	if {[catch {seq_msgRecvReply $cmdHandle errorNumber \
		lastReply} reply]} {
	    seq_errDisplay
	    iracqBusy 0 $doRes
	    return 0
	}
    }

    if {$gvar(iracqAborted)} {
	set diagString "aborted\n"
    } else {
	set diagString "$reply\n"
    }

    scan $func "%s" cmdString
    
    # evaluate result
    if {$doRes && $show} {
	# enable result box for insertion
	$gvar(iracqText) configure -state normal
	
	# get result-text index
	set i1 [$gvar(iracqText) index end]

	# append result to result-text
	$gvar(iracqText) insert end "\n$cmdString:\n$diagString"
	$gvar(iracqText) see end 

	# get result-text index
	set i2 [$gvar(iracqText) index end]
	
	# disable insertion 
	$gvar(iracqText) configure -state disabled
    }

    if {$errorNumber != 0} {
	if {$doRes} {
	    if {$show} {
		$gvar(iracqText) tag add err $i1 $i2
	    }
	    if {$gvar(iracqAborted)} {
		iracqTkDialog1 "Error Result" "$cmdString: aborted"
	    } else {
		seq_errDisplay
	    }
	}
	iracqBusy 0 $doRes
	return 0
    } 

    if {$doRes && $show} {
	$gvar(iracqText) tag remove err $i1 $i2
    }

    iracqBusy 0 $doRes
    
    return 1
}

# add event-id to event list
proc iracqAddEvent {eventId {attr ""}} {
    global gvar 

    lappend gvar(iracqEvents) $eventId
    if {$attr != ""} {
	set gvar(iracqEvent$attr) $eventId
    }
}

# detach all database events in event list
proc iracqDetachEvents {} {
    global gvar 

    foreach event $gvar(iracqEvents) {seq_evtDetach $event}
    set gvar(iracqEvents) {}
}

# attach event update
proc iracqAttachUpdateCcs {} {
    global gvar 

    if {$gvar(iracqSrvOpen)} {
	set gvar(iracqUpdateEvt) 1
	seq_evtSingleEnable $gvar(iracqEvtIdSubState)
	seq_evtSingleEnable $gvar(iracqEvtIdUpdate)
	set gvar(iracqSrvSubState) \
		[seq_dbReadSymbolic $gvar(iracqDbAttrSubState)]
    }
}

# detach event update
proc iracqDetachUpdateCcs {} {
    global gvar 

    if {$gvar(iracqSrvOpen)} {
	set gvar(iracqUpdateEvt) 0
	seq_evtSingleDisable $gvar(iracqEvtIdSubState)
	seq_evtSingleDisable $gvar(iracqEvtIdUpdate)
    }
}

# send CCS command
proc iracqSendCmd {cmd param {errDisp 1}} {
    global gvar 

    if {!$gvar(iracqCCS)} {
	iracqTkDialog1 "Error" \
		"This function is available only in CCS-compliant version!"
	return 0
    }

    # send command
    if {[catch \
	    {seq_msgSendCommand $gvar(iracqEnv) $gvar(iracqSrvName) \
	    $cmd $param} \
	    cmdHandle]} {
	if {$errDisp} {
	    seq_errDisplay
	}
	return 0
    }

    # wait for result
    set lastReply 0
    while {$lastReply == 0} {
	if {[catch {seq_msgRecvReply $cmdHandle errorNumber \
		lastReply} reply]} {
	    if {$errDisp} {
		seq_errDisplay
	    }
	    return 0
	}
    }

    if {$errorNumber != 0 && $errDisp} {
	seq_errDisplay
	return 0
    }

    return 1
}

proc iracqWaitExit {process {timeout 5}} {
    set downCounter [expr {$timeout * 5}]
    
    while {$downCounter > 0} {
	if {[catch {set cmdId \
		[seq_msgSendCommand "" $process PING ""]}]} {
	    catch {seq_msgFlush all -nowait}
	    return 1
	}
	if {[catch {set reply \
		[seq_msgRecvReply -100 $cmdId errorNumber -last]}]} {
	    catch {seq_msgFlush all -nowait}
	}
	
	set downCounter [expr {$downCounter - 1}]
	after 200
    }
    
    return 0
}

#
# ___oOo___
