#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: iracqUifLib.tcl,v 1.113 2005/09/13 07:27:25 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# jstegmei  20/05/99  created
# jstegmei  08/12/99  added restart server function
# jstegmei  05/01/99  resolve also -xterm, -v option when restarting 
#                     the server
# jstegmei  21/06/00  use panSelectFile instead of UifFileSelBox
# jstegmei  28/06/00  added file manager
# jstegmei  11/07/00  added stack for entry widgets
# jstegmei  24/07/01  added file history enable/disable feature
# jstegmei  13/02/03  flexible rounding routine
# jstegmei  29/08/03  fixed floating point error - VLTSW20030362
# jstegmei  23/04/04  fixed bug when aborting file-manager
# jstegmei  29/06/04  increased maximum FITS-header size
# jstegmei  15/07/04  enable action-history by default
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#


# lock whole panel
proc iracqLockPanel {} {
    global gvar cv
    
    set gvar(iracqPanLocked) 1
 
}

# release panel lock
proc iracqUnlockPanel {} {
    global gvar cv

    set gvar(iracqPanLocked) 0

}

# verbose output
proc iracqVB {data} {
    global gvar
    
    if {![info exists gvar(iracqVerbose)]} {
	set vb 1
    } else {
	set vb $gvar(iracqVerbose)
    }

    if {![info exists gvar(iracqProcName)]} {
	set procName "iracq"
    } else {
	set procName $gvar(iracqProcName)
    }

    if {$vb} {
	puts "$procName: $data"
    }
}


# scan 
proc iracqScanString {istr} {
    set i1 [expr {1 + [string first ":" $istr]}]
    set ele [string trim [string range $istr $i1 end]]
}

# get line from string
proc iracqSgets {istr} {
    upvar $istr strLoc

    set str $strLoc

    set i1 [string first "\n" $str]
    if {$i1 == -1} {
	return -1
    }
    set ele [string trim [string range $str 0 $i1]]
    set str [string range $str [expr {$i1 + 1}] end]

    set strLoc $str
    return $ele
}

proc iracqSelectText {w x y stat} {
    if {$stat} {
	set i1 [$w index "@$x,$y linestart"]
	set i2 [$w index "@$x,$y lineend"]
	$w tag add high $i1 $i2
    } else {
	$w tag remove high 1.0 end
    }
}

# text result box
proc iracqTextBox {w width height name} {
    global gvar 

    frame $w.box 
    pack $w.box -side top -fill both -expand 1

    text $w.text -relief sunken -foreground black -background white -bd 2 \
	    -cursor left_ptr \
	    -width $width -height $height \
	    -selectforeground black \
	    -selectbackground white \
	    -selectborderwidth 0 \
	    -wrap none -state disabled \
	    -xscrollcommand "$w.scrollx set" \
	    -yscrollcommand "$w.scrolly set" 


    set gvar($name) $w.text

    $w.text tag configure err -foreground red3
    $w.text tag configure high -foreground white -background black
    $w.text tag bind act1 <Enter> \
	    "$w.text configure -cursor hand2; \
	    iracqSelectText $w.text %x %y 1"
    $w.text tag bind act1 <Leave> \
	    "$w.text configure -cursor left_ptr; \
	    iracqSelectText $w.text %x %y 0"
    $w.text tag bind act2 <Enter> \
	    "$w.text configure -cursor hand2; \
	    iracqSelectText $w.text %x %y 1"
    $w.text tag bind act2 <Leave> \
	    "$w.text configure -cursor left_ptr; \
	    iracqSelectText $w.text %x %y 0"

    scrollbar $w.scrolly -command "$w.text yview" \
	    -orient vertical -cursor left_ptr
    
    scrollbar $w.scrollx -command "$w.text xview" \
	    -orient horizontal -cursor left_ptr
   
    pack $w.scrollx -side bottom -fill x -in $w.box
    pack $w.scrolly -side right -fill y -in $w.box
    pack $w.text -side left -fill both -expand 1 -in $w.box
}

# option box workaround
proc iracqOptionBox {w length name var} {
    global gvar 

    if {$var != ""} {
	UifOptionButton $w.option -variable gvar($var) -length $length
    } else {
	UifOptionButton $w.option 
    }
    pack $w.option  -side top -fill both -expand 1

    set gvar($name) $w.option
}

# clear file history
proc iracqClearFileHistory {} {
    global gvar

    # clear file history
    $gvar(iracqFileHistoryText) configure -state normal
    $gvar(iracqFileHistoryText) delete 1.0 end
    $gvar(iracqFileHistoryText) configure -state disabled
}

# clear action history
proc iracqClearActionHistory {} {
    global gvar
    
    if {[winfo exists $gvar(iracqActionHistoryText)]} {
	# clear action history
	$gvar(iracqActionHistoryText) configure -state normal
	$gvar(iracqActionHistoryText) delete 1.0 end
	$gvar(iracqActionHistoryText) configure -state disabled
    }
}

# clear diagnostic result display
proc iracqClearResult {} {
    global gvar

    # clear diagnostic result disply
    $gvar(iracqText) configure -state normal
    $gvar(iracqText) delete 1.0 end
    $gvar(iracqText) configure -state disabled

    # clear action history
    iracqClearActionHistory
}

proc iracqFileHistoryOnOff {} {
    global gvar

    if {$gvar(iracqCCS)} {
	if {$gvar(iracqFileHistory)} {
	    if {$gvar(iracqFileEvtEnabled) == 0} {
		seq_evtSingleEnable $gvar(iracqFileEvt)
		seq_evtSingleEnable $gvar(iracqEventDtStatus)
		set gvar(iracqFileEvtEnabled) 1
	    }
	} else {
	    if {$gvar(iracqFileEvtEnabled) == 1} {
		seq_evtSingleDisable $gvar(iracqFileEvt)
		seq_evtSingleDisable $gvar(iracqEventDtStatus)
		set gvar(iracqFileEvtEnabled) 0
	    }
	}
    }
}

proc iracqActionHistoryOnOff {} {
    global gvar

    if {$gvar(iracqCCS)} {
	if {$gvar(iracqActionHistory)} {
	    if {$gvar(iracqActionEvtEnabled) == 0} {
		seq_evtSingleEnable $gvar(iracqActionEvt)
		set gvar(iracqActionEvtEnabled) 1
	    }
	} else {
	    if {$gvar(iracqActionEvtEnabled) == 1} {
		seq_evtSingleDisable $gvar(iracqActionEvt)
		set gvar(iracqActionEvtEnabled) 0
	    }
	}
    }
}

# ad to file history
proc iracqFileHistory {outputString} {
    global gvar 
    global iracqCurTag

    if {$gvar(iracqFileHistory)} {
	# enable verbose box for insertion
	$gvar(iracqFileHistoryText) configure -state normal
	
	# append result to diagText
	if {![info exists iracqCurTag]} {
	    set iracqCurTag 0
	}
	if {$iracqCurTag == 0} {
	    set iracqCurTag 1
	    $gvar(iracqFileHistoryText) insert end \
		    "$outputString" {act act1} "\n" {}
	} else {
	    set iracqCurTag 0
	    $gvar(iracqFileHistoryText) insert end \
		    "$outputString" {act act2} "\n" {}
	}
	
	$gvar(iracqFileHistoryText) see end 
	
	# disable insertion 
	$gvar(iracqFileHistoryText) configure -state disabled
    }
}

# add to action history
proc iracqActionLog {outputString} {
    global gvar 

    if {$gvar(iracqActionHistory)} {
	if {[winfo exists $gvar(iracqActionHistoryText)]} {
	    set w $gvar(iracqActionHistoryText)
	} else {
	    set w $gvar(iracqText)
	}

	# enable verbose box for insertion
	$w configure -state normal
	
	# append result to diagText
	$w insert end "$outputString"
	$w see end 
	
	# disable insertion 
	$w configure -state disabled
    }  
}

# OK dialog 
proc iracqDialog1 {text msg} {
    set w .iracqDialog

    iracqDialog $w $text $msg warning 0 "OK"
}

# (OK, Dismiss) dialog
proc iracqDialog2 {text msg} {
    set w .iracqDialog

    if {[iracqDialog $w $text $msg warning 0 "OK" "Dismiss"]} {
	return 0
    }
    
    return 1
}

# OK dialog 
proc iracqTkDialog1 {text msg} {
    set w .iracqDialog

    tk_dialog $w $text $msg warning 0 "OK"
}

# (OK, Dismiss) dialog
proc iracqTkDialog2 {text msg} {
    set w .iracqDialog

    if {[tk_dialog $w $text $msg warning 0 "OK" "Dismiss"]} {
	return 0
    }
    
    return 1
}

# show version
proc iracqShowVersion {} {
    global gvar

    if {$gvar(iracqCCS)} {
	set res [tk_dialog .version "Version" \
		"Infrared Acquisition Module $gvar(iracqVersion)" \
		"" 0 "OK" "More"]
    } else {
	set res [tk_dialog .version "Version" \
		"Infrared Acquisition Module $gvar(iracqVersion) (NOCCS)" \
		"" 0 "OK" "More"]
    }
    
    if {$res == 1} {
	#iracqMessageBox
    }
}

# start panel
proc iracqStartPanel {panelName args} {
    global gvar env

    set panelBaseName $panelName
    set panelExecName "$panelName [join $args]"

    # we enter in the directory where the binary of the panel is
    # stored to avoid too long path names when doing ps -ef
    if {[catch {eval "exec which $panelBaseName"} res]} {
	iracqTkDialog1 "Error" \
		"Problem locating panel: $panelBaseName\n\n\
		Error Message: $res"
	return -1
    } else {
	set idx [string last "/" $res]
	if {$idx != -1} {
	    set path [string range $res 0 $idx]
	    if {[catch {cd $path} res]} {
		iracqTkDialog1 "Error" \
			"Problem entering directory: $path\n\n\
			Error Message: $res"
		return -1
	    }
	}
    }
    
    if {[catch {eval "exec $panelExecName &"} res]} {
	iracqTkDialog1 "Error" \
		"Error starting panel $panelBaseName\n\n\
		Error Message: $res"
	return -1
    }
}

# display panel
proc iracqDisplayPanel {} {
    if {[winfo viewable .]} {
	raise .
    } else {
	wm deiconify .
    }
}

# pop up widget
proc iracqPopUp {w} {

    if {[winfo exists $w]} {
	wm deiconify $w
	raise $w
	return 1
    } else {
	return 0
    }

}

# pop down widget
proc iracqPopDown {w} {
    if {[winfo exists $w]} {
	wm withdraw $w
	return 1
    } else {
	return 0
    }
}


# check, if process is running
proc iracqGetPid {proc} {
    global env

    # check process name
    if {[catch {set result \
	    [exec ps -u $env(USER) -ef \
	    | grep "$proc" \
	    | grep -v "iracqCtrl" \
	    | grep -v "iracqStart" \
	    | grep -v "iracqStop" \
	    | grep -v "display" \
	    | grep -v grep \
		 | grep "$env(USER) "]}]} {
	return 0
    }
    

    if { $result != "" } {
	set pid  [lindex $result 1]
	set parentPid  [lindex $result 2]
	
	return $pid
    }

    return 0    
}

# writing flag
proc iracqHandleDtStatus {status} {
    global gvar

    if {$status == 1} {
	set gvar(iracqDtRecv) 1
	set gvar(iracqDtStore) 0
    } else {
	if {$status == 2} {
	    set gvar(iracqDtRecv) 0
	    set gvar(iracqDtStore) 1
	} else {
	    set gvar(iracqDtRecv) 0
	    set gvar(iracqDtStore) 0
	}
    }    
}

# exposure status handler
proc iracqHandleExpStatus {status} {
    global gvar

    if {$gvar(iracqCCS)} {
	return
    }

    if {$gvar(iracqExpStatus) == "COMPLETED" || \
	    $gvar(iracqExpStatus) == "INACTIVE" || \
	    $gvar(iracqExpStatus) == "ABORTED" || \
	    $gvar(iracqExpStatus) == "FAILURE"} {
	if {$status == "INTEGRATING"} {
	    set gvar(iracqExpCountdown) $gvar(iracqExpTime)

	    iracqStartTimer iracqExpCountdown 1
	}
    }

    if {$gvar(iracqExpStatus) == "INTEGRATING" && $status == "PAUSED"} {
	    iracqStopTimer iracqExpCountdown
    }

    if {$gvar(iracqExpStatus) == "PAUSED" && $status == "INTEGRATING"} {
	    iracqStartTimer iracqExpCountdown 1
    }

    if {$gvar(iracqExpStatus) == "INTEGRATING" || \
	    $gvar(iracqExpStatus) == "TRANSFERRING"} {
	if {$status == "ABORTED" || \
	    $status == "COMPLETED" || \
	    $status == "INACTIVE" || \
	    $status == "FAILURE"} {
	    iracqStopTimer iracqExpCountdown
	}
    }

    set gvar(iracqExpStatus) $status
}

# start timer
proc iracqStartTimer {var step} {
    global gvar 

    set seconds [iracqStr2Sec $gvar($var)]

    if {[info exists gvar(iracqTimer_$var)]} {
	set value [expr {$seconds - $step}]
	set gvar($var) [iracqSec2Str $value]
    }

    set tval [round ([expr {$step * 1000}])]
    set gvar(iracqTimer_$var) [after $tval\
	    [list iracqStartTimer $var $step]]

}

#stop timer
proc iracqStopTimer {var} {
    global gvar

    if {[info exists gvar(iracqTimer_$var)]} {
	after cancel $gvar(iracqTimer_$var)
	unset gvar(iracqTimer_$var)
    }
}

# round function
proc iracqRound {var digits {exp "f"}} {
    upvar $var value

    if {($exp == "e")  && ($value >= 10000.0 || $value < 0.001) && $value != 0.0} {
	set c "e"
    } else {
	set c "f"
    }

    set formatString "%.$digits$c"
    set value [format "$formatString" $value]
}

proc iracqSelectFileXX {fileId title dir filter args} {

    global par fsbReturn gvar iracqFileActive

    set c ".iracqSelFile"; 
    set fsb $c.fsb
    set fsbOld 0
    
    set gvar($fileId) ""
    set gvar(iracqFileId) $fileId

    if {![file isdirectory $dir]} {
	set dir [pwd]
    }

    if {[winfo exists $c] == 0} {
	# get current directory
	set home [exec pwd]
	toplevel $c
	set fsb  $c.fsb
	
	UifFileSelBox  $fsb -topLevel $c -dirGeometry 30x10 \
		-fileGeometry 25x10 -iconify 1
	pack $fsb -fill both -expand 1 
	
	bind $c <Visibility> "raise $c"
	wm group $c [winfo toplevel [winfo parent $c]]
	wm minsize $c 400 395
	wm title   $c $title

	# center
	wm withdraw $c
	update idletasks
	set x [expr {[winfo screenwidth $c]/2 - [winfo reqwidth $c]/2 \
			 - [winfo vrootx [winfo parent $c]]}]
	set y [expr {[winfo screenheight $c]/2 - [winfo reqheight $c]/2 \
			 - [winfo vrooty [winfo parent $c]]}]
	wm geom $c +$x+$y
	wm deiconify $c
	
	catch {$fsb CD $dir}

	if {[catch {$fsb filter $filter}]} {
	    set fsbOld 1

	    catch {$fsb FILTER $filter}
	}
    } else {
	catch {$fsb CD $dir}
	if {[catch {$fsb filter $filter}]} {
	    set fsbOld 1
	    catch {$fsb FILTER $filter}
	}
	wm title     $c $title
	wm deiconify $c
    }

    # raise widget
    raise $c

    if {[info exists iracqFileActive($fileId)]} {
	if {$iracqFileActive($fileId)} {
	    set filePath ""
	    return filePath
	}
    }

    # set active flag for fileId
    set iracqFileActive($fileId) 1

    # wait until the user does something ...
    if {$fsbOld} {
	vwait fsbReturn
	set filePath $fsbReturn
    } else {
        uplevel #0 vwait ${fsb}_fsbReturn
        set filePath [uplevel #0 set ${fsb}_fsbReturn]
    }

    # hide widget
    wm withdraw $c

    # set filename
    if {[string length $filePath] < 1} { 
	set gvar($gvar(iracqFileId)) ""
    } else {	
	set gvar($gvar(iracqFileId)) $filePath
    }

    # clear active flags
    foreach i [array names iracqFileActive] {
	set iracqFileActive($i) 0
    }
    
    return $filePath
}

proc iracqSelectFileX {fileId title dir filter args} {
    global gvar iracqFileActive

    set c ".panSelFile"; 

    if {[winfo exists $c]} {
	wm deiconify $c
	wm title     $c $title
	raise $c
    }

    set gvar($fileId) ""
    set gvar(iracqFileId) $fileId

    if {[info exists iracqFileActive($fileId)]} {
	if {$iracqFileActive($fileId)} {
	    set filePath ""
	    return filePath
	}
    }

    set iracqFileActive($fileId) 1

    set panSelFileName [panSelectFile $dir $filter -title $title]

    # set filename
    set gvar($gvar(iracqFileId)) $panSelFileName

    # clear active flags
    foreach i [array names iracqFileActive] {
	set iracqFileActive($i) 0
    }

    return $panSelFileName
    
}

# toplevel file selection box
proc iracqSelectFile {fileId title dir filter args} {
    global gvar 

    set panSelFileName [panSelectFile $dir $filter \
	    -title $title -top ".$fileId" -cd]

    # set filename
    set gvar($fileId) $panSelFileName

    return $panSelFileName
}

# dialog box
proc iracqDialog {w title text bitmap default args} {
    global button

    if {[winfo exists $w]} {
	wm deiconify $w
	raise $w
	tkwait window $w
    }

    set button -1

    toplevel $w
    wm title $w $title
    wm iconname $w dialog
    wm protocol $w WM_DELETE_WINDOW { }
    wm transient $w [winfo toplevel [winfo parent $w]]

    frame $w.top -relief raised -bd 1
    pack $w.top -side top -fill both
    frame $w.bot -relief raised -bd 1 
    pack $w.bot -side bottom -fill both

    # message box
    message $w.top.msg -aspect 1000 -text $text 
    pack $w.top.msg -side right -expand 1 -fill both -padx 3m 
    
    # bitmap
    if {$bitmap != ""} {
	label $w.top.bitmap -bitmap $bitmap
	pack $w.top.bitmap -side left -padx 3m -pady 3m
    }

    # buttons
    set i 0
    foreach but $args {
	button $w.bot.button$i -text $but -command "set button $i; destroy $w"
	if {$i == $default} {
	    frame $w.bot.default -relief sunken -bd 1
	    raise $w.bot.button$i
	    pack $w.bot.default -side left -expand 1 -padx 3m -pady 2m
	    pack $w.bot.button$i -in $w.bot.default \
		    -side left -padx 2m -pady 2m -ipadx 2m -ipady 1m
	} else {
	    pack $w.bot.button$i -side left -expand 1 \
		    -padx 3m -pady 3m -ipadx 2m -ipady 1m
	}
	incr i
    }
    
    if {$default >= 0} {
	bind $w <Return> "$w.bot.button$default flash; \
		set button $default; destroy $w"
    }

    # center
    wm withdraw $w
    update idletasks
    set x [expr {[winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
		     - [winfo vrootx [winfo parent $w]]}]
    set y [expr {[winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
		     - [winfo vrooty [winfo parent $w]]}]
    wm geom $w +$x+$y
    wm deiconify $w

    # wait until one button has been pressed and window is destroyed
    tkwait window $w

    return $button
}

# dump result window to file
proc iracqDumpToFile {} {
    global gvar env

    set text [$gvar(iracqText) get 1.0 end]
    if {[string trim $text] == ""} {
	iracqTkDialog1 "Info" "Result window contains no data!"
	return
    }

    set dir $env(HOME)
    set filter "*.*"
 
    iracqSelectFile iracqDumpFile "Dump To File" $dir $filter

    if {$gvar(iracqDumpFile) == ""} {
	return
    }
    
    set fileName $gvar(iracqDumpFile)

    if {[file exists $fileName]} {
	if {![iracqTkDialog2 \
		"Warning" "The file $fileName already exists!\
		\nOverwrite?"]} {
	    return 0
	}
    }

    if {[catch {set fd [open $fileName w]}]} {
	iracqTkDialog1 "File Error" "The file $fileName cannot be accessed"
	return 0
    }
   

    puts $fd $text

    close $fd
    return 1
}

# configure panel editor class
proc iracqUpdatePanClass { w { relief flat }} {
    global shArea gvar
    
    $w configure -cwp $gvar(iracqDbPoint)
    $w init
    catch { $w configure -helparea $shArea } 
    $w display

    # remove initial borders
    $w configure -relief $relief

    return $w
}  

# create notebook
proc iracqNoteBookCreate {w} {
    global gvar

    panAddNamespace iwidgets

    set tabPath [tabnotebook $w.tabB \
	    -tabpos n -equaltabs false -raiseselect 1]

    pack $tabPath -fill both -side left -expand 1

    return $tabPath
}

# add notebook page
proc iracqNoteBookAdd {w library page name instance} {
    global gvar

    if {$w == ""} {
	return 1
    }

    set childPath [$w add -label $name]

    if {[catch {set childClass [$page $childPath.$instance \
	    -library $library -cwp $gvar(iracqDbPoint)]}]} {
	$w delete end
	return 0
    }

    pack [iracqUpdatePanClass $childClass flat] \
	    -padx 2 -pady 2 -fill both -expand 1

    return 1
}

# load uif-classes into notebook
proc iracqNoteBookLoadPlugIn {w prefix path {classList ""}} {
    global gvar auto_path

    iracqVB "loading plug-in's in $path"

    # set up indexing information
    set wd [pwd] 
    cd $path
    lappend auto_path [pwd]
    cd $wd 

    if {$classList == ""} {
	# get all uif-classes with matching prefix
	if {[catch {set flist [glob $path/$prefix*_uifClass.tcl]}]} {
	    return 0
	}
    } else {
	set flist ""
	foreach ele $classList {
	    set fullName "$path/$prefix$ele\_uifClass.tcl"
	    if {![file exists $fullName]} {
		return 0
	    } else {
		lappend flist $fullName
	    }
	}
    }

    set library [file rootname [file tail $path]]
    foreach ele $flist {
	set className [file rootname [file tail $ele]]
	iracqVB "loading class $className (instance is $prefix)..."
	set last [string last "_uifClass" $className]
	set last [expr {$last - 1}]
	if {[string first $prefix $className] != -1} {
	    set first [string length $prefix]
	} else {
	    set first 0
	}
	set procName [string range $className $first $last]
	# add uif-class to notebook
	if {![iracqNoteBookAdd $w $library $className "$procName" $prefix]} {
	    iracqVB "class $className not loaded \
		    (wrong indexing information)..."
	}
    }

}

# intitialize notebook
proc iracqNoteBookInit {w} {
    global gvar
    
    if {$w == ""} {
	return 1
    }

    if {[catch {set n [$w childsite]}]} {
	$w add -label "No Plug-In"
    }  

    $w select 0
}

# get notebock path
proc iracqNoteBookPath {w child} {

    if {$w == ""} {
	return ""
    }

    set childpath [$w childsite $child]

    return $childpath
}

# search module
proc iracqSearchModule {path name libList} {
    upvar $libList tmpList

    if {[catch {set tmpList [glob $path/$name]}]} {
	return 0
    }

    return 1
}

# load module
proc iracqLoadModule {w module} {
    global gvar env

    iracqVB "loading module $module..."

    set prefix $module
    set libName lib$module*.tcl

    # search in ../lib
	if {[iracqSearchModule "../lib" $libName libList]} {
	    foreach lib $libList {
		iracqNoteBookLoadPlugIn $w $prefix $lib
	    }
	    return 1
	}

    # search in $INTROOT
    if {![catch {set INTROOT $env(INTROOT)}]} {
	if {[iracqSearchModule "$INTROOT/lib" $libName libList]} {
	    foreach lib $libList {
		iracqNoteBookLoadPlugIn $w $prefix $lib
	    }
	    return 1
	}
    }
    
    # search in $VLTROOT
    if {![catch {set VLTROOT $env(VLTROOT)}]} {
	if {[iracqSearchModule "$VLTROOT/lib" $libName libList]} {
	    foreach lib $libList {
		iracqNoteBookLoadPlugIn $w $prefix $lib
	    }
	    return 1
	}
    }

    iracqVB "module $module not found..."
    return 0
}


# load module list
proc iracqLoadModList {w modList} {
    global gvar

    foreach module $modList {
	if {[iracqLoadModule $w $module]} {
	    catch {$module\Init}
	}
    }
}

# load IRACE module plug-in's
proc iracqLoadIraceModule {nb} {
    global gvar env

    # search in ../lib
    if {[file exists ../lib/libiracqIrace.tcl]} {
	iracqNoteBookLoadPlugIn $nb iracq ../lib/libiracqIrace.tcl 
	return 1
    }
    
    # search in $INTROOT
    if {![catch {set INTROOT $env(INTROOT)}]} {
	if {[file exists $INTROOT/lib/libiracqIrace.tcl]} {
	    iracqNoteBookLoadPlugIn $nb iracq $INTROOT/lib/libiracqIrace.tcl
	    return 1
	}
    }

    # search in $VLTROOT
    if {![catch {set VLTROOT $env(VLTROOT)}]} {
	if {[file exists $VLTROOT/lib/libiracqIrace.tcl]} {
	    iracqNoteBookLoadPlugIn $nb iracq $VLTROOT/lib/libiracqIrace.tcl
	    return 1
	}
    }

    return 0
}

# load standard extension plug-in's
proc iracqLoadStdExt {nb {classList "stdext"}} {
    global gvar env

    if {$classList == ""} {
	return 1 
    }

    if {[lsearch $classList "stdext"] != -1} {
	set classListLoad ""
    } else {
	set classListLoad $classList
    }

    # search in ../lib
    if {[file exists ../lib/libiracqIrace.tcl]} {
	iracqNoteBookLoadPlugIn $nb iracq ../lib/libiracqStdExt.tcl \
		$classListLoad
        iracqStdExtInit $classList
	return 1
    }
    
    # search in $INTROOT
    if {![catch {set INTROOT $env(INTROOT)}]} {
	if {[file exists $INTROOT/lib/libiracqIrace.tcl]} {
	    iracqNoteBookLoadPlugIn $nb iracq $INTROOT/lib/libiracqStdExt.tcl \
		    $classListLoad
	    iracqStdExtInit $classList
	    return 1
	}
    }

    # search in $VLTROOT
    if {![catch {set VLTROOT $env(VLTROOT)}]} {
	if {[file exists $VLTROOT/lib/libiracqIrace.tcl]} {
	    iracqNoteBookLoadPlugIn $nb iracq $VLTROOT/lib/libiracqStdExt.tcl \
		    $classListLoad
	    iracqStdExtInit $classList
	    return 1
	}
    }

    return 0
}

proc iracqIsStdExt {name} {

    set stdExt {stdext Irace1 Chopper}

    if {[lsearch $stdExt $name] != -1} {
	return 1
    } else {
	return 0
    }
}

proc iracqPerfMeterInit {n} {
    global cv

    if {![winfo exists $cv.iracqPerfMeter]} {
	return
    }

    set w [$cv.iracqPerfMeter component uifFrame]

    for {set i 1} {$i <= $n} {incr i 1} {
	UifFrame $w.p$i -background green \
		-width 10 -height 5
	pack $w.p$i -side left
    }

    iracqPerfMeter $n
}

proc iracqPerfMeter {n} {
    global cv

    if {![winfo exists $cv.iracqPerfMeter]} {
	return
    }

    set w [$cv.iracqPerfMeter component uifFrame]
    set x [iracqCmdPerf 0]
    set step [round ([expr {100 / $n}])]
    for {set i 1} {$i <= $n} {incr i 1} {
	if {[expr {$i * $step}] < $x} {
	    $w.p$i configure -background red
	} else {
	    $w.p$i configure -background green
	}
    }

    after 1000 [list iracqPerfMeter $n]

}

proc iracqStartRtd {} {    
    global gvar env seq_ccsEnvName

    if {[string trim $gvar(iracqRtdCmd)] == ""} {
	return
    }

    set os [eval "exec uname"]
    if {$gvar(iracqRtdHost) == $env(HOST)} {
	set rtdStartCmd "$gvar(iracqRtdCmd) -display $gvar(iracqRtdDisp)"
    } else {
	if {$os == "Linux"} {
	    set rtdStartCmd "rsh $gvar(iracqRtdHost) -n $gvar(iracqRtdCmd) -display $gvar(iracqRtdDisp)"
	} else {
	    set rtdStartCmd "remsh $gvar(iracqRtdHost) -n $gvar(iracqRtdCmd) -display $gvar(iracqRtdDisp)"
	}
    }

    if {$gvar(iracqRtdInt)} {
	if {[catch {eval exec $rtdStartCmd &} stat]} {
	    iracqTkDialog1 "RTD-Error" \
		    "Error starting RTD <$gvar(iracqRtdCmd)>:\n\n$stat"
	    return 0
	}

	return 1
    }

    if {!$gvar(iracqCCS) || $gvar(iracqRtdHost) != $env(HOST)} {
	if {[catch {eval exec $rtdStartCmd &} stat]} {
	    iracqTkDialog1 "RTD-Error" \
		    "Error starting RTD <$gvar(iracqRtdCmd)>:\n\n$stat"
	    return 0
	}
    } else {
	set rtdStartCmd "$rtdStartCmd \
		-server $gvar(iracqSrvName) -rtapEnv $seq_ccsEnvName"
	
	# try first to see if the application is running
	set env(NO_ERR_DISPLAY) true
	if {[catch {exec msgSend -n $seq_ccsEnvName \
		$gvar(iracqRtdCmd) SCRIPT "docmd RAISE" 10000}]} {
	    # have to launch the application
	    if {[catch {eval exec $rtdStartCmd &} stat]} {
		iracqTkDialog1 "RTD-Error" \
			"Error starting RTD <$gvar(iracqRtdCmd)>:\n\n$stat"
		unset env(NO_ERR_DISPLAY)
		return 0
	    }
	}
	unset env(NO_ERR_DISPLAY)
	
    }

    return 1
}

proc iracqCheckServer {} {
    global gvar

    if {$gvar(iracqCCS)} {
	# send VERSION command
	if {![iracqSendCmd VERSION "" 0]} {
	    return 0
	}
	
	return 1
    } else {
	# try to open server socket connection
	if {![iracqOpenServerNoCcs]} {
	    return 0
	}

	# open event socket connection
	if {![iracqOpenEventNoCcs $gvar(iracqEvtPort)]} {
	    iracqCloseServerNoCcs
	    return 0
	}

	return 1
    }
}

proc iracqRestartServer {} {    
    global gvar env seq_ccsEnvName
    
    iracqBusy 1
    if {$gvar(iracqCCS)} {
	# send exit command
	set reply [panSendCommand $gvar(iracqEnv) $gvar(iracqSrvName) \
		"EXIT" "CHECK_ALL -noErrDisplay" 10000 ""]

	if {![iracqWaitExit $gvar(iracqSrvName)]} {
	    iracqTkDialog1 "Error" \
		    "Unable to stop server <$gvar(iracqSrvName)>"
	    iracqBusy 0
	    return 0
	}
    } else {
	# close socket connections
	iracqCloseEventNoCcs
	iracqCloseServerNoCcs

	# kill server process
	set pid [iracqGetPid $gvar(iracqSrvNameOrig)]
	if {$pid != 0} {
	    eval exec "kill $pid"
	}

	set gvar(iracqSrvState) "OFF"
	set gvar(iracqSrvSubState) "IDLE"
	set gvar(iracqSrvStateRst) 1
    }
    
    update

    if {$gvar(iracqInstance) == ""} {
	set cmd "$gvar(iracqSrvNameOrig)"
    } else {
	set cmd "$gvar(iracqSrvNameOrig) -inst $gvar(iracqInstance)"
    }
    
    set cmd "$cmd -port $gvar(iracqPort) \
	    -cfg [file tail $gvar(iracqSysConfigFile)] \
	    -log $gvar(iracqLogLevel) \
	    -dtt $gvar(iracqDttName)"

    if {$gvar(iracqVerbose) > 0 && $gvar(iracqInXterm)} {
	set cmd "$cmd -v $gvar(iracqVerbose)"
    }
    if {$gvar(iracqInXterm)} {
	set cmd "$cmd -xterm"
	set cmd "xterm -geometry 75x15+0+300 \
		-display $env(DISPLAY) -e $cmd"
    } else {
	set cmd "$cmd >&/dev/null"
    }

    iracqVB "$cmd"
    # restart server process
    eval exec "$cmd" &

    set downCounter 50
    
    while {$downCounter > 0} {
	if {![iracqCheckServer]} {
	    set downCounter [expr {$downCounter - 1}]
	    after 200
	} else {
	    iracqBusy 0
	    iracqGetConfig
	    iracqFileChanged sysCfg \
		    "$gvar(iracqSysConfigFile)" 1
	    iracqFileChanged detCfg \
		    "$gvar(iracqDetConfigFile)" 1
	    return 1
	}
    }
    
    iracqBusy 0
    return 0
}

proc iracqSelFits {} {
    global gvar env

    set filter "*.fits"
    set dir $gvar(iracqFitsBase)
   
    iracqSelectFile iracqFitsFile "Show Fits Header" $dir $filter

    if {$gvar(iracqFitsFile) == ""} {
	return
    }
 
    set gvar(iracqFitsBase) [file dirname $gvar(iracqFitsFile)]
       
    iracqShowFits $gvar(iracqFitsFile)
}

proc iracqShowFits {fileName} {
    global gvar env cv

    # open the fits file
    if {[catch {set fd [open $fileName]} reply]} {
	iracqTkDialog1 "Error" $reply
	return
    }
    
    set line ""
    set cnt 0

    # enable result box for insertion
    $gvar(iracqText) configure -state normal
    set i1 [$gvar(iracqText) index end]
    $gvar(iracqText) insert end "\n[file tail $fileName]:\n"		
    set i0 [$gvar(iracqText) index end]
    
    while {$line != "END"} {
	# read next fits line
	if {[catch {set line [read $fd 80]} reply]} {
	    iracqTkDialog1 "Error" $reply
	    break
	}
	incr cnt 1

	if {$cnt > 2304} {
	    iracqTkDialog1 "Error" \
		    "Invalid FITS-header size (max. 576 lines)"
	    break
	}

	if {[string trim $line] != ""} {
	    if {[string compare "END " [string range $line 0 3]] == 0} {
		set line "END"
	    } else {
		# append result to result-text
		$gvar(iracqText) insert end "\n$line"		
	    }
	}
    }

    close $fd
    $gvar(iracqText) insert end "\n"		
    set i2 [$gvar(iracqText) index end]
    $gvar(iracqText) tag remove err $i1 $i2
    $gvar(iracqText) see $i0
    $gvar(iracqText) configure -state disabled
}

# plot
proc iracqPlot {x0 y0 x1 var w updateTime minY maxY} {
    global gvar

    catch {[$var\_update]}

    set y1 $gvar($var)

    set deltaX [expr {$x1 - $x0}]
    set sy [expr {$maxY - $minY}]

    set xw [$w cget -width]
    set yw [$w cget -height]

    set yy0 [expr {($y0-$minY)*($yw/$sy)}]
    set yy1 [expr {($y1-$minY)*($yw/$sy)}]
    set ly0 [expr {$yw-$yy0}]
    set ly1 [expr {$yw-$yy1}]

    if {$x1 > $xw} {
	$w move lineTag -${deltaX} 0
	set x1 $x0
	set x0 [expr {$x0 - $deltaX}]
    }

    set id [$w create line ${x0} ${ly0} ${x1} ${ly1} \
	    -fill yellow]
    $w addtag lineTag withtag $id

    set x0 $x1
    set y0 $y1

    set x1 [expr {$x0 + $deltaX}]

    set gvar($w\_monId) [after $updateTime [list iracqPlot $x0 $y0 $x1 $var \
	    $w $updateTime $minY $maxY]]
}

# clear plot area
proc iracqPlotClear {w} {
    global gvar

    $w delete lineTag
}

# start plotting
proc iracqPlotStart {w var updateTime minY maxY} {
    global gvar

    if {[info exists gvar($w\_monId)]} {
	# already running
	return
    }

    iracqPlotClear $w
    iracqPlot 0 0 2 $var $w $updateTime $minY $maxY
}

# stop plotting
proc iracqPlotStop {w} {
    global gvar

    if {[info exists gvar($w\_monId)]} {
	after cancel $gvar($w\_monId) 
	unset gvar($w\_monId) 
    }
}

# plotter widget
proc iracqPlotWidget {w} {
    global gvar
    global px0 py0

    $w configure -relief sunken -bd 2

    # performance canvas
    canvas $w.canvas -width 120 -height 50 \
	    -relief flat -bd 0 -bg black 
    pack $w.canvas -side left -padx 0m -pady 0m -fill both

    set id [$w.canvas create line 0 5 120 5 -fill red]
    $w.canvas addtag alarmTag withtag $id

    for {set x 0} {$x < 120} {incr x 10} {
	set id [$w.canvas create line $x 0 $x 50 -fill grey]
	$w.canvas addtag gridTag withtag $id
    }

    for {set y 10} {$y < 50} {incr y 5} {
	set id [$w.canvas create line 0 $y 120 $y -fill grey]
	$w.canvas addtag gridTag withtag $id
    }

    bind $w <Destroy> "iracqPlotStop $w"

    return $w.canvas
}

# start performance monitor
proc iracqPerfStart {} {
    global gvar cv

    if {!$gvar(iracqCCS)} {
	iracqPlotStart $gvar(iracqPerfMon) iracqPerf 2000 0.0 100.0
    } else {
	if {[iracqSendCmd "MISC" "-perfStart"]} {
	    iracqPlotStart $gvar(iracqPerfMon) iracqPerf 2000 0.0 100.0
	} else {
	    return 0
	}
    }

    return 1
}

# stop performance monitor
proc iracqPerfStop {} {
    global gvar cv

    iracqPlotStop $gvar(iracqPerfMon)

    if {$gvar(iracqCCS) && $gvar(iracqSrvState) != "OFF"} {
	iracqSendCmd "MISC" "-perfStop" 0
    }

    return 1
}

# enable/disable function
proc iracqPerfEnable {} {
    global gvar
    
    if {$gvar(iracqPerfEnabled)} {
	if {![iracqPerfStart]} {
	    set gvar(iracqPerfEnabled) 0
	}
    } else {
	if {![iracqPerfStop]} {
	    set gvar(iracqPerfEnabled) 1
	}
    }
}

# update script for iracqPerf variable
proc iracqPerf_update {} {
    global gvar

    if {!$gvar(iracqCCS)} {
	set gvar(iracqPerf) [iracqCmdPerf 0]
    } else {
	set gvar(iracqPerf) [seq_dbReadSymbolic $gvar(iracqDbAttrPerf)]
    }
    
}

proc iracqCancelInfo {w} {
    global iracqInfo

    if {[info exists iracqInfo(id_$w)]} {
	after cancel $iracqInfo(id_$w)
    }
}

# running info message
proc iracqRunInfo {w text speed} {
    global iracqInfo

    if {![winfo exists $w]} {
	return
    }

    set len [$w cget -width] 

    if {![info exists iracqInfo(infoText_$w)]} {
	set iracqInfo(infoText_$w) $text
	set iracqInfo(infoText1_$w) $iracqInfo(infoText_$w)
	set iracqInfo(infoText2_$w) ""
	set iracqInfo(newInfoText_$w) ""
	set iracqInfo(saveInfoText_$w) $iracqInfo(infoText_$w)
	$w configure -cursor watch -anchor e
	bind $w <Enter> "iracqCancelInfo $w"
	bind $w <Leave> "iracqRunInfo $w \"\" $speed"
	bind $w <Destroy> "unset iracqInfo(infoText_$w); \
		unset iracqInfo(id_$w)"
    }

    if {$text != "" && [info exists iracqInfo(id_$w)]} {
	set iracqInfo(saveInfoText_$w) $text
	set iracqInfo(newInfoText_$w) "$iracqInfo(newInfoText_$w) $text"
	return
    }

    if {[string length $iracqInfo(infoText1_$w)] == 0} {
	set i1 [string length $iracqInfo(infoText_$w)]
	set i2 [string length $iracqInfo(infoText2_$w)]
	if {[expr {($i2 - $i1)/2}] >= $len} {
	    set iracqInfo(infoText2_$w) ""
	    if {$iracqInfo(newInfoText_$w) != ""} {
		set tmp [iracqSgets iracqInfo(newInfoText_$w)]
		if {$tmp != -1} {
		    set iracqInfo(infoText_$w) $tmp
		} else {
		    set iracqInfo(infoText_$w) $iracqInfo(newInfoText_$w)
		    set iracqInfo(newInfoText_$w) ""
		}
		set iracqInfo(infoText1_$w) $iracqInfo(infoText_$w)
	    } else {
		set iracqInfo(newInfoText_$w) $iracqInfo(saveInfoText_$w)
		set tmp [iracqSgets iracqInfo(newInfoText_$w)]
		if {$tmp != -1} {
		    set iracqInfo(infoText_$w) $tmp
		} else {
		    set iracqInfo(infoText_$w) $iracqInfo(newInfoText_$w)
		    set iracqInfo(newInfoText_$w) ""
		}
		set iracqInfo(infoText1_$w) $iracqInfo(infoText_$w)
		$w configure -text ""
		return
	    }
	} else {
	    set iracqInfo(infoText2_$w) "$iracqInfo(infoText2_$w) "
	    $w configure -anchor e -text $iracqInfo(infoText2_$w)
	}
    } else {
	set tmpStr [string range $iracqInfo(infoText1_$w) 0 0]
	set iracqInfo(infoText2_$w) "$iracqInfo(infoText2_$w)$tmpStr"
	
	$w configure -text $iracqInfo(infoText2_$w)
	
	set iracqInfo(infoText1_$w) \
		[string range $iracqInfo(infoText1_$w) 1 end]
    }

    set iracqInfo(id_$w) \
	    [after $speed [list iracqRunInfo $w "" $speed]]
}

proc iracqEditFile {file} {
    global gvar env

    if {$file == ""} {
	return
    }

    if {[info exists gvar(iracqFileMgr)]} {
	if {$gvar(iracqFileMgr) != 0} {
	    set pid ""
	    catch {set pid [eval exec ps -p $gvar(iracqFileMgr)]}
	    if {$pid != ""} {
		catch [eval exec kill $gvar(iracqFileMgr)]
	    }
	    set gvar(iracqFileMgr) 0
	}
    }

    if {[catch {set editor $env(EDITOR)}]} {
	set editor "emacs"
    }

    set cmd "$editor $file"

    set gvar(iracqFileMgr) [eval exec "$cmd" &]    
}

proc iracqFileMgr {} {
    global gvar env

    iracqSelectFile iracqFileSel "File Manager" \
	    $gvar(iracqCfgBase) "*"

    if {$gvar(iracqFileSel) != ""} {
	iracqEditFile $gvar(iracqFileSel)
    }
}

proc iracqFileChanged {fileId fileName rst} {
    global iracqFileMod

    if {![file exists $fileName] || ![file isfile $fileName]} {
	return 0
    }

    if {[catch {file stat $fileName info}]} {
	return 0
    }

    set flist [list $info(mtime) $info(size) $info(ctime)]

    if {![info exists iracqFileMod($fileId)]} {
	set iracqFileMod($fileId) $flist
	return 0
    }
    
    if {$flist == $iracqFileMod($fileId)} {
	set res 0
    } else {
	set res 1
    }

    if {$rst} {
	set iracqFileMod($fileId) $flist
    }

    return $res
}

proc iracqStackPush {name ele} {
    global gvar

    if {$ele == ""} {
	return
    }

    if {![info exists gvar(iracqStack_$name)]} {
	set gvar(iracqStack_$name) ""
    }

    set idx [lsearch -exact $gvar(iracqStack_$name) $ele]
    if {$idx != -1} {
	set gvar(iracqStack_$name) \
		[lreplace $gvar(iracqStack_$name) $idx $idx]
    } else {
	if {[llength $gvar(iracqStack_$name)] >= 64} {
	    set gvar(iracqStack_$name) \
		    [lreplace $gvar(iracqStack_$name) 0 0]
	}
    }

    lappend gvar(iracqStack_$name) $ele
    set gvar(iracqStackIdx_$name) [llength $gvar(iracqStack_$name)]
}

proc iracqStackIdx {name} {
    global gvar
    if {![info exists gvar(iracqStackIdx_$name)]} {
	return 0
    }
    
    return $gvar(iracqStackIdx_$name)
}

proc iracqStackSize {name} {
    global gvar

    if {![info exists gvar(iracqStack_$name)]} {
	return 0
    }

    return [llength $gvar(iracqStack_$name)]
}

proc iracqStackClear {name} {
    global gvar

    if {![info exists gvar(iracqStack_$name)]} {
	return
    }

    set idx [expr {$gvar(iracqStackIdx_$name) + 1}]

    if {$idx >= [llength $gvar(iracqStack_$name)]} {
	return
    }

    set gvar(iracqStack_$name) [lreplace $gvar(iracqStack_$name) $idx end]
}

proc iracqStackReset {name} {
    global gvar
    set gvar(iracqStackIdx_$name) 0
    set gvar(iracqStack_$name) ""
}

proc iracqStackDown {name} {
    global gvar


    if {![info exists gvar(iracqStack_$name)]} {
	return ""
    }

    incr gvar(iracqStackIdx_$name) -1
    if {$gvar(iracqStackIdx_$name) < 0} {
	set gvar(iracqStackIdx_$name) 0
    }

    set ele [lindex $gvar(iracqStack_$name) $gvar(iracqStackIdx_$name)]

    return $ele
}

proc iracqStackUp {name} {
    global gvar

    if {![info exists gvar(iracqStack_$name)]} {
	return ""
    }

    incr gvar(iracqStackIdx_$name) 1
    if {$gvar(iracqStackIdx_$name) >= [llength $gvar(iracqStack_$name)]} {
	set gvar(iracqStackIdx_$name) [llength $gvar(iracqStack_$name)]
	return ""
    }

    set ele [lindex $gvar(iracqStack_$name) $gvar(iracqStackIdx_$name)]

    return $ele
}

proc iracqSec2Str {value} {
    
    if {$value >= 0} {
	set sign " "
    } else {
	set sign "-"
    }

    set absValue [expr {abs($value)}]
    set seconds [expr {round($absValue)}]
    set hours [expr {$seconds / 3600}]
    set rem [expr {$seconds % 3600}]
    set min [expr {$rem / 60}]
    set seconds [expr {$rem % 60}]
    set result [format "$sign%02d:%02d:%02d" $hours $min $seconds]
    return $result
}

proc iracqStr2Sec {value} {
    if {[string index $value 0] == "-"} {
	set factor -1
	set value [string range $value 1 end]
    } else {
	set factor 1
    }
    scan $value "%d:%d:%d" hours min seconds
    set result [expr {$factor * \
			  (($hours * 3600) + ($min * 60) + $seconds)}]
    return $result
}



#
# ___oOo___
