#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: iracqUifParam.tcl,v 1.113 2005/09/13 07:27:26 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# jstegmei  13/10/99  created
# jstegmei  14/03/00  optimized dynamic-parameter update
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

# parameter entry
proc iracqParamEntry {name} {

    set x "wdg"
    while {[set i [string first "." $name]] != -1} {
	set sub1 [string range $name 0 [expr {$i - 1}]]
	set sub2 [string range $name [expr {$i + 1}] end]
	set x "$x$sub1"
	set name $sub2
    }
    set x "$x$name"

    return $x
}


# parameter toplevel widget
proc iracqParamSetup {} {
    global gvar
    global iracqParamWdg
    global iracqParam
    global iracqParamSet

    if {$gvar(iracqSrvState) == "OFF"} {
	iracqTkDialog1 "Error" "System is switched off"
	return
    }

    set iracqParamWdg .iracqParam

    set w $iracqParamWdg
    
    if {!$gvar(iracqCCS)} {
	# get parameter setup
	if {![iracqCmdGetParam 0]} {
	    if {[winfo exists $w]} {
		destroy $w
	    }
	    return
	}
    }

    # retrieve number of parameters
    set cnt 0
    set plist [lsort [array names iracqParam]]
    foreach ele $plist {
	if {$iracqParam($ele) != "undefined"} {
	    incr cnt 1
	}
    }

    if {[iracqPopUp $w]} {
	return
    }

    toplevel $w
    bind $w <Destroy> {}

    wm iconname $w "Parameter"
    wm title $w "Parameter Setup"

    # top/bottom area
    frame $w.top 
    frame $w.bot -relief raised -bd 1
    pack $w.top -side top -fill both -expand 1
    pack $w.bot -side bottom -fill both -expand 0

    if {$cnt > $gvar(iracqMaxParam)} {
	label $w.top.info -width 60 -fg black \
		-text "PARAMETER LIST IS TOO LONG!" 
	pack $w.top.info -side top -fill both -pady 20 
    } else {
	set m 0
	frame $w.top.left -bd 1 -relief raised
	pack $w.top.left -side left -fill both -expand 1
	if {$cnt > [expr {$gvar(iracqMaxParam) / 2}]} {
	    frame $w.top.right -bd 1 -relief raised
	    pack $w.top.right -side right -fill both -expand 1
	}
	# generate entry fields
	foreach i $plist {
	    if {$iracqParam($i) != "undefined"} {
		set e [iracqParamEntry $i]
		if {$m && $cnt > [expr {$gvar(iracqMaxParam) / 2}]} {
		    frame $w.top.$e -bd 1 
		    pack $w.top.$e -side top -in $w.top.right \
			    -fill both -expand 1
		} else {
		    frame $w.top.$e -bd 1 
		    pack $w.top.$e -side top -in $w.top.left \
			    -fill both -expand 1
		}
		set m [expr {1 - $m}]
		set ptext $i
		label $w.top.$e.entryL -anchor nw -width 30 -fg black \
			-text $ptext 
		entry $w.top.$e.entry -relief sunken -bd 2 -fg black \
			-textvariable iracqParamSet($i) -width 16 
		entry $w.top.$e.output -relief ridge -bd 2 -fg blue \
			-textvariable iracqParam($i) -width 20 \
			-state disabled -justify center
		pack $w.top.$e.entryL -side left -anchor nw \
			-fill x -expand 0 
		pack $w.top.$e.output -side right -anchor ne \
			-fill x -expand 1 -padx 10
		pack $w.top.$e.entry -side right -anchor ne \
			-fill x -expand 0
		bind $w.top.$e.entry <Return> "iracqCmdParamSet $i"
	    }
	}
    }


    frame $w.bot.cent
    pack $w.bot.cent -side top -pady 3m

    # Apply button
    button $w.bot.apply -text Apply -command {
	iracqCmdParamApply
    }
    pack $w.bot.apply -side left  -padx 1c -in $w.bot.cent

    # default button
    button $w.bot.def -text Default -command {
	iracqCmdParamDefault
    }
    pack $w.bot.def -side left  -padx 1c -in $w.bot.cent

    # dismiss button
    button $w.bot.dismiss -text Dismiss -command {
	wm withdraw $iracqParamWdg
    }
    pack $w.bot.dismiss -side left -padx 1c -in $w.bot.cent
    # center

    wm withdraw $w
    update idletasks
    set x [expr {[winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
		     - [winfo vrootx [winfo parent $w]]}]
    set y [expr {[winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
		     - [winfo vrooty [winfo parent $w]]}]
    wm geom $w +$x+$y
    wm deiconify $w
    
}

# update parameter widget
proc iracqParamUpdate {} {
    global gvar
    global iracqParamWdg
    global iracqParam
    global iracqParamTrace

    set instance $gvar(iracqParamInstance)

    if {$instance != ""} {
	set w $gvar(iracqParamWidget)
	$w.iracqParam-paramList deleteItem 0 end
	foreach i [lsort [array names iracqParam]] {
	    if {![info exists iracqParamTrace($i)]} {
		set iracqParamTrace($i) 0
	    }
	    if {$iracqParam($i) != "undefined"} {
		set line [format "%-30s %s" $i $iracqParam($i)]
		set parArray($line) $line
		
		if {!$iracqParamTrace($i)} {
		    trace variable iracqParam($i) [string tolower w] \
			    iracqParamUpdateValue
		    set iracqParamTrace($i) 1
		}
	    } else {
		if {$iracqParamTrace($i)} {
		    trace vdelete iracqParam($i) [string tolower w] \
			    iracqParamUpdateValue
		    set iracqParamTrace($i) 0
		}
	    }
	}
	
	$w.iracqParam-paramList updateSortList -increasing -ascii parArray
    }

    set iracqParamWdg .iracqParam

    set w $iracqParamWdg

    if {![winfo exists .iracqParam]} {
	return
    }

    wm geometry $w {}

    # retrieve number of parameters
    set cnt 0
    set plist [lsort [array names iracqParam]]
    foreach ele $plist {
	if {$iracqParam($ele) != "undefined"} {
	    incr cnt 1
	}
    }

    # bottom area
    destroy $w.top
    frame $w.top 
    pack $w.top -side top -fill both -expand 1

    if {$cnt > $gvar(iracqMaxParam)} {
	label $w.top.info -width 60 -fg black \
		-text "PARAMETER LIST IS TOO LONG!" 
	pack $w.top.info -side top -fill both -pady 20 
	return
    }

    # generate entry fields
    set m 0
    frame $w.top.left -bd 1 -relief raised
    pack $w.top.left -side left -fill both -expand 1
    if {$cnt > [expr {$gvar(iracqMaxParam) / 2}]} {
	frame $w.top.right -bd 1 -relief raised
	pack $w.top.right -side right -fill both -expand 1 
    }
    foreach i $plist {
	if {$iracqParam($i) != "undefined"} {
	    set e [iracqParamEntry $i]
	    if {$m && $cnt > [expr {$gvar(iracqMaxParam) / 2}]} {
		frame $w.top.$e -bd 1 
		pack $w.top.$e -side top -in $w.top.right \
			-fill both -expand 1
	    } else {
		frame $w.top.$e -bd 1 
		pack $w.top.$e -side top -in $w.top.left \
			-fill both -expand 1
	    }
	    set m [expr {1 - $m}]
	    set ptext $i
	    label $w.top.$e.entryL -anchor nw -width 30 -fg black \
		    -text $ptext
	    entry $w.top.$e.entry -relief sunken -bd 2 -fg black \
		    -textvariable iracqParamSet($i) -width 16 
	    entry $w.top.$e.output -relief ridge -bd 2 -fg blue \
		    -textvariable iracqParam($i) -width 20 \
		    -state disabled -justify center
	    pack $w.top.$e.entryL -side left -anchor nw \
		    -fill x -expand 0
	    pack $w.top.$e.output -side right -anchor ne \
		    -fill x -expand 1 -padx 10
	    pack $w.top.$e.entry -side right -anchor ne \
		    -fill x -expand 0 
	    bind $w.top.$e.entry <Return> "iracqCmdParamSet $i"
	}
    }
 
}

proc iracqParamSelect {par1 par2 par3} {
    global gvar iracqParam iracqParamSet
    
    set i1 [string first "_paramSel" $par2]
    set instance [string range $par2 0 [expr {$i1 - 1}]]
    set gvar($instance\_paramSelName)  [lindex $gvar($par2) 0]
    set gvar($instance\_paramSelValue) ""
    
}

proc iracqParamUpdateValue {par1 par2 par3} {
    global gvar iracqParam iracqParamSet
    
    set instance $gvar(iracqParamInstance)
    set w $gvar(iracqParamWidget)
    set tmp [$w.iracqParam-paramList getList 0 end]
    set pos 0
    foreach i $tmp {
	scan $i "%s %s" p1 p2
	set pval $iracqParam($par2)
	if {$p1 == $par2 && $pval != $p2 && $pval != "undefined"} {
	    $w.iracqParam-paramList deleteItem $pos $pos
	    set line   [format "%-30s %s" $p1 $pval]
	    $w.iracqParam-paramList addItem $pos $line
	}
	incr pos
    }
}

proc iracqSetParam {instance detIdx} {
    global gvar

    if {$gvar($instance\_paramSelValue) != ""} {
	set name $gvar($instance\_paramSelName)
	set i1 [string first "." $name]
	set str [string range $name $i1 end]
	set name "DET$detIdx$str"
	iracqSendCmd \
		"SETUP" \
		"-function $name $gvar($instance\_paramSelValue)"
	set gvar($instance\_paramSelValue) ""
    }
}

proc iracqInitParam {instance w} {
    global gvar iracqParam iracqParamSet
    
    set gvar(iracqParamInstance) $instance
    set gvar(iracqParamWidget) $w

    trace variable gvar($instance\_paramSel) [string tolower w] \
	    iracqParamSelect

    bind [$w.iracqParam-paramEntry component uifEntry] <Return> \
	    "iracqSetParam $instance \"$gvar(iracqDetIdx)\""

    if {!$gvar(iracqCCS)} {
	$w.iracqParam-paramDefault disable
    }

    iracqAttachParamEvent
}

#
# ___oOo___
