#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: irlDebugLib.tcl,v 1.104 2005/09/12 09:21:01 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# jstegmei  09/02/99  created
#

#************************************************************************
#   NAME
#   irlDebugLib.tcl - debugger for irace loops
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#   debugger for irace loops
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc irlDebugInitX {} {
    global argc argv argv0 gvar env auto_path cv bcv
    global irlVar irlInitFlag irlDebug 

    set gvar(irlDebug-LoopFile) ""
    set gvar(irlBaseDir) [pwd]
    set gvar(irlDebug-VarSel) ""
    set gvar(irlDebug-FrameSel) ""
    set gvar(irlDebug-VarName) ""
    set gvar(irlDebug-VarVal) 0
    set gvar(irlDebug-FrameName) ""
    set gvar(irlDebug-FrameTime) 0

    # pack result box
    irlTextBox [$cv.irlDebug-ResultBox component uifFrame] \
	    100 10 irlText

    bind [$cv.irlDebug-LoopFile component uifEntry] <Return> { 
	unset irlDebug 
	catch {set res [irlDebugLoop $gvar(irlDebug-LoopFile)]} 
	irlInitVars
	irlInitFrames
	irlUpdateVars
	irlUpdateFrames
    }

    bind [$cv.irlDebug-VarVal component uifEntry] <Return> {
	set irlVar($gvar(irlDebug-VarName)) $gvar(irlDebug-VarVal)
	irlUpdateVars
    }

    bind [$cv.irlDebug-FrameTime component uifEntry] <Return> { 
	set gvar(T_$gvar(irlDebug-FrameIdx)) $gvar(irlDebug-FrameTime)
	irlUpdateFrames
    }

    trace variable gvar(irlDebug-VarSel) w irlSelectVar
    trace variable gvar(irlDebug-FrameSel) w irlSelectFrame
}

proc irlDebugExitX {} {
    global argc argv argv0 gvar env auto_path 
    global irlVar irlInitFlag irlDebug 
}

proc irlDebugLoop {fileName} {
    global argc argv argv0 gvar env auto_path errorInfo
    global irlVar irlInitFlag irlDebug 
    
    set res ""

    if {![info exists irlDebug]} {
	set irlDebug 1
	irlInit
	if {[catch {set res [source $fileName]} msg]} {
	    return ""
	}
    } else {
	set irlDebug 1
	irlInit
	if {[catch {set res [source $fileName]} errMsg]} {
	    set savedInfo $errorInfo
	    error $errMsg $savedInfo
	    return ""
	}
    }

    return $gvar(irlOut)
}

proc irlDebugStart {} {
    global gvar

    set res [irlDebugLoop $gvar(irlDebug-LoopFile)]
    
    if {$res != ""} {
	irlAddResult "$res\n"
    irlUpdateVars
    irlUpdateFrames
    }
}

proc irlOpenLoopFile {} {
    global gvar cv
    
    set dir $gvar(irlBaseDir)
    set filter "*"

    irlSelectFile irlDebug-LoopFile "Load System Configuration" \
	    $dir $filter -cd

    if {$gvar(irlDebug-LoopFile) == ""} {
	return
    }

    set gvar(irlBaseDir) [file dirname $gvar(irlDebug-LoopFile)]

    [$cv.irlDebug-LoopFile component uifEntry] xview end

    catch {set res [irlDebugLoop $gvar(irlDebug-LoopFile)]}
    irlInitVars
    irlInitFrames
    irlUpdateVars
    irlUpdateFrames
}

proc irlSelectVar {par1 par2 par3} {
    global gvar 

    set gvar(irlDebug-VarName)  [lindex $gvar(irlDebug-VarSel) 0]
    set gvar(irlDebug-VarVal) [lindex $gvar(irlDebug-VarSel) 1]
}

proc irlSelectFrame {par1 par2 par3} {
    global gvar cv

    set gvar(irlDebug-FrameName)  [lindex $gvar(irlDebug-FrameSel) 0]
    set gvar(irlDebug-FrameTime) [lindex $gvar(irlDebug-FrameSel) 1]
    set newList [join $gvar(irlFrameList) " "]
    set ele [expr {[lsearch $newList $gvar(irlDebug-FrameName)] + 1}]
    set gvar(irlDebug-FrameIdx)  [lindex $newList $ele]
}

proc irlInitVars {} {
    global gvar
    global irlVar

    foreach ele $gvar(irlVarList) {
	if {![info exists irlVar($ele)]} {
	    set irlVar($ele) 0
	}
    }
}

proc irlInitFrames {} {
    global gvar
    global irlVar

    foreach ele $gvar(irlFrameList) {
	set name [lindex $ele 0]
	set idx  [lindex $ele 1]
	if {![info exists gvar(T_$idx)]} {
	    set gvar(T_$idx) 0
	}
    }
}

proc irlUpdateVars {} {
    global cv gvar env
    global irlVar

    $cv.irlDebug-Vars deleteItem 0 end
    
    foreach ele $gvar(irlVarList) {
	set line   [format "%-20s %s" $ele $irlVar($ele)]
	set parArray($line) $line
    }

    $cv.irlDebug-Vars updateSortList -increasing -ascii parArray
}

proc irlUpdateFrames {} {
    global cv gvar env
    global irlVar

    $cv.irlDebug-Frames deleteItem 0 end
    
    set idx 0
    foreach ele $gvar(irlFrameList) {
	set name [lindex $ele 0]
	set idx  [lindex $ele 1]
	set line   [format "%-20s %s" $name $gvar(T_$idx)]
	set parArray($line) $line
	incr idx 1
    }

    $cv.irlDebug-Frames updateSortList -increasing -ascii parArray
}

#
# ___oOo___
