#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: irtdCountDisp.tcl,v 1.22 2005/04/18 14:33:35 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  15/02/00  created

#************************************************************************
#   NAME
#   irtdCountDisp.tcl - count down display for image events
# 
#   SYNOPSIS
#   irtdCountDisp <path> ?options?
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------

itk::usual irtdCountDisp  {
    keep -activebackground -activerelief -background -borderwidth -cursor \
         -elementborderwidth -foreground -highlightcolor -highlightthickness \
         -jump -labelfont -selectbackground -selectborderwidth \
         -selectforeground -textbackground -textfont -troughcolor 
}

class ::irtdCountDisp {
    inherit util::TopLevelWidget

    constructor {args} {
	eval itk_initialize $args
    }

    # method init is called by the TopLevelWidget after all options have been evaluated

    method init {} {
	wm title $w_ "Image counter"
	# make the layout
	make_countdisp
	updateSince
    }

    method make_countdisp {} {
	global ::irtd

	wm resizable $w_ 0 0
        itk_component add cframe {
            frame $w_.cframe
        }
        pack $itk_component(cframe) -side top -fill none -expand 0 -pady 1

        global ::irtd

        set cconf "-anchor w -borderwidth 2 -relief groove"

        itk_component add framelabel {
            label $itk_component(cframe).framelabel \
		    -text "Frame & count" \
		    -width 10
        }

        itk_component add sincelabel {
            label $itk_component(cframe).sincelabel \
		    -text "Last update since..." \
		    -width 10
        }

        itk_component add cmain {
            eval LabelValue $itk_component(cframe).cmain \
		    -text {"Main:"} \
		    -textvariable irtd(cmain) \
		    -labelwidth 5 \
		    -justify right \
		    -valuewidth 5 \
		    $cconf
        }
	[$itk_component(cmain) component entry] config -justify right
	set irtd(cmain) 0

        itk_component add tmain {
            eval LabelValue $itk_component(cframe).tmain \
		    -text {"sec."} \
		    -textvariable irtd(tmain) \
		    -labelwidth 4 \
		    -justify right \
		    -valuewidth 10 \
		    $cconf
        }
	[$itk_component(tmain) component entry] config -justify right

        itk_component add crf {
            eval LabelValue $itk_component(cframe).crf \
		    -text {"Rapid:"} \
		    -textvariable irtd(crf) \
		    -labelwidth 5 \
		    -justify right \
		    -valuewidth 5 \
		    $cconf
        }
	[$itk_component(crf) component entry] config -justify right
	set irtd(crf) 0

        itk_component add trf {
            eval LabelValue $itk_component(cframe).trf \
		    -text {"sec."} \
		    -textvariable irtd(trf) \
		    -labelwidth 4 \
		    -justify right \
		    -valuewidth 10 \
		    $cconf
        }
	[$itk_component(trf) component entry] config -justify right

	itk_component add cclose {
	    button $itk_component(cframe).cclose \
		    -text "Close" \
		    -padx 1m \
		    -command "destroy $w_"
	}

	blt::table $itk_component(cframe) \
		$itk_component(framelabel)    1,0 -fill x -anchor w \
		$itk_component(sincelabel)    1,1 -fill x -anchor w \
		$itk_component(cmain)         2,0 -fill x -anchor w \
		$itk_component(tmain)         2,1 -fill x -anchor w \
		$itk_component(crf)           3,0 -fill x -anchor w \
		$itk_component(trf)           3,1 -fill x -anchor w \
		$itk_component(cclose)        4,0 -fill none -columnspan 2
    }
    
    # mothod to update the "last update since..." values

    method updateSince {} {
	global ::irtd
	if {$rapid1FrameCount_ != -1} {
	    set irtd(trf) $rapid1FrameCount_
	    incr rapid1FrameCount_
	}
	if {$rtdFrameCount_ != -1} {
	    set irtd(tmain) $rtdFrameCount_
	    incr rtdFrameCount_
	}
	after 1000 [code catch "$this updateSince"]
    }
    
    method reset {frame} {
	if {"$frame" == "cmain"} {
	    set rtdFrameCount_ 0
	} else {
	    set rapid1FrameCount_ 0
	}
    }

    # -- public vars --

    # -- protected vars -- 

    # for display of remaining time
    protected variable timeDiff_ 0
    protected variable timeDiffId_ ""
    protected variable timeLastFrame2_ 0
    protected variable timeLastFrame4_ 0

    # frame counters
    protected variable rapid1FrameCount_ -1
    protected variable rtdFrameCount_ -1
}
