#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: irtdRateDisp.tcl,v 1.22 2005/04/18 14:33:34 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  15/02/00  created

#************************************************************************
#   NAME
#   irtdRateDisp.tcl - display transfer rates
# 
#   SYNOPSIS
#   irtdRateDisp <path> ?options?
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------

itk::usual irtdRateDisp  {
    keep -activebackground -activerelief -background -borderwidth -cursor \
         -elementborderwidth -foreground -highlightcolor -highlightthickness \
         -jump -labelfont -selectbackground -selectborderwidth \
         -selectforeground -textbackground -textfont -troughcolor 
}

class ::irtdRateDisp {
    inherit util::TopLevelWidget

    constructor {args} {
	eval itk_initialize $args
    }

    # method init is called by the TopLevelWidget after all options have been evaluated

    method init {} {
	wm title $w_ "Data rates in KB/sec."
	# make the layout
	make_ratedisp
    }

    method make_ratedisp {} {
	wm resizable $w_ 0 0
        itk_component add rframe {
            frame $w_.rframe
        }
        pack $itk_component(rframe) -side top -fill none -expand 0 -pady 1

        set cconf "-anchor w -borderwidth 2 -relief groove"

        itk_component add ratelabel {
            label $itk_component(rframe).ratelabel \
		    -text "Data transmission rates" \
		    -width 10
        }

        itk_component add dratelabel {
            label $itk_component(rframe).dratelabel \
		    -text "Display rates incl. scaling" \
		    -width 10
        }

	set labels {"Actual rate" "Mean rate" "Max. rate" "Min. rate"}
	set i 0
	foreach el "trate dtrate mrate dmrate ratemax dratemax ratemin dratemin" {
	    if {![expr {$i % 2}]} {
		set text [lindex $labels [expr {$i / 2}]]
	    }
	    incr i
	    addLabelValue $el $cconf $text
	}

	itk_component add rclose {
	    button $itk_component(rframe).rclose \
		    -text "Close" \
		    -padx 1m \
		    -command "destroy $w_"
	}

	itk_component add rreset {
	    button $itk_component(rframe).rreset \
		    -text "Reset" \
		    -padx 1m \
		    -command "$this initRateVals"
	}

	blt::table $itk_component(rframe) \
		$itk_component(ratelabel)    1,0 -fill x -anchor w \
		$itk_component(dratelabel)   1,1 -fill x -anchor w \
		$itk_component(trate)        2,0 -fill x -anchor w \
		$itk_component(dtrate)       2,1 -fill x -anchor w \
		$itk_component(mrate)        3,0 -fill x -anchor w \
		$itk_component(dmrate)       3,1 -fill x -anchor w \
		$itk_component(ratemax)      4,0 -fill x -anchor w \
		$itk_component(dratemax)     4,1 -fill x -anchor w \
		$itk_component(ratemin)      5,0 -fill x -anchor w \
		$itk_component(dratemin)     5,1 -fill x -anchor w \
		$itk_component(rreset)       6,0 -fill none \
		$itk_component(rclose)       6,1 -fill none

	initRateVals
    }

    method initRateVals {} {
	global ::stc
	foreach el "trate srate nrate mrate ratemax ratemin dtrate dsrate dnrate dmrate dratemax dratemin" {
	    set stc($el) 0
	}
    }

    # set value for actual display rate and calc. mean rate

    method drate { dtrate } {
	global ::stc
	set stc(dtrate) $dtrate
	set stc(dsrate) [expr {$stc(dsrate) + $dtrate}]
	incr stc(dnrate)
	set stc(dmrate) [format {%.1f} [expr {$stc(dsrate) / $stc(dnrate)}]]
	if {$stc(dratemin)} {
	    if {$stc(dtrate) < $stc(dratemin)} {
		set stc(dratemin) $stc(dtrate)
	    }
	    if {$stc(dtrate) > $stc(dratemax)} {
		set stc(dratemax) $stc(dtrate)
	    }
	} else {
	    set stc(dratemin) $stc(dtrate)
	    set stc(dratemax) $stc(dtrate)
	}
    }

    # set value for actual transmission rate and calc. mean rate

    method rate { trate size } {
	global ::stc
	set stc(trate) $trate
	set stc(srate) [expr {$stc(srate) + $trate}]
	incr stc(nrate)
	set stc(mrate) [format {%.1f} [expr {$stc(srate) / $stc(nrate)}]]
	if {$stc(ratemin)} {
	    if {$stc(trate) < $stc(ratemin)} {
		set stc(ratemin) $stc(trate)
	    }
	    if {$stc(trate) > $stc(ratemax)} {
		set stc(ratemax) $stc(trate)
	    }
	} else {
	    set stc(ratemin) $stc(trate)
	    set stc(ratemax) $stc(trate)
	}
    }

    method addLabelValue {compo cconf text} {
	global ::stc
        itk_component add $compo {
            eval LabelValue $itk_component(rframe).$compo \
		    -text {"$text:"} \
		    -textvariable stc($compo) \
		    -labelwidth 15 \
		    -justify right \
		    -valuewidth 10 \
		    $cconf
        }
    }

    # -- public vars --

    # -- protected vars -- 
}
