#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: oslxInsMan_Util.tcl,v 1.253 2005/08/17 13:07:56 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# jknudstr  23/05/95  created
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#


# Initialize the panel
proc oslxInsManInit {} {
    global gvar env

    set gvar(INS_ROOT) ""
    set gvar(INS_USER) ""
}


# Select and adjust a new INS_ROOT
proc oslxInsManSetInsRoot {} {
    global gvar env
    
    catch "set basePath $env(HOME)"
    set dirName [panSelectFile $basePath *]
    if {$dirName == ""} return 

    set gvar(INS_ROOT) $dirName
    set gvar(INS_USER) ""
}

# Select and adjust a new INS_USER
proc oslxInsManSetInsUserRoot {} {
    global gvar env

    if {[catch "set basePath $gvar(INS_ROOT)"]} {
	oslxInsManCheckRes -1 "No INS_ROOT defined - set this first"
	return -1
    }
    set dirName [panSelectFile $basePath *]
    if {$dirName == ""} return 

    set gvar(INS_USER) $dirName
}


# Procedure to create the base INS Directory structure
proc oslxInsManCreateBaseDir {{insRoot ""}} {
    global gvar env 

    if {$insRoot != ""} {
	set gvar(INS_ROOT) $insRoot
    }

    # Check if this INS_ROOT already exists
    set insDir [glob -nocomplain $gvar(INS_ROOT)]
    if {$insDir != ""} {
	tk_dialog .ins_root_exists "Dialog Box" \
		"The INS_ROOT $gvar(INS_ROOT) already exists!!" info 0 ok
	return -1
    }

    # Create the INS_ROOT root directory
    if {[catch "exec mkdir $gvar(INS_ROOT)"]} {
	oslxInsManCheckRes -1 "Error occurred while creating $gvar(INS_ROOT)"
	return -1
    }
    
    # Create the basic structure
    if {[oslxAddUserBranch $gvar(INS_ROOT)/SYSTEM] != 0} {
	oslxInsManCheckRes -1 \
		"Error occurred while creating the INS_ROOT/SYSTEM directories"
	return -1 
    }
     
    # Create the Dictionary and Alias directories
    if {[catch "exec mkdir $gvar(INS_ROOT)/SYSTEM/Dictionary"]} {
	oslxInsManCheckRes -1 \
		"Error occurred while creating Dictionary directory"
	return -1
    }
    if {[catch "exec mkdir $gvar(INS_ROOT)/SYSTEM/Alias"]} {
	oslxInsManCheckRes -1 "Error occurred while creating Alias directory"
	return -1
    }

    # Create the MAINTENANCE branch
    if {[oslxAddUserBranch $gvar(INS_ROOT)/MAINTENANCE] != 0} {
	oslxInsManCheckRes -1 \
		"Could not create the INS_ROOT/MAINTENANCE sub-directories"
	return -1
    }
    return 0
}

proc oslxInsManCheckRes {result errorMsg} {
    if {$result != 0} {
	tk_dialog .ins_root_error "Dialog Box" $errorMsg info 0 ok
	return
    }
}

proc oslxAddUserBranchPrompt {} {
    global gvar env
    
    # Check if INS_ROOT is defined
    if {[catch "set basePath $gvar(INS_ROOT)"]} {
	oslxInsManCheckRes -1 "No INS_ROOT is defined"
	return -1
    }
  
    # Check if the INS_ROOT exists
    set res [glob -nocomplain $gvar(INS_ROOT)]
    if {$res == ""} {
	oslxInsManCheckRes -1 "No INS_ROOT is created - create this first"
	return -1
    }

    # Get the name of the new User Directory
    set userDir [promptDialog .getUserDir "Enter New User Directory"]
    if {$userDir == ""} return 

    if {[oslxAddUserBranch $gvar(INS_ROOT)/$userDir] != 0} {
	oslxInsManCheckRes -1 \
		"Error in creating user branch $gvar(INS_ROOT)/$userDir"
	return -1
    }

    return 0
}

# Add the one user branch to the INS_ROOT
proc oslxAddUserBranch {userDir} {
    global gvar env

    # Create the User Branch
    if {[catch "exec mkdir $userDir"]} {return -1}

    # Create the User Branch/DETDATA
    if {[catch "exec mkdir $userDir/DETDATA"]} {return -1}

    # Create the User Branch/BADDATA
    if {[catch "exec mkdir $userDir/BADDATA"]} {return -1}

    # Create the User Branch/ARCDATA
    if {[catch "exec mkdir $userDir/ARCDATA"]} {return -1}

    # Create the User Branch/MISC
    if {[catch "exec mkdir $userDir/MISC"]} {return -1}

    if {[oslxAddModeBranch $userDir/COMMON] != 0} {return -1}

    return 0
}

# Prompt for the INS_USER and add a Mode Branch
proc oslxAddModeBranchPrompt {} {
    global gvar env

    if {[catch "set basePath $gvar(INS_USER)"]} {
	oslxInsManCheckRes -1 "INS_USER not defined"
	return -1
    }
    set dirName [panSelectFile $basePath *]
    if {$dirName == ""} return

    if {[oslxAddModeBranch $dirName] != 0} {
	oslxInsManCheckRes -1 "Could not add Mode Branch $dirName"
	return -1
    }
    
    return 0
}

# Create an Observation Mode Branch in the indicated location
proc oslxAddModeBranch {modeDir} {
    global gvar env

    # Create the $modeDir
    if {[catch "exec mkdir $modeDir"]} {return -1}

    # $modeDir/CONFIGFILES
    if {[catch "exec mkdir $modeDir/CONFIGFILES"]} {return -1}

    # $modeDir/SETUPFILES
    if {[catch "exec mkdir $modeDir/SETUPFILES"]} {return -1}

    # $modeDir/SETUPFILES/REF
    if {[catch "exec mkdir $modeDir/SETUPFILES/REF"]} {return -1}

    # $modeDir/SETUPFILES/DET
    if {[catch "exec mkdir $modeDir/SETUPFILES/DET"]} {return -1}

    # $modeDir/SETUPFILES/INS
    if {[catch "exec mkdir $modeDir/SETUPFILES/INS"]} {return -1}

    # $modeDir/SETUPFILES/TARG
    if {[catch "exec mkdir $modeDir/SETUPFILES/TARG"]} {return -1}

    # $modeDir/TEMPLATES
    if {[catch "exec mkdir $modeDir/TEMPLATES"]} {return -1}
    # $modeDir/TEMPLATES/OBD
    if {[catch "exec mkdir $modeDir/TEMPLATES/OBD"]} {return -1}
    # $modeDir/TEMPLATES/SEQ
    if {[catch "exec mkdir $modeDir/TEMPLATES/SEQ"]} {return -1}
    # $modeDir/TEMPLATES/TSF
    if {[catch "exec mkdir $modeDir/TEMPLATES/TSF"]} {return -1}

    # $modeDir/MOBS
    if {[catch "exec mkdir $modeDir/MOBS"]} {return -1}

    # $modeDir/MIDAS
    if {[catch "exec mkdir $modeDir/MIDAS"]} {return -1}

    # $modeDir/MIDAS/PROCS
    if {[catch "exec mkdir $modeDir/MIDAS/PROCS"]} {return -1}

    # $modeDir/MIDAS/REF 
    if {[catch "exec mkdir $modeDir/MIDAS/REF"]} {return -1}
    
    return 0
}

#
# ___oOo___




