#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bindings.tcl,v 5.34 2005/09/26 12:44:16 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.Bernini 24/05/94  created 
# M.Comin   05/07/94  Add comments to the code
# M.COMIN   20/03/95  Add edit binding for listbox
# M.COMIN   06/04/95  Add edit binding for UifText
# M.COMIN   04/05/95  Add insertion cursor
#
# mcomin    27/02/96  Porting to TK4.0
#           28/04/96  Fixed bug setting edit binding for pubClass
#
#************************************************************************
#   NAME
#
#   bindings -  This files contains a set of routine used
#               to configure the binding mechanisms
# 
#   SYNOPSIS
#
#  
# 
#   DESCRIPTION
#
#
#   FILES
#    
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#------------------------------------------------------------------------
#



#----------------------------------------------------------------------------
# PROCEDURE : panSetClassBind -  Defines specific bindings for specific 
#                                widget classes.
#----------------------------------------------------------------------------

proc panSetClassBind { cv } {

   #
   #  freeText widget bindings
   #  

   # Insert key in current active freeText widget

   $cv bind text <KeyPress>  "$cv insert activetext insert %A" 
   $cv bind text <Delete>    "panSetTextBS $cv" ; # Backspace and delete functions
   $cv bind text <BackSpace> "panSetTextBS $cv"

   $cv bind text <Right>    { set char [%W index activetext insert]; \
                              %W  icursor activetext [expr $char + 1] }

   $cv bind text <Left>     { set char [%W index activetext insert]; \
                              %W icursor activetext [expr $char - 1] }

   $cv bind text <B1-Motion>  "panTextSelect     $cv %x %y"
   $cv bind text <1>          "panSetTextFocus   %x %y"

   # Disable cut/paste otherwise there are problems with entry fields
   # during editing 
#   bind Entry <ButtonRelease-2> {}
}


proc panGetAllWidgets { w } {
  set wdgList ""

  foreach item [winfo children $w] {
     set wdgList [concat $wdgList $item]

     if {[set tmp [panGetAllWidgets $item]] != "" } {
         set wdgList [concat $wdgList $tmp]
     }
#     puts "DBG > wdgList = $wdgList"
  }

  return $wdgList
}


proc panSetWidgetBind { cv name type } {
  global par panWidget i_clRoot

  # Define binding for a canvas item

  if {[cequal $type freeText]} {
     $cv bind $name <2>         "panStartDrag  %X %Y %W $cv $name ; break"
     $cv bind $name <B2-Motion> "panDragWidget %X %Y $name $cv"
     $cv bind $name <ButtonRelease-2> "panSetEditMode create $type"
     $cv bind $name <3>         "createConfigPanel   $type $name $cv"
     return
  }

  # Define binding for a widget

  switch $type {
    UifClass        { set me $cv.$name.cv }
    UifBarChart     { set me [$cv.$name component uifBarChart] }
    UifBitmap       { set me [$cv.$name component uifBitmap]}
    UifListBox      { set me [$cv.$name component uifListBox]}
    UifText         { set me [$cv.$name component uifText]}
    UifOptionButton { set me [$cv.$name component uifOptionButton]}
    UifCheckButton  { set me [$cv.$name component uifCheckButton] }
    UifRadioButton  { set me [$cv.$name component uifRadioButton] }
    UifButton       { set me [$cv.$name component uifButton]}
    UifFrame        { set me [$cv.$name component uifFrame]}
    UifOutput       { set me [$cv.$name component uifOutput]}
    UifOutputString { set me [$cv.$name component uifOutputString]}
    UifEntry        { set me [$cv.$name component uifEntry]}
    UifScale        { set me [$cv.$name component uifScale]}
    UifActionButton { set me [$cv.$name component uifActionButton]}

    default         {
       set     wdgList [panGetAllWidgets $cv.$name]
       lappend wdgList $cv.$name
     
       foreach item $wdgList {
#          puts "DBG> $type   ...  bind item $item"
          bind $item <2>         "panStartDrag  %X %Y %W $cv   $name; break"
          bind $item <B2-Motion> "panDragWidget %X %Y $name $cv"
          bind $item <ButtonRelease-2> "panSetEditMode create"
          bind $item <3>         "createConfigPanel   $type $name $cv"
       }
       return
    }
  } ;# End of switch ...

  bind $me <2>          "panStartDrag  %X %Y  %W $cv $name ; break"
  bind $me <B2-Motion>  "panDragWidget %X %Y  $name $cv; break"
  bind $me <3>          "createConfigPanel $type $name $cv; break"

  # Reset the cursor to the default mode
  bind $me <ButtonRelease-2> "panSetEditMode create $type"
}


#----------------------------------------------------------------------------
# PROCEDURE : panStartDrag - Starts dragging operation
#----------------------------------------------------------------------------

proc panStartDrag {x y w cv name} {
   global par panMaxWdg copyName

   set par(lastX) $x
   set par(lastY) $y
   incr par(SAVE_FLAG) 

   set id [$cv find withtag $name]

   if { $par(EDIT_MODE) == "copy" } {

      if {[$cv type $id] == "window"} {
         set copyName [string trimleft "$par(ROOT_PANEL)-wdg${panMaxWdg}" "."]
         incr panMaxWdg
         panCopyWidget $cv $name $copyName 
      }

   } elseif { $par(EDIT_MODE) == "delete" } {

      if {[$cv type $id] == "window" } {
          panDeleteWidget $cv $name
      } else {
          $cv delete $name; 
      }
      panSetEditMode create  ;# Reset to default mode

   } else {

      # Get current widget coordinates

      $cv configure -cursor gobbler
      lassign   [$cv coords $name] xc yc

      set movex [expr {[$cv canvasx $xc $par(GRID)] - $xc} ]
      set movey [expr {[$cv canvasy $yc $par(GRID)] - $yc} ]

      # Align the widget with the grid
      $cv move $name $movex $movey

      panSaveCoords $cv $name [$cv coords $name]
   }
}


proc panSetEditMode { what {type {} } } {
   global par panBox

   set par(EDIT_MODE) $what
   panGetCvNames $par(ROOT_PANEL) cv bcv

   if {[cequal $type UifActionButton]} {
      set cv $bcv
   }

   switch $par(EDIT_MODE) {

      copy    { $cv configure -cursor hand2 }

      delete  { 
              $cv configure -cursor pirate

              bind $cv <B1-Motion>       {panBoxDrag      %W %x %y}
              bind $cv <ButtonRelease-1> "panDeleteObject %W %x %y $cv"
      }

      move    { 
              $cv configure -cursor gobbler

              bind $cv <ButtonPress-2>   {panBoxInitDrag %x %y}
              bind $cv <B1-Motion>       {panBoxDrag     %W %x %y}
              bind $cv <ButtonRelease-1> {panSelectMove  %W %x %y}
              bind $cv <B2-Motion>       "panMoveDrag %x %y $cv"
              bind $cv <ButtonRelease-2> "panMoveEnd  %W %x %y $cv"
      }

      default { $cv configure -cursor left_ptr

              # Reset all edit-specific bindings

              bind $cv <ButtonPress-2>   {}
              bind $cv <B1-Motion>       {}
              bind $cv <ButtonRelease-1> {}
              bind $cv <B2-Motion>       {}
              bind $cv <ButtonRelease-2> {}
      }
   }
}


# Calculate grid index 

proc panGetGridIdx { coord } {
    global par
    set tmp [expr {(2*$coord + $par(GRID))/ (2*$par(GRID))}]
    return [expr {int($tmp)}]
}

#----------------------------------------------------------------------------
# PROCEDURE : panGetDragCoords - Calculate coordinates for dragging an object
#
# The procedure takes into account if a grid has been defined and returns the
# proper increase in X,Y coordinates.
#----------------------------------------------------------------------------

proc panGetDragCoords { x y } {
    global par
    
    set dx [expr {$x - $par(lastX)}]
    set dy [expr {$y - $par(lastY)}]
    
    # calculate grid steps
    
    set movex 0 ; set movey 0
    
    if {[set nx [panGetGridIdx $dx]] != 0} {
	set movex [expr {$nx * $par(GRID)}]
	set par(lastX) $x
    }
    
    if {[set ny [panGetGridIdx $dy]] != 0} {
	set movey [expr {$ny * $par(GRID)}]
	set par(lastY) $y
    }
    
    return "$movex $movey"
}

#----------------------------------------------------------------------------
# PROCEDURE : panDragWidget - Move current widget to a new position
#----------------------------------------------------------------------------

proc panDragWidget {x y name cv} {
   global par
  
   if {$par(EDIT_MODE) != "create" && $par(EDIT_MODE) != "move"} { return }

   lassign [panGetDragCoords $x $y] movex movey

#   puts "panDragWidget   movex $movex   movey  $movey"

   if {$movex == 0 && $movey == 0} {return }

   foreach item $name {

      $cv move $item $movex $movey

      # Update widget coordinates : ignore canvas items
    
      panSaveCoords $cv $item [$cv coords $item] 
   }
}

proc panSaveCoords { cv item coords } {
   global par  i_posX i_posY

   set id [$cv find withtag $item]

   if {[$cv type $id] == "window" } {
       lassign $coords posx posy
       saveWidget $item $i_posX [expr {int($posx)}]
       saveWidget $item $i_posY [expr {int($posy)}]
   }
}

#----------------------------------------------------------------------------
# PROCEDURE : panSetTextFocus - Setup freeText to get input from the keyboard
#----------------------------------------------------------------------------

proc panSetTextFocus {x y} {
   global cv
   
   $cv icursor current @$x,$y;     # set cursor inside current freeText
   $cv focus current;              # Set keyboard focus to current freeText
   focus $cv
   $cv dtag activetext activetext;         # Delete tag from previous widget
   $cv addtag activetext withtag current;  # Set  tag to current widget
   $cv select from current @$x,$y
}

#----------------------------------------------------------------------------
# PROCEDURE : panSetTextBS - Implement DELETE and BACKSPACE 
#----------------------------------------------------------------------------

proc panSetTextBS { cv } {
   set char [expr {[$cv index activetext insert] - 1}]
   if {$char >= 0} {$cv dchar activetext $char}
}


#----------------------------------------------------------------------------
# PROCEDURE : panTextSelect - 'Select' part of a text
#----------------------------------------------------------------------------

proc panTextSelect {cv x y} {
   $cv select to current @$x,$y
}

