#
# E.S.O. - VLT project
#
# "@(#) $Id: configButton.tcl,v 5.34 2005/09/26 12:44:19 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.Comin   ........  created 
# M.Comin   13/03/95  panelUifButton is used also for UifActionButtons.
# M.COMIN   14/06/95  Bitmap label only for normal buttons.
# ------------------
# M.COMIN   17/07/96  Fixed bug when displaying initial status
#           16/09/96  Add automatic path for bitmap search
#
#

#----------------------------------------------------------------------------
# PROCEDURE : buttonPanel
#----------------------------------------------------------------------------

proc buttonPanel { idx top type w } {
   global  wdgConfig ckDots ckXbm ckStatus

   set row1 $top.f4
   set row2 $top.f10

   set wdgLabel [getWdgConfig $idx wdgLabel]
   set selXbm   $row1.selXbm ; setWdgConfig $idx selXbm $selXbm

   checkbutton ${wdgLabel}DOTS -text "Popup (...)" -variable ckDots($idx) \
               -command "checkDots $idx"

   checkbutton $row1.xbm -text "Bitmap" -variable ckXbm($idx) \
                         -command "checkXbm $idx $w $type"

   button $selXbm -text "Select Bitmap ..." -width 15 \
                  -command "setXbm $idx $w $type"

   label $row1.lblType  -text " Type of Label" -anchor w\
                        -font $wdgConfig(lblFont) -width $wdgConfig(lblWidth)
   label $row2.lstatus  -text " Initial Status" -anchor w\
                        -font $wdgConfig(lblFont) -width $wdgConfig(lblWidth)
  
   radiobutton $row2.e -width 8 -text "Enabled " -value 1 \
                       -variable ckStatus($idx) 
   radiobutton $row2.d -width 8 -text "Disabled" -value 0 \
                       -variable ckStatus($idx)
   #
   #  Create configuration panel
   #

   pack $row1.lblType  $row1.xbm -side left -pady 10 
   pack $selXbm      -side left -padx 20

   if {[cequal $type UifButton]} {
      pack ${wdgLabel}DOTS -side left -padx 10
   } else {
      $row1.xbm    config -state disabled
      $row1.selXbm config -state disabled
   }
  
   pack $row2.lstatus  $row2.e $row2.d  -side left -pady 10
}


#-----------------------------------------------------------------------------
# PROCEDURE : checkDots - Check configuration of (...) for pushButtons
#-----------------------------------------------------------------------------

proc checkDots { idx } {
    global ckDots

    if {$ckDots($idx) == 0} {     
       [getWdgConfig $idx wdgAct] configure -state normal
       [getWdgConfig $idx wdgPan] configure -state disabled
    } else {
       [getWdgConfig $idx wdgAct] configure -state disabled
       [getWdgConfig $idx wdgPan] configure -state normal
    }
}

#-----------------------------------------------------------------------------
# PROCEDURE : setXbm - 
#-----------------------------------------------------------------------------

proc setXbm {idx w type} {
    global ckXbm 

    if { $ckXbm($idx) == 0 } { return }

    if {[file exists "../bitmaps"]} {
       set tmp "../bitmaps" 
    } else {
       set tmp "." ;# Search file in current directory
    }

    if {[set file [panSelectFile $tmp "*.xbm"]] == "" } { 
       set ckXbm($idx) 0 ; return 
    }

    set myBitmap  [file tail $file]; 
    setConfigEntry $idx wdgLabel $myBitmap  
   
    if { [set file [findVLTFile bitmaps/$myBitmap]] == "" } { 
       set ckXbm($idx) 0 ; return
    }

    $w configure -bitmapFile $myBitmap
}

#-----------------------------------------------------------------------------
# PROCEDURE : checkXbm - Check configuration of (...) for pushButtons
#-----------------------------------------------------------------------------

proc checkXbm {idx w type } {
    global ckDots ckXbm 
          
    if {$ckXbm($idx)} { 
 
       # Set type of possible operations

       [getWdgConfig $idx wdgAct] configure -state normal
       [getWdgConfig $idx wdgPan] configure -state disabled
       [getWdgConfig $idx selXbm] configure -state normal

       set ckDots($idx) 0     ;# Disable popup (...)   
       [getWdgConfig $idx wdgLabel]DOTS configure -state disabled  

    } else {

       $w configure -label [getConfigInput $idx wdgLabel] 

       if {[cequal $type UifButton]} { $w configure -bitmapFile "" }

       [getWdgConfig $idx wdgLabel]DOTS  configure -state normal
       [getWdgConfig $idx selXbm]        configure -state disabled
    }
}


#-----------------------------------------------------------------------------
# PROCEDURE : configUifButton - Updates the data structures describing the
#                               configuration of the UifButton Class
#-----------------------------------------------------------------------------

proc configUifButton { idx cv type name } {
   global ckDots  ckXbm ckStatus  i_status  i_stateVar i_bitmap
  
   if {[cequal $type UifButton]} {
      $cv.$name configure -popup $ckDots($idx)
   }

   if {$ckXbm($idx)} {
       saveWidget $name $i_bitmap  [getConfigInput $idx wdgLabel]
       $cv.$name configure -label  ""
   } else {
       saveWidget $name $i_bitmap  ""
   }

   saveWidget $name $i_status  $ckStatus($idx)
                
   if { $ckStatus($idx) == "0" } {
        $cv.$name configure -initialState disabled
   } else {
        $cv.$name configure -initialState normal
   }
}

#-----------------------------------------------------------------------------
# PROCEDURE : displayUifButton - Updates the configuration panel according to
#                                the current UifButton Class configuration
#-----------------------------------------------------------------------------

proc displayUifButton { idx cv type name } {
   global ckDots  ckDbVar  ckStatus ckXbm
   global i_status  i_dbVar i_bitmap

   #
   # Set Current Options 
   # 

   set ckDots($idx)   0
   set ckXbm($idx)    0
   set ckDbVar($idx)  [getWidget $name $i_dbVar  ]
   set ckStatus($idx) [getWidget $name $i_status ]

   if { [set myBitmap [getWidget $name $i_bitmap]] != "" } {
      setConfigEntry $idx wdgLabel $myBitmap
      set ckXbm($idx) 1 ;   
   }

   checkXbm $idx $cv.$name $type

   if { [cequal $type UifButton] } { 
      set ckDots($idx) [$cv.$name cget -popup] 
      checkDots $idx
   }
}
