#
# E.S.O. - VLT project
#
# "@(#) $Id: configOutput.tcl,v 5.34 2005/09/26 12:44:18 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.COMIN   ??/??/??  created 
# M.COMIN   08/05/95  Add processing of real number format
# M.COMIN   18/10/95  Add event mechanism
#
# mcomin    27/02/96  Porting to TK4.0
#

#----------------------------------------------------------------------------
# PROCEDURE : uifOutputPanel -  Creates config. panel for UifOutput Widget
#----------------------------------------------------------------------------

proc uifOutputPanel { idx top type } {
    addFormatAndUnits $idx $type
    configAnchor $idx

    if {[cequal $type UifOutputList]} {
	addKeyList $idx $top  
    }
}



#------------------------------------------------------
# PROCEDURE : addKeyList
# Creates the part to configure list of keywords and colors
#------------------------------------------------------
proc addKeyList {idx top } {
   global wdgConfig

   set wdgKeyList  [getWdgConfig $idx wdgKeyList]
   set wdgColList  [getWdgConfig $idx wdgColList]
    set fl          [expr {$wdgConfig(fieldLen) + 10}]

   UifEntry $wdgKeyList -label " Keywords List" -labelFont $wdgConfig(lblFont)\
            -width $fl -labelWidth $wdgConfig(lblWidth)

   UifEntry $wdgColList -label " Color List" -labelFont $wdgConfig(lblFont)\
            -width $fl -labelWidth $wdgConfig(lblWidth)

   pack $wdgKeyList $wdgColList -pady 5 -anchor w

    set configHelp [getWdgConfig $idx configHelp]

    set tmp "List of keywords that need a different color"
    UifShortHelp::add $configHelp $wdgKeyList $tmp

    set tmp "List of colors associated to each keyword"
    UifShortHelp::add $configHelp $wdgKeyList $tmp
}


#----------------------------------------------------------------------------
# PROCEDURE : configUifOutput - Updates the data structures describing the
#                               configuration of the UifOutput Class
#----------------------------------------------------------------------------

proc configUifOutput { idx cv top type name } {
   global wdgConfig panOutputAnchor
   global ckFrame i_outFormat i_precision 

   if {$ckFrame($idx) == 0} {    
      set tmp  [$cv.$name cget -variable ] ;# In case of no frame this helps
      uplevel #0 set $tmp 0                ;# identify the widget position
   }

   $cv.$name configure -color  [getWdgConfig $idx color] \
                       -font   [getWdgConfig $idx font] \
                       -anchor $panOutputAnchor($idx) -frame $ckFrame($idx)

   if { ! [cequal $type UifOutput] && ! [cequal $type UifOutputList] } { return }

   if { [cequal $type UifOutputList]} {
       $cv.$name configure -listKey [getConfigInput $idx wdgKeyList] \
	       -listColor  [getConfigInput $idx wdgColList]
   }

   set outFormat [getConfigInput $idx wdgFormat]
   saveWidget $name $i_outFormat $outFormat

    #
    #  Format of reals in UifOutput widgets only. Variable tcl_precision is
    # used within seq_dbReadSymbolic to convert the number into a string
    # using given number of digits, for example :
    #
    #  tcl_precision      value
    #  ----------------------------
    #      4            123.4
    #      7            123.1235
    #

    set prec ""
    if {[regexp {(%+|%-|%)([0-9]*).([0-9]*)([a-z])}  $outFormat \
	     all tmp prec field what ] } {
	if { $what != "f"} { set prec "" }
    }
    
    $cv.$name configure -units [getInput [getWdgConfig $idx wdgUnits]]

    saveWidget $name $i_precision $prec
}

#----------------------------------------------------------------------------
# PROCEDURE : displayUifOutput - Updates the configuration panel according
#                                to the current UifOutput Class configuration
#----------------------------------------------------------------------------

proc displayUifOutput { idx cv top name } {
   global panOutputAnchor panSelectedColor panSelectedFont
   global wdgConfig ckFrame i_outFormat

   set ckFrame($idx)         [$cv.$name cget -frame]
   set panOutputAnchor($idx) [$cv.$name cget -anchor]

   set panSelectedFont    [$cv.$name cget -font]
   setWdgConfig $idx font [$cv.$name cget -font]

   set wtype [winfo class $cv.$name]

   # Parameters common to UifOutput and UifOutputList

   if {[cequal $wtype UifOutput] || [cequal $wtype UifOutputList]} {

      set panSelectedColor    [$cv.$name cget -color]
      setWdgConfig $idx color [$cv.$name cget -color]

      setEntry [getWdgConfig $idx wdgUnits] [$cv.$name cget -units]
      setConfigEntry $idx wdgFormat  [getWidget $name $i_outFormat] 
      [getWdgConfig $idx wdgColor]   configure -fg $panSelectedColor
   }

   # Parameters specific to UifOutputList

   if {[cequal $wtype UifOutputList]} {
      setEntry [getWdgConfig $idx wdgKeyList] [$cv.$name cget -listKey]
      setEntry [getWdgConfig $idx wdgColList] [$cv.$name cget -listColor]
   }
}
