#***************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: configPanels.tcl,v 5.34 2005/09/26 12:44:18 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.Bernini 24/05/94  created 
# M.Comin   05/07/94  Store it into a separate file .. with comments
# M.Comin   27/09/94  Add env. name entry widget
# ------------------
# M.COMIN   08/02/95  Added new frame (tcl3) in the definition of TCL commands
# M.COMIN   15/02/95  Add user defined widget name.
# M.COMIN   16/03/95  Add configuration for 'UifFrames'
# M.COMIN   21/03/95  Add configuration for 'freeText'
# M.COMIN   30/10/95  panParseVar strips away class cwp.
# ------------------
# M.COMIN   31/01/96  New implementation of var. parsing
#
# mcomin    27/02/96  Porting to TK4.0
#           28/04/96  myCwp is now accessible through a method.
#
#
#************************************************************************
#   NAME
#
#   configPanels - Contains routines to create the panels to 
#                  configure the widgets
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   This procedure creates all parts which are common to several
#   widget configuration panels.
#   The configuration panel is divided in 'row' by means of into frames.
#
#   f1       widget coord. , widget length and env. name
#   f2       widget name
#   f3       widget label, class name
#   f4       Length of the label and label font
#            also used by UifBitmap, UifButton, UifClass
#   f5       widget variable, class working point
#   f6       Database Configuration
#   f7       default value , class frame
#   f8       units : UifOutput, UifEntry and used by  UifText
#   f9       free
#
#   f10      Used by : UifFrame, UifBitmap, UifButton, uifOptionButton
#            UifOutput, UifOutputString, UifArrowsButton, UifOutputLed
#
#   f11      Used to contain the stateVariable
#   f12      Ranges (Min,Max  Off,On values)
#            ON colors for UifOutputLed
#   f13      Geometry ; OFF colors for UifOutputLed ; Used by UifOutputArray
#   f14      Orientation, selectmode
#   f15      Anchor Configuration + Used by : uifBarChart, UifListBox, UifEntry
#   f16      Configure List Of Values : UifOutputString    
#   f17      free            
#   f18      Tcl Command
#   f17      free
#   f20      Short Help
#
#   FILES 
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#----------------------------------------------------------------------------
#
#
#----------------------------------------------------------------------------
# PROCEDURE : setConfigWdgName
#----------------------------------------------------------------------------

proc initWdgConfig { idx top bot } {
   global wdgConfig

   setWdgConfig $idx wdgX        $top.f1.posx
   setWdgConfig $idx wdgY        $top.f1.posy
   setWdgConfig $idx wdgLen      $top.f1.wdgLen
   setWdgConfig $idx wdgEnv      $top.f1.wdgEnv

   setWdgConfig $idx wdgName     $top.f2.wdgName
   setWdgConfig $idx wdgLabel    $top.f3.wdgLabel
   setWdgConfig $idx wdgLblSize  $top.f3.wdgLblSize

   setWdgConfig $idx lblParFrame  $top.f4
  
   setWdgConfig $idx wdgVar      $top.f5.wdgVar
   setWdgConfig $idx wdgDef      $top.f7.wdgDef

   setWdgConfig $idx frameFormat $top.f8   
   setWdgConfig $idx wdgUnits    $top.f8.wdgUnits 
   setWdgConfig $idx wdgFormat   $top.f8.wdgFormat

   setWdgConfig $idx wdgKeyList    $top.f9.wdgKeyList
   setWdgConfig $idx wdgColList    $top.f9.wdgColList

   setWdgConfig $idx frameFont      $top.f10
   setWdgConfig $idx frameStep      $top.f10
   setWdgConfig $idx frameLedFlash  $top.f10

   setWdgConfig $idx wdgStateVar $top.f11.wdgStateVar

   setWdgConfig $idx frameRange  $top.f12 ;# Ranges (Min,Max  Off,On values)

   setWdgConfig $idx frameArray  $top.f13 ;# Size + padding for OutputArray

   setWdgConfig $idx frameOrient $top.f14

   setWdgConfig $idx frameAnchor    $top.f15
   setWdgConfig $idx wdgDigits      $top.f15.digits
   setWdgConfig $idx wdgResolution  $top.f15.resolution

   setWdgConfig $idx frameListValues  $top.f16  
  
   setWdgConfig $idx frameCmd    $top.f18 ;# Tcl Command

   setWdgConfig $idx wdgSH       $top.f20.wdgSH ;# Short Help

   # Set as dummy variable (default)
   setWdgConfig $idx lblFont     "-Adobe-helvetica-medium-r-normal--14*-iso8859-1"

   set wdgConfig(lblFont)  "-Adobe-courier-bold-r-normal--12*-iso8859-1"
   set wdgConfig(lblWidth) 16
   set wdgConfig(shWidth)  58
   set wdgConfig(fieldLen) 40
}

#-----------------------------------------------------------------------------
# PROCEDURE : getWdgConfig -  Retrieves the configuration parameter from one
#                             one of the active widget configuration panels.
#
#             setWdgConfig -  Set the new widget config parameter
#-----------------------------------------------------------------------------

proc getWdgConfig { idx what } {
   return [uplevel #0 set wdgConfig${idx}($what)]
}

proc setWdgConfig { idx what value } {
   uplevel #0 set wdgConfig${idx}($what) \"$value\"
}


proc getConfigInput { idx what } {
   set w [getWdgConfig $idx $what]
   return [getInput $w]
}

#-----------------------------------------------------------------------------
# PROCEDURE : createConfigPanel -  Create the widget configuration panel.
#                                  The panel is formed by a common part and a 
#                                  part configured according to the type of
#                                  widget                           
#-----------------------------------------------------------------------------

proc createConfigPanel {type name cv } {
   global par wdgConfig  i_dbVar
   global wdgConfig ckDbVar ckDbOnvalue ckDbOffvalue ckDbMin ckDbMax

   set idx 0
   set topWdg $par(CONFIG_PANEL)${idx}

   while {[winfo exists $topWdg] && $idx < 3 } {

       # Check if the used panel refers to the current widget
       set tmp [getWdgConfig $idx oldWdgPath]
       if {[cequal $tmp "$cv.$name"]} { raise $topWdg ; return }
       incr idx
       set topWdg $par(CONFIG_PANEL)${idx}
   }

   if {$idx >= 3 } {
      set    msg "Too Many Configuration Panel\nOpened at the same time !!"
      append msg "\n\n CANNOT OPEN A NEW ONE"
      tk_dialog .dialog "Dialog Box" $msg info 0 OK
      return
   }

#   puts "DBG > createConfigPanel --- $cv $type, $name  idx = $idx"

   #
   # Initialize some configuration variables 
   #
   set ckDbVar($idx) 0 ; set ckDbOnvalue($idx) 0 ; set ckDbOffvalue($idx) 0 
   set ckDbMin($idx) 0 ; set ckDbMax($idx) 0

   setWdgConfig $idx topLevel $topWdg

   toplevel $topWdg          ;# Define the outer most part of the panel 
   wm withdraw $topWdg;		# needed to make "tkwait visibility" work
   wm resizable $topWdg 0 0  ;# disable resize

   label $topWdg.lbl -font "-Adobe-courier-bold-r-normal--14*-iso8859-1" \
                     -relief groove -bd 2

   set top  [frame $topWdg.top]
   set bot  [frame $topWdg.bot -relief raised -border 2] 

   pack $topWdg.lbl -fill both 
   pack $top -fill both  -padx 3 -pady 10 ;# Create top    frame
   pack $bot -fill both  -padx 3 -pady 3  ;# Create bottom frame

   initWdgConfig $idx $top $bot

   #  Create Main Frames
   #  These frames structure the whole panel in rows

   set padFrames [list 0 3 3 3 3 3 10 3 3 3 3 3 3 3 3 3 3 3 3 3 3]
   set maxFrames 20
   for {set i 1} { $i <= $maxFrames } {incr i } {
       frame  $top.f${i}
   }

   setWdgConfig $idx oldName    $name
   setWdgConfig $idx oldWdgPath $cv.$name

   set lblFont     $wdgConfig(lblFont)
   set lblWidth    $wdgConfig(lblWidth)
   set fieldLen    $wdgConfig(fieldLen)

   createBottomPart $idx $cv $name $topWdg $type $bot

   set tmp [wm protocol $topWdg WM_DELETE_WINDOW]
   wm protocol $topWdg  WM_DELETE_WINDOW "$tmp ; Cancel $idx $topWdg $type $cv"

   #----------------------------------------------------------
   #               Definition of common widgets
   #----------------------------------------------------------
   
   set configHelp  [getWdgConfig $idx configHelp]
   set w1 [getWdgConfig $idx wdgX]
   set w2 [getWdgConfig $idx wdgY]
   set w3 [getWdgConfig $idx wdgName]

   UifEntry $w1 -label " X" -labelFont $lblFont -width 4
   UifEntry $w2 -label " Y" -labelFont $lblFont -width 4

   set tmp "Widget Coordinates (upper left corner is the origin)"
   UifShortHelp::add $configHelp $w1 $tmp
   UifShortHelp::add $configHelp $w2 $tmp

   UifEntry $w3 -label " Widget Name" -width $fieldLen -labelWidth $lblWidth \
            -labelFont $lblFont

   pack $w1 $w2 -side left -pady 10
   pack $w3 -side left

   #----------------------------------------------------------
   #                    Creation of the panel
   #
   # Widget have different configuration options which are enabled
   # by setting the optList variable
   #----------------------------------------------------------

   set optList [panGetOptionList $type]

   setWdgConfig $idx optList $optList

   if {[lsearch $optList "length"] != -1 } {
      set w [getWdgConfig $idx wdgLen]
      UifEntry $w  -label " Length" -labelFont $lblFont -width 4
      pack $w -side left
   }

   # Special case for xxx_uifClass widgets

   if {[cequal $type UifClass]} {
      set w [getWdgConfig $idx wdgEnv]
      UifEntry $w -label " Env." -labelFont $lblFont -width 10
      pack $w -side right -padx 10 
   }

   if {[lsearch $optList "noVar"] == -1  } {
      #    
      # Environment Name
      #
      set w [getWdgConfig $idx wdgEnv]
      UifEntry $w -label " Env." -labelFont $lblFont -width 10

      UifShortHelp::add $configHelp $w\
      {CCS environment : starts with 'w' for WS and 'l' for LCU }

      pack $w -side left

      #
      # Widget Variable
      #
      set w [getWdgConfig $idx wdgVar]

      UifEntry $w -label " Variable" -width $fieldLen\
               -labelWidth $lblWidth -labelFont $lblFont

      # Set Short Help

      switch $type {
         UifButton - UifActionButton {
             set tmp "Name of the variable used to enable/disable the widget"
         }
         UifListBox - UifRadioButton {
             set tmp "Name of the variable where to store the selection"
         } 
         UifEntry - UifScale {
             set tmp "Name of the variable where to store the input"
         }
         default { 
             set tmp "Name of the variable to read the displayed quantity"
         }    
      }

      UifShortHelp::add $configHelp $w $tmp

      checkbutton ${w}DB  -text  "DB" -variable ckDbVar($idx) \
               -command "checkDbVar $idx $cv.$name $top.f6"

      if {[cequal $type UifArrowsButton]} {
         ${w}DB configure -state disabled
      }

      pack $w      -side left
      pack ${w}DB  -side right -padx 3
   }

   if {[lsearch $optList "stateVar"] != -1 } {
      #    
      # State Variable
      #
      set w [getWdgConfig $idx wdgStateVar]

      UifEntry $w -label " State Variable" -width $fieldLen\
               -labelWidth $lblWidth -labelFont $lblFont
      pack $w -side left

#      checkbutton ${w}DB -text "DB" -variable ckDbDef($idx) -state disabled
#      pack ${w}DB -side left -padx 15

      UifShortHelp::add $configHelp $w \
           {Name of the variable used to control the state of the widget }
    }


   if {[lsearch $optList "default"] != -1 } {
      #    
      # Default Value
      #
      set w [getWdgConfig $idx wdgDef]

      UifEntry $w -label " Default" -width $fieldLen\
               -labelWidth $lblWidth -labelFont $lblFont

      checkbutton ${w}DB -text "DB" -variable ckDbDef($idx) -state disabled

      pack $w -side left
      pack ${w}DB -side right -padx 3

      set tmp "Value set by 'default' when the widget is created"
      UifShortHelp::add $configHelp $w $tmp
   }

   #    
   # Widget Label and label size for a few widgets
   #
   if {[lsearch $optList "label"] != -1 } {

      set w [getWdgConfig $idx wdgLabel]

      UifEntry $w -label " Widget Label" -width $fieldLen\
               -labelWidth $lblWidth -labelFont $lblFont

      pack $w -side left
   }

   #    
   # Add fields to configure length and font associated with the label
   #

   if {[lsearch $optList lblOptions] != -1 } {

      set w [getWdgConfig $idx lblParFrame]

      UifEntry $w.lblSize -label " Label Size" -width 5 \
               -labelWidth $lblWidth -labelFont $lblFont

      if {[cequal $type freeText]} { $w.lblSize disable }
      label $w.lblFont -text "   Label Font " -font $lblFont

      createFontButton $w.selFont 10 wdgConfig${idx}(lblFont)\
                       [uplevel #0 set freeTextFonts]

      pack $w.lblSize $w.lblFont $w.selFont -side left
   }

   #----------------------------------------------------------
   #         Creates specific configuration 
   #----------------------------------------------------------

   #  Creates the part to configure the short help

   configShortHelp $idx $top $type

   if {[lsearch $optList "event"] != -1 }   { databaseConfig $idx $top.f6 }
   if {[lsearch $optList "command"] != -1 } { configCommand $idx $top $type }
   if {[lsearch $optList "range"] != -1 }   { configRange $idx }
   if {[lsearch $optList "orient"] != -1 }  { configOrientation $idx }
   if {[lsearch $optList "font"] != -1 }    { configFont $idx $type}

   set frameOrient [getWdgConfig $idx frameOrient]

   switch $type {
      UifBitmap        { uifBitmapPanel   $idx $top }
      UifArrowsButton  {arrowsButtonPanel $idx $top $type $cv.$name} 
      UifActionButton  { buttonPanel      $idx $top $type $cv.$name}      
      UifButton        { buttonPanel      $idx $top $type $cv.$name}
      UifCheckButton   { panelRCButton    $idx $type }
      UifRadioButton   { panelRCButton    $idx $type }
      UifEntry         { uifEntryPanel    $idx $top $name }
      UifClass         { uifClassPanel    $idx $top }
      PubClass         { pubClassPanel    $idx $top }
      UifFrame         { uifFramePanel    $idx $top }
      UifText          { uifTextPanel     $idx $top }
      UifListBox       { uifListBoxPanel      $idx $top $type } 
      UifOptionButton  { uifOptButtonPanel    $idx $top $type }
      UifBarChart      { barChartPanel        $idx $top.f15 }
      UifOutput        { uifOutputPanel       $idx $top $type }
      UifOutputList    { uifOutputPanel       $idx $top $type }
      UifOutputLed     { uifOutputLedPanel    $idx $top $type }
      UifOutputString  { uifOutputStringPanel $idx $top $type }
      UifOutputIcon    { uifOutputIconPanel   $idx $top $type }
      UifOutputArray   { uifOutputArrayPanel  $idx $top $type }
      UifScale         { uifScalePanel        $idx $top $type }
      freeText         { freeTextPanel        $idx $top }
   } ;# end of switch 

   if {[lsearch $optList "format"] != -1 }  { configFormat $idx }
   if {[lsearch $optList "units"] != -1 }   { configUnits $idx }


   wm group     $topWdg .
#   wm transient $topWdg .
   wm geometry  $topWdg +$par(POPUP_POSX)+$par(POPUP_POSY)
   wm title     $topWdg "Configuration Panel"
   $topWdg.lbl  configure -text "Type Of Widget  --  $type" \
                          -background [lindex $wdgConfig(lockColor) $idx]  

   #
   # Update  panel with current config.
   #

#   panSetConfigHelp $idx $top $bot $topWdg.bot.sh $type 
   displayConfig    $idx $cv $top $name $type ;# Update config panel

   # Highlight the widget under editing by changing the background color 

   if {[cequal $type freeText]} {
      if {[set oldBgColor [$cv itemcget $name -fill]] == "" } {
         set oldBgColor Black
      }
   } else {
      if {[set oldBgColor [$cv.$name cget -background]] == "" } {
         set oldBgColor [$cv.$name cget -background]
      }
   }

   setWdgConfig $idx oldBgColor $oldBgColor
   set configColor [lindex $wdgConfig(lockColor) $idx]

   switch $type {
      freeText { $cv itemconfigure $name -fill $configColor }
      UifClass { $cv.$name.cv config -background $configColor }
      default  { $cv.$name config    -background $configColor }
   }

   for {set i 1} { $i <= $maxFrames} {incr i } {
       if {[winfo children $top.f${i}] != ""} { 
           pack $top.f${i} -fill x -pady [lindex $padFrames $i]
       }
   }

   wm deiconify $topWdg
   tkwait visibility $topWdg
}


#-----------------------------------------------------------------------------
# PROCEDURE : createBottomPart - Adds all the buttons wllowing the user
# to apply changes to the widget's configuration
#-----------------------------------------------------------------------------

proc createBottomPart { idx cv name c type bot } {
   set line1 [frame $bot.line1]
   set line2 [frame $bot.line2]
   setWdgConfig $idx configHelp $bot.sh

   UifShortHelp $bot.sh

   pack $bot.sh  -fill x 
   pack $line1 $line2 -fill x -pady 5

   # Add buttons

   UifActionButton $line2.delete  -label "Delete" -width 10 \
	  -activeforeground yellow -activebackground plum \
          -background red -foreground yellow

   # This is configure depending on the type of 'item', a widget or a canvas
   # item

   if {[cequal $type freeText]} {
      set tmp "$cv delete $name; Cancel $idx $c $type $cv"
   }  else {
      set tmp "panDeleteWidget $cv $name; Cancel $idx $c $type $cv"
   }

   $line2.delete configure -command $tmp


   UifActionButton $line1.default  -label "Defaults" -width 10 \
                                   -command "setDefaults $idx $type "
                                
   UifActionButton $line2.ok   -label "OK"     -width 10 \
                   -command "panApplyConfig $idx $cv $type  $c Ok"
                              
   UifActionButton $line1.clear   -label "Clear" -width 10 \
                                  -command "clearConfig $idx"

   UifActionButton $line2.cancel  -label "Cancel" -width 10 \
                                  -command "Cancel $idx $c $type $cv"

   UifActionButton $line1.apply   -label "Apply" -width 10 \
                   -command "panApplyConfig $idx $cv $type $c Apply"

   UifActionButton $line1.help  -label "Help" -width 10 -state disabled

   pack $line1.apply $line1.default $line1.clear  -side left  -padx 7
   pack $line1.help              -side right -padx 3
   pack $line2.ok $line2.cancel  -side left  -padx 7
   pack $line2.delete            -side right -padx 3
}


##################################################################
#
#     Set of routines to check the configuration options
#
##################################################################

#-----------------------------------------------------------------------------
# PROCEDURE : panApplyConfig
#-----------------------------------------------------------------------------
proc panApplyConfig {idx cv type top what} {
   if {[saveConfig $idx $type $cv] == "FAILURE"} { return }

   if {$what == "Ok"} { Cancel $idx $top $type $cv }
}


#-----------------------------------------------------------------------------
# PROCEDURE : checkShHelp - Create or Destroy short help area depending on
#                           the panel configuration
#-----------------------------------------------------------------------------
proc checkShHelp { } {
   global ckShHelp shArea

   if { $ckShHelp } { 
      panCreateShArea $shArea
   } else {      
      destroy $shArea
   }
}

#-----------------------------------------------------------------------------
# PROCEDURE : checkHeartBeat - Create or Destroy the heart-beat
#-----------------------------------------------------------------------------
proc checkHeartBeat { } {
   global ckShHelp shArea ckHeartBeat

   if { $ckShHelp } { 
      $shArea configure -showHeartBeat $ckHeartBeat
   }
}


#----------------------------------------------------------------------------
# PROCEDURE : checkMenuBar - Create or Destroy the Menu Bar area depending on
#                            the panel configuration
#----------------------------------------------------------------------------
proc checkMenuBar { } {
   global ckMenuBar menuBar mbArea
   
   if { $ckMenuBar == 0 } {
      pack forget $menuBar 
   } else {

      if {[winfo children $menuBar] == "" } {
         # The menuBar is empty .. so set the default configuration
         panCreateNewMenuBar $menuBar
         panSetMenuBind $mbArea
      }

      pack $menuBar -fill x
   }
}

#----------------------------------------------------------------------------
# PROCEDURE : getConfigColor
#----------------------------------------------------------------------------

proc getConfigColor { idx paramName } {
   global  panSelectedColor
   set panSelectedColor [getWdgConfig $idx $paramName]
   set tmp [panelColor]
   setWdgConfig $idx $paramName $tmp
   return $tmp
}


#----------------------------------------------------------------------------
# PROCEDURE : configColor -
#----------------------------------------------------------------------------
proc configColor idx {
   global  panSelectedColor

   set panSelectedColor [getWdgConfig $idx color]
   set w   [getWdgConfig $idx wdgColor]
   set tmp [panelColor]
   $w configure -foreground $tmp
   setWdgConfig $idx color $tmp
}

#----------------------------------------------------------------------------
# PROCEDURE : configShortHelp
#----------------------------------------------------------------------------
proc configShortHelp { idx top type } {
   global wdgConfig

   set w [getWdgConfig $idx wdgSH]

   label ${w}lbl -text "Short Help Text " -anchor w  -font $wdgConfig(lblFont)
   entry $w -width $wdgConfig(shWidth) -state normal

   pack ${w}lbl -anchor w -pady 10
   pack $w      -anchor w

   UifShortHelp::add [getWdgConfig $idx configHelp] $w \
                     { Define Short Help Text for the widget }

   if {[cequal $type freeText]} {
      $w configure -state disabled -relief flat
   }
}

#------------------------------------------------------
# PROCEDURE : configCommand
# Creates the part to configure the -command option
#------------------------------------------------------

proc configCommand { idx top type } {
   global wdgConfig

   set row1 [frame [getWdgConfig $idx frameCmd].r1]
   set row2 [frame [getWdgConfig $idx frameCmd].r2]

   pack $row1 $row2 -fill x

   set wdgPan $row1.wdgPan ;   setWdgConfig $idx wdgPan $wdgPan
   set wdgAct $row1.wdgAct ;   setWdgConfig $idx wdgAct $wdgAct
   set wdgCmd $row2.wdgCmd ;   setWdgConfig $idx wdgCmd $wdgCmd


   label  ${wdgPan}lbl -text "TCL Command" -font $wdgConfig(lblFont)
   button $wdgAct -text "Select Action ..." -width 15  \
                  -command "selectCmd $idx browse"
   button $wdgPan -text "Select Panel  ..." -width 15 \
                  -command "selectCmd $idx browse_dots"

   UifEntry $wdgCmd -width $wdgConfig(shWidth)

   # Select mode 

   if {! [cequal $type UifButton] && ! [cequal $type UifBitmap]} { 
      $wdgPan configure -state disabled 
   }

   if {[cequal $type UifOptionButton] } { 
      $wdgAct configure -state disabled 
   }

   switch $type {
     UifActionButton -
     UifCheckButton  -
     UifRadioButton  -
     UifOptionButton { set what "actButton" }
     default         { set what "browse" }
   }

   $wdgAct configure -command "selectCmd $idx $what"
   
   pack ${wdgPan}lbl -side left  -padx 3
   pack $wdgCmd      -side left  -pady 5
   pack $wdgPan $wdgAct -side right -pady 5 -padx 10
}

#------------------------------------------------------
# PROCEDURE : configRange
# Creates the part to configure the Min/max values
#------------------------------------------------------
proc configRange {idx } {
   global wdgConfig

   set row1 [frame [getWdgConfig $idx frameRange].r1]
   set row2 [frame [getWdgConfig $idx frameRange].r2]

   pack $row1 $row2 -fill x

   set wdgMin $row1.wdgMin ;   setWdgConfig $idx wdgMin $wdgMin
   set wdgMax $row2.wdgMax ;   setWdgConfig $idx wdgMax $wdgMax

   UifEntry $wdgMax -label " Max. Value" -labelFont $wdgConfig(lblFont)\
            -width $wdgConfig(fieldLen) -labelWidth $wdgConfig(lblWidth)

   UifEntry $wdgMin -label " Min. Value" -labelFont $wdgConfig(lblFont)\
            -width $wdgConfig(fieldLen) -labelWidth $wdgConfig(lblWidth)

   checkbutton ${wdgMin}DB -text "DB" -variable ckDbMin($idx)  
   checkbutton ${wdgMax}DB -text "DB" -variable ckDbMax($idx)   

   pack $wdgMin -side left
   pack ${wdgMin}DB -side left -padx 15

   pack $wdgMax   -side left
   pack ${wdgMax}DB -side left -padx 15

   set tmp "Set check button ON to declare the parameter as DB attribute"
 
   set configHelp [getWdgConfig $idx configHelp]

#   UifShortHelp::add $configHelp [getWdgConfig $idx wdgVar]DB $tmp
   UifShortHelp::add $configHelp ${wdgMin}DB $tmp
   UifShortHelp::add $configHelp ${wdgMax}DB $tmp
}

#------------------------------------------------------
# PROCEDURE : configOrientation
#------------------------------------------------------
proc configOrientation { idx } {
   global par wdgConfig

   set frameOrient [getWdgConfig $idx frameOrient]

   set wdgVert  $frameOrient.wdgVert  ; setWdgConfig $idx wdgVert  $wdgVert
   set wdgHoriz $frameOrient.wdgHoriz ; setWdgConfig $idx wdgHoriz $wdgHoriz

   label $frameOrient.lbl -text " Orientation" \
         -font $wdgConfig(lblFont) -anchor w -width $wdgConfig(lblWidth)

   radiobutton $wdgHoriz -text "Horizontal" -width 8\
              -variable ckOrientation($idx) -value $par(HORIZONTAL) -anchor w

   radiobutton $wdgVert -text "Vertical" -width 8\
              -variable ckOrientation($idx) -value $par(VERTICAL) -anchor w

   pack $frameOrient.lbl $wdgHoriz $wdgVert -side left 
}

#------------------------------------------------------
# PROCEDURE : configAnchor
#------------------------------------------------------

proc configAnchor { idx } {
   global wdgConfig panOutputAnchor

   set top [getWdgConfig $idx frameAnchor]


   label $top.layout -text " Position :" -anchor w -font $wdgConfig(lblFont)\
                   -width $wdgConfig(lblWidth)

   radiobutton $top.left    -text "Left" -width 8 -value "w" \
                             -variable panOutputAnchor($idx) -anchor w

   radiobutton $top.center  -text "Center" -width 8 -value "center" \
                            -variable panOutputAnchor($idx) -anchor w

   radiobutton $top.right   -text "Right"  -width 8 -value "e" \
                            -variable panOutputAnchor($idx) -anchor w 

   pack $top.layout $top.left $top.center $top.right -side left
}


#------------------------------------------------------
# PROCEDURE : configFormat
#------------------------------------------------------
proc addFormatAndUnits { idx type } {
   global wdgConfig

   set wdgFormat [getWdgConfig $idx wdgFormat]
   set wdgUnits  [getWdgConfig $idx wdgUnits ]

   UifEntry $wdgFormat -label " Format" -width 8\
            -labelWidth $wdgConfig(lblWidth) -labelFont $wdgConfig(lblFont)

   UifEntry $wdgUnits -label "  Units" -width 20\
             -labelWidth 10 -labelFont $wdgConfig(lblFont)

   $wdgFormat disable
   $wdgUnits  disable

   pack $wdgFormat $wdgUnits -side left

   # Set configuration short help

   set configHelp [getWdgConfig $idx configHelp]

   UifShortHelp::add $configHelp $wdgFormat \
        {Display format for Database values (same as printf format)}

   UifShortHelp::add $configHelp $wdgUnits \
        {String describing the nature of the displayed value }
}

proc configFormat { idx } {
   global wdgConfig
   [getWdgConfig $idx wdgFormat] enable
}


proc configUnits { idx } {
   global wdgConfig
   [getWdgConfig $idx wdgUnits] enable
}


#------------------------------------------------------
# PROCEDURE : configFont
#------------------------------------------------------
proc configFont { idx type } {
   global wdgConfig ckFrame

   set top [getWdgConfig $idx frameFont]

   set wdgColor  $top.wdgColor  ; setWdgConfig $idx wdgColor  $wdgColor
   set wdgFont   $top.wdgFont   ; setWdgConfig $idx wdgFont   $wdgFont

   button $wdgColor -text "Color ..." -width 10 -command "configColor $idx"\
                    -font "-Adobe-courier-bold-r-normal--12*-iso8859-1"

   label $top.lblFont -text " Font " -font $wdgConfig(lblFont)\
                      -width  $wdgConfig(lblWidth) -anchor w
 
   createFontButton $wdgFont  10 wdgConfig${idx}(font)\
                       [uplevel #0 set outputFonts]

   checkbutton $top.frame  -text "Frame"  -variable ckFrame($idx)

   pack $top.lblFont $wdgFont -side left

   if { ![cequal $type UifOutputString] && ![cequal $type UifText] } {
      pack $wdgColor -side left -padx 20
   }

   if { ![cequal $type UifText] } {
      pack $top.frame -side left -padx 20
   }

   # Set configuration short help

   set configHelp [getWdgConfig $idx configHelp]

   UifShortHelp::add $configHelp $top.frame \
        {Enable the drawing of a frame around the widget}
}


#-----------------------------------------------------------------------------
# PROCEDURE : Cancel - Close configuration panel without applying any change
#-----------------------------------------------------------------------------
proc Cancel { idx c type cv} {
   global wdgConfig

   set oldWdgPath [getWdgConfig $idx oldWdgPath]

   if {[uplevel #0 info exists wdgConfig${idx}(oldBgColor)]} {
      set oldBgColor [getWdgConfig $idx oldBgColor]

      switch $type {
         freeText {       
             set pos  [string last "." $oldWdgPath] ; incr pos
             set name [string range $oldWdgPath $pos end]
             $cv itemconfigure $name -fill $oldBgColor
         }
         UifClass { 
             if {[winfo exists $oldWdgPath] } {
                $oldWdgPath.cv configure -background $oldBgColor
             }
         }
         default  {
             if {[winfo exists $oldWdgPath] } {
                $oldWdgPath configure -background $oldBgColor
             }
         }
      }
   }

   setWdgConfig $idx oldWdgPath ""
   setWdgConfig $idx oldName    ""

   destroy $c
}

#-----------------------------------------------------------------------------
# PROCEDURE : panGetOptionList - 
#
#            Widget have different configuration options which are enabled
#            by setting the optList variable
#-----------------------------------------------------------------------------
proc panGetOptionList { type } {

   set optList "noVar" ;# Default configuration

   switch $type {
      UifBitmap        {set optList "noVar label command"  } 
      UifArrowsButton  {set optList "command stateVar"} 
      UifActionButton  {set optList "noVar label command length stateVar"}
      UifButton        {set optList "noVar label command length stateVar"}
      UifCheckButton   {set optList "label default command length" }
      UifRadioButton   {set optList "label default command length" }
      UifEntry         {set optList "units label lblOptions default length range"}
      UifScale         {set optList "label default length range orient"}
      UifFrame         {set optList "noVar length" }
      UifText          {set optList "noVar length font" }
      UifListBox       {set optList "label default"} 
      UifOptionButton  {set optList "length default command"}
      UifBarChart      {set optList "event label length range orient"}
      UifOutput        {set optList "font format units event length \
                                    label lblOptions" }
      UifOutputList    {set optList "font format units event length \
                                    label lblOptions listKey listColor" }
      UifOutputString  {set optList "font event length label lblOptions"}
      UifOutputIcon    {set optList "event label lblOptions listPictures"}
      UifOutputLed     {set optList "event label lblOptions"}
      UifOutputArray   {set optList "event label lblOptions orient \
                                     length format font"}
      freeText         {set optList "label noVar lblOptions" }
   }; # end of switch

   return $optList
}

#----------------------------------------------------------------------------
# PROCEDURE : createFontButton - Creates a menu button with the list
#                                of possible fonts.
# The procedure takes as parameters :
# - widget name
# - widget label
# - length of the widget
# - name of the variable where to store the associated element
# - set of fonts.
#----------------------------------------------------------------------------

proc createFontButton { w width var fontList } {
   set lblList [list Small Normal Big "Small Bold"  Bold "Big Bold" ]

   # The widget is configure with the first font to show the user 
   # the type of font : courier or helvetica.

   UifOptionButton $w -length $width -selectMode value -variable $var \
                      -itemList $lblList  -itemValue $fontList \
                      -font [lindex $fontList 0]
}

