#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: configPubClass.tcl,v 5.34 2005/09/26 12:44:27 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin   28/06/95  created
# M.COMIN  14/08/95  improvements on config. panel.
# M.COMIN  05/09/95  Re-arrangement of panel layout
# M.COMIN  21/12/95  Array resArray was not properly cleared
#
# mcomin   27/02/96  Porting to TK4.0
#          30/06/96  Add class type in config panel
#          31/06/96  Fixed bug for configuration of "" options.
#          04/12/96  Fixed bug : Background was set to green after 
#                    changing the widget name
#
#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

#--------------------------------------------------------------------------- 
# PROCEDURE : pubClassPanel -  Creates config. panel for a mega-widget
#----------------------------------------------------------------------------

proc pubClassPanel { idx top } {
   global wdgConfig resValue resName resList resIdx resArray clLib clName
                  
   #
   #  Definition of widgets
   #

   set resList  $top.f5.resList
   set resName  $top.f6.res
   set resValue $top.f7.val

   set lblFont   $wdgConfig(lblFont)
   set lblWidth  $wdgConfig(lblWidth)
   set fieldLen  37

   UifEntry $top.f3.clName -label " Class Name" -width $fieldLen\
            -labelWidth $lblWidth -labelFont $lblFont -variable clName($idx)

   UifEntry $top.f4.clLib -label " Library" -width $fieldLen\
            -labelWidth $lblWidth -labelFont $lblFont -variable clLib($idx)

   UifEntry $resName -label " Pub. Variable" -width $fieldLen\
            -labelWidth $lblWidth -labelFont $lblFont 

   UifEntry $resValue -label " Value" -width $fieldLen\
            -labelWidth $lblWidth -labelFont $lblFont 

   UifListBox $resList  -geometry 58x5 -label "Public Variables" \
                        -scrollx 1 -scrolly 1 -font $wdgConfig(lblFont)
   #
   #  Create configuration panel
   #

   pack $top.f3.clName $top.f4.clLib -side left
   pack $resList -pady 10  -expand 1
   pack $resName $resValue -side left

   #
   # Set Current Options 
   #  
   $top.f3.clName disable
   $resList   deleteItem 0 end
   set resArray(none) none

   #
   # Set Short Help Text
   # 

   set configHelp [getWdgConfig $idx configHelp]
   UifShortHelp::add $configHelp  $resName \
	         {Resources Name : type <RETURN> to change value}

   UifShortHelp::add $configHelp  $resValue \
	         {Current Resource Value : type <RETURN> to change value}

   UifShortHelp::add $configHelp $resList {List of all Class Public variables }

   #
   # Set Specific bindings
   # 

   $resList setBind <B1-ButtonRelease> "getClassRes"
   bind $resValue   <Return>  "storeClassRes"
   bind $resName    <Return>  "storeClassRes"
}

#------------------------------------------------------------------------------
# PROCEDURE :  Used to setup the current  configuration
#
# - getClassRes   , select a class resource item
# - storeClassRes , update current definition
# - updateListRes , update resource list 
#
#------------------------------------------------------------------------------

proc getResList { text } {
   set name  [lindex $text 0]
   set len   [string length $name] ; incr len
   set value [string range $text $len end]
   set value [string trimleft $value " "]
   return [list $name "$value"]
}

proc getClassRes {} {
   global par resValue resName resList resIdx resArray

   set what [$resList getSelection]; # Get selected resource
   set idx  [$resList getIndex]
   
   storeClassRes ;# Store previous definition

   $resValue delete  0 end ;  $resName delete 0 end

   set tmp [getResList [string trim $what "\{\}"]]
   $resName  insert 0 [lindex $tmp 0]
   $resValue insert 0 [lindex $tmp 1]

    if { [set tmp [expr {$idx - 2}]] > 0 } {
       set w [$resList getComponentPath uifListBox]
       $w yview $tmp     
   }
}

proc storeClassRes { } {
   global par resValue resName resList resIdx resArray

   set name  [$resName get]
   set value [$resValue get]

   if {[info exists resArray($name)] == 0 } {
      $resValue delete 0 end ;  $resName delete 0 end ; return
   }

   set resArray($name) "$name $value"

   # Update list

   $resList deleteItem 0 end
   $resList updateSortList "" "" resArray formatResource
}



#------------------------------------------------------------------------------
# PROCEDURE : displayPubClass -
#                               
#------------------------------------------------------------------------------

proc displayPubClass { idx cv w name } {
   global par resValue resName resList resIdx resArray clLib i_clib clName

   catch {unset resArray}

   foreach item [panGetWdgResources $cv $name list all ] {
      set res [lindex $item 0]
      set resArray($res) $item 
   }

   set clName($idx) [winfo class $cv.$name]
   set clLib($idx)  [getWidget $name $i_clib]

   if {[info exists resArray] == 0} { return }

   $resList updateSortList "" "" resArray formatResource
}


proc formatResource { text } {
   set tmp   [getResList $text]
   set name  [lindex $tmp 0]
   set value [lindex $tmp 1]
   set newText [format "%-18s  %-s" $name "$value"]
   return $newText
}   


#----------------------------------------------------------------------------
# PROCEDURE : configPubClass - Updates the data structures describing the
#                              configuration of the Class
#----------------------------------------------------------------------------

proc configPubClass { idx cv top name } {
   global par resValue resName resList resIdx resArray
   global par clLib clCWP i_clib
    
   storeClassRes
   set resources {}
   foreach item [$resList getList] {

     set tmp [getResList $item]

     if {[set res [lindex $tmp 0]] == "-background"} {
          set value [getWdgConfig $idx oldBgColor]
     } else { 
          set value [lindex $tmp 1]
     }

     if {[string trim $value " "] != "" } {
        if {$value == "\"\""} {
            lappend resources $res ""
        } else {
            lappend resources $res "$value"
        }
     }
   }
  
   # Specific parameters must be configured first
   # -wdgLibrary  belongs to classFrame and classContainer mega-widgets

   if {[set pos [lsearch $resources "-wdgLibrary"]] != -1} {
       $cv.$name configure -wdgLibrary [lindex $resources [expr {$pos +1}]]
       set resources [lreplace $resources $pos [expr {$pos +1}]]
   }

   eval $cv.$name configure $resources
   saveWidget $name $i_clib $clLib($idx)
}

#
# ___oOo___
