
#
# E.S.O. - VLT project
#
# "@(#) $Id: configTclCommand.tcl,v 5.34 2005/09/26 12:44:19 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.Comin   05/07/94  Created
# M.COMIN   27/03/95  TCL command added to Check/Radio Buttons
# M.COMIN   12/04/95  Add user Init/Exit procedures
# M.COMIN   04/05/95  Add check on proc. name
#
# mcomin    27/02/96  Porting to TK4.0
#           04/07/96  Fixed buf for default values in procedures
#                     processed by panGetProcDef 
#           11/12/96  Cleanup of empty lines at the end of procedures
#           06/03/97  Porting to TK 4.2
#           22/11/97  Add .tcl extension to library names (if required)
#           01/05/98  Add new standard procedure xxxInitData
#


#-----------------------------------------------------------------------------
# PROCEDURE : panEditProcedure - Create sub-panel to edit procedures
#-----------------------------------------------------------------------------

proc panEditProcedure { } {
   global par procName panUsrProcs panProcIndex glVarButton fileName

   set c    $par(EDIT_PROC_ROOT) ;# Root name for config. panel
   set FONT $par(FONT_TEXT)

   if {[winfo exists $c]} { wm deiconify $c ; raise $c ; return }

   toplevel $c
   wm withdraw $c

   frame $c.bot -relief groove -border 2;      
   frame $c.top -relief raised -border 2

   set tl1          [frame $c.top.line1]
   set bl1          [frame $c.bot.l1]
   set shHelp       $c.bot.hlp.sh
   set editWdg      $par(EDIT_TEXT)
   set glVarButton  $bl1.glVar

   frame $c.bot.hlp
                                  
   #
   #  Definition of common widgets
   #

   UifMenuButton  $par(EDIT_MENU)  -label "Edit proc ..." -width 14
   label          $tl1.lname       -text "Name of the procedure  "
   entry          $tl1.name        -textvariable procName -bd 2 \
                                   -relief sunken  -width 30
   button $tl1.reLoad -text "Re-Load Procs" -command panReLoadProcs -width 14

   UifText $editWdg 

   UifActionButton $bl1.save -label "Register proc"    -width 14 \
                   -command "panSaveProc $shHelp $editWdg $tl1.proc "

   UifActionButton $bl1.delete -label "Delete proc"  -width 14 \
                   -command "panDeleteProc $editWdg $tl1.proc" \
		   -activeforeground yellow  -activebackground plum \
                   -background red -foreground yellow

   UifActionButton $bl1.clear -label "Clear Text"  -width 14 \
                   -command "$editWdg deleteText 1.0 end"

   UifActionButton $bl1.glVar -label "Set Global Var."  -width 14 \
                   -command "panSetUsrVar $editWdg $tl1.proc"

   UifActionButton $bl1.cancel -label "Cancel"  -width 12 -command "destroy $c"

   #
   #  Create panel
   #
   pack $c.top      -fill both -expand 1 -padx 5 -pady 3 
   pack $c.bot       -fill both  -padx 5 -pady 3
 
   pack $tl1        -fill both  -pady 5
   pack $c.top.edit  -fill both  -expand 1 -pady 5

   pack $c.bot.hlp  $bl1         -fill x

   pack $tl1.lname $tl1.name $tl1.proc  -side left -padx 5 -pady 5
   pack $tl1.reLoad -side left -padx 15
   pack $editWdg               -padx 5

   pack $bl1.glVar $bl1.save $bl1.delete  -side left  -padx 5 -pady 5
   pack $bl1.clear             -side left  -padx 20 -pady 5
   pack $bl1.cancel            -side right -padx 5 -pady 5

   $editWdg config -height 16 -font $par(COUR_R14)

   #
   # Define short help
   #
   UifShortHelp $shHelp ; pack $shHelp -fill x

   UifShortHelp::add $shHelp $bl1.save   {Save/Register the proc definition}
   UifShortHelp::add $shHelp $bl1.delete {Remove proc definition}
   UifShortHelp::add $shHelp $bl1.cancel {Exit the panel}
   UifShortHelp::add $shHelp $bl1.clear  {Clear the contant of the edit window}
   UifShortHelp::add $shHelp $bl1.glVar  {Set User's defined global variables : these variables can be re-defined run-time }
   UifShortHelp::add $shHelp $tl1.proc   {List of known procedures}
   UifShortHelp::add $shHelp $tl1.reLoad \
   {Re-Loads the definitons of procs included in the user's defined libraries}

   panInitProcEditor ;# Initialize proc list

   set myFile   [getMyRoot $fileName]
   set dataFile "${myFile}GlobalData"

   if {[lsearch $panUsrProcs "$dataFile"] != -1 } {
      $bl1.glVar configure -state disabled
   }

   wm title $c "Edit Procedure"
   wm minsize $c 680 400

   wm deiconify $c
   tkwait visibility $c ;# Returns when the panel is 'visible' on screen
}

proc panReLoadProcs {} {
   global auto_path panUsrLib panDEBUG

   catch { auto_reset }

   foreach item $panUsrLib {

      # Add the .tcl extension if required

      if {[cequal [file extension $item] ""]} {
         append item ".tcl"
      }

      foreach filePath $auto_path {

	 if { ! [cequal $item [file tail $filePath]]} { continue }

         # Auto_load files in that directory

         foreach fileName [glob -nocomplain ${filePath}/*.tcl] {

            # Do not auto_load widget class definition again
	    if {[string match "*_uifClass.tcl" $fileName]} { continue }

            if {[catch { auto_load_file $fileName } ret ]} {
               puts "DBG > $ret"
            } else {
               puts "DBG > reLoad $fileName"
            }
         }
      }
   }
}

#-----------------------------------------------------------------------------
# PROCEDURE : panInitProcEditor - Initialization of the proc editor
#-----------------------------------------------------------------------------

proc panInitProcEditor { } {

   global par procName panUsrProcs fileName

   set root [getMyRoot $fileName]

   set initFile "${root}Init"
   set exitFile "${root}Exit"
   set dataFile "${root}GlobalData"
   set initData "${root}InitData"

   set userInitTemplate "proc ${initFile} { } { \n\n\
   # Declare global variables \n\n   global <var> \n\n\
   # User Initialization\n\n   set <var> <value> ; # comment .. \
   \n} "

   set userExitTemplate "proc $exitFile { } { \n\n\
   # Declare global variables \n\n   global <var> \n\n\
   # Statements to be executed when exiting\n\n   \
   \n} "

   set initDataTemplate "proc $initData { } { \n\n\
   # Declare global variables \n\n   global <var> \n\n\
   # Statements to be executed when exiting\n\n   \
   \n} "

   #
   # Initialize menu button
   #

   if {[winfo exists $par(EDIT_PROC_ROOT)] == 0} return;

   $par(EDIT_MENU) deleteEntry  0 ; # Delete previous entries  

   foreach name $panUsrProcs {
      $par(EDIT_MENU) addCommand   $name 0 \
                      "panLoadProc $name $par(EDIT_TEXT) $par(EDIT_MENU)"
   }

   if { [lsearch $panUsrProcs $initFile] == -1} {

       eval $userInitTemplate ; set name $initFile

       lappend panUsrProcs $name ; # Add new proc
       $par(EDIT_MENU) addCommand  $name 0 \
                       "panLoadProc $name $par(EDIT_TEXT) $par(EDIT_MENU)"
   }

   if { [lsearch  $panUsrProcs $exitFile] == -1} {
       eval $userExitTemplate ; set name $exitFile

       lappend panUsrProcs $name ; # Add new proc
       $par(EDIT_MENU) addCommand $name 0 \
                       "panLoadProc $name $par(EDIT_TEXT) $par(EDIT_MENU)"
   }

   if { [lsearch $panUsrProcs $initData] == -1} {
       eval $initDataTemplate ; set name $initData

       lappend panUsrProcs $name ; # Add new proc
       $par(EDIT_MENU) addCommand  $name 0 \
                       "panLoadProc $name $par(EDIT_TEXT) $par(EDIT_MENU)"
   }

   set procName "" ; $par(EDIT_TEXT) deleteText 1.0 end;  # Clear text editor
}

#-----------------------------------------------------------------------------
# PROCEDURE : panSetUsrVar - This routine defines user's global variables
#-----------------------------------------------------------------------------

proc panSetUsrVar { editWdg  procWdg } {
   global procName panUsrProcs fileName

   #
   # Define Empty Template
   #
   set myFile   [getMyRoot $fileName]
   set dataFile "${myFile}GlobalData"

   set globalDataTemplate "proc ${dataFile} { } { \n\n\
   # Declare any variable as global\n\n   global <var> \n\n\
   # Variable Initialization\n\n   set <var> <value> ; # comment .. \
   \n} "

   #
   # Register empty template to the interpreter
   #
   if { [lsearch $panUsrProcs $dataFile] == -1} {
       eval $globalDataTemplate ; # Register template
   }

   panLoadProc $dataFile  $editWdg $procWdg
}

#-----------------------------------------------------------------------------
# PROCEDURE : panSaveProc - This routine change the definition fo the procedure
#                           according to the editing done by the user
#-----------------------------------------------------------------------------

proc panSaveProc { shHelp editWdg  procWdg } {
   global procName panUsrProcs glVarButton fileName par

   incr par(SAVE_FLAG)
   set buf       [$editWdg  getText 1.0 end]
   set procName  [lindex $buf 1]

   if { [lindex $buf 0] != "proc" }           { return }
   if { [string trim $procName " {}"] == "" } { return }

   eval $buf;   # Register the new definition 

   if { [lsearch $panUsrProcs $procName] < 0 } {
     lappend panUsrProcs $procName ; # Add new proc
     $procWdg addCommand $procName 0 "panLoadProc $procName $editWdg $procWdg"
   } 

   set myFile   [getMyRoot $fileName]
   set dataFile "${myFile}GlobalData"

   if {[lsearch $procName $dataFile] != -1 } {
      $glVarButton configure -state disabled
      eval $dataFile;     # Execute the procedure to enforce modifications
   }
}

#-----------------------------------------------------------------------------
# PROCEDURE : panDeleteProc - This routine deletes the procedure from the set
#                          of those known by the TCL interpreter.
#-----------------------------------------------------------------------------

proc panDeleteProc { editWdg procWdg } {
   global procName panUsrProcs par

   if { [set index  [lsearch  $panUsrProcs $procName]] < 0} {
       return;    # proc not present in the list
   } 

   incr par(SAVE_FLAG)
   set panUsrProcs  [lreplace $panUsrProcs $index $index]

   rename $procName {};                # Delete proc from interpreter
   set procName "";                    # Clear proc name from entry
   $editWdg deleteText 1.0 end;        # Clear edit window
   $procWdg deleteEntry $index $index; # Update list of procs
}

#-----------------------------------------------------------------------------
# PROCEDURE : panLoadProc - This routine retrieves the body of a given 
#                           procedure as known by the interpreter and 
#                           display it in the editor text widget
#-----------------------------------------------------------------------------

proc panLoadProc { proc editWdg procWdg } {
   global procName 

   set procName $proc
   $editWdg deleteText 1.0 end
   $editWdg insertText 1.0 [panGetProcDef $proc]
}

#-----------------------------------------------------------------------------
# PROCEDURE : panSetProcList - 
#-----------------------------------------------------------------------------

proc panSetProcList { oldFile } {
   global panUsrProcs fileName

   if { $oldFile == "" } { return }  
   
   set root [getMyRoot $fileName]
   set oldRoot [getMyRoot $oldFile]

##  puts "DBG > panSetProcList : oldRoot = $oldRoot ,  root = $root"
##   puts "DBG > panSetProcList : panUsrProcs = $panUsrProcs"


   if { [set idx [lsearch $panUsrProcs ${oldRoot}Init ]] != -1} {
      set panUsrProcs [lreplace $panUsrProcs $idx $idx "${root}Init"]
      if { [info procs ${root}Init] == "" } {
         rename ${oldRoot}Init ${root}Init
      }
   }

   if { [set idx [lsearch $panUsrProcs ${oldRoot}GlobalData ]] != -1} {
      set panUsrProcs [lreplace $panUsrProcs $idx $idx "${root}GlobalData"]
      if { [info procs ${root}GlobalData] == "" } {
         rename ${oldRoot}GlobalData ${root}GlobalData
      }
   }

   if { [set idx [lsearch $panUsrProcs ${oldRoot}Exit ]] != -1} {
      set panUsrProcs [lreplace $panUsrProcs $idx $idx "${root}Exit"]
      if { [info procs ${root}Exit] == "" } {
         rename ${oldRoot}Exit ${root}Exit  
      }
   }

   if { [set idx [lsearch $panUsrProcs ${oldRoot}InitData ]] != -1} {
      set panUsrProcs [lreplace $panUsrProcs $idx $idx "${root}InitData"]
      if { [info procs ${root}InitData] == "" } {
         rename ${oldRoot}InitData ${root}InitData  
      }
   }

##   puts "DBG > panSetProcList : NEW panUsrProcs = $panUsrProcs"
}


#-----------------------------------------------------------------------------
# PROCEDURE : panStoreProc - This routine stores the procedure definition
#                            in a file
#-----------------------------------------------------------------------------

proc panStoreProc { fid root } {
   global panUsrProcs
   
   foreach proc $panUsrProcs {
     if { $proc == "${root}GlobalData" } { continue }
     if { $proc == "${root}Init" }       { continue }
     if { $proc == "${root}InitData" }   { continue }
     puts $fid "\n"
     puts $fid [panGetProcDef $proc]
   }
}

#-----------------------------------------------------------------------------
# PROCEDURE : panGetProcDef - This routine retrieves the procedure definition
#-----------------------------------------------------------------------------
   
proc panGetProcDef { proc } {
   set argList ""
   set proc    [string trim $proc " \n"]

   foreach arg [info args $proc] {
      if { [info default $proc $arg default] } {
	  append argList " \{$arg \{${default}\}\}"
      } else {
	  append argList " $arg"
      }
   }  
 
   set srcList  [split [info body $proc] "\n"]
   set numLines [llength $srcList]
  
   # Clean up proc definition of empty lines at the end

   for {set i [expr {$numLines - 1}]} {$i > 0} { incr i -1} {
       if {[string trimright [lindex $srcList $i]] != ""} { break }
       set srcList [lreplace $srcList $i $i]
   }

   foreach line $srcList  {
      append tmp "[string trimright $line " "]\n"
   }

   return "proc $proc {[string trim $argList " "]} {$tmp}"
}


#-----------------------------------------------------------------------------
# PROCEDURE :
#-----------------------------------------------------------------------------

proc selectCmd { idx myList } {
   global par panCmdList cmdSelect

   set cmdSelect -1
   showCmdList $myList

   update idletasks
   vwait cmdSelect ; # Suspend until cmdSelect change value

   if {$cmdSelect} {
      set i [.cmdWdg.top.lbox getIndex]
      setConfigEntry $idx wdgCmd [lindex $panCmdList(ec$myList) $i]
      destroy .cmdWdg
   }
   
   return
}
    
#-----------------------------------------------------------------------------
# PROCEDURE :
#-----------------------------------------------------------------------------

proc showCmdList { myList } {
   global par panCmdList cmdSelect

   set c .cmdWdg ; toplevel $c; wm withdraw $c
    
   frame $c.top -relief raised -border 1 
   frame $c.bot -relief raised -border 1

   label      $c.top.lbl  -text "List of Items" 
   UifListBox $c.top.lbox -geometry 25x8
   
   UifButton $c.bot.cancel -label "Cancel" -width 10\
	     -command "destroy $c ; set cmdSelect 0"

   pack $c.top $c.bot -side top -fill x
   
   pack $c.top.lbl  $c.top.lbox  -pady 5
   pack $c.bot.cancel            -pady 10

   wm deiconify $c; tkwait visibility $c ; grab $c
   update idletasks

   #
   # Initialization
   #

   foreach item $panCmdList($myList) {
      $c.top.lbox addItem end $item
   }

##   $c.top.lbox updateList array
   $c.top.lbox setBind <B1-ButtonRelease> "set cmdSelect 1"

   wm title  $c "Eso Selection Box"
}
