#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: createMainPanel.tcl,v 5.34 2005/09/26 12:44:20 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.Comin   27/09/94  created
# M.Comin   25/11/94  Modifications to support widget classes 
#
# ------------------
# M.COMIN   07/02/95 Widget showing filename changed to 'message'
#                    Add check button to disable polling
#
# M.COMIN   13/02/95 Add button to edit local procedures
# M.COMIN   13/03/95 Add button to control size of action area
# M.COMIN   16/03/95 Add dummy frame widget
# M.COMIN   20/03/95 Add New Widgets : list box, text, options menu,
#                    menu button, output string (ony on main panel)
# M.COMIN   22/03/95 Activate  uifListBox
# M.COMIN   22/03/95 Activate  uifOutputString
# M.COMIN   10/07/95 Changes HelvB18 and HelvB12 to full names.
# M.COMIN   02/10/95 Add new Std. comp. 'msg reply'
# ------------------
# mcomin    24/01/96 Add local environment to main panel.
#
# mcomin    27/02/96  Porting to TK4.0
#           07/05/96  Add break to editing bindings
#           11/06/96  Disable binding for UifOutputStatus and UifOutputBitmap
#
#************************************************************************
#   NAME
#
#   createMainPanel - Creates the main window of the VLT Panel Editor.
# 
#   SYNOPSIS
#
#   createMainPanel
# 
#   DESCRIPTION
#
#
#   FILES
#    
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#------------------------------------------------------------------------
#

proc panSetLaptopSize {} {
   global panLaptopSize par
   scan $panLaptopSize "%dx%d" w h
   wm geometry $par(ROOT_PANEL) ${w}x${h}
}


proc panCheckFile { fileName } {

   if {[file extension $fileName] != ".pan"} {
      set tmp "Cannot Load\n <$fileName> \n Wrong file Extension"
      tk_dialog .dialog "Dialog Box" $tmp info 0 OK
      return ""
   }

   if {[file exists $fileName] == 0 } {
      tk_dialog .dialog "Dialog Box" "<$fileName> NOT FOUND " info 0 OK
      return ""
   }

   return $fileName
}


proc createMainPanel { } {

global par fileName ckDbPoll mainHelp pollRate optLbl panLaptopSize
global mainGrid  wdgHelp editButton clearPosButton wdgConfig
global panWinXSize panWinYSize panCvXSize panCvYSize panXPos panYPos 

#
# The main panel is devided in rows
#

set wdgHelp "Click on button <1> to add the widget"
append wdgHelp "  -  Use button <2> to move the widget"
append wdgHelp "  -  Click on button <3> to configure the widget"

set FONT     $par(FONT_TEXT) 
set helv_b18 "-Adobe-helvetica-bold-r-normal--18*-iso8859-1" 
set helv_b12 "-Adobe-helvetica-bold-r-normal--12*-iso8859-1" 
set helv_b14 "-Adobe-helvetica-bold-r-normal--14*-iso8859-1" 

#---------------------------------------------------------
#    Define and create main frames and short help area
#---------------------------------------------------------

frame $par(MAIN_ROOT) ; pack  $par(MAIN_ROOT)

createMainMenuBar  ;# Create the Panel Editor menuBar

# Create main sub-frames

for {set i 1} { $i < 4 } { incr i } {
    set row${i} [frame $par(MAIN_ROOT).row${i}]
    eval pack \$row${i} -fill x -pady 3
}

set editArea   [frame $row2.edit      -relief raised -border 2] 
set botArea    [frame $row3.help      -relief groove -border 2]
set actArea    [frame $botArea.action -relief raised -border 2]
set mainHelp    $botArea.sh;   # Name of the short help widget
                     
pack  $editArea  -fill both  -padx 5
pack  $botArea   -fill x

UifShortHelp $mainHelp

pack $mainHelp $actArea  -fill x

#---------------------------------------------------------
#    Create widgets for global infos
#---------------------------------------------------------

for {set i 1} { $i < 4 } { incr i } {
    set f${i} [frame $row1.f${i}]
    eval pack \$f${i} -fill x
}

UifOutput  $f1.file -label "File Name" -labelFont $helv_b12 \
         -font "-Adobe-courier-bold-r-normal--14*-iso8859-1" \
         -anchor w -labelWidth 15 -width 102 -variable fileName -frame 0 \
         -color Blue

UifEntry $f2.argv -label "Command Line" -labelFont  $helv_b12\
         -labelWidth 15 -width 102 -variable panArgv

set    tmp "This line is automatically interpreted as panel-command-line"
append tmp " when a panel is loaded : press <RETURN> to manually executed it"

UifShortHelp::add $mainHelp $f2.argv $tmp

bind [$f2.argv component uifEntry] <Return> { \
                       global panArgv; set argv [eval list $panArgv]; \
                       panCmdLine [llength $argv] $argv 
                       }

button $f2.clearArgv -text Clear -command { global panArgv; set panArgv "" }

UifEntry $f3.localEnv -label "Environment" -labelFont $helv_b12\
         -labelWidth 15 -width 8 -variable localEnv 

UifEntry $f3.cwp -label "Current Working Point" -labelFont $helv_b12\
         -width 55 -variable cwp

$f2.argv     component uifEntry configure -fg Blue
$f3.localEnv component uifEntry configure -fg Blue
$f3.cwp      component uifEntry configure -fg Blue

bind [$f3.cwp component uifEntry]  <Return> "panUserSetCwp $f3.cwp"

UifShortHelp::add $mainHelp $f3.cwp \
     { Set Current Working Point : Press RETURN to change it to new value }

pack  $f1.file                    -side left -padx 5 -pady 3;
pack  $f2.argv      $f2.clearArgv -side left -padx 5 -pady 3;
pack  $f3.localEnv  $f3.cwp       -side left -padx 5 -pady 3;

#---------------------------------------------------------
#    Fill Action Button Area
#---------------------------------------------------------


UifActionButton $actArea.but3  -label "Change Title ..."   -width 15 \
                               -command "changeTitle" 

UifActionButton $actArea.but2 -label "Add Action Button " -width 20 \
	        -command { newWidget UifActionButton }

UifActionButton $actArea.but4 -label "Import UifClass ..."   -width 22 \
	        -command { newWidget UifClass }

UifActionButton $actArea.but5 -label "Import Mega Widget ..."   -width 22 \
	        -command { newWidget PubClass }

UifActionButton $actArea.but1 -label "Edit procedure ..."   -width 15 \
                                    -command {panEditProcedure}
set editButton  $actArea.but1

#
# Add Short Help
#
                     
UifShortHelp::add $mainHelp $actArea.but2  \
                 {Add Action Button below the short help}
UifShortHelp::add $mainHelp $actArea.but3 {Change the title of the panel  }
UifShortHelp::add $mainHelp $actArea.but4 {Add a sub-panel defined as 'class' }

UifShortHelp::add $mainHelp $actArea.but1\
                 {Add a user defined procedure local to the panel}

pack $actArea.but1 $actArea.but3 -side left   -pady 10 -padx 5
pack $actArea.but2 -side left -pady 10 -padx 15
pack $actArea.but5 $actArea.but4 -side right  -pady 10 -padx 5

#######################################################
#          Create the "Editing Functions" palette
#######################################################

set ecv $editArea.cv

canvas $ecv -height 105 -width 980;
pack   $ecv -fill both -padx 5

#------------------------------------------
#          Frame :  Editing Mode
#------------------------------------------

set modeFrame [frame $ecv.f1 -relief flat]

label $modeFrame.lbl1 -text "Editing Mode" -font $helv_b14  -anchor w

radiobutton $modeFrame.create  -text "Create" -variable par(EDIT_MODE) \
            -value create -width 5 -command {panSetEditMode create} -anchor w
radiobutton $modeFrame.delete  -text "Delete" -variable par(EDIT_MODE) \
             -value delete -width 5 -command {panSetEditMode delete} \
             -foreground red -anchor w
radiobutton $modeFrame.copy -text "Copy  " -variable par(EDIT_MODE) \
            -value copy -width 5 -command {panSetEditMode copy} -anchor w
radiobutton $modeFrame.move -text "Move  " -variable par(EDIT_MODE) \
            -value move -width 5 -command {panSetEditMode move} -anchor w

pack $modeFrame.lbl1 -side left -padx 10 -pady 5

pack $modeFrame.create $modeFrame.delete $modeFrame.copy \
     $modeFrame.move -side left -padx 5 -pady 5


UifShortHelp::add $mainHelp $modeFrame.create \
             { Create a new widget by selecting one from the 'Palette'}

set tmp1 "(DOES NOT APPLY to Free Text widgets)" 
set tmp  "move cursor on the widget and click the mouse-middle button"

UifShortHelp::add $mainHelp $modeFrame.copy   "Duplicate a widget $tmp1 : $tmp"
UifShortHelp::add $mainHelp $modeFrame.delete "Delete a widget : $tmp"


#------------------------------------------
#     Frame :  with generic functions
#------------------------------------------

set optFrame [frame $ecv.f2 -relief flat]

# Set Height of action area

UifButton $optFrame.actArea -width 16 -label "Size Action Area" -popup 1\
       -command {
                global par cv bcv ;\
                set tmp       "New Size of Action Area" ;\
                set currSize  [lindex [$bcv configure -height] 4] ;\
                set newSize   [promptDialog $par(SUB_PANEL) $tmp $currSize] ;\
                panSetActionAreaSize $newSize
                }

UifShortHelp::add $mainHelp $optFrame.actArea {Sets the height of action area}

pack $optFrame.actArea -side left -padx 10

checkbutton $ecv.ccsFlag -text "Use CCS" -variable ckCCS \
            -command {panInitCCS} -anchor w

UifShortHelp::add $mainHelp $ecv.ccsFlag\
	{ Enable/Disable  initialization with CCS at panel startup }

#------------------------------------------
#     Frame :  with geometry infos.
#------------------------------------------

set geoFrame    [frame $ecv.f3 -relief groove -bd 2]
set warnFrame   [frame $geoFrame.warn  ]
set panFrame    [frame $geoFrame.panel ]
set defFrame    [frame $geoFrame.defPos ]

pack $warnFrame $panFrame $defFrame  -pady 2 -fill x

label $warnFrame.warnText -textvariable panWarnFrame -width 30\
                         -fg red -font $helv_b12 -anchor w

label  $warnFrame.lblTop -text "Laptop Size" -font $helv_b12 \
       -width 12 -anchor w

UifOptionButton $warnFrame.ltpSize -variable panLaptopSize -length 7 \
                -itemList {800x600 1024x768} -command panLaptopWarn

UifButton $warnFrame.setSize -label "Set" -width 5 \
          -command panSetLaptopSize

UifShortHelp::add $mainHelp $warnFrame.setSize \
"Set the panel size equal to the Laptop size"

label $panFrame.winx -width 5 -relief ridge -bd 2 -textvariable panWinXSize
label $panFrame.winy -width 5 -relief ridge -bd 2 -textvariable panWinYSize
label $panFrame.lbl1  -width 12 -text "Panel Size" -font $helv_b12  -anchor w

label $panFrame.cvx -width 5 -relief ridge -bd 2 -textvariable panCvXSize
label $panFrame.cvy -width 5 -relief ridge -bd 2 -textvariable panCvYSize
label $panFrame.lbl2 -width 12 -text "Canvas Size" -font $helv_b12 -anchor w
button $panFrame.setCvSize -text "Set ..." -command {panSetGeometry CANVAS}

label $defFrame.posx -width 5 -relief ridge -bd 2 -textvariable panXPos 
label $defFrame.posy -width 5 -relief ridge -bd 2 -textvariable panYPos
label $defFrame.lbl2 -width 12 -text "Panel Position" -font $helv_b12 \
                     -anchor w

set wdgConfig(setDEFAULT_POS) $defFrame.setPos

button $wdgConfig(setDEFAULT_POS) -width 5 -text "Set" \
       -command panSetPanelPosition

UifShortHelp::add $mainHelp $wdgConfig(setDEFAULT_POS)\
  { Sets Default panel position : if not specified it appears in the same position where it has been saved }

set wdgConfig(clearDEFAULT_POS) $defFrame.clearPos

button $wdgConfig(clearDEFAULT_POS) -width 5 -text "Clear" \
       -command {global panXPos panYPos;  set panXPos ""; set panYPos "" }

UifShortHelp::add $mainHelp $wdgConfig(clearDEFAULT_POS) \
                 { Clears the Default panel position }

label $defFrame.curx -width 5 -relief ridge -bd 2 -textvariable par(X) -fg blue
label $defFrame.cury -width 5 -relief ridge -bd 2 -textvariable par(Y) -fg blue
label $defFrame.lbl1 -width 12 -text "Cursor Position" -font $helv_b12 \
                     -anchor w

pack  $warnFrame.lblTop  $warnFrame.ltpSize $warnFrame.setSize \
      -side left -padx 3
pack  $warnFrame.warnText -side right

pack $panFrame.lbl1 $panFrame.winx $panFrame.winy \
     $panFrame.lbl2 $panFrame.cvx  $panFrame.cvy  $panFrame.setCvSize \
     -side left -padx 3

pack $defFrame.lbl1 $defFrame.curx $defFrame.cury\
     $defFrame.lbl2 $defFrame.posx $defFrame.posy $wdgConfig(setDEFAULT_POS) \
     $wdgConfig(clearDEFAULT_POS) -side left -padx 3

#
# Create the Editing Function
#

$ecv create window 5   10 -window $modeFrame -anchor nw
$ecv create window 5   65 -window $optFrame  -anchor nw
$ecv create window 460 10 -window $geoFrame  -anchor nw

$ecv create window 330 75 -window $ecv.ccsFlag -anchor w

}


#----------------------------------------------------------------------------
# PROCEDURE : panUserSetCwp - 
#----------------------------------------------------------------------------

proc panUserSetCwp { w } {
   global cwp mainHelp cv

   if {[info exists cwp] == 0 } { set cwp "" }
   if {$cwp == ""} { return }

   if {[panSetMainCwp $cwp] == "FAILURE"} {
      $w configure -foreground Red
      tk_dialog .dialog "Dialog Box" "Cannot Set CWP to : \n $cwp" info 0 OK
   } else {
      $mainHelp displayText "Current Working Point SUCCESSFUL Set !!"
      $w configure -foreground Black
   }
}
