#
# E.S.O. - VLT project
#
# "@(#) $Id: displayConfig.tcl,v 5.34 2005/09/26 12:44:20 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.Comin   ??/??/94   created 
# M.COMIN   12/05/95   Fixed length for UifOptionButton.
# M.COMIN   21/11/95   Added check for empty var. names
#
# mcomin    27/02/96  Porting to TK4.0
#
#

#------------------------------------------------------------------------------
# PROCEDURE : displayConfig - Takes the current widget configuration and 
#                             display it on the widget configuration panel
#------------------------------------------------------------------------------

proc displayConfig {idx cv top name type } {
   global gvar panDbData  panWidgetHlp  par
   global ckDbVar ckReSize 
   global i_posX  i_posY i_dbVar  i_var i_reSize i_default db_env 

#   puts "DBG > displayConfig : $cv, $name idx = $idx \n"

   set optList  [getWdgConfig $idx optList]

   #---------------------------------------------------------
   #   Display Common Configuration parameters
   #---------------------------------------------------------

    if {[info exists panWidgetHlp($name)]} {   
	setConfigEntry $idx wdgSH $panWidgetHlp($name) ;# Display Short Help
    }
    
    ###
    ###  Set widget coordinates and widget name 
    ###
    
    set pos [$cv coords $name];
    set x   [expr {int([lindex $pos 0])}]
    set y   [expr {int([lindex $pos 1])}]
    
##   puts "DBG > setWidgetCoord : cv = $ cv , name = $name , (X,Y) = $pos"

    setConfigEntry $idx wdgName $name  ;# Display Widget name
    setConfigEntry $idx wdgX    $x
    setConfigEntry $idx wdgY    $y
    
   ###
   ###  Display Widget Length
   ###

   if {[lsearch $optList "length"] != -1 } {
      switch $type {
        UifScale         { set tmp [$cv.$name cget -length] }
        UifBarChart      { set tmp [$cv.$name cget -length] }
        UifOptionButton  { set tmp [$cv.$name cget -length] }
        UifListBox       { set tmp ""}
        default          { set tmp [$cv.$name cget -width]  }           
      }; #end of switch

      if {[string length $tmp] == 0 }  { set tmp $par(DEF_LENGTH) }
      setConfigEntry $idx wdgLen $tmp 
   }
 
   ###
   ###  Display Command
   ###

   if {[lsearch $optList "command"] != -1 } {

      if {[cequal $type UifArrowsButton]} {
          setConfigEntry $idx wdgCmd [$cv.$name cget -script]
      } else {
          setConfigEntry $idx wdgCmd [$cv.$name cget -command]
      }
   }

   ###
   ###  Display Widget Label
   ###

   if {[lsearch $optList label] != -1 } {
      if {! [cequal $type freeText]} {
         setConfigEntry $idx wdgLabel [$cv.$name cget -label]
      }
   }

   if {[lsearch $optList lblOptions] != -1 } {
      set w [getWdgConfig $idx lblParFrame]

      if {[cequal $type freeText]} {
         set tmp  [lindex [$cv itemconfigure $name -font] 4]
      } else {
         set tmp  [$cv.$name cget -labelFont]
         setEntry $w.lblSize [$cv.$name cget -labelWidth]
      }

      setWdgConfig $idx lblFont $tmp
   }


   if {[lsearch $optList noVar] == -1 } {

      ###
      ###  Display Environment Name
      ###
   
      if {[info exists panDbData($name)]} {
         set envName [getDbConfig $name $db_env ]         
         if { ![cequal $envName local]} { setConfigEntry $idx wdgEnv $envName }
      } 
      
      ###
      ###  Display Variable 
      ###

      if {[set var [panParseVar [$cv.$name cget -variable]]] == "" } {
         set var "${name}Var" 
      }

      set ckDbVar($idx)  [getWidget $name $i_dbVar]
      setWdgConfig   $idx oldVar $var  ;# Record current name
      setConfigEntry $idx wdgVar $var
   }

   ###
   ###  Display State Variable 
   ###

   if {[lsearch $optList "stateVar"] != -1 } {

      if {[set var [panParseVar [$cv.$name cget -stateVar]]] == "" } {
         set var "${name}Var" 
      }
      setWdgConfig   $idx oldStateVar $var  ;# Record current name
      setConfigEntry $idx wdgStateVar $var
   }

   ###
   ### Display Default Value
   ###
       
   if {[lsearch $optList "default"] != -1 } {
      setConfigEntry $idx wdgDef [getWidget $name $i_default]
   }

   #---------------------------------------------------------
   #   Display  Specific  Configuration parameters
   #---------------------------------------------------------

   switch $type {
   
      UifBitmap        { displayUifBitmap $idx $top $cv.$name ; return }  
      UifClass         { displayUifClass  $idx $cv $top $name ; return }
      PubClass         { displayPubClass  $idx $cv $top $name ; return }
      UifText          { displayUifText   $idx $cv $top $name ; return }
      freeText         { displayFreeText  $idx $cv $name      ; return }
      UifFrame         { displayUifFrame  $idx $cv $top $name ; return }

      UifButton        { displayUifButton       $idx $cv $type $name }
      UifArrowsButton  { displayArrowsButton    $idx $cv $type $name }
      UifActionButton  { displayUifButton       $idx $cv $type $name }
      UifCheckButton   { displayRCButton        $idx $cv $type $name }
      UifRadioButton   { displayRCButton        $idx $cv $type $name }
      UifOutput        { displayUifOutput       $idx $cv $top $name }
      UifOutputList    { displayUifOutput       $idx $cv $top $name }
      UifEntry         { displayUifEntry        $idx $cv $top $name }
      UifScale         { displayUifScale        $idx $cv $top $name }
      UifBarChart      { displayUifBarChart     $idx $cv $top $name }      
      UifListBox       { displayUifListBox      $idx $cv $top $name }      
      UifOptionButton  { displayUifOptButton    $idx $cv $top $name }
      UifOutputLed     { displayOutputLed       $idx $cv $top $name }
      UifOutputArray   { displayOutputArray     $idx $cv $top $name }  
      UifOutputString  { 
                         displayUifOutput       $idx $cv $top $name 
                         displayUifOutputStr    $idx $cv $top $name 
      }
      UifOutputIcon    { displayUifOutputIcon   $idx $cv $top $name }

      default          { puts "Not Yet Implemented" }            
   }; # end of switch 


   # Display Database Event Configuration

   if {[catch {$cv.$name cget -dbMode}] == 0} {
      checkDbVar     $idx $cv.$name $top.f6 ; # Set initial conditions
      displayDbEvent $idx $cv $name
   }                
}  
       
#------------------------------------------------------------------------------
# PROCEDURE : displayMin - Display the minimum value
#                            
#------------------------------------------------------------------------------

proc displayMin { idx name } {
   global par ckDbMin i_dbMin i_min db_min

   if { [ set ckDbMin($idx) [ getWidget  $name $i_dbMin ]] } {      
      set from [getDbConfig  $name $db_min ]                  
   } else {       
      set from [getWidget    $name $i_min ]
   } 

   if { [string length $from ] > 0 } {
      setConfigEntry $idx wdgMin $from 
   } else {
      setConfigEntry $idx wdgMin $par(RANGE_MIN)
   }
}


#------------------------------------------------------------------------------
# PROCEDURE : displayMax - Display the minimum value
#                            
#------------------------------------------------------------------------------

proc displayMax { idx name } {
   global par ckDbMax i_dbMax i_max db_max

   if { [ set ckDbMax($idx) [getWidget  $name $i_dbMax ]] } {      
      set to [ getDbConfig  $name $db_max ]  
   } else {       
      set to [ getWidget $name $i_max ]
   }

   if { [string length $to ] > 0 } {
      setConfigEntry $idx wdgMax $to
   } else {
      setConfigEntry $idx wdgMax $par(RANGE_MIN) 
   }
}      
 
