#***************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: panDbgWidgetInfo.tcl,v 5.34 2005/09/26 12:44:28 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    18/10/96  created
#           19/04/98  Increase space for names
#                     Add filter type for events
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc panDbgWidgetInfo {} {
   global cv par panWidgetInfo panClassPath

   if {[info exists panWidgetInfo]} { unset panWidgetInfo }

   set wdgList [panWdgReferenceList $cv all all]

   foreach name $wdgList {
      set wdgPath [panGetWdgPath $cv $name]

      if {[winfo exists $wdgPath] == 0} {
          puts "WARNING : widget $name ($wdgPath) NOT FOUND"
          continue
      }

      set type [winfo class $wdgPath]

      if {"$cv.$name" != "$wdgPath"} {
         set class "class"
      } else {
         set class "-"
      }

      if {[catch {set dbMode [$wdgPath cget -dbMode]}]} { set dbMode " " }
      
      if {[cequal $dbMode dbEvent]} {
          set evtFilter [$wdgPath cget -evtFilter]
      } else {
          set evtFilter "-"
      }

      if {[cequal $type UifButton] || [cequal $type UifActionButton]} {
         if {[catch {set var [$wdgPath cget -stateVar]}]}  { set var "-" }
      } else {
         if {[catch {set var [$wdgPath cget -variable]}]}  { set var "-" }
      }

      # Get label to identify widget on the panel

      if {[catch {set wdgLabel [$wdgPath cget -label]}]} { set wdgLabel "-" }

      set tmp [split $name "."]
      if {[llength $tmp] > 1} {
         set myInstance [lindex $tmp 0]
         set tmp        [winfo class [winfo parent $panClassPath($myInstance)]]
         regexp -nocase {([a-z0-9]+)_([a-z0-9]+)} $tmp all myClass
      } else {
         set myInstance "-"
         set myClass    "-"
      }

      set panWidgetInfo($name) [list $type $dbMode $evtFilter $myClass $myInstance $name $wdgPath $var $wdgLabel ]
#     puts "DBG> panDbgWidgetInfo($name)  $panWidgetInfo($name)"

   };# end for ...

   panWdgInfoList
}

proc formatInfoList { text } {
   global par totalPollWidgets totalEventWidgets

   switch [lindex $text 1] {
       "dbPoll"  { set dbMode poll ; incr totalPollWidgets}
       "dbEvent" { set dbMode event; incr totalEventWidgets}
       "default" { set dbMode "-"}
   }

   set newText [format $par(WIDGET_INFO_FORMAT)  "[lindex $text 0]" $dbMode\
               [lindex $text 2] [lindex $text 3] [lindex $text 4] \
               [lindex $text 5]  "[lindex $text 8]" [lindex $text 7]]

   return $newText
}

proc panWdgInfoList {} {
   global panWidgetInfo par wdgInfoScrollX wdgInfoTitle wdgInfoList

   set top .panWdgInfoList;  # Define the outer most part of the panel

   # If the window already exist , just update the contents

   if {[winfo exists $top]} { 
      panUpdateWdgInfo
      wm deiconify $top
      raise $top
      return
   }

   toplevel $top

   for {set i 0} {$i <= 4} {incr i} {
       set f${i} [frame $top.f${i}]
   }

   pack $f0 -fill x
   pack $f1 -fill x -padx 3 -pady 5
   pack $f2 -fill both -expand 1 -padx 3
   pack $f3 $f4 -fill x -padx 3

   #################################      
   #     Creation of panel
   #################################

   set wdgInfoTitle $f1.title
   set wdgInfoList  $f2.wdgList
   set wdgInfoScrollX $f3.srollx
   set scrolly  $f2.sy 

   label $f0.lbl1 -text "Total Widgets "
   label $f0.totWdg -textvariable totalWidgets -width 4 -fg blue
   label $f0.lbl2 -text "UifClass Widgets"
   label $f0.totUifClass -textvariable totalUifClass -width 4 -fg blue

   label $f0.lbl3 -text "Polling Widgets"
   label $f0.totPoll -textvariable totalPollWidgets -width 4 -fg blue
   label $f0.lbl4 -text "Event Driven Widgets"
   label $f0.totEvent -textvariable totalEventWidgets -width 4 -fg blue

   pack $f0.lbl1 $f0.totWdg $f0.lbl2 $f0.totUifClass \
        $f0.lbl3 $f0.totPoll $f0.lbl4 $f0.totEvent \
        -side left -anchor w -padx 3 -pady 5

   set maxLen 120

   listbox $wdgInfoTitle -bd 1 -relief raised -width $maxLen -height 1 \
           -font "-Adobe-courier-medium-r-normal--12*-iso8859-1" 

   listbox $wdgInfoList -width $maxLen -height 15 -xscrollcommand infoScrollX\
                    -font "-Adobe-courier-bold-r-normal--12*-iso8859-1" \
                    -yscrollcommand "$scrolly  set" -bd 1

   scrollbar $wdgInfoScrollX -orient h  -command "infoSetX xview"
   scrollbar $scrolly -orient v -command [code $wdgInfoList yview] 

   pack $wdgInfoTitle  -fill x -anchor w
   pack $wdgInfoList   -side left  -fill both -anchor w -expand yes
   pack $scrolly   -side right -fill y
   pack $wdgInfoScrollX   -fill x  -anchor w 

   button $f4.cancel -text Cancel -width 12 -command "cancelWidgetInfo $top"
   button $f4.update -text "Update Widget List" -width 20 \
                     -command "panDbgWidgetInfo"

   pack $f4.update $f4.cancel -side left -expand 1 -pady 10
   
   panUpdateWdgInfo

   bind $wdgInfoList <ButtonRelease-1> "panShowWidgetLocation $wdgInfoList"

   wm resizable $top 1 0
   wm title     $top "Widget Infos"
}

proc panUpdateWdgInfo {} {
   global panWidgetInfo par wdgInfoScrollX wdgInfoTitle wdgInfoList
   global totalWidgets totalUifClass totalPollWidgets totalEventWidgets

   $wdgInfoList delete 0 end

   set elemList [lsort -increasing -ascii [array names panWidgetInfo]]

   set maxLen 0
   set totalWidgets  [llength $elemList]
   set totalUifClass 0
   set totalPollWidgets 0
   set totalEventWidgets 0
   set classList {}

   foreach item $elemList {
      set tmp [formatInfoList $panWidgetInfo($item)]
      $wdgInfoList insert end $tmp
      if {[set len [string length $tmp]] > $maxLen} { set maxLen $len }
     
      if {![cequal [lindex $tmp 3] "-"]} {lappend classList [lindex $tmp 3]}
   }

   set totalUifClass [llength [lrmdups $classList]]

   set tmp [format $par(WIDGET_INFO_FORMAT) "Widget Type" \
           "dbMode" "Filter" XXX_uifClass  "uifClass Instance" "Widget Name" "Label" "Variable"][replicate " " 100]

   # Change title in order to be able to scroll syncrounously with the
   # main list box.

   $wdgInfoTitle delete 0 end 
   $wdgInfoTitle insert 0 [string range $tmp 0 $maxLen]

}

proc infoScrollX {args} {
   global wdgInfoScrollX
   eval $wdgInfoScrollX set $args
   infoSetX xview moveto [lindex $args 0]
}

proc infoSetX {args} {
   global wdgInfoTitle wdgInfoList
   eval $wdgInfoList  $args
   eval $wdgInfoTitle $args
}

proc cancelWidgetInfo { top } {
   global oldWdgPath oldBgColor panWidgetInfo

   if {[info exists oldWdgPath] && [winfo exists $oldWdgPath]} {
      catch { $oldWdgPath configure -background $oldBgColor }
   }

   set oldWdgPath ""
   destroy $top
}

proc panShowWidgetLocation { w } {
   global oldWdgPath oldBgColor panWidgetInfo

   if {[winfo exists .configPanel]} { return }

   set tmp  [$w get [$w curselection]]
   set name [lindex $tmp 5] ;

   if {[info exists oldWdgPath] && [winfo exists $oldWdgPath]} {
      catch { $oldWdgPath configure -background $oldBgColor }
   }

   if {[info exists panWidgetInfo($name)] == 0 } { return }

   set wdgPath [lindex $panWidgetInfo($name) 6]
   
   if {[winfo exists $wdgPath]} {
      set oldWdgPath    $wdgPath
      set oldBgColor [$wdgPath cget -background]
      $wdgPath configure -background green
   }
}

#
# ___oOo___
