#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: panGeometry.tcl,v 5.34 2005/09/26 12:44:27 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin   24/10/95   created
# mcomin   25/10/95   fixed bug during calculation of actual size
#
# mcomin   27/02/96   Porting to TK4.0
#          09/05/96   Fixed bug in panInstallGeoHandle for bind configuration
#                     Fixed big : Canvas geometry does not include borderwidth.
#          21/05/96   Patch for crazy re-sizing with short help
#                     xdiff = 0 if smaller than 4
# mcomin   24/10/96   Re-implementation ... easier and more robust
#

#------------------------------------------------------------------------------
# PROCEDURE : panSetGeometry -
#------------------------------------------------------------------------------

proc panSetGeometry { what } {
   global panCvXSize panCvYSize par mainHelp

   set c .panGeometry
#   if {[winfo exists .panGeometry]} { return }

   toplevel $c;            # Define the outer most part of the panel 

   frame $c.f1;
   frame $c.f2;
   frame $c.f3 -bd 2 -relief raised

   label $c.lbl2 -font "-Adobe-courier-bold-r-normal--12*-iso8859-1"
   label $c.xlbl -text "X Size : " -font "-Adobe-courier-bold-r-normal--12*-iso8859-1"  
   label $c.ylbl -text "Y Size : " -font "-Adobe-courier-bold-r-normal--12*-iso8859-1"  

   entry $c.x -width 4
   entry $c.y -width 4
   
   button $c.f3.accept -text "Accept" -width 8 -command "panGeoAccept $c $what"
   button $c.f3.cancel -text "Cancel" -width 8 -command "destroy $c"

   set title "Canvas Dimensions"
   $c.x insert 0 $panCvXSize 
   $c.y insert 0 $panCvYSize
   $c.lbl2 configure -text "Size of the Canvas"

   pack $c.f1 $c.f2 $c.f3 -fill x
   pack $c.lbl2 -in $c.f1 -pady 5 
   pack $c.xlbl $c.x $c.ylbl $c.y  -in $c.f2  -side left  -pady 5 -padx 5 
   pack $c.f3.accept $c.f3.cancel -side left -expand yes -pady 10
   
   #
   # Wait for the user to respond, then restore the focus and
   # return the index of the selected button.
   #

   wm group      $c .
   tkwait window $c
}

proc panLaptopWarn { {dummy ""} } {
   global panWinXSize panWinYSize par cv panWarnFrame panLaptopSize

   scan $panLaptopSize "%dx%d" w h

   if {($panWinXSize > $w) || ($panWinYSize > $h)} {
      set panWarnFrame "DOES NOT Fit in Laptop Screen"
   } else {
      set panWarnFrame ""
   }
}

proc panDisableGeoHandle {w} {
#   puts "panDisableGeoHandle ... $w "
   bind $w <Configure> {}
}

proc panEnableGeoHandle {w} {
#   puts "panEnableGeoHandle ... $w "
   bind $w <Configure> "panWindowResize $w %W %w %h; break"
}

proc panGeoAccept {c what} {
    global panWinXSize panWinYSize panCvXSize panCvYSize  par cv
    
    set xsize  [getInput $c.x]
    set ysize  [getInput $c.y]
    set bottomSize  [winfo height $par(ROOT_PANEL).bot]
    
    if {$xsize == "" } { set xsize $panCvXSize }
    if {$ysize == "" } { set ysize $panCvYSize }
    
    set xdiff [expr {$panCvXSize - $xsize}]
    set ydiff [expr {$panCvYSize - $ysize}]
    
    # Adjust Panel Geometry 
    
    set panWinXSize [expr {$panWinXSize - $xdiff}]
    set panWinYSize [expr {$panWinYSize - $ydiff}]
    set borderSize  [$cv cget -borderwidth]

    set panCvXSize  [expr {$xsize - 2*$borderSize}]
    set panCvYSize  [expr {$ysize - 2*$borderSize}]
    
    panLaptopWarn
    panDisableGeoHandle $par(ROOT_PANEL) ;# Disable temporarily bind 
    
   # Set new geometry

   $cv configure -width $panCvXSize -height $panCvYSize
   wm geometry $par(ROOT_PANEL) ${panWinXSize}x${panWinYSize}
   update

   panEnableGeoHandle $par(ROOT_PANEL)

   destroy $c
}


#------------------------------------------------------------------------------
# PROCEDURE : panGetSizeWindow - 
#------------------------------------------------------------------------------


proc panWindowResize { widgetRef w width height } {
    global panWinXSize panWinYSize panCvXSize panCvYSize
    global panResizeEvent par
    
    if {$widgetRef != $w} { return }
    
    panGetCvNames [string range $w 1 end] cv bcv ;# Get canvas names
    
    set xdiff [expr {$width  - $panWinXSize}]
    set ydiff [expr {$height - $panWinYSize}]
    
#    puts "DBG > panWindowResize $w ($width,$height)  diff = ($xdiff,$ydiff)"
    
    if {[winfo exists $cv] == 0} { return }
    
    # Get Current Canvas Size
    
    panDisableGeoHandle $par(ROOT_PANEL) ;# Disable temporarily bind 
 
    set borderSize   [$cv cget -borderwidth]

    if {$xdiff != 0} {
	set panCvXSize   [expr {$panCvXSize + $xdiff - 2*${borderSize}}]
	$cv configure   -width $panCvXSize
    }

    if {$ydiff != 0} {
	set panCvYSize   [expr {$panCvYSize + $ydiff - 2*${borderSize}}]
	$cv configure   -height $panCvYSize
    }

   # Keep track of actual sizes

   set panWinXSize $width 
   set panWinYSize $height

   panLaptopWarn

   update
   panEnableGeoHandle $par(ROOT_PANEL)
}


#------------------------------------------------------------------------------
# PROCEDURE : panInstallGeoHandle - 
#------------------------------------------------------------------------------

proc panInstallGeoHandle { root } {
   global panWinXSize panWinYSize panCvXSize panCvYSize  

   # This bind is triggered by all other widgets, so the first parameter
   # it is used to check when to really execute the procedure.

   update

   # Get Panel geometry
   set panWinXSize [winfo width  $root]
   set panWinYSize [winfo height $root]

   # Get widget canvas geometry
   panGetCvNames $root cv bcv
   set panCvXSize [winfo width  $cv]
   set panCvYSize [winfo height $cv]

   panEnableGeoHandle $root
   bind $root <Destroy>  "bind $root <Configure> {}"
}

#------------------------------------------------------------------------------
# PROCEDURE : panSetPanelPosition - 
#------------------------------------------------------------------------------

proc panSetPanelPosition {} {
   global par mainHelp panXPos panYPos 

   set geo    [wm geometry $par(ROOT_PANEL)]
   scan $geo "%dx%d+%d+%d" w h panXPos panYPos 

   $mainHelp displayText \
	   "Panel position set to coordinates (${panXPos},${panYPos})"
}


#
# ___oOo___
