#***************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: panel.tcl,v 5.34 2005/09/26 12:44:20 vltsccm Exp $" 
#
# who       when      what
# --------  --------  ----------------------------------------------
# M.Bernini 24/05/94  created 
# pforstma  21/06/94  VLT standards applied
# M.Comin   05/07/94  Add comments to the code
# M.Comin   06/07/94  Changed bitmap with NTT bitmap
# M.Comin   25/07/94  Re-arrangment of the main panel
# M.COMIN   27/09/94  Start using the sequencer TCL-shell
# M.COMIN   28/03/95  Add trick to get version number from cmm
# M.COMIN   08/05/95  Check for proper CCS initialization
# M.COMIN   10/05/95  Add version number to title
# M.COMIN   17/07/95  Makefile takes care of the configuration : RTAP
#                     CCS_LIGHT or nothing !!
#
# mcomin    27/02/96  Porting to TK4.0
#
#
#************************************************************************
#   NAME
#
#   panel - VLT Panel Editor applying ESO common conventions.
# 
#   SYNOPSIS
#
#       panel [&]
# 
#   DESCRIPTION
#
#   The Panel Editor gives the software developer the possibility to
#   build interactively a panel: choosing widgets, placing then on the
#   panel to be build and configuring each widget.
#   The User Manual provides a complete description of available widgets
#   and the way to configure them.
#
#   To work properly in the VLT Standard Environments:
#
#        - develop your panels under the src/ directory of your module
#          and add each panel in the Makefile as a SCRIPT: 
#
#              PANELS    =  panel1 panel2 ....
#                      .
#              SCRIPTS   =  .... $(PANELS)
#                      .
#          The standard treatment of a shell script is good also
#          for panels.
#
#        - if you need bitmaps, create them under a separate directory:
#
#             ..(mod)../
#                      .
#                      |--bitmaps/              
#                      |         |-- shutter.xbm
#                      .         |-- .....
#
#          and in the Makefile add the following lines:
#
#              install : install_all
#                          .
#                      cp ../bitmaps/bitmap.xbm $(VLTTOP)/bitmaps
#                          .
#                      @echo " . . . installation done"
#
#          This will install the bitmaps.
#
#
#   FILES
#    
#   $VLTROOT/app_resources/Xpanel.*   READ    X resources files
#   mod/bitmaps/                      READ    where to locate bitmaps
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#------------------------------------------------------------------------
#

set Id dummy
set myVersion "@(#) $Id: panel.tcl,v 5.34 2005/09/26 12:44:20 vltsccm Exp $"
set sccsId "[lrange $myVersion 2 3]"


############################################
#             Variable  Set-up             #
############################################

global menuList par colordef initDone filePath auto_path env wdgConfig
global ckHotSpot fileName  myName mainCursorGrid
global cwpEnv localEnv seqFlag release
global TCLTK_ROOT VLTROOT INTROOT lastFilePath

set strictMotif      0 ;         # Set MOTIF behaviour:  1 (true) or 0 (false)

set colordef         winter;     # Set Default Color Set
set par(MAIN_ROOT)   .mainPanel
set par(SUB_PANEL)   .subRoot
set par(EDIT_WHAT)   .pan
set par(MYCLASS)     ""
set par(SAVE_FLAG)   0
set filePath         "."
set wdgConfig(lockColor) { green PaleGreen aquamarine }

set home [pwd]
set lastFilePath $home

# Re-define INTROOT/VLTROOT with complete path to avoid problems 
# with remotely mounted disks

set MODULE [file dirname $home]

if {[info exists env(INTROOT)] } {
    if {[file exist $env(INTROOT)]} {
	cd $env(INTROOT)
	set INTROOT [pwd]
    } else {
	set INTROOT .
    }
}     

if {[info exists env(TCLTK_ROOT)] } {
   cd $env(TCLTK_ROOT)
   set TCLTK_ROOT [pwd]
}  


if {[info exists env(VLTROOT)] } {
   cd $env(VLTROOT)
   set VLTROOT [pwd]
} 

if {[info exists env(VLTROOT)]} {
   set par(VLT) [file dirname $env(VLTROOT)]
   cd $home
} else {
   set par(VLT) dummy
}

cd $home

#
#  variable Initialization
#

panConfig ;# Configure the panel to work with CCS

set EDITING  1 ;# Set "Editing" Mode

panLoadPackage Rtd

set par(MY_NAME)       panel_[exec whoami]
set par(PANEL_EDITOR)  panel_[exec whoami]

#
#  Read X Resources   
#

panReadOption ;    # Read general resources

option  readfile   [findVLTFile app-defaults/Xpanel.private]

############################################
#           Start panel Editor             #
############################################

set tk_strictMotif $strictMotif

createMainPanel    ;# Create main Panel
panCreateWdgSet    ;# Create palette of widgets

set release  [string trim [lindex $myVersion 3] "v +"]

# The title specifies the current release number and the 'copy' number
# in case that several instances of the panel are running at the same time

set title [wm title .]

if {[llength [winfo name .]] > 1} {
   append title " [lindex  [winfo name .] 1]"
}

if {[llength $title] > 1} {
   wm  title . "Panel Editor [lindex $title 1] - Release $release"   
} else {
   wm  title . "Panel Editor - Release $release"
}

wm  geometry  . 1000x335+10+10  ;# Set global geometry
wm  resizable . 0 0             ;# Disable resize 
update  ;# Immediate execution of all 'delayed' operations.

if {$argc != 0} {
   
   # Load panel specified in the command line
   # It also checks for the file extension and if the file exists

   if {[set fileName [panCheckFile [lindex $argv 0]]] != ""} { 
      loadPanel [lindex $argv 0] no 
   } else {
      set argc 0 ;# Force to create a new panel
   }
}

if {$argc == 0} {
   newPanel no ;# Create an empty panel
   set initDone  0 ; panInitCCS
   panDrawGrid
}

update

#___oOo___
