#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: panScrollCanvas.tcl,v 5.34 2005/09/26 12:44:31 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin  15/05/96  created
#

#************************************************************************
#   NAME
#
#   panCreateScrollCanvas  -  Creates a scrollable canvas
#
#   SYNOPSIS
#
#   panCreateScrollCanvas <frame> <width> <height> <vbar> <hbar> <region>
#
#   DESCRIPTION
#
#   This procedure create a scrollable canvas inside a frame.
#   The canvas can be used to create a list of uifClass widgets.
#    
#      <frame>              Full name of the frame widget where to create the canvas.
#      <width> <height>     Canvas dimensions. The frame widget provided by the 
#                           panel editor is able to automatically re-size according
#                           to the canvas dimensions.
#      <vbar> <hbar>        Flags to select 'vertical' and 'horizontal' scroll bars
#
#      <region>             List of four coordinates describing the left,top, right
#                           and bottom of a rectangular scrolling region.
#   
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   Returns the full name of the canvas widget.
#
#   CAUTIONS
#
#   EXAMPLES
#
#   Let's suppose that we have a set of uifClass widgets showing the
#   diagnostics of the beam alignment control device (called lgialcDiag1_uifClass).
#   Now we want to display the diagnostics of all ALC devices using a scrolled
#   canvas in order to save screen space.
#
#   With the panel editor one should create a frame using the uifFrame widget
#   with the name wdgAlcFrame.
#   
#   If the panel is called lgialcXXX.pan , we add the following code to the 
#   lgialcXXXInit procedure.
#
#   Get path to the 'real' frame widget
#
#      set top [$cv.wdgAlcFrame component uifFrame]
#
#   Create scrolled canvas
#
#      set scanvas [panCreateScrollCanvas $top 700 300 1 1 {0 0 800 600 }]
#
#   Add the widget to the scroll canvas
#
#      lgialcAddDiagWidgets $fcv
#
#
#
#   proc lgialcAddDiagWidgets { scanvas } {
#      global lgialcLcu gvar lgialcMinIdx lgialcMaxIdx
#
#      set max [expr $lgialcMaxIdx - $lgialcMinIdx + 2]
#      set idx 1
#      set alcIdx [expr $lgialcMinIdx + 1]
#  
#       while {$alcIdx < $max } {
#          set w $scanvas.wdgALC${alcIdx} ;# Create widget instance 
#     
#          lgialcDiag1_uifClass $w -cwp "<alias>ALC${alcIdx}" -relief raised \
#     	              -envName {$lgialcLcu} -library liblgialcGUI
#     
#          set ypos [expr ($alcIdx -1) * 100] ;# Set class position along Y
#     
#          $scanvas create window 10 $ypos -window $w -anchor nw  \
#     	            -tags "uifClass wdgALC${alcIdx} class"
#     
#     	   $w display ;# Display uifClass
#
#          incr alcIdx 1
#         }
#     }
#     
#
#   SEE ALSO
#
#   UifFileSelectionBox
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc panCreateScrollCanvas { top width height vbar hbar region } {
    
    set dcv $top.diagCanvas
    set dxs $top.xscroll
    set dys $top.yscroll

    canvas $dcv -width $width -height $height -scrollregion $region

    if {$vbar} {
	$dcv configure -yscrollcommand [list $dys set]
	scrollbar $dys -orient vertical -command [list $dcv yview]
        pack $dys -side right  -fill y
    }

    if {$hbar} {
	$dcv configure -xscrollcommand [list $dxs set]
	scrollbar $dxs -orient horizontal -command [list $dcv xview]
	pack $dxs -side bottom -fill x 
    }

    pack $dcv -side left -fill both -expand true

    return $dcv
}

#
# ___oOo___
