#**************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ptlibCheckDbData.tcl,v 1.21 2003/11/18 12:40:55 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    21/09/95  created
# mcomin    07/11/95  Modified to support classes
# mcomin    14/12/95  panGetWidgetReference replaced with panWdgReferenceList
#

#************************************************************************
#   NAME
# 
#   SYNOPSIS
#      
#      ptlibCheckDbData <appl> <widget root>
#
#   DESCRIPTION
#      
#      Checks if the current DB values shown by the widgets have the same
#      values of the corresponding values in the DB.
#
#      appl         <IN>  Name of the Tcl/Tk application
#      widget root  <IN>  Full name of the parent-widget.
#
#      The procedure generates a log on the standard output of the user action.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc ptlibCheckDbData { appl root } {

   global tcl_precision
   #
   # Set indexes to access database interface data structure
   #
   
   set i_type       0;  # Widget Type
   set i_precision  11; # Precision specification for real numbers   
   set i_outFormat  10; # Output format description  
    
   set db_var       2;  # Variable symbolic adddress 
    
   set wdgList [send $appl panWdgReferenceList $root all uifType]       
   set errFlag 0
   
   puts "\nTEST WIDGET DATABASE DATA IN $appl UNDER $root"
             
   foreach name [lsort $wdgList] {
   
#     set wdgName [string range $name $rootLen end]      
      set wdgName $name ; # Patch not to risk to mess all up
      
      # Get Widget Data
    
      if {[catch { set dbList [send $appl set panDbData($wdgName)] } ret]} {
         continue
      }      

      if {[set var [lindex $dbList $db_var]] == "0" } { continue }
      
      if {[catch { set attr [send $appl set panDbSymbAddr($wdgName)] } ret]} {
         puts "SYMBOLIC ADDRESS FOR WIDGET $wdgName NOT PROPERLY DEFINED"
         continue
      }  
   
      set wdgBuff  [send $appl set panWidget($wdgName)]   
      set type     [lindex $wdgBuff $i_type]
      set prec     [lindex $wdgBuff $i_precision]      
      set wdgPath  [send $appl panGetWdgPath $root $wdgName]
      
      if {$type == "UifScale"} {
      
         set myVal [send $appl $wdgPath get]
         catch {seq_dbReadSymbolic $attr} refVal
         
      } elseif { $type == "UifOutput"} { 
      
         # UifOutput widget can display the information according to
         # a specific format and precision (in case of numerical values )      
                 
         set orig_tcl_precision   $tcl_precision; 
         set prec                 [lindex $wdgBuff $i_precision]
         set myFormat             [lindex $wdgBuff $i_outFormat]   
              
         if { $prec != "" } {
            set tcl_precision $prec ; # Read DB value with the same precision
         }                
         
         set myVal  [ptlibGetValue $appl gvar($var)]         
         
         #
         # Read DB value and format it according to the widget format and
         # precision.
         #
         
         catch {seq_dbReadSymbolic $attr} dbVal                 
         
         set tcl_precision $orig_tcl_precision;  # Set to default value
               
         if { $myFormat != ""} {
             set refVal [format $myFormat $dbVal] ;
         } else {
             set refVal $dbVal
         }         
         
      } elseif { $type == "UifListBox" || $type == "UifText" } {
      
         set myVal  0
         set refVal 0
                  
      } else {  
          
         set myVal  [ptlibGetValue $appl gvar($var)]
         catch {seq_dbReadSymbolic $attr} refVal
      }  
            
      if { $myVal != $refVal } {         
         lappend wrongData " $wdgName  val=($myVal) dbval=($refVal)"; 
         incr errFlag    
      }                     
   
   } ; # end foreach 
    
   if {$errFlag} {
      puts "WRONG DATA FOR WIDGETS : $wrongData"    
   } else {
      puts "CORRECT DATABASE WIDGETS VALUES"   
   }     
}

#
# ___oOo___
