#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ptlibLogAction.tcl,v 1.21 2003/11/18 12:40:56 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    21/09/95  created
# ------------------
# mcomin    18/06/96  Add mechanism to find toplevel window
#
#************************************************************************
#   NAME
# 
#   SYNOPSIS
#
#      ptlibLogAction <appl> <wdgName> ?action? ?text?
#
#   DESCRIPTION
#
#      Logs on the standard output the user action.
#
#      appl     <IN>  Name of the Tcl/Tk application
#      wdgName  <IN>  Full widget name
#      action   <IN>  ...
#      text     <IN>  ...
#
#      The log has the format :
#
#         USER - ON WINDOW <title> <action> ?text? IN WIDGET <widget>
#
#      If this does not apply to the specified widgets , it prints the 
#      widget name.
#      This call is automatically executed whenever the user interacts 
#      with a widget.
#
#      A grep on the output file allows the user to retrieve all the actions
#      performed during the test.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc ptlibGetTitle { appl wdg } {
   global ptlibTitleList

#   puts "DBG>             ptlibGetTitle $appl $wdg"
   if {$wdg == "." || $wdg == ""} { set wdg ".$appl" }

   set myroot [lindex [split $wdg "."] 1]

   if {[info exists ptlibTitleList($myroot)]} {
#      puts "DBG> return ptlibTitleList($myroot)"
      return $ptlibTitleList($myroot)
   }

   # Get toplevel window for current widget

   if { [send $appl winfo exists $wdg] } {
      set topWdg [send $appl winfo toplevel $wdg]
   } else {
      set topWdg "."
   }

   set tmp [send $appl wm title $topWdg ] ;# Get window title

#   puts "DBG>             title = $tmp"
   if { [set len [string first " - @" $tmp]] != -1  } {      
       set title [string range $tmp 0 [expr {$len - 1}]]
   } else {
      set title $tmp 
   }

   set ptlibTitleList($myroot) $title
   return $title
}

proc ptlibLogAction { appl wdg {action {}} {text {}} } {

   if { $action == "" } { set action CLICK }

   set title [ptlibGetTitle $appl $wdg]

   if {[send $appl winfo exists $wdg ] == 0} {
      puts "WINDOW \"$title\" WIDGET $wdg DOES NOT EXIST"
      return 1
   }

   set label ""
   catch { set label [send $appl $wdg cget -text] }
   catch { set label [send $appl $wdg cget -label] }

   set msg "WINDOW \"${title}\" USER $action"

   if { $label == "" } { set label $wdg }

   switch $action {
     "TYPE"                  { puts "$msg \"$text\" IN WIDGET \"$label\" " }
     "CLICK ITEM"            { puts "$msg \"$text\" IN MENU   \"$label\" " }
     "SELECTS"               { puts "$msg \"$text\" IN MENU   \"$label\" " }
     "SELECT OPTION"         { puts "$msg \"$text\" IN WIDGET \"$wdg\" "   } 
     "SELECT LIST BOX ITEM"  { puts "$msg \"$text\" IN WIDGET \"$label\" " }   
     "SET SCALE TO"          { puts "$msg \"$text\" IN WIDGET \"$label\" " }
     "default"               { puts "$msg \"$label\"" }
   }

   flush stdout
   return 0
}

#
# ___oOo___
