#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ptlibWidgetConfig.tcl,v 1.21 2003/11/18 12:40:57 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    21/09/95  created
# mcomin    06/11/95  Modify to support classes
# mcomin    08/11/95  Skip empty configuration parameters
# 
# mcomin    09/03/96  Porting to Tk4.0 and itcl2.0
#           16/09/96  Add list of options to ignore
#
#************************************************************************
#   NAME
# 
#   SYNOPSIS
#      
#      ptlibWidgetConfig  <appl> <parent widget>  <args>
#
#   DESCRIPTION
#      
#      ptlibWidgetConfig
#
#        Dumps the configuration of a set of widgets as children of a 
#        specific <parent widget> window -i.e.- a canvas.
#
#        appl             <IN>  Name of the Tcl/Tk application
#        parent widget    <IN>  Full name of the parent-widget.
#        flag             <IN>  If this is set to 'SHORT_CONFIG' the result
#                               is a reduced configuration set, namely only
#                               widget type and current value 
#        list of widgets  <IN> 
#
#      The procedures generates a log on the standard output of the user 
#      action.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

global ptlibConfigList

set ptlibConfigList [ list -text -command -label -state -color -ledColor ]
set ptlibIgnoreList [ list -qualityBg -qualityFg -offset -activefill \
			 -disabledstipple -disabledfill -activestipple \
			 -repeatdelay -overrelief]


proc ptlibWidgetConfig { appl root args } {

   global ptlibFullConfig ptlibConfigList ptlibIgnoreList

   puts " "; # Add space in the reference file

   set title [ptlibGetTitle $appl $root]

    if {[llength $args] == 0 } {
      puts "MISSING WIDGET NAMES IN ptlibWidgetConfig"
      puts "TEST ABORTED ... FIX THE PROBLEM !! \n"      
      exit
   }
   
   set flagSHORT_CONFIG 0
   set extraConfig      {}

   set idx 0
   foreach item $args {
 
      if {$item == "SHORT_CONFIG"} {
         set flagSHORT_CONFIG 1
      } elseif { $item == "DELAY" } {
         set pollRate [ptlibGetValue $appl pollRate]
	  after [int [expr {1.5 * $pollRate}]] ; # Wait proper data update.
      } elseif {[string index $item 0] == "-"} {
         lappend extraConfig $item
      } else {
         incr idx
         continue
      }

      set args [lreplace $args $idx $idx] ; # Take element away and do not
                                            # icrement index idx.
   }

#   if {[set idx [lsearch $args "SHORT_CONFIG"]] != -1 } {
#      set args [lreplace $args $idx $idx]
#      set flagSHORT_CONFIG 1
#   }
#
#   if {[set idx [lsearch $args "DELAY"]] != -1 } {
#      set args     [lreplace $args $idx $idx]
#      set pollRate [ptlibGetValue $appl pollRate]
#
#      after [int [expr 1.5 * $pollRate]] ; # Wait proper data update.
#   }
#

   foreach name $args {

      if {[string trim $name " "] == ""} {
         puts "NULL WIDGET NAMES IN ptlibWidgetConfig"
         puts "TEST ABORTED ... FIX THE PROBLEM !! \n"      
         exit
      }
      
      if {[catch { set wdgPath [send $appl panGetWdgPath $root $name] } ret]} {
         puts "WIDGET $name DOES NOT EXIST IN \"$title\" "
         continue
      }

      set type    [send $appl winfo class $wdgPath]; # Get Widget Type
      set config  [send $appl $wdgPath configure]      
    
      puts "WINDOW \"$title\" CONFIGURATION OF WIDGET $name TYPE $type"      
      foreach item $config {

         set what  [lindex $item 0]
         set value [lindex $item 4]

         # Some otpions are always skipped

#         puts "what  $what  value $value  [lsearch $ptlibIgnoreList $what]"
         if {[lsearch $ptlibIgnoreList $what] != "-1" } { continue }

	 if { $flagSHORT_CONFIG && [lsearch $extraConfig $what] == "-1" } {
	      if {[lsearch $ptlibConfigList $what] == "-1"} { continue };
         }
 
         #
         # Strip the first and the last '"' character in the string
         #

         if {$value != "" } {
            set idx1 0
	     set idx2 [expr {[string length $value] -1}]

            if {[string index $value $idx1] == "\"" }          {incr idx1   }
            if {[string index $value $idx2] == "\"" && $idx1 } {incr idx2 -1}

            set value [string range $value $idx1 $idx2]
#            puts "new value = ($value) "
            puts "OPTION $what \"${value}\"" 
         } 
      }  
       
      ptlibWidgetData $appl $wdgPath $name 1; # Dump current variable
      puts " ";                               # Add space in the reference file
   }
}

#
# ___oOo___
