#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: rtdbPanel.tcl,v 1.24 2005/06/19 19:36:30 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  15/02/00  created
# pbiereic  12/12/00  use the 'pack' geometry manager instead of 'blt::table'
#

#************************************************************************
#   NAME
#   rtdbPanel.tcl - itcl widget combining the RtdImage itcl widget with
#                   a control panel, zoom and panning windows.
# 
#   SYNOPSIS
#   rtdbPanel <path> ?options?
# 
#   DESCRIPTION
#   rtdbPanel builds the control panel which consists of a zoom window, a pan
#   window, a cursor coordinates display, an image control panel, a color ramp etc.
#   It inherits the RtdImagePanel class and adds/redefines method to:
#   - arrange the control widgets vertically on the left side of the window
#   - add user defined control widgets and pack them according to the definitions
#     in the config file
#   - adding callbacks
#
#   PARENT CLASS
#        ::rtd::RtdImagePanel
#
#   PUBLIC METHODS
#   The public methods are called via rtdbRtd(n) and are used internally
#   by the IRTD software. They are not supposed to be used by external plugins.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#   RTD documentation, RtdImagePanel(n)
#
#   BUGS     
#
#------------------------------------------------------------------------

itk::usual rtdbPanel {}

class ::rtdbPanel {
    inherit rtd::RtdImagePanel

    constructor { args } {
	global ::rtdb
	set rtdb(Panel) $w_
	eval itk_initialize $args

	# rtd::RtdImagePanel::make_layout needs the $args list
	rtd::RtdImagePanel::make_layout

	# XXX add component trans here since Rtd.tcl needs it
	set trans $w_.trans
	itk_component add trans {
	    rtd::RtdImageTrans $trans -relief flat -image $itk_option(-image) \
		-show_trans 0 -show_Zz_buttons 0 \
		-min_scale $itk_option(-min_scale) \
		-max_scale $itk_option(-max_scale)
	} { keep -state }
        catch {destroy $trans.trans_frame}

	# XXX we are going to use the 'pack' geometry manager.
        foreach wdg "$w_ [winfo children $w_]" {
	    catch {blt::table forget $wdg}
       }
    }
 
    protected method make_layout {} { }

    # change the layout of the demo widget
	
    public method init {} {
	global ::rtdb

	# add another image transformation widget with different options
	set trans2 $w_.trans2
	itk_component add trans2 {
	    rtd::RtdImageTrans $trans2 -relief flat -image $itk_option(-image) \
		-show_trans 1 -show_Zz_buttons 1 \
		-min_scale $itk_option(-min_scale) \
		-max_scale $itk_option(-max_scale)
	} { keep -state }
        catch {destroy $trans2.trans_frame}
	pack forget [$trans2 component choose]
	$trans2 config -show_trans 0    ; # avoids setting the trace variables

	global ::$w_.trans2.var
	foreach el [array names $trans2.var] {
	    trace variable $trans2.var($el) w "$this transCmd"
	}

	# change packing order of transformation widget
	foreach el "larger smaller flipx flipy rotate" {
	    pack [$trans2 component $el] -fill both -expand 1 -side left
	}

	$itk_component(cameraStatus) config -height 30
        catch {destroy $itk_component(lrframe)}

	set controls [addUserWdgs]
	packCtrlWdgs $controls

	# workaround for bug in itcl2.0

        # the RtdImage code sets this array for us to speed up the panel
        # update by using the -textvariable option
        set var $image_
        global ::$var

	foreach compo "x y value ra dec" {
	    $itk_component($compo) config -textvariable ${var}([string toupper $compo])
	}
	$w_ transCmd
    }
    

    # add user defined control widgets to the list of the standard widgets

    protected method addUserWdgs {} {
	global ::rtdb
	set i 1
	foreach el "cameraStatus x y value ra dec low high autocut trans trans2" {
	    lappend controls "$itk_component($el) RTDB-$i"
	    incr i
	}

	if {[info exists rtdb(ADD_CTRLS)]} {
	    foreach spec $rtdb(ADD_CTRLS) {
		set name [lindex $spec 0]
		set type [lindex $spec 1]
		if {"$name" == "" || "$type" == ""} { continue }
		set wdg   "$w_.$name"
		set opts  [lindex $spec 4]
		set help  "[lindex $spec 2]"
		global ::$wdg
		if {[catch {eval $type $wdg $opts} msg]} {
		    rtdbExit_dialog "Control widget '$name': $msg"
		}
		add_short_help $wdg $help
		catch {$wdg config -variable $wdg}
		lappend controls "$w_.$name [lindex $spec 5]-[lindex $spec 3]"
	    }
	}
	return $controls
    }

    # pack the control widgets

    protected method packCtrlWdgs {controls} {
	global ::rtdb
	set indices [rtdbIdx $controls CTRLWDG]
	foreach idx $indices {
	    foreach els $controls {
		lassign $els name i
		lassign [split $i "-"] ctgry n

		if {! [regsub "{$name $ctgry-$idx}" $controls {} controls] && \
			! [regsub "{$name $idx}" $controls {} controls]} { 
		    continue
		}

		rtdbFeedback "Adding control widget: $name"
		    
		# force unique appearance of control widgets
		foreach child [winfo children $name] {
		    foreach el "$child [winfo children $child]" {
			if {"$el" == "[$itk_component(trans2) component choose]"} { break }
			if {"[$el cget -relief]" == "sunken"} {
			    catch {$el config -highlightthickness 2}
			} else {
			    catch {$el config -highlightthickness 0}
			    catch {$el config -padx 0 -pady 2}
			}
			catch {pack $el -padx 0 -pady 0 -ipadx 0 -ipady 0 -expand 1 -fill both}
		    }
		}
		catch {$name config -pady 2}
		pack $name -padx 1 -pady 1 -ipadx 0 -ipady 0 -expand 1 -fill both
		incr tableIdx_
		break
	    }
	}
	set pattern "$this\*"
	foreach obj "[[rtdbItclInfo] objects $pattern -class ::util::LabelValue] \
		[[rtdbItclInfo] objects $pattern -class ::util::LabelEntry] \
		[[rtdbItclInfo] objects $pattern -class ::util::LabelMenu]" {
	    catch {$obj.label config -pady 0 -padx 2}
	    set lw $itk_option(-labelwidth)
	    set vw $itk_option(-valuewidth)
	    if {[$obj isa ::util::LabelMenu]} {
		set vw 9
	    }
	    catch {$obj config -labelwidth $lw -valuewidth $vw}
	}
    }
    
    # return name of the panel control widget

    public method get_panel {args} {
	return $w_
    }

    # configure the X, Y labels.
    # flag = 0: Motion event disabled
    # flag = 1: Motion event enabled
    # flag = 2: Motion event with Caps Lock 

    public method stopXY {flag} {
	if {$flag == 2} {
	    foreach el "x y value" {
		$itk_component($el) config -value "Caps Lock!"
	    }
	    return
	}
	set col red
	if {$flag} { set col black }
	foreach el "x y value" {
	    $itk_component($el) config -foreground $col
	}	    
    }

    # configure the camera status widget

    public method camera_ready {flag} {
	if {$flag == -1} {
	    set flag $camera_ready_
	}
	if {$flag} {
	    $itk_component(cameraStatus) config -borderwidth 2 -relief flat
	    $itk_component(cameraStatus) itemconfig cameraStatus -fill black
	} else {
	    $itk_component(cameraStatus) config  -borderwidth 2 -relief sunken
	    $itk_component(cameraStatus) itemconfig cameraStatus -fill red
	}
	set camera_ready_ $flag
    }

    # set the flash widget on/off

    public method set_flash {on} {
	if {$on} {
	    flash 1
	    eval $itk_component(cameraStatus) itemconfigure flash -fill green4
	} else {
	    flash 0
	}
    }

    # add command which is evaluated whenever image transformations or
    # cut levels change

    method add_updateValues { cmd } {
	lappend updateValues_ $cmd
    }

    # set the cut levels automatically using median filtering...

    public method auto_set_cut_levels {} {
        busy {$image_ autocut -percent 99.5}
        updateValues
    }

    # redefine method updateValues

    public method updateValues {} {
	rtd::RtdImagePanel::updateValues
	lassign [$image_ cut] low high
	$itk_component(low) config -value $low
	$itk_component(high) config -value $high
	evalCmds $updateValues_
    }

    method add_transCmd { cmd } {
	lappend transCmd_ $cmd
    }

    method transCmd {args} {
	$itk_component(trans) update_trans
	$itk_component(trans2) update_trans
	evalCmds $transCmd_
    }

    # evaluate user commands

    public method evalCmds {args} {
	if {"$args" == ""} { return }
	foreach cmd [lindex $args 0] {
	    if {[catch {eval $cmd} msg]} {
		rtdbFeedback $msg
	    }
	}
    }

    # -- options --

    # -- protected vars -- 

    protected variable tableIdx_ 1
    protected variable camera_ready_ 1

    protected variable updateValues_ ""
    protected variable transCmd_ ""
}

# ___oOo___
