#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: rtdbSpectrum.tcl,v 1.24 2005/06/19 19:36:28 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  22/10/03  created
#

#************************************************************************
#   NAME
#   rtdbSpectrum.tcl - itcl widget for displaying a graph of image data values 
#                      along a line
# 
#   SYNOPSIS
#   rtdbSpectrum <path> ?options?
# 
#   DESCRIPTION
#   rtdbSpectrum inherits RtdImageSpectrum and adds widgets to allow
#   setting the cut levels
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#   none
#
#   EXAMPLES
#
#   SEE ALSO
#   RTD documentation
#
#   BUGS     
#
#------------------------------------------------------------------------

itk::usual rtdbSpectrum {}

class ::rtdbSpectrum {
    inherit rtd::RtdImageSpectrum

    constructor {args} {
	# the constructor of RtdImageSpectrum needs the args list
	eval RtdImageSpectrum::constructor $args
    } {
	make_cutlevel_wdgs
    }

    # add widgets for setting the cut levels

    protected method make_cutlevel_wdgs {} {
	global ::rtdb
	set b $w_.buttons
	catch {pack forget $b.print $b.close}
	pack \
	    [LabelEntry $b.low -text "Low:" -command [code $this setCuts] \
		 -validate real -valuewidth 8 -labelwidth 4 -labelfont -Adobe-helvetica-bold-r-normal-*-12*] \
	    [LabelEntry $b.high -text "High:" -command [code $this setCuts] \
		 -validate real -valuewidth 8 -labelwidth 4 -labelfont -Adobe-helvetica-bold-r-normal-*-12*] \
	    [button $b.auto -text "Auto" -command [code $this auto]] \
	    $b.print \
	    $b.close \
	    -side left -expand 1 -pady 1m
	foreach but "print close auto" {
	    $b.$but config -width 10
	}

	add_short_help $b.low  {Low: Low cut value}
	add_short_help $b.high {High: High cut value}
	add_short_help $b.auto {Auto: Auto scaling}

	# restore settings from previous spectrum

	if { [info exists rtdb(spectrumLow)] } {
	    $b.low config -value $rtdb(spectrumLow)
	}
	if { [info exists rtdb(spectrumHigh)] } {
	    $b.high config -value $rtdb(spectrumHigh)
	}
	if { [info exists rtdb(spectrumAuto)] && $rtdb(spectrumAuto) } {
	    auto
	} else {
	    setCuts 
	}
    }
    
    # set cut levels

    private method setCuts { args } {
	global ::rtdb
	set b $w_.buttons
	set rtdb(spectrumLow)  [$b.low get]
	set rtdb(spectrumHigh) [$b.high get]
	if { "$rtdb(spectrumLow)" != "" } {
	    $graph_ axis configure y -min $rtdb(spectrumLow)
	    set rtdb(spectrumAuto) 0
	}
	if { "$rtdb(spectrumHigh)" != "" } {
	    $graph_ axis configure y -max $rtdb(spectrumHigh)
	    set rtdb(spectrumAuto) 0
	}
    }
    
    # set auto scaling

    public method auto { args } {
	global ::rtdb
	$graph_ axis configure y -min "" -max ""
	set rtdb(spectrumAuto) 1
    }

    # -- public vars --

    # -- protected vars --
}

# ___oOo___





