#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: rtdbStat.tcl,v 1.24 2005/06/19 19:36:29 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  15/02/00  created
# pbiereic  27/06/01  subtract fixed pattern for statistics. Use
#                     new rtdimage subcommand 'statistics noise'
#

#************************************************************************
#   NAME
#   rtdbStat.tcl - itk class to define a window for statistics computation
# 
#   SYNOPSIS
#   rtdbStat <path> ?options?
# 
#   DESCRIPTION
#   rtdbStat allows to define interactively a window for statistics computation.
#   The size and position of the window are specified with a rectangle in
#   the main window.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#   none
#
#   EXAMPLES
#
#   SEE ALSO
#   RTD documentation
#
#   BUGS     
#
#------------------------------------------------------------------------

itk::usual rtdbStat {}

class ::rtdbStat {
    inherit ::rtdbDefwin

    #  constructor: create a new instance of this class

    constructor {args} {
	eval itk_initialize $args
    }

    protected method init {} {
	::rtdbDefwin::init
	# make the control panel
        makeStatLabels
	makeStatHelp
	$itk_option(-target_image) add_postCmd [code $this notify_cmd move]
	config -notify [code $this changed]
	raise_dwin
        if {[catch {$target_image_ statistics noise 1 1 5 5}]} {
            error_dialog "Need newer Rtd version >= 2.50"
        } else {
	    set initialized_ 1
	}
	global ::$target_image_
	trace variable ${target_image_}(BIAS) w [code $this notify_cmd move]
    }

    # make the window to display the statistics in the given frame

    protected method makeStatLabels {} {
        # frame for labels
        itk_component add labelf {
	    frame $w_.labelf \
		    -bd 2 -relief groove
	}
        pack $itk_component(labelf) -side top -fill none -expand 0

        set i 0
	foreach el $elements_ {
	    # convert 1'st char to upper case
	    set lab [string toupper [string range $el 0 0]]
	    append lab [string range $el 1 [string length $el]]
	    
	    itk_component add $el {
		entry $itk_component(labelf).$el -width $itk_option(-valuewidth) \
			-relief groove \
			-justify left \
			-state disabled
	    }
	    itk_component add $el\lab {
		label $itk_component(labelf).$el\lab -text "$lab:" \
			-width 8 \
			-relief flat \
			-anchor w
	    }
	    blt::table $itk_component(labelf) $itk_component($el\lab) $i,0 \
		    -fill x -anchor w -padx 1
	    blt::table $itk_component(labelf) $itk_component($el) $i,1 \
		    -fill x -anchor w -padx 1 -columnspan 2
	    incr i
        }
	pack $itk_component(buttons) -after $itk_component(labelf)
    }

    public method raise_dwin {} {
	rtdbDefwin::raise_dwin
	notify_cmd move
    }

    # short help text

    protected method makeStatHelp {} {
        add_short_help $itk_component(max) {Max. value of statistics image area}
        add_short_help $itk_component(min) {Min. value of statistics image area}
        add_short_help $itk_component(average) {Average value of statistics image area}
        add_short_help $itk_component(stdDev) {Standard Deviation of statistics image area}
        add_short_help $itk_component(samples) {Number of samples taken for statistics}
	add_short_help $itk_component(cancel) \
		{CANCEL: {bitmap b1} = Cancel this window}
	add_short_help $itk_component(default) \
		{DEFAULT: {bitmap b1} = Set the window to the center of the image with a default size.}
    }

    public method changed {args} {
	update
	if { [$target_image_ isclear] || ! $initialized_ } { return }
	lassign $args op x0 y0 nx ny
        # compute min, max, avarage, RMS
	set offs 0
	lassign [rtdbChipToImage $x0 $y0 1 1 $target_image_] x0 y0 dum dum
	lassign [$target_image_ statistics noise $x0 $y0 $nx $ny] min max average stdDev samples
	set list "$min $max $average $stdDev $samples"
        set i 0
        foreach el $elements_ {
            if {[info exists itk_component($el)]} {
                $itk_component($el) config -state normal
                $itk_component($el) delete 0 end
                $itk_component($el) insert 0 [lindex $list $i]
                $itk_component($el) config -state disabled
            }
            incr i
        }
    }

    # -- public vars --

    # -- protected vars --

    # list of elements to be calculated
    protected variable elements_ "min max average stdDev samples"

    # initialized, bool
    protected variable initialized_ 0
}

# ___oOo___
