#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: rtdbPubGen.tcl,v 1.24 2005/06/19 19:36:28 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  15/02/00  created

#************************************************************************
#   NAME
#   rtdbPubGen.tcl - general public library for the IRTD base module
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#        rtdbIsVideoImage {}
#              Return 1 when the current image is a real-time image else 0
#
#        rtdbIsImageCleared {}
#              Return 1 when the image is cleared else 0
#
#        rtdbCamera {}
#              return 1 when the camera name was set else 0
#
#        rtdbGetDataDir {}
#              return the data directory of the INS user
#
#        rtdbRtdCmd {cmd}
#              Evaluate a command executed by the main widget
#
#        rtdbPopup {pathName}
#              Popup a window and return 1 when ok else 0
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------

# popup a window

proc rtdbPopup {win} {
    if {! [winfo exists $win]} { return 0 }
    if {[winfo viewable $win]} {
	raise $win
    } else {
	wm deiconify $win
    }
    return 1
}

# is main image a video image?

proc rtdbIsVideoImage {} {
    global ::rtdb

    if {"[$rtdb(RtdImage) cget -file]" != "" || [$rtdb(RtdImage) isclear]} {
	return 0
    }
    return 1
}

# check if Rtd image is cleared

proc rtdbIsImageCleared {} {
    global ::rtdb
    return [$rtdb(RtdImage) isclear]
}

# return the data directory of the INS user

proc rtdbGetDataDir {} {
    global ::env

    if {! [info exists env(INS_ROOT)] || ! [info exists env(INS_USER)]} {
	return "."
    }
    set dir $env(INS_ROOT)/$env(INS_USER)/DETDATA
    if {[file isdirectory $dir]} {
	return $dir
    }
    return "."
}

# return 1 if env. RTD_CAMERA was set else 0

proc rtdbCamera {} {
    global ::env
    return [info exists env(RTD_CAMERA)]
}

# call a method of the main rtdbRtd image widget

proc rtdbRtdCmd {args} {
    global ::rtdb
    eval $rtdb(Rtd) $args
}

# call a proc/method invoked via the SCRIPT command

proc rtdbCall {args} {
    global ::rtdb

    rtdbFeedback "Received command:\t $args"
    set rtdb(asyncCmd) {}
    eval $args
    if {"$rtdb(asyncCmd)" == ""} {
	tkwait variable rtdb(asyncCmd)
    }
    rtdbFeedback "Returning reply:\t $rtdb(asyncCmd)"
    return $rtdb(asyncCmd)
}

# set result from a SCRIPT command

proc rtdbReply {result} {
    global ::rtdb
    set rtdb(asyncCmd) $result
}

# return additional options from run string; w is the object name.

proc rtdbArgs {w} {
    global ::argv ::rtdb

    if { ! [info exists argv] } { return "" }

    set arglist ""
    foreach el [$w configure] {
	append arglist "[lindex $el 0] "
    }
    set argvs ""
    set i 0
    set n 1
    set opts "$argv $rtdb(STRTOPT)"
    foreach el $opts {
	if {[set i [expr {! $i}]]} {
	    if {[lsearch -exact $arglist $el] != -1} {
		append argvs "$el [lindex $opts $n] "
	    }
	}
	incr n
    }
    return $argvs
}

# ___oOo___
