#******************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifChartConfig.tcl,v 3.49 2005/09/07 11:46:00 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin  11/12/96  created
#

class uifChartConfig {
   inherit itk::Widget
   constructor { w args } {} 
   
   itk_option define -topLevel  topLevel  TopLevel  "." 
   
   public method applyConfig   {}
   public method okConfig      {}
   public method defaultConfig {}
   public method cancelConfig  {}
   public method getValue      {}
   public method getIdx        {}
   public method insertValue   {}
   public method viewList      {} 
      
   protected variable val         ""
   protected variable idx         0
   protected variable wdg         ""
   protected variable defaultList ""    
}



#---------------------------------------------------------------------------
# Constructor
#---------------------------------------------------------------------------

body uifChartConfig::constructor { w args } {

   set wdg $w
   set helv14 "-Adobe-helvetica-bold-r-normal--14*"

   itk_component add f {
      frame $itk_interior.f -borderwidth 2 -relief groove
   }
   
   itk_component add options {
      UifListBox $itk_component(f).options -width 25 -height 5 \
                 -font "-Adobe-courier-medium-r-normal--12*" 
   } {
   }
           
   itk_component add value {
      UifEntry $itk_component(f).value -label "Option Value" -width 30 
   }
   
   itk_component add fbuttons {
      frame $itk_component(f).fbuttons -borderwidth 2 -relief raised
   }
           
   itk_component add apply {
      button $itk_component(fbuttons).apply -text "Apply" -width 6\
      -command "$this applyConfig" -font $helv14
   } {
   } 
            
   itk_component add ok {
      button $itk_component(fbuttons).ok -text "OK" -width 6\
      -command "$this okConfig" -font $helv14
   } {
   } 
   
   itk_component add cancel {
      button $itk_component(fbuttons).cancel -text "Cancel" -width 6\
      -command "$this cancelConfig" -font $helv14
   } {
   }
   
   itk_component add default {
      button $itk_component(fbuttons).default -text "Default" -width 9\
      -command "$this defaultConfig" -font $helv14
   } {
   } 
      
   pack $itk_component(f) -fill both -expand 1
   pack $itk_component(options)  -expand 1 -fill both -pady 5
   pack $itk_component(value)    -anchor w -pady 5
   pack $itk_component(fbuttons) -fill x
   pack $itk_component(apply) $itk_component(ok) \
        $itk_component(default) $itk_component(cancel) \
        -side left -expand 1  -pady 10
   
  
   $itk_component(options) setBind <1> "$this getValue; $this getIdx"

   set w [$itk_component(value) component uifEntry]
   bind $w  <Return>  "$this insertValue"

#   configure -font "-Adobe-courier-medium-r-normal--12*"  
   eval itk_initialize $args ;  # User's configuration
}


#---------------------------------------------------------------------------
# METHOD : getIdx
#---------------------------------------------------------------------------

body uifChartConfig::getIdx {} {
   set idx [$itk_component(options) getIndex]
}

#---------------------------------------------------------------------------
# METHOD : getValue
#---------------------------------------------------------------------------

body uifChartConfig::getValue {} {

   set val [$itk_component(options) getSelection]
   $itk_component(value) delete 0 end
   $itk_component(value) insert 0 [lindex $val 1]
}

#---------------------------------------------------------------------------
# METHOD : insertValue
#---------------------------------------------------------------------------

body uifChartConfig::insertValue {} {

   set optName  [lindex $val 0]
   set optValue [$itk_component(value) get]
   $itk_component(options) deleteItem $idx $idx 
   $itk_component(options) addItem $idx [myFormat [list $optName $optValue]]
}

#---------------------------------------------------------------------------
# METHOD : viewList
#---------------------------------------------------------------------------

body uifChartConfig::viewList {} {

   set optionsList [list valueUpperLim valueLowerLim datasource samplingTime \
   			 minY maxY updateMode updateProc dataType]
		
   foreach option $optionsList {
      set array($option) "$option [$wdg cget -$option]"
   }
		
   $itk_component(options) updateSortList "-increasing" "-ascii" array myFormat
   
   set defaultList [$itk_component(options) getList]   
}


#---------------------------------------------------------------------------
# METHOD : applyConfig
#---------------------------------------------------------------------------

body uifChartConfig::applyConfig {} {

   set List [$itk_component(options) getList]          
   foreach elem $List {
       $wdg configure -[lindex $elem 0] [lindex $elem 1]
   }
#   $itk_component(value) delete 0 end    
}

#---------------------------------------------------------------------------
# METHOD : okConfig
#---------------------------------------------------------------------------

body uifChartConfig::okConfig {} {
   $this applyConfig
   $this cancelConfig
}

#---------------------------------------------------------------------------
# METHOD : defaultConfig
#---------------------------------------------------------------------------

body uifChartConfig::defaultConfig {} {
   $itk_component(value) delete 0 end     
   foreach elem $defaultList {
     set array($elem) "[lindex $elem 0] [lindex $elem 1]"     
     $wdg configure -[lindex $elem 0] [lindex $elem 1]     
   }
   $itk_component(options) updateSortList "-increasing" "-ascii" array myFormat   
}

#---------------------------------------------------------------------------
# METHOD : cancelConfig
#---------------------------------------------------------------------------

body uifChartConfig::cancelConfig {} {
   destroy $itk_option(-topLevel) 
   
}
   
proc myFormat { text } {        
   set name  [lindex $text 0]
   set value [lindex $text 1]
   set newText [format "%-15s %-15s" $name $value]
   return $newText
}   


