#***********************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: wpActAdd.tcl,v 1.23 1998/03/09 08:55:26 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    12/04/95  created
# gfilippi  31/05/95  set as a separate file from wpAccountLib.tcl
# gfilippi  01/06/95  save/restore pwd before/after panSelectFile
# gfilippi  01/06/95  set current position to the added item
# mcomin    21/07/95  check maximum number of activities in list
# mcomin    11/09/97  Fixed bug to get filter for file selection box
# gfilippi  09/03/98  do not skip first char: UifListBox has a new interface
#

#************************************************************************
#   NAME
#
#     wpActAdd - handle Add Activity to activity list
#
#   DESCRIPTION
#
#     Internal procedure for 'Activity Account' panel (wpActivity.pan)
#
#------------------------------------------------------------------------

#------------------------------------------------------------------------
# PROCEDURE : wpActAdd  - Select an activity from the database and
#                         add it to the list
#------------------------------------------------------------------------

proc wpActAdd { } {
   global wpUser  wpActivityDir wpDummyRec wpListWidget wpActRecord
   global wp_name shArea wpMyList wpFilter

   #
   # Add new activity to the list
   #
   if { [$wpListWidget getSize] == 20 } {
      set tmp "Cannot add more activities !"
      tk_dialog .dialog "Dialog Box" $tmp info 0 OK
      return 
   }      

   #
   # check WP index availability: if not "*" is used

   set wpIndex "${wpActivityDir}/../wpList"

   if { [file exists $wpIndex] != 1} {
      set t1 "wpActivity - WARNING"
      set q1 "$wpIndex not accessible.\n\n \"*\" will be used"
      set ans [tk_dialog .dialog $t1 $q1 info 1 OK]
      set filter "*" 
   } else {
      #
      # define the list box panel 
      toplevel     .subRoot    
      set c        .subRoot
      set wpMyList .subRoot

      frame $c.list
      frame $c.bot
       
      pack  $c.list $c.bot -fill both

      UifListBox $c.list.box -geometry 80x30 -scrollx 1
                          
      button $c.bot.apply  -text Apply -width 12 \
                           -command wpApplyList
                                                                
      button $c.bot.cancel -text Cancel -width 12 \
                           -command { global wpFilter wpMyList; \
                                       set wpFilter "" ; destroy $wpMyList }
    
      pack $c.list.box -padx 5 -pady 10
      pack $c.bot.apply $c.bot.cancel -side left -expand 1 -pady 5
    
      #
      #  load the list to be displayed from $wpActivityDir/../wpList    
      set myList ""
      set fh [open $wpIndex r];
      while {[gets $fh line] >= 0} {
         append myList "\{$line\} "
      }
      close $fh

      $c.list.box  deleteItem  0 end
      $c.list.box  addItemList 0 $myList
 
      #
      # post the panel on the display and wait for user choice
      wm title $c "wp - Set Activity Filter"
      tkwait window $c

      #
      # check user's choice (only the first 10 chars are meaningful)

      if { [ string length $wpFilter] < 10} { return } 

      set filter [string range $wpFilter 0 9]
      if { $filter == "          " } { return }

   } ; # end if file exists $wpIndex

   set filter [string trim $filter]

   #
   # prompt the user to select the activity to be added to the list
   
   set home [exec pwd]
   set tmp [panSelectFile $wpActivityDir $filter]
   cd  $home
   
   if { [ string length $tmp ] == 0 } { update idletasks ; return }

   set newAct [file tail $tmp];
   set pos [$wpListWidget getSize]

   # Check if activity is already in the list

   if { [lsearch [$wpListWidget getList] $newAct] != "-1" } {
      set tmp "Activity  $newAct  \nALREADY IN THE LIST !"
      tk_dialog .dialog "Dialog Box" $tmp info 0 OK
      return 
   }

   $wpListWidget addItem end $newAct  
   set wpActRecord($pos) $wpDummyRec 
   saveRecField $pos $wp_name $newAct

   #
   # Set the currently selected to the just added one
   $wpListWidget setCursor end        
   wpSetSelection
   
#   puts "DBG> Add activity $newAct"
#   showData
}

#----------------------------------------------------------------------------
# PROCEDURE : wpApplyList  -  execute "Apply"
#----------------------------------------------------------------------------

proc wpApplyList { } {
   global wpFilter wpMyList
   set wpFilter [$wpMyList.list.box getSelection]
   destroy $wpMyList 
}

#
# ___oOo___
