#***********************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: wpMove.tcl,v 1.23 1998/03/09 08:55:28 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    12/04/95  created
# gfilippi  31/05/95  set as a separate file from wpAccountLib.tcl
#           11/09/97  Fixed bug when setting cursor position and widget color
#
#************************************************************************
#   NAME
#
#     wpMove - handle Up/Down displacement of an activity in the list
#
#   DESCRIPTION
#
#     Internal procedure for 'Activity Account' panel (wpActivity.pan)
#
#------------------------------------------------------------------------
#

#------------------------------------------------------------------------
# PROCEDURE : wpSetSelection  - Sets the current selection
#
#------------------------------------------------------------------------

proc wpSetSelection { } {
   global cv wpLastSelection wpListWidget wpEntryBg wpOutputBg

   # Check if selection is the same as 'before'. This typically happens
   # by double clicking on an activity in the list.

   if {[set idx [$wpListWidget getIndex]] == ""} { return }
   if {$wpLastSelection == $idx}                 { return }

   # Configure last selected entry widgets to old configuration

   set idx $wpLastSelection ; incr idx

   for { set i 1 } {$i < 8} {incr i} {
       $cv.cell_${i}_${idx} configure -background $wpEntryBg -foreground Black
   }
  
   $cv.cell_8_${idx} configure -background $wpOutputBg -color Black

   # Configure current selected entry widgets

   set wpLastSelection [$wpListWidget getIndex];
   set idx $wpLastSelection ; incr idx  

   for { set i 1 } {$i < 8} {incr i} {
       $cv.cell_${i}_${idx} config -background lavender -foreground Red
   }

   $cv.cell_8_${idx} configure -background lavender -color Red
}



#------------------------------------------------------------------------
# PROCEDURE : wpMoveUp  - Moves a list element 1 position up
#
#------------------------------------------------------------------------

proc wpMoveUp { } {
   global wpListWidget wpActRecord

   # Get Selected item and check if it is the first in the list

   if { [set pos [$wpListWidget getIndex]] == 0} { return }
   if { $pos == "" }                             { return }

   set newPos [expr $pos - 1]

   # Swap the two elements

   set tmp                  $wpActRecord($newPos)
   set wpActRecord($newPos) $wpActRecord($pos)
   set wpActRecord($pos)    $tmp

#  puts "DBG > wpActRecord($newPos) $wpActRecord($newPos)"
   wpUpdateRowSwap $newPos $pos
   $wpListWidget   setCursor $newPos
   wpSetSelection 
#   showData
}

#------------------------------------------------------------------------
# PROCEDURE : wpMoveDown  - Moves a list element 1 position down
#
#------------------------------------------------------------------------

proc wpMoveDown { } {
   global wpListWidget wpActRecord

   # Get Selected item and check if it is the first in the list

   set max [$wpListWidget getSize]
   if {[set pos [$wpListWidget getIndex]]  == $max} { return }
   if { $pos == "" }                                { return }

   set newPos [expr $pos + 1]

   if { $newPos == [$wpListWidget getSize] } { return }

   # Swap the two elements

   set tmp                  $wpActRecord($newPos)
   set wpActRecord($newPos) $wpActRecord($pos)
   set wpActRecord($pos)    $tmp

#   puts "DBG > wpActRecord($newPos) $wpActRecord($newPos)"
   wpUpdateRowSwap $pos $newPos

   $wpListWidget setCursor $newPos 
   wpSetSelection 
#   showData
}


#------------------------------------------------------------------------
# PROCEDURE : wpUpdateRowSwap  - Moves a list element 1 position up
#
# <pos1>,<pos2> are the two positions to swap and must be pos1 < pos2
#------------------------------------------------------------------------

proc wpUpdateRowSwap { pos1 pos2 } {

   global gvar wpListWidget wp_name

   # Delete list elements

   $wpListWidget deleteItem $pos1 $pos2

   # Replace elements

   $wpListWidget addItem $pos1 [getRecField $pos1 $wp_name]
   $wpListWidget addItem $pos2 [getRecField $pos2 $wp_name]

   wpSetRowValues $pos1
   wpSetRowValues $pos2
}

#------------------------------------------------------------------------
# PROCEDURE : wpSetRowValues  - Initialize the values of a row
#------------------------------------------------------------------------

proc wpSetRowValues { pos } {

   global gvar 
   global wpActRecord

   set values $wpActRecord($pos) ; # Get list of values
   incr pos

   for { set i 1 } {$i < 9} {incr i} {
       if { [set val [lindex $values $i]] == "0" } { set val "" }
       set gvar(cell_${i}_${pos}) $val
   }
}

#----------------------------------------------------------------------------
# PROCEDURE : getRecField - 
#----------------------------------------------------------------------------

proc getRecField  { name i } {
   global  wpActRecord 
   return  [lindex $wpActRecord($name)  $i]
}

#----------------------------------------------------------------------------
# PROCEDURE : saveRecField - 
#----------------------------------------------------------------------------

proc saveRecField  { name i value } {
   global wpActRecord
   set list               $wpActRecord($name)  
   set wpActRecord($name) [lreplace  $list $i $i $value ] 
}


#
# ___oOo___
